@@ -1,19 +1,15 @@
-#ifndef __LYNX_H__
-#define __LYNX_H__
+#ifndef LYNX_H
+#define LYNX_H
+
 #include "driver.h"
+#include "sound/custom.h"
 
 extern UINT32 lynx_palette[0x10];
 
-int lynx_vh_start(void);
-void lynx_vh_stop(void);
-void lynx_vh_screenrefresh (struct mame_bitmap *bitmap, int full_refresh);
 void lynx_draw_lines(int newline);
 
-
-extern UINT32 lynx_partialcrc(const unsigned char *,unsigned int);
-
-extern int debug_pos;
-extern char debug_strings[16][30];
+void lynx_partialhash(char *dest, const unsigned char *data,
+	unsigned long length, unsigned int functions);
 
 
 #define PAD_UP 0x80
@@ -21,7 +17,14 @@ extern char debug_strings[16][30];
 #define PAD_LEFT 0x20
 #define PAD_RIGHT 0x10
 
-extern void lynx_machine_init(void);
+MACHINE_INIT( lynx );
+DRIVER_INIT( lynx );
+
+extern UINT8 *lynx_mem_0000;
+extern UINT8 *lynx_mem_fc00;
+extern UINT8 *lynx_mem_fd00;
+extern UINT8 *lynx_mem_fe00;
+extern UINT8 *lynx_mem_fffa;
 
 extern UINT16 lynx_granularity;
 extern int lynx_rotate;
@@ -29,32 +32,26 @@ extern int lynx_rotate;
 typedef struct {
 	UINT8 data[0x100];
 } MIKEY;
+
 extern MIKEY mikey;
-WRITE_HANDLER( lynx_memory_config );
-WRITE_HANDLER(mikey_write);
-READ_HANDLER(mikey_read);
-WRITE_HANDLER(suzy_write);
-READ_HANDLER(suzy_read);
+READ8_HANDLER( lynx_memory_config_r );
+WRITE8_HANDLER( lynx_memory_config_w );
+WRITE8_HANDLER(mikey_write);
+READ8_HANDLER(mikey_read);
+WRITE8_HANDLER(suzy_write);
+READ8_HANDLER(suzy_read);
 void lynx_timer_count_down(int nr);
 
-#ifdef RUNTIME_LOADER
-# ifdef __cplusplus
-extern "C" void lynx_runtime_loader_init(void);
-# else
-extern void lynx_runtime_loader_init(void);
-# endif
-#endif
-
-void lynx_audio_debug(struct mame_bitmap *bitmap);
 void lynx_audio_reset(void);
 void lynx_audio_write(int offset, UINT8 data);
 UINT8 lynx_audio_read(int offset);
 void lynx_audio_count_down(int nr);
-extern int lynx_custom_start (const struct MachineSound *driver);
-extern int lynx2_custom_start (const struct MachineSound *driver);
-extern void lynx_custom_stop (void);
-extern void lynx_custom_update (void);
-#endif
+void *lynx_custom_start(int clock, const struct CustomSound_interface *config);
+void *lynx2_custom_start(int clock, const struct CustomSound_interface *config);
+void lynx_custom_update (void);
+
+#endif /* LYNX_H */
+
 
 #ifdef INCLUDE_LYNX_LINE_FUNCTION
 	int j, xi, wi, i;
