@@ -10,7 +10,9 @@
 
 #include <ctype.h>
 #include "driver.h"
+#include "image.h"
 #include "cpu/m6502/m6502.h"
+#include "sound/sid6581.h"
 
 #define VERBOSE_DBG 1
 #include "includes/cbm.h"
@@ -19,7 +21,6 @@
 #include "includes/vc1541.h"
 #include "includes/vc20tape.h"
 #include "includes/ted7360.h"
-#include "includes/sid6581.h"
 
 #include "includes/c16.h"
 
@@ -28,7 +29,7 @@ static UINT8 keyline[10] =
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
 };
 
-static int rom_specified[2] = {0};
+static mame_file *rom_fp[2] = {0};
 
 /*
  * tia6523
@@ -43,11 +44,10 @@ static int rom_specified[2] = {0};
  * 7 ack input edge ready for next datum
  */
 
-static UINT8 port6529, port7501, ddr7501;
+static UINT8 port6529;
 
 static int lowrom = 0, highrom = 0;
 
-UINT8 *c16_memory;
 static UINT8 *c16_memory_10000;
 static UINT8 *c16_memory_14000;
 static UINT8 *c16_memory_18000;
@@ -57,6 +57,8 @@ static UINT8 *c16_memory_24000;
 static UINT8 *c16_memory_28000;
 static UINT8 *c16_memory_2c000;
 
+static int c16_rom_load(mess_image *img);
+
 /**
   ddr bit 1 port line is output
   port bit 1 port line is high
@@ -80,21 +82,10 @@ static UINT8 *c16_memory_2c000;
   p6 serial clock in
   p7 serial data in, serial bus 5
  */
-WRITE_HANDLER(c16_m7501_port_w)
+void c16_m7501_port_write(UINT8 data)
 {
 	int dat, atn, clk;
 
-	if (offset)
-	{
-		if (port7501 != data)
-			port7501 = data;
-	}
-	else
-	{
-		if (ddr7501 != data)
-			ddr7501 = data;
-	}
-	data = (port7501 & ddr7501) | (ddr7501 ^ 0xff);
 	/* bit zero then output 0 */
 	cbm_serial_atn_write (atn = !(data & 4));
 	cbm_serial_clock_write (clk = !(data & 2));
@@ -103,69 +94,66 @@ WRITE_HANDLER(c16_m7501_port_w)
 	vc20_tape_motor (data & 8);
 }
 
-READ_HANDLER(c16_m7501_port_r)
+UINT8 c16_m7501_port_read(void)
 {
-	if (offset)
-	{
-		int data = (ddr7501 & port7501) | (ddr7501 ^ 0xff);
+	UINT8 data = 0xFF;
+	UINT8 c16_port7501 = (UINT8) cpunum_get_info_int(0, CPUINFO_INT_M6510_PORT);
 
-		if (!(ddr7501 & 0x80)
-			&& (((ddr7501 & 1) && (port7501 & 1)) || !cbm_serial_data_read ()))
-			data &= ~0x80;
-		if (!(ddr7501 & 0x40)
-			&& (((ddr7501 & 2) && (port7501 & 2)) || !cbm_serial_clock_read ()))
-			data &= ~0x40;
-		if (!(ddr7501 & 0x10) && !vc20_tape_read ())
-			data &= ~0x10;
-/*      data&=~0x20; //port bit not in pinout */
-		return data;
-	}
-	else
-	{
-		return ddr7501;
-	}
+	if ((c16_port7501 & 0x01) || !cbm_serial_data_read())
+		data &= ~0x80;
+
+	if ((c16_port7501 & 0x02) || !cbm_serial_clock_read())
+		data &= ~0x40;
+
+	if (!vc20_tape_read())
+		data &= ~0x10;
+
+/*	data &= ~0x20; //port bit not in pinout */
+	return data;
 }
 
 static void c16_bankswitch (void)
 {
+	memory_set_bankptr(9, mess_ram);
+
 	switch (lowrom)
 	{
 	case 0:
-		cpu_setbank (2, c16_memory + 0x10000);
+		memory_set_bankptr (2, memory_region(REGION_CPU1) + 0x10000);
 		break;
 	case 1:
-		cpu_setbank (2, c16_memory + 0x18000);
+		memory_set_bankptr (2, memory_region(REGION_CPU1) + 0x18000);
 		break;
 	case 2:
-		cpu_setbank (2, c16_memory + 0x20000);
+		memory_set_bankptr (2, memory_region(REGION_CPU1) + 0x20000);
 		break;
 	case 3:
-		cpu_setbank (2, c16_memory + 0x28000);
+		memory_set_bankptr (2, memory_region(REGION_CPU1) + 0x28000);
 		break;
 	}
 	switch (highrom)
 	{
 	case 0:
-		cpu_setbank (3, c16_memory + 0x14000);
-		cpu_setbank (8, c16_memory + 0x17f20);
+		memory_set_bankptr (3, memory_region(REGION_CPU1) + 0x14000);
+		memory_set_bankptr (8, memory_region(REGION_CPU1) + 0x17f20);
 		break;
 	case 1:
-		cpu_setbank (3, c16_memory + 0x1c000);
-		cpu_setbank (8, c16_memory + 0x1ff20);
+		memory_set_bankptr (3, memory_region(REGION_CPU1) + 0x1c000);
+		memory_set_bankptr (8, memory_region(REGION_CPU1) + 0x1ff20);
 		break;
 	case 2:
-		cpu_setbank (3, c16_memory + 0x24000);
-		cpu_setbank (8, c16_memory + 0x27f20);
+		memory_set_bankptr (3, memory_region(REGION_CPU1) + 0x24000);
+		memory_set_bankptr (8, memory_region(REGION_CPU1) + 0x27f20);
 		break;
 	case 3:
-		cpu_setbank (3, c16_memory + 0x2c000);
-		cpu_setbank (8, c16_memory + 0x2ff20);
+		memory_set_bankptr (3, memory_region(REGION_CPU1) + 0x2c000);
+		memory_set_bankptr (8, memory_region(REGION_CPU1) + 0x2ff20);
 		break;
 	}
-	cpu_setbank (4, c16_memory + 0x17c00);
+	memory_set_bankptr (4, memory_region(REGION_CPU1) + 0x17c00);
 }
 
-WRITE_HANDLER(c16_switch_to_rom)
+WRITE8_HANDLER(c16_switch_to_rom)
 {
 	ted7360_rom = 1;
 	c16_bankswitch ();
@@ -184,7 +172,7 @@ WRITE_HANDLER(c16_switch_to_rom)
  * 0  1  plus4 hi
  * 1  0  c1 high
  * 1  1  c2 high */
-WRITE_HANDLER(c16_select_roms)
+WRITE8_HANDLER(c16_select_roms)
 {
 	lowrom = offset & 3;
 	highrom = (offset & 0xc) >> 2;
@@ -192,39 +180,13 @@ WRITE_HANDLER(c16_select_roms)
 		c16_bankswitch ();
 }
 
-WRITE_HANDLER(c16_switch_to_ram)
+WRITE8_HANDLER(c16_switch_to_ram)
 {
 	ted7360_rom = 0;
-	switch (DIPMEMORY)
-	{
-	case MEMORY64K:
-		cpu_setbank (2, c16_memory + 0x8000);
-		cpu_setbank (3, c16_memory + 0xc000);
-		cpu_setbank (4, c16_memory + 0xfc00);
-		cpu_setbank (8, c16_memory + 0xff20);
-		break;
-	case MEMORY32K:
-		cpu_setbank (2, c16_memory);
-		cpu_setbank (3, c16_memory + 0x4000);
-		cpu_setbank (4, c16_memory + 0x7c00);
-		cpu_setbank (8, c16_memory + 0x7f20);
-		break;
-	case MEMORY16K:
-		cpu_setbank (2, c16_memory);
-		cpu_setbank (3, c16_memory);
-		cpu_setbank (4, c16_memory + 0x3c00);
-		cpu_setbank (8, c16_memory + 0x3f20);
-		break;
-	}
-}
-
-WRITE_HANDLER(plus4_switch_to_ram)
-{
-	ted7360_rom = 0;
-	cpu_setbank (2, c16_memory + 0x8000);
-	cpu_setbank (3, c16_memory + 0xc000);
-	cpu_setbank (4, c16_memory + 0xfc00);
-	cpu_setbank (8, c16_memory + 0xff20);
+	memory_set_bankptr (2, mess_ram + (0x8000 % mess_ram_size));
+	memory_set_bankptr (3, mess_ram + (0xc000 % mess_ram_size));
+	memory_set_bankptr (4, mess_ram + (0xfc00 % mess_ram_size));
+	memory_set_bankptr (8, mess_ram + (0xff20 % mess_ram_size));
 }
 
 int c16_read_keyboard (int databus)
@@ -271,12 +233,12 @@ int c16_read_keyboard (int databus)
  * output low means keyboard line selected
  * keyboard line is then read into the ted7360 latch
  */
-WRITE_HANDLER(c16_6529_port_w)
+WRITE8_HANDLER(c16_6529_port_w)
 {
 	port6529 = data;
 }
 
-READ_HANDLER(c16_6529_port_r)
+ READ8_HANDLER(c16_6529_port_r)
 {
 	return port6529 & (c16_read_keyboard (0xff /*databus */ ) | (port6529 ^ 0xff));
 }
@@ -291,11 +253,11 @@ READ_HANDLER(c16_6529_port_r)
  * p6 Userport j
  * p7 Userport f
  */
-WRITE_HANDLER(plus4_6529_port_w)
+WRITE8_HANDLER(plus4_6529_port_w)
 {
 }
 
-READ_HANDLER(plus4_6529_port_r)
+ READ8_HANDLER(plus4_6529_port_r)
 {
 	int data = 0;
 
@@ -304,7 +266,7 @@ READ_HANDLER(plus4_6529_port_r)
 	return data;
 }
 
-READ_HANDLER(c16_fd1x_r)
+ READ8_HANDLER(c16_fd1x_r)
 {
 	int data = 0;
 
@@ -350,14 +312,14 @@ READ_HANDLER(c16_fd1x_r)
    1111 19200
  control register
   */
-WRITE_HANDLER(c16_6551_port_w)
+WRITE8_HANDLER(c16_6551_port_w)
 {
 	offset &= 3;
 	DBG_LOG (3, "6551", ("port write %.2x %.2x\n", offset, data));
 	port6529 = data;
 }
 
-READ_HANDLER(c16_6551_port_r)
+ READ8_HANDLER(c16_6551_port_r)
 {
 	int data = 0;
 
@@ -366,42 +328,12 @@ READ_HANDLER(c16_6551_port_r)
 	return data;
 }
 
-static WRITE_HANDLER(c16_write_3f20)
+static READ8_HANDLER(ted7360_dma_read)
 {
-	c16_memory[0x3f20 + offset] = data;
+	return mess_ram[offset % mess_ram_size];
 }
 
-static WRITE_HANDLER(c16_write_3f40)
-{
-	c16_memory[0x3f40 + offset] = data;
-}
-
-static WRITE_HANDLER(c16_write_7f20)
-{
-	c16_memory[0x7f20 + offset] = data;
-}
-
-static WRITE_HANDLER(c16_write_7f40)
-{
-	c16_memory[0x7f40 + offset] = data;
-}
-
-static READ_HANDLER(ted7360_dma_read_16k)
-{
-	return c16_memory[offset & 0x3fff];
-}
-
-static READ_HANDLER(ted7360_dma_read_32k)
-{
-	return c16_memory[offset & 0x7fff];
-}
-
-static READ_HANDLER(ted7360_dma_read)
-{
-	return c16_memory[offset];
-}
-
-static READ_HANDLER(ted7360_dma_read_rom)
+static  READ8_HANDLER(ted7360_dma_read_rom)
 {
 	/* should read real c16 system bus from 0xfd00 -ff1f */
 	if (offset >= 0xc000)
@@ -434,16 +366,8 @@ static READ_HANDLER(ted7360_dma_read_rom)
 			return c16_memory_28000[offset & 0x7fff];
 		}
 	}
-	switch (DIPMEMORY)
-	{
-	case MEMORY16K:
-		return c16_memory[offset & 0x3fff];
-	case MEMORY32K:
-		return c16_memory[offset & 0x7fff];
-	case MEMORY64K:
-		return c16_memory[offset];
-	}
-	exit (0);
+
+	return mess_ram[offset % mess_ram_size];
 }
 
 void c16_interrupt (int level)
@@ -453,7 +377,7 @@ void c16_interrupt (int level)
 	if (level != old_level)
 	{
 		DBG_LOG (3, "mos7501", ("irq %s\n", level ? "start" : "end"));
-		cpu_set_irq_line (0, M6510_IRQ_LINE, level);
+		cpunum_set_input_line (0, M6510_IRQ_LINE, level);
 		old_level = level;
 	}
 }
@@ -465,6 +389,10 @@ static void c16_common_driver_init (void)
 #endif
 	C1551_CONFIG config= { 1 };
 
+	/* configure the M7501 port */
+	cpunum_set_info_fct(0, CPUINFO_PTR_M6510_PORTREAD, (genf *) c16_m7501_port_read);
+	cpunum_set_info_fct(0, CPUINFO_PTR_M6510_PORTWRITE, (genf *) c16_m7501_port_write);
+
 	c16_select_roms (0, 0);
 	c16_switch_to_rom (0, 0);
 
@@ -488,28 +416,23 @@ static void c16_common_driver_init (void)
 	tpi6525[3].c.read=c1551_1_read_handshake;
 	tpi6525[3].c.output=c1551_1_write_handshake;
 
-	c16_memory_10000 = c16_memory + 0x10000;
-	c16_memory_14000 = c16_memory + 0x14000;
-	c16_memory_18000 = c16_memory + 0x18000;
-	c16_memory_1c000 = c16_memory + 0x1c000;
-	c16_memory_20000 = c16_memory + 0x20000;
-	c16_memory_24000 = c16_memory + 0x24000;
-	c16_memory_28000 = c16_memory + 0x28000;
-	c16_memory_2c000 = c16_memory + 0x2c000;
+	c16_memory_10000 = memory_region(REGION_CPU1) + 0x10000;
+	c16_memory_14000 = memory_region(REGION_CPU1) + 0x14000;
+	c16_memory_18000 = memory_region(REGION_CPU1) + 0x18000;
+	c16_memory_1c000 = memory_region(REGION_CPU1) + 0x1c000;
+	c16_memory_20000 = memory_region(REGION_CPU1) + 0x20000;
+	c16_memory_24000 = memory_region(REGION_CPU1) + 0x24000;
+	c16_memory_28000 = memory_region(REGION_CPU1) + 0x28000;
+	c16_memory_2c000 = memory_region(REGION_CPU1) + 0x2c000;
 
-	/*    memset(c16_memory, 0, 0xfd00); */
 	/* need to recognice non available tia6523's (iec8/9) */
-	memset (c16_memory + 0xfdc0, 0xff, 0x40);
+	memset(mess_ram + 0xfdc0, 0xff, 0x40);
 
 
-	memset (c16_memory + 0xfd40, 0xff, 0x20);
+	memset(mess_ram + 0xfd40, 0xff, 0x20);
 
 	c16_tape_open ();
 
-	cbm_drive_open ();
-	cbm_drive_attach_fs (0);
-	cbm_drive_attach_fs (1);
-
 	if (REAL_C1551)
 		c1551_config (0, 0, &config);
 
@@ -529,50 +452,53 @@ void c16_driver_init (void)
 void c16_driver_shutdown (void)
 {
 	vc20_tape_close ();
-	cbm_drive_close ();
 }
 
-static WRITE_HANDLER(c16_sidcart_16k)
+static WRITE8_HANDLER(c16_sidcart_16k)
 {
-	c16_memory[0x1400+offset]=data;
-	c16_memory[0x5400+offset]=data;
-	c16_memory[0x9400+offset]=data;
-	c16_memory[0xd400+offset]=data;
+	mess_ram[0x1400 + offset]=data;
+	mess_ram[0x5400 + offset]=data;
+	mess_ram[0x9400 + offset]=data;
+	mess_ram[0xd400 + offset]=data;
 	sid6581_0_port_w(offset,data);
 }
 
-static WRITE_HANDLER(c16_sidcart_32k)
+static WRITE8_HANDLER(c16_sidcart_32k)
 {
-	c16_memory[0x5400+offset]=data;
-	c16_memory[0xd400+offset]=data;
+	mess_ram[0x5400 + offset]=data;
+	mess_ram[0xd400 + offset]=data;
 	sid6581_0_port_w(offset,data);
 }
 
-static WRITE_HANDLER(c16_sidcart_64k)
+static WRITE8_HANDLER(c16_sidcart_64k)
 {
-	c16_memory[0xd400+offset]=data;
+	mess_ram[0xd400 + offset]=data;
 	sid6581_0_port_w(offset,data);
 }
 
-void c16_init_machine (void)
+MACHINE_INIT( c16 )
 {
 	int i;
 
 	tpi6525_2_reset();
 	tpi6525_3_reset();
+	c364_speech_init();
 
-	sid6581_reset(0);
-	if (SIDCARD) {
-		sid6581_set_type(0, MOS8580);
-		install_mem_read_handler (0, 0xfd40, 0xfd5f, sid6581_0_port_r);
-		install_mem_write_handler (0, 0xfd40, 0xfd5f, sid6581_0_port_w);
-		install_mem_read_handler (0, 0xfe80, 0xfe9f, sid6581_0_port_r);
-		install_mem_write_handler (0, 0xfe80, 0xfe9f, sid6581_0_port_w);
-	} else {
-		install_mem_read_handler (0, 0xfd40, 0xfd5f, MRA_NOP);
-		install_mem_write_handler (0, 0xfd40, 0xfd5f, MWA_NOP);
-		install_mem_read_handler (0, 0xfe80, 0xfe9f, MRA_NOP);
-		install_mem_write_handler (0, 0xfe80, 0xfe9f, MWA_NOP);
+	sndti_reset(SOUND_SID8580, 0);
+
+	if (SIDCARD)
+	{
+		memory_install_read8_handler (0, ADDRESS_SPACE_PROGRAM, 0xfd40, 0xfd5f, 0, 0, sid6581_0_port_r);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xfd40, 0xfd5f, 0, 0, sid6581_0_port_w);
+		memory_install_read8_handler (0, ADDRESS_SPACE_PROGRAM, 0xfe80, 0xfe9f, 0, 0, sid6581_0_port_r);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xfe80, 0xfe9f, 0, 0, sid6581_0_port_w);
+	}
+	else
+	{
+		memory_install_read8_handler (0, ADDRESS_SPACE_PROGRAM, 0xfd40, 0xfd5f, 0, 0, MRA8_NOP);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xfd40, 0xfd5f, 0, 0, MWA8_NOP);
+		memory_install_read8_handler (0, ADDRESS_SPACE_PROGRAM, 0xfe80, 0xfe9f, 0, 0, MRA8_NOP);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xfe80, 0xfe9f, 0, 0, MWA8_NOP);
 	}
 
 #if 0
@@ -581,86 +507,52 @@ void c16_init_machine (void)
 #endif
 	if (TYPE_C16)
 	{
-		cpu_setbank (1, (DIPMEMORY == MEMORY16K) ? c16_memory : c16_memory + 0x4000);
-		switch (DIPMEMORY)
-		{
-		case MEMORY16K:
-			cpu_setbank (5, c16_memory);
-			cpu_setbank (6, c16_memory);
-			cpu_setbank (7, c16_memory);
-#ifdef NEW_BANKHANDLER
-			/* causes problems to do this */
-			/* seeable with c16 and 32k ram extension */
-			install_mem_write_handler (0, 0x8000, 0xbfff, MWA_BANK6);
-			install_mem_write_handler (0, 0x4000, 0x7fff, MWA_BANK5);
-			install_mem_write_handler (0, 0xc000, 0xfcff, MWA_BANK7);
-#endif
-			install_mem_write_handler (0, 0xff20, 0xff3d, c16_write_3f20);
-			install_mem_write_handler (0, 0xff40, 0xffff, c16_write_3f40);
-			if (SIDCARD_HACK) {
-				install_mem_write_handler (0, 0xd400, 0xd41f, c16_sidcart_16k);
-			}
-			ted7360_set_dma (ted7360_dma_read_16k, ted7360_dma_read_rom);
-			break;
-		case MEMORY32K:
-#ifdef NEW_BANKHANDLER
-			install_mem_write_handler (0, 0x4000, 0x7fff, MWA_RAM);
-			cpu_setbank (5, c16_memory);
-			install_mem_write_handler (0, 0x8000, 0xfcff, MWA_BANK5);
-			cpu_setbank (6, (c16_memory + 0x7f20));
-			install_mem_write_handler (0, 0xff20, 0xff3d, MWA_BANK6);
-			cpu_setbank (7, (c16_memory + 0x7f40));
-			install_mem_write_handler (0, 0xff40, 0xffff, MWA_BANK7);
-#else
-			cpu_setbank (5, c16_memory + 0x4000);
-			cpu_setbank (6, c16_memory);
-			cpu_setbank (7, c16_memory + 0x4000);
-			install_mem_write_handler (0, 0xff20, 0xff3d, c16_write_7f20);
-			install_mem_write_handler (0, 0xff40, 0xffff, c16_write_7f40);
-#endif
-			ted7360_set_dma (ted7360_dma_read_32k, ted7360_dma_read_rom);
-			if (SIDCARD_HACK) {
-				install_mem_write_handler (0, 0xd400, 0xd41f, c16_sidcart_32k);
-			}
-			break;
-		case MEMORY64K:
-			install_mem_write_handler (0, 0x4000, 0xfcff, MWA_RAM);
-			if (SIDCARD_HACK) {
-				install_mem_write_handler (0, 0xd400, 0xd41f, c16_sidcart_64k);
-			}
-			install_mem_write_handler (0, 0xff20, 0xff3d, MWA_RAM);
-			install_mem_write_handler (0, 0xff40, 0xffff, MWA_RAM);
-			ted7360_set_dma (ted7360_dma_read, ted7360_dma_read_rom);
-			break;
-		}
-	}
-	else
-	{
-		install_mem_write_handler (0, 0x4000, 0xfcff, MWA_RAM);
-		if (SIDCARD_HACK) {
-			install_mem_write_handler (0, 0xd400, 0xd41f, c16_sidcart_64k);
-		}
+		memory_set_bankptr(1, mess_ram + (0x4000 % mess_ram_size));
+
+		memory_set_bankptr(5, mess_ram + (0x4000 % mess_ram_size));
+		memory_set_bankptr(6, mess_ram + (0x8000 % mess_ram_size));
+		memory_set_bankptr(7, mess_ram + (0xc000 % mess_ram_size));
+
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM, 0xff20, 0xff3d, 0, 0, MWA8_BANK10);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM, 0xff40, 0xffff, 0, 0, MWA8_BANK11);
+		memory_set_bankptr(10, mess_ram + (0xff20 % mess_ram_size)); 
+		memory_set_bankptr(11, mess_ram + (0xff40 % mess_ram_size)); 
+
+		if (SIDCARD_HACK)
+			memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xd400, 0xd41f, 0, 0, c16_sidcart_16k);
+
 		ted7360_set_dma (ted7360_dma_read, ted7360_dma_read_rom);
 	}
-	if (IEC8ON||REAL_C1551)
+	else
 	{
-		install_mem_write_handler (0, 0xfee0, 0xfeff, tpi6525_2_port_w);
-		install_mem_read_handler (0, 0xfee0, 0xfeff, tpi6525_2_port_r);
+		memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM, 0x4000, 0xfcff, 0, 0, MWA8_BANK10);
+		memory_set_bankptr(10, mess_ram + 0x4000);
+
+		if (SIDCARD_HACK)
+			memory_install_write8_handler (0, ADDRESS_SPACE_PROGRAM,  0xd400, 0xd41f, 0, 0, c16_sidcart_64k);
+
+		ted7360_set_dma (ted7360_dma_read, ted7360_dma_read_rom);
+	}
+
+	if (IEC8ON || REAL_C1551)
+	{
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM,  0xfee0, 0xfeff, 0, 0, tpi6525_2_port_w);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfee0, 0xfeff, 0, 0, tpi6525_2_port_r);
 	}
 	else
 	{
-		install_mem_write_handler (0, 0xfee0, 0xfeff, MWA_NOP);
-		install_mem_read_handler (0, 0xfee0, 0xfeff, MRA_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM,  0xfee0, 0xfeff, 0, 0, MWA8_NOP);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfee0, 0xfeff, 0, 0, MRA8_NOP);
 	}
 	if (IEC9ON)
 	{
-		install_mem_write_handler (0, 0xfec0, 0xfedf, tpi6525_3_port_w);
-		install_mem_read_handler (0, 0xfec0, 0xfedf, tpi6525_3_port_r);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM,  0xfec0, 0xfedf, 0, 0, tpi6525_3_port_w);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfec0, 0xfedf, 0, 0, tpi6525_3_port_r);
 	}
 	else
 	{
-		install_mem_write_handler (0, 0xfec0, 0xfedf, MWA_NOP);
-		install_mem_read_handler (0, 0xfec0, 0xfedf, MRA_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM,  0xfec0, 0xfedf, 0, 0, MWA8_NOP);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfec0, 0xfedf, 0, 0, MRA8_NOP);
 	}
 
 	if (SERIAL8ON)
@@ -688,36 +580,24 @@ void c16_init_machine (void)
 
 	cbm_serial_reset_write (0);
 
-	for (i = 0; rom_specified[i] && (i < sizeof (rom_specified) / sizeof (rom_specified[0])); i++)
-		c16_rom_load (i);
-
+	for (i = 0;  (i < sizeof (rom_fp) / sizeof (rom_fp[0])) && rom_fp[i]; i++)
+		c16_rom_load(image_from_devtype_and_index(IO_CARTSLOT, i));
 }
 
-void c16_shutdown_machine (void)
-{
-}
-
-int c16_rom_id (int id)
+static int c16_rom_id(mess_image *img, mame_file *romfile)
 {
     /* magic lowrom at offset 7: $43 $42 $4d */
 	/* if at offset 6 stands 1 it will immediatly jumped to offset 0 (0x8000) */
 	int retval = 0;
 	char magic[] = {0x43, 0x42, 0x4d}, buffer[sizeof (magic)];
-	const char *name = device_filename(IO_CARTSLOT,id);
-	FILE *romfile;
+	const char *name = image_filename(img);
 	char *cp;
 
 	logerror("c16_rom_id %s\n", name);
 	retval = 0;
-	if (!(romfile = (FILE*)image_fopen (IO_CARTSLOT, id, OSD_FILETYPE_IMAGE, 0)))
-	{
-		logerror("rom %s not found\n", name);
-		return 0;
-	}
 
-	osd_fseek (romfile, 7, SEEK_SET);
-	osd_fread (romfile, buffer, sizeof (magic));
-	osd_fclose (romfile);
+	mame_fseek (romfile, 7, SEEK_SET);
+	mame_fread (romfile, buffer, sizeof (magic));
 
 	if (memcmp (magic, buffer, sizeof (magic)) == 0)
 	{
@@ -725,9 +605,9 @@ int c16_rom_id (int id)
 	}
 	else if ((cp = strrchr (name, '.')) != NULL)
 	{
-		if ((stricmp (cp + 1, "rom") == 0) || (stricmp (cp + 1, "prg") == 0)
-			|| (stricmp (cp + 1, "bin") == 0)
-			|| (stricmp (cp + 1, "lo") == 0) || (stricmp (cp + 1, "hi") == 0))
+		if ((mame_stricmp (cp + 1, "rom") == 0) || (mame_stricmp (cp + 1, "prg") == 0)
+			|| (mame_stricmp (cp + 1, "bin") == 0)
+			|| (mame_stricmp (cp + 1, "lo") == 0) || (mame_stricmp (cp + 1, "hi") == 0))
 			retval = 1;
 	}
 
@@ -738,71 +618,60 @@ int c16_rom_id (int id)
 	return retval;
 }
 
-int c16_rom_init (int id)
+DEVICE_LOAD(c16_rom)
 {
-	rom_specified[id] = device_filename(IO_CARTSLOT,id) != NULL;
-	return rom_specified[id] && !c16_rom_id(id) ? INIT_FAIL: INIT_PASS;
+	int id = image_index_in_device(image);
+	rom_fp[id] = file;
+	return (rom_fp[id] && !c16_rom_id(image, rom_fp[id])) ? INIT_FAIL : INIT_PASS;
 }
 
-
-int c16_rom_load (int id)
+DEVICE_UNLOAD(c16_rom)
 {
-	const char *name = device_filename(IO_CARTSLOT,id);
-    UINT8 *mem = memory_region (REGION_CPU1);
-	FILE *fp;
-	int size, read;
-	char *cp;
+	int id = image_index_in_device(image);
+	rom_fp[id] = NULL;
+}
+
+static int c16_rom_load(mess_image *img)
+{
+	UINT8 *mem = memory_region (REGION_CPU1);
+	mame_file *fp = image_fp(img);
+	int size, read_;
+	const char *filetype;
 	static unsigned int addr = 0;
 
-	if (name==NULL) return 1;
-	if (!c16_rom_id (id))
+	if (fp == NULL)
+		return INIT_FAIL;
+	if (!c16_rom_id(img, fp))
 		return 1;
-	fp = (FILE*)image_fopen (IO_CARTSLOT, id, OSD_FILETYPE_IMAGE, 0);
-	if (!fp)
+
+	size = mame_fsize (fp);
+
+	filetype = image_filetype(img);	
+	if (filetype && !mame_stricmp (filetype, "prg"))
 	{
-		logerror("%s file not found\n", name);
-		return 1;
-	}
+		unsigned short in;
 
-	size = osd_fsize (fp);
-
-	if ((cp = strrchr (name, '.')) != NULL)
-	{
-		if (stricmp (cp, ".prg") == 0)
-		{
-			unsigned short in;
-
-			osd_fread_lsbfirst (fp, &in, 2);
-			logerror("rom prg %.4x\n", in);
-			addr = in+0x20000;
-			size -= 2;
-		}
+		mame_fread_lsbfirst(fp, &in, 2);
+		logerror("rom prg %.4x\n", in);
+		addr = in + 0x20000;
+		size -= 2;
 	}
 	if (addr == 0)
 	{
 		addr = 0x20000;
 	}
-	logerror("loading rom %s at %.5x size:%.4x\n", name, addr, size);
-	read = osd_fread (fp, mem + addr, size);
+	logerror("loading rom at %.5x size:%.4x\n", addr, size);
+	read_ = mame_fread (fp, mem + addr, size);
 	addr += size;
-	osd_fclose (fp);
-	if (read != size)
+	if (read_ != size)
 		return 1;
 	return 0;
 }
 
-
-int c16_frame_interrupt (void)
+INTERRUPT_GEN( c16_frame_interrupt )
 {
-	static int quickload = 0;
 	int value;
 
-	sid6581_update();
-
-	if (!quickload && QUICKLOAD)
-		cbm_quick_open (0, 0, c16_memory);
-	quickload = QUICKLOAD;
-
 	value = 0xff;
 	if (KEY_ATSIGN)
 		value &= ~0x80;
@@ -1007,6 +876,4 @@ int c16_frame_interrupt (void)
 	vc20_tape_buttons (DATASSETTE_PLAY, DATASSETTE_RECORD, DATASSETTE_STOP);
 	set_led_status (1 /*KB_CAPSLOCK_FLAG */ , KEY_SHIFTLOCK ? 1 : 0);
 	set_led_status (0 /*KB_NUMLOCK_FLAG */ , JOYSTICK_SWAP ? 1 : 0);
-
-	return ignore_interrupt ();
 }
