@@ -14,7 +14,11 @@
 #include "vidhrdw/generic.h"
 #include "includes/cgenie.h"
 #include "includes/wd179x.h"
-#include "includes/basicdsk.h"
+#include "devices/basicdsk.h"
+#include "devices/cartslot.h"
+#include "sound/ay8910.h"
+#include "sound/dac.h"
+#include "image.h"
 
 #define AYWriteReg(chip,port,value) \
 	AY8910_control_port_0_w(0,port);  \
@@ -25,9 +29,6 @@
 UINT8 *cgenie_fontram;
 
 
-extern char cgenie_frame_message[64];
-extern int cgenie_frame_time;
-
 int cgenie_tv_mode = -1;
 int cgenie_load_cas = 0;
 static int port_ff = 0xff;
@@ -80,11 +81,9 @@ static UINT8 irq_status = 0;
 static UINT8 motor_drive = 0;
 static UINT8 head = 0;
 
-static int cass_specified = 0;
 /* current tape file handles */
 static char tape_name[12+1];
-static void *tape_put_file = 0;
-static void *tape_get_file = 0;
+static mame_file *tape_get_file = 0;
 
 /* tape buffer for the first eight bytes at write (to extract a filename) */
 static UINT8 tape_buffer[9];
@@ -120,103 +119,99 @@ static void tape_put_close(void);
 static OPBASE_HANDLER (opbaseoverride)
 {
 	UINT8 *RAM = memory_region(REGION_CPU1);
+	mess_image *img;
+	UINT8 *buff, *s, data;
+	UINT16 size, entry = 0, block_len, block_ofs = 0;
+	mame_file *cmd;
+
 	/* check if the BASIC prompt is visible on the screen */
 	if( cgenie_load_cas && RAM[0x4400+3*40] == 0x3e )
 	{
 		cgenie_load_cas = 0;
-		if( cass_specified && strlen(device_filename(IO_CASSETTE,0)) )
-		{
-			UINT8 *buff = (UINT8*)malloc(65536), *s, data;
-			UINT16 size, entry = 0, block_len, block_ofs = 0;
-			void *cmd;
 
+		img = image_from_devtype_and_index(IO_CASSETTE, 0);
+
+		if (image_exists(img))
+		{
+			buff = (UINT8*) malloc(65536);
 			if( !buff )
 			{
 				logerror("failed to allocate 64K buff\n");
 				return address;
 			}
-			cmd = image_fopen(IO_CASSETTE, 0, OSD_FILETYPE_IMAGE, OSD_FOPEN_READ);
-			if( !cmd )
-				  cmd = image_fopen(IO_SNAPSHOT, 0, OSD_FILETYPE_IMAGE, OSD_FOPEN_READ);
-			if( !cmd )
+
+			cmd = image_fp(img);
+			size = mame_fread(cmd, buff, 65536);
+			s = buff;
+			if( memcmp(s, TAPE_HEADER, sizeof(TAPE_HEADER)-1) == 0 )
 			{
-				logerror("failed to open '%s'\n", device_filename(IO_CASSETTE,0));
-			}
-			else
-			{
-				size = osd_fread(cmd, buff, 65536);
-				s = buff;
-				if( memcmp(s, TAPE_HEADER, sizeof(TAPE_HEADER)-1) == 0 )
+				s = (UINT8*)memchr(s, 26, size);
+				if( s )
 				{
-					s = (UINT8*)memchr(s, 26, size);
-					if( s )
-					{
-						*s++ = '\n';
-						*s++ = '\0';
-						logerror("%s",s);
-					}
-					size -= s - buff;
+					*s++ = '\n';
+					*s++ = '\0';
+					logerror("%s",s);
 				}
-				if( s[0] == 0x66 && s[1] == 0x55 && s[8] == 0x3c )
+				size -= s - buff;
+			}
+			if( s[0] == 0x66 && s[1] == 0x55 && s[8] == 0x3c )
+			{
+				logerror("image name: [%-6.6s]\n",s+1);
+				s += 8;
+				size -= 8;
+				while( size > 3 )
+				{
+					data = *s++;
+					switch( data )
 					{
-					logerror("image name: [%-6.6s]\n",s+1);
-					s += 8;
-					size -= 8;
-					while( size > 3 )
-					{
-						data = *s++;
-						switch( data )
+					case 0x01:		   /* CMD file header */
+					case 0x07:		   /* another type of CMD file header */
+					case 0x3c:		   /* CAS file header */
+						block_len = *s++;
+						/* on CMD files size zero means size 256 */
+						if( block_len == 0 )
+							block_len = 256;
+						block_ofs = *s++;
+						block_ofs += 256 * *s++;
+						if( data != 0x3c )
 						{
-						case 0x01:		   /* CMD file header */
-						case 0x07:		   /* another type of CMD file header */
-						case 0x3c:		   /* CAS file header */
-							block_len = *s++;
-							/* on CMD files size zero means size 256 */
+							block_len -= 2;
 							if( block_len == 0 )
 								block_len = 256;
-							block_ofs = *s++;
-							block_ofs += 256 * *s++;
-							if( data != 0x3c )
-							{
-								block_len -= 2;
-								if( block_len == 0 )
-									block_len = 256;
-							}
-							size -= 4;
-							logerror("cgenie_cmd_load block ($%02X) %d at $%04X\n", data, block_len, block_ofs);
-							while( block_len && size )
-							{
-								cpu_writemem16(block_ofs, *s);
-								s++;
-								block_ofs++;
-								block_len--;
-								size--;
-							}
-							if( data == 0x3c )
-								s++;
-							break;
-						case 0x02:
-							block_len = *s++;
-							size -= 1;
-						case 0x78:
-							block_ofs = *s++;
-							block_ofs += 256 * *s++;
-							if( !entry )
-								entry = block_ofs;
-							logerror( "cgenie_cmd_load entry ($%02X) at $%04X\n", data, entry);
-							size -= 3;
-							if( size <= 3 )
-							{
-								logerror("starting program at $%04X\n", block_ofs);
-							}
-							break;
-						default:
+						}
+						size -= 4;
+						logerror("cgenie_cmd_load block ($%02X) %d at $%04X\n", data, block_len, block_ofs);
+						while( block_len && size )
+						{
+							program_write_byte(block_ofs, *s);
+							s++;
+							block_ofs++;
+							block_len--;
 							size--;
 						}
+						if( data == 0x3c )
+							s++;
+						break;
+					case 0x02:
+						block_len = *s++;
+						size -= 1;
+					case 0x78:
+						block_ofs = *s++;
+						block_ofs += 256 * *s++;
+						if( !entry )
+							entry = block_ofs;
+						logerror( "cgenie_cmd_load entry ($%02X) at $%04X\n", data, entry);
+						size -= 3;
+						if( size <= 3 )
+						{
+							logerror("starting program at $%04X\n", block_ofs);
+						}
+						break;
+					default:
+						size--;
 					}
 				}
-				osd_fclose(cmd);
-				cpunum_set_pc(0,entry);
+				activecpu_set_reg(REG_PC, entry);
 			}
 			free(buff);
 		}
@@ -233,7 +228,7 @@ void init_cgenie(void)
 	 * Every fitfth cycle is a wait cycle, so I reduced
 	 * the overlocking by one fitfth
 	 */
-	timer_set_overclock(0, 0.80);
+	cpunum_set_clockscale(0, 0.80);
 
 	/* Initialize some patterns to be displayed in graphics mode */
 	for( i = 0; i < 256; i++ )
@@ -242,7 +237,7 @@ void init_cgenie(void)
 
 static void cgenie_fdc_callback(int);
 
-void cgenie_init_machine(void)
+MACHINE_INIT( cgenie )
 {
 	UINT8 *ROM = memory_region(REGION_CPU1);
 
@@ -273,31 +268,30 @@ void cgenie_init_machine(void)
 	}
 	else
 	{
-				logerror("cgenie floppy discs disabled\n");
+		logerror("cgenie floppy discs disabled\n");
 	}
 
 	/* copy DOS ROM, if enabled or wipe out that memory area */
 	if( readinputport(0) & 0x40 )
 	{
-
 		if ( readinputport(0) & 0x080 )
 		{
-			install_mem_read_handler(0, 0xc000, 0xdfff, MRA_ROM);
-			install_mem_write_handler(0, 0xc000, 0xdfff, MWA_ROM);
+			memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MRA8_ROM);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MWA8_ROM);
 			logerror("cgenie DOS enabled\n");
 			memcpy(&ROM[0x0c000],&ROM[0x10000], 0x2000);
 		}
 		else
 		{
-			install_mem_read_handler(0, 0xc000, 0xdfff, MRA_NOP);
-			install_mem_write_handler(0, 0xc000, 0xdfff, MWA_NOP);
+			memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MRA8_NOP);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MWA8_NOP);
 			logerror("cgenie DOS disabled (no floppy image given)\n");
 		}
 	}
 	else
 	{
-		install_mem_read_handler(0, 0xc000, 0xdfff, MRA_NOP);
-		install_mem_write_handler(0, 0xc000, 0xdfff, MWA_NOP);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MRA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xdfff, 0, 0, MWA8_NOP);
 		logerror("cgenie DOS disabled\n");
 		memset(&memory_region(REGION_CPU1)[0x0c000], 0x00, 0x2000);
 	}
@@ -305,16 +299,16 @@ void cgenie_init_machine(void)
 	/* copy EXT ROM, if enabled or wipe out that memory area */
 	if( readinputport(0) & 0x20 )
 	{
-		install_mem_read_handler(0, 0xe000, 0xefff, MRA_ROM);
-		install_mem_write_handler(0, 0xe000, 0xefff, MWA_ROM);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe000, 0xefff, 0, 0, MRA8_ROM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe000, 0xefff, 0, 0, MWA8_ROM);
 		logerror("cgenie EXT enabled\n");
 		memcpy(&memory_region(REGION_CPU1)[0x0e000],
 			   &memory_region(REGION_CPU1)[0x12000], 0x1000);
 	}
 	else
 	{
-		install_mem_read_handler(0, 0xe000, 0xefff, MRA_NOP);
-		install_mem_write_handler(0, 0xe000, 0xefff, MWA_NOP);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe000, 0xefff, 0, 0, MRA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe000, 0xefff, 0, 0, MWA8_NOP);
 		logerror("cgenie EXT disabled\n");
 		memset(&memory_region(REGION_CPU1)[0x0e000], 0x00, 0x1000);
 	}
@@ -322,177 +316,122 @@ void cgenie_init_machine(void)
 	/* check for 32K RAM */
 	if( readinputport(0) & 0x04 )
 	{
-		install_mem_read_handler(0, 0x8000, 0xbfff, MRA_RAM);
-		install_mem_write_handler(0, 0x8000, 0xbfff, MWA_RAM);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0xbfff, 0, 0, MRA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0xbfff, 0, 0, MWA8_RAM);
 	}
 	else
 	{
-		install_mem_read_handler(0, 0x8000, 0xbfff, MRA_NOP);
-		install_mem_write_handler(0, 0x8000, 0xbfff, MWA_NOP);
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0xbfff, 0, 0, MRA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0xbfff, 0, 0, MWA8_NOP);
 	}
 
 	cgenie_load_cas = 1;
 	memory_set_opbase_handler(0, opbaseoverride);
 }
 
-void cgenie_stop_machine(void)
+MACHINE_STOP( cgenie )
 {
-	wd179x_exit();
 	tape_put_close();
 }
 
-int cgenie_cassette_init(int id)
+DEVICE_LOAD( cgenie_cassette )
 {
-	cass_specified = device_filename(IO_CASSETTE,id) != NULL;
-	return 0;
+	return INIT_PASS;
 }
 
-#if 0
-			if( file == REAL_FDD )
-			{
-				PDRIVE *pd = (PDRIVE *)memory_region(REGION_CPU1) + 0x5a71 + drive * sizeof(PDRIVE);
-				/* changed pdrive parameters for drive ? */
-				if( memcmp(&pdrive[drive], pd, sizeof(PDRIVE)) )
-				{
-					/* copy them and set new geometry */
-					memcpy(&pdrive[drive], pd, sizeof(PDRIVE));
-					tracks[drive] = pd->TRK;
-					heads[drive] = (pd->SPT > 18) ? 2 : 1;
-					spt[drive] = pd->SPT / heads[drive];
-					dir_sector[drive] = pd->DDSL * pd->GATM * pd->GPL + pd->SPT;
-					dir_length[drive] = pd->DDGA * pd->GPL;
-					wd179x_set_geometry(drive, tracks[drive], heads[drive], spt[drive], 256, dir_sector[drive], dir_length[drive], 0);
-				}
-				return;
-			}
-#endif
-
 /* basic-dsk is a disk image format which has the tracks and sectors
  * stored in order, no information is stored which details the number
  * of tracks, number of sides, number of sectors etc, so we need to
  * set that up here
  */
-int cgenie_floppy_init(int id)
+DEVICE_LOAD( cgenie_floppy )
 {
-		void *file;
+	int i, j, dir_offset;
+	UINT8 buff[16];
+	UINT8 tracks = 0;
+	UINT8 heads = 0;
+	UINT8 spt = 0;
+	short dir_sector = 0;
+	short dir_length = 0;
 
-	if (basicdsk_floppy_init(id) != INIT_PASS)
+	/* A Floppy Isnt manditory, so return if none */
+	if (device_load_basicdsk_floppy(image, file) != INIT_PASS)
 		return INIT_FAIL;
 
-	/* open file and determine image geometry */
-	file = image_fopen(IO_FLOPPY, id, OSD_FILETYPE_IMAGE, OSD_FOPEN_READ);
+	/* determine image geometry */
+	mame_fseek(file, 0, SEEK_SET);
 
-	if (file)
+	/* determine geometry from disk contents */
+	for( i = 0; i < 12; i++ )
 	{
-		int i, j, dir_offset;
-		UINT8 buff[16];
-		UINT8 tracks = 0;
-		UINT8 heads = 0;
-		UINT8 spt = 0;
-		short dir_sector = 0;
-		short dir_length = 0;
-		/* determine geometry from disk contents */
-		for( i = 0; i < 12; i++ )
+		mame_fseek(file, pd_list[i].SPT * 256, SEEK_SET);
+		mame_fread(file, buff, 16);
+		/* find an entry with matching DDSL */
+		if (buff[0] != 0x00 || buff[1] != 0xfe || buff[2] != pd_list[i].DDSL)
+			continue;
+		logerror("cgenie: checking format #%d\n", i);
+
+		dir_sector = pd_list[i].DDSL * pd_list[i].GATM * pd_list[i].GPL + pd_list[i].SPT;
+		dir_length = pd_list[i].DDGA * pd_list[i].GPL;
+
+		/* scan directory for DIR/SYS or NCW1983/JHL files */
+		/* look into sector 2 and 3 first entry relative to DDSL */
+		for( j = 16; j < 32; j += 8 )
 		{
-			osd_fseek(file, pd_list[i].SPT * 256, SEEK_SET);
-			osd_fread(file, buff, 16);
-			/* find an entry with matching DDSL */
-			if (buff[0] != 0x00 || buff[1] != 0xfe || buff[2] != pd_list[i].DDSL)
-				continue;
-			logerror("cgenie: checking format #%d\n", i);
-
-			dir_sector = pd_list[i].DDSL * pd_list[i].GATM * pd_list[i].GPL + pd_list[i].SPT;
-			dir_length = pd_list[i].DDGA * pd_list[i].GPL;
-
-			/* scan directory for DIR/SYS or NCW1983/JHL files */
-			/* look into sector 2 and 3 first entry relative to DDSL */
-			for( j = 16; j < 32; j += 8 )
+			dir_offset = dir_sector * 256 + j * 32;
+			if( mame_fseek(file, dir_offset, SEEK_SET) < 0 )
+				break;
+			if( mame_fread(file, buff, 16) != 16 )
+				break;
+			if( !strncmp((char*)buff + 5, "DIR     SYS", 11) ||
+				!strncmp((char*)buff + 5, "NCW1983 JHL", 11) )
 			{
-				dir_offset = dir_sector * 256 + j * 32;
-				if( osd_fseek(file, dir_offset, SEEK_SET) < 0 )
-					break;
-				if( osd_fread(file, buff, 16) != 16 )
-					break;
-				if( !strncmp((char*)buff + 5, "DIR     SYS", 11) ||
-					!strncmp((char*)buff + 5, "NCW1983 JHL", 11) )
-				{
-					tracks = pd_list[i].TRK;
-					heads = (pd_list[i].SPT > 18) ? 2 : 1;
-					spt = pd_list[i].SPT / heads;
-					dir_sector = pd_list[i].DDSL * pd_list[i].GATM * pd_list[i].GPL + pd_list[i].SPT;
-					dir_length = pd_list[i].DDGA * pd_list[i].GPL;
-					memcpy(memory_region(REGION_CPU1) + 0x5A71 + id * sizeof(PDRIVE), &pd_list[i], sizeof(PDRIVE));
-					break;
-				}
+				tracks = pd_list[i].TRK;
+				heads = (pd_list[i].SPT > 18) ? 2 : 1;
+				spt = pd_list[i].SPT / heads;
+				dir_sector = pd_list[i].DDSL * pd_list[i].GATM * pd_list[i].GPL + pd_list[i].SPT;
+				dir_length = pd_list[i].DDGA * pd_list[i].GPL;
+				memcpy(memory_region(REGION_CPU1) + 0x5A71 + image_index_in_device(image) * sizeof(PDRIVE), &pd_list[i], sizeof(PDRIVE));
+				break;
 			}
-
-			logerror("cgenie: geometry %d tracks, %d heads, %d sec/track\n", tracks, heads, spt);
-			/* set geometry so disk image can be read */
-			basicdsk_set_geometry(id, tracks, heads, spt, 256, 0, 0);
-
-			logerror("cgenie: directory sectors %d - %d (%d sectors)\n", dir_sector, dir_sector + dir_length - 1, dir_length);
-			/* mark directory sectors with deleted data address mark */
-			/* assumption dir_sector is a sector offset */
-			for (j = 0; j < dir_length; j++)
-			{
-				UINT8 track;
-				UINT8 side;
-				UINT8 sector_id;
-				UINT16 track_offset;
-				UINT16 sector_offset;
-
-				/* calc sector offset */
-				sector_offset = dir_sector + j;
-
-				/* get track offset */
-				track_offset = sector_offset / spt;
-
-				/* calc track */
-				track = track_offset / heads;
-
-				/* calc side */
-				side = track_offset % heads;
-
-				/* calc sector id - first sector id is 0! */
-				sector_id = sector_offset % spt;
-
-				/* set deleted data address mark for sector specified */
-				basicdsk_set_ddam(id, track, side, sector_id, 1);
-			}
-
 		}
 
-		osd_fclose(file);
-		return INIT_PASS;
+		logerror("cgenie: geometry %d tracks, %d heads, %d sec/track\n", tracks, heads, spt);
+		/* set geometry so disk image can be read */
+		basicdsk_set_geometry(image, tracks, heads, spt, 256, 0, 0, FALSE);
+
+		logerror("cgenie: directory sectors %d - %d (%d sectors)\n", dir_sector, dir_sector + dir_length - 1, dir_length);
+		/* mark directory sectors with deleted data address mark */
+		/* assumption dir_sector is a sector offset */
+		for (j = 0; j < dir_length; j++)
+		{
+			UINT8 track;
+			UINT8 side;
+			UINT8 sector_id;
+			UINT16 track_offset;
+			UINT16 sector_offset;
+
+			/* calc sector offset */
+			sector_offset = dir_sector + j;
+
+			/* get track offset */
+			track_offset = sector_offset / spt;
+
+			/* calc track */
+			track = track_offset / heads;
+
+			/* calc side */
+			side = track_offset % heads;
+
+			/* calc sector id - first sector id is 0! */
+			sector_id = sector_offset % spt;
+
+			/* set deleted data address mark for sector specified */
+			basicdsk_set_ddam(image, track, side, sector_id, 1);
+		}
+
 	}
-
-	return INIT_FAIL;
-}
-
-int cgenie_rom_load(int id)
-{
-	int result = 0;
-	UINT8 *ROM = memory_region(REGION_CPU1);
-	void *rom;
-	const char *filename;
-
-	/* Initialize memory */
-	memset(&ROM[0x4000], 0xff, 0xc000);
-
-	filename = "newe000.rom";
-	rom = osd_fopen(Machine->gamedrv->name, filename, OSD_FILETYPE_IMAGE, 0);
-	if( rom )
-	{
-		logerror("%s found '%s' ROM\n", Machine->gamedrv->name, filename);
-		osd_fread(rom, &ROM[0x12000], 0x1000);
-		osd_fclose(rom);
-	}
-	else
-	{
-		logerror("%s optional ROM image '%s' not found\n", Machine->gamedrv->name, filename);
-	}
-
-	return result;
+	return INIT_PASS;
 }
 
 /*************************************
@@ -523,21 +462,12 @@ static void tape_put_byte(UINT8 value)
 				sprintf(tape_name, "%-6.6s.cas", tape_buffer + 2);
 			else
 				strcpy(tape_name, "unknown.cas");
-			osd_fopen(Machine->gamedrv->name, tape_name, OSD_FILETYPE_IMAGE, OSD_FOPEN_WRITE);
-			if( tape_put_file )
-				osd_fwrite(tape_put_file, tape_buffer, 9);
+			mame_fopen(Machine->gamedrv->name, tape_name, FILETYPE_IMAGE, OSD_FOPEN_WRITE);
 		}
 	}
 	else
 	{
 		tape_count++;
-		if( tape_put_file )
-			osd_fwrite(tape_put_file, &value, 1);
-	}
-	if( tape_put_file )
-	{
-		cgenie_frame_time = 30;
-		sprintf(cgenie_frame_message, "Tape write '%s' $%04X bytes", tape_name, tape_count);
 	}
 }
 
@@ -548,42 +478,7 @@ static void tape_put_byte(UINT8 value)
  *******************************************************************/
 static void tape_put_close(void)
 {
-	/* file open ? */
-	if( tape_put_file )
-	{
-		if( put_bit_count )
-		{
-			UINT8 value;
-			while( put_bit_count < 16 )
-			{
-				tape_bits <<= 1;
-				put_bit_count++;
-			}
-			value = 0;
-			if( tape_bits & 0x8000 )
-				value |= 0x80;
-			if( tape_bits & 0x2000 )
-				value |= 0x40;
-			if( tape_bits & 0x0800 )
-				value |= 0x20;
-			if( tape_bits & 0x0200 )
-				value |= 0x10;
-			if( tape_bits & 0x0080 )
-				value |= 0x08;
-			if( tape_bits & 0x0020 )
-				value |= 0x04;
-			if( tape_bits & 0x0008 )
-				value |= 0x02;
-			if( tape_bits & 0x0002 )
-				value |= 0x01;
-			tape_put_byte(value);
-		}
-		osd_fclose(tape_put_file);
-		cgenie_frame_time = 30;
-		sprintf(cgenie_frame_message, "Tape output closed");
-	}
 	tape_count = 0;
-	tape_put_file = 0;
 }
 
 /*******************************************************************
@@ -592,7 +487,7 @@ static void tape_put_close(void)
  *******************************************************************/
 static void tape_put_bit(void)
 {
-	int now_cycles = cpu_gettotalcycles();
+	int now_cycles = activecpu_gettotalcycles();
 	int diff = now_cycles - put_cycles;
 	int limit = 12 * (memory_region(REGION_CPU1)[0x4310] + memory_region(REGION_CPU1)[0x4311]);
 	UINT8 value;
@@ -615,8 +510,7 @@ static void tape_put_bit(void)
 				case 0: 	   /* look for sync AA */
 					if( (tape_bits & 0xffff) == 0xcccc )
 					{
-						cgenie_frame_time = 30;
-						sprintf(cgenie_frame_message, "Tape sync1 written");
+						ui_popup("Tape sync1 written");
 						in_sync = 1;
 					}
 					break;
@@ -624,8 +518,7 @@ static void tape_put_bit(void)
 					if( (tape_bits & 0xffff) == 0x3c3c )
 					{
 						in_sync = 2;
-						cgenie_frame_time = 30;
-						sprintf(cgenie_frame_message, "Tape sync2 written");
+						ui_popup("Tape sync2 written");
 						put_bit_count = 16;
 					}
 					break;
@@ -642,16 +535,14 @@ static void tape_put_bit(void)
 				case 0: 	   /* look for sync AA */
 					if( (tape_bits & 0xffff) == 0xcccc )
 					{
-						cgenie_frame_time = 30;
-						sprintf(cgenie_frame_message, "Tape sync1 written");
+						ui_popup("Tape sync1 written");
 						in_sync = 1;
 					}
 					break;
 				case 1: 	   /* look for sync 66 */
 					if( (tape_bits & 0xffff) == 0x3c3c )
 					{
-						cgenie_frame_time = 30;
-						sprintf(cgenie_frame_message, "Tape sync2 written");
+						ui_popup("Tape sync2 written");
 						in_sync = 2;
 						put_bit_count = 16;
 					}
@@ -706,15 +597,13 @@ static void tape_get_byte(void)
 	{
 		if( tape_count < 32 )
 		{
-			cgenie_frame_time = 30;
-			sprintf(cgenie_frame_message, "Tape load sync1");
+			ui_popup("Tape load sync1");
 			value = 0xaa;
 		}
 		else
 		{
-			cgenie_frame_time = 30;
-			sprintf(cgenie_frame_message, "Tape load '%s' $%04X bytes", tape_name, tape_count);
-			osd_fread(tape_get_file, &value, 1);
+			ui_popup("Tape load '%s' $%04X bytes", tape_name, tape_count);
+			mame_fread(tape_get_file, &value, 1);
 		}
 		tape_bits |= 0xaaaa;
 		if( value & 0x80 )
@@ -759,26 +648,25 @@ static void tape_get_open(void)
 		if (tape_name[0] != ' ')
 		{
 			logerror("tape_get_open '%s'\n", tape_name);
-			tape_get_file = osd_fopen(Machine->gamedrv->name, tape_name, OSD_FILETYPE_IMAGE, OSD_FOPEN_READ);
+			tape_get_file = mame_fopen(Machine->gamedrv->name, tape_name, FILETYPE_IMAGE, OSD_FOPEN_READ);
 		}
 		if( tape_get_file )
 		{
-			cgenie_frame_time = 30;
-			sprintf(cgenie_frame_message, "Tape load '%s'", tape_name);
-			osd_fread(tape_get_file, buffer, sizeof(TAPE_HEADER));
+			ui_popup("Tape load '%s'", tape_name);
+			mame_fread(tape_get_file, buffer, sizeof(TAPE_HEADER));
 			if( strncmp(buffer, TAPE_HEADER, sizeof(TAPE_HEADER) - 1) == 0 )
 			{
 				UINT8 data;
 				/* skip data until zero byte */
 				do
 				{
-					osd_fread(tape_get_file, &data, 1);
+					mame_fread(tape_get_file, &data, 1);
 				} while( data );
 			}
 			else
 			{
 				/* seek back to start of tape */
-				osd_fseek(tape_get_file, 0, SEEK_SET);
+				mame_fseek(tape_get_file, 0, SEEK_SET);
 			}
 		}
 		tape_count = 0;
@@ -787,7 +675,7 @@ static void tape_get_open(void)
 
 static void tape_get_bit(void)
 {
-	int now_cycles = cpu_gettotalcycles();
+	int now_cycles = activecpu_gettotalcycles();
 	int limit = 10 * memory_region(REGION_CPU1)[0x4312];
 	int diff = now_cycles - get_cycles;
 
@@ -796,10 +684,9 @@ static void tape_get_bit(void)
 	{
 		if( tape_get_file )
 		{
-			osd_fclose(tape_get_file);
+			mame_fclose(tape_get_file);
 			tape_get_file = NULL;
-			cgenie_frame_time = 30;
-			sprintf(cgenie_frame_message, "Tape file closed");
+			ui_popup("Tape file closed");
 		}
 		get_bit_count = tape_bits = tape_time = 0;
 	}
@@ -847,7 +734,7 @@ static void tape_get_bit(void)
 #define FF_BGD2 0x80		   /* background color select 2 */
 #define FF_BGD	(FF_BGD0 | FF_BGD1 | FF_BGD2)
 
-WRITE_HANDLER( cgenie_port_ff_w )
+WRITE8_HANDLER( cgenie_port_ff_w )
 {
 	int port_ff_changed = port_ff ^ data;
 
@@ -906,8 +793,7 @@ WRITE_HANDLER( cgenie_port_ff_w )
 				b = 15;
 			}
 		}
-		osd_modify_pen(0, r, g, b);
-		osd_modify_pen(Machine->pens[0], r, g, b);
+		palette_set_color(0, r, g, b);
 	}
 
 	/* character mode changed ? */
@@ -934,7 +820,7 @@ WRITE_HANDLER( cgenie_port_ff_w )
 	port_ff = data;
 }
 
-READ_HANDLER( cgenie_port_ff_r )
+ READ8_HANDLER( cgenie_port_ff_r )
 {
 	/* virtual tape ? */
 
@@ -960,12 +846,12 @@ static UINT8 psg_b_out = 0x00;
 static UINT8 psg_a_inp = 0x00;
 static UINT8 psg_b_inp = 0x00;
 
-READ_HANDLER( cgenie_psg_port_a_r )
+ READ8_HANDLER( cgenie_psg_port_a_r )
 {
 	return psg_a_inp;
 }
 
-data8_t cgenie_psg_port_b_r(offs_t port)
+UINT8 cgenie_psg_port_b_r(offs_t port)
 {
 	if( psg_a_out < 0xd0 )
 	{
@@ -1000,17 +886,17 @@ data8_t cgenie_psg_port_b_r(offs_t port)
 	return psg_b_inp;
 }
 
-WRITE_HANDLER( cgenie_psg_port_a_w )
+WRITE8_HANDLER( cgenie_psg_port_a_w )
 {
 	psg_a_out = data;
 }
 
-WRITE_HANDLER( cgenie_psg_port_b_w )
+WRITE8_HANDLER( cgenie_psg_port_b_w )
 {
 	psg_b_out = data;
 }
 
-READ_HANDLER( cgenie_status_r )
+ READ8_HANDLER( cgenie_status_r )
 {
 	/* If the floppy isn't emulated, return 0 */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1018,7 +904,7 @@ READ_HANDLER( cgenie_status_r )
 	return wd179x_status_r(offset);
 }
 
-READ_HANDLER( cgenie_track_r )
+ READ8_HANDLER( cgenie_track_r )
 {
 	/* If the floppy isn't emulated, return 0xff */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1026,7 +912,7 @@ READ_HANDLER( cgenie_track_r )
 	return wd179x_track_r(offset);
 }
 
-READ_HANDLER( cgenie_sector_r )
+ READ8_HANDLER( cgenie_sector_r )
 {
 	/* If the floppy isn't emulated, return 0xff */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1034,7 +920,7 @@ READ_HANDLER( cgenie_sector_r )
 	return wd179x_sector_r(offset);
 }
 
-READ_HANDLER(cgenie_data_r )
+ READ8_HANDLER(cgenie_data_r )
 {
 	/* If the floppy isn't emulated, return 0xff */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1042,7 +928,7 @@ READ_HANDLER(cgenie_data_r )
 	return wd179x_data_r(offset);
 }
 
-WRITE_HANDLER( cgenie_command_w )
+WRITE8_HANDLER( cgenie_command_w )
 {
 	/* If the floppy isn't emulated, return immediately */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1050,7 +936,7 @@ WRITE_HANDLER( cgenie_command_w )
 	wd179x_command_w(offset, data);
 }
 
-WRITE_HANDLER( cgenie_track_w )
+WRITE8_HANDLER( cgenie_track_w )
 {
 	/* If the floppy isn't emulated, ignore the write */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1058,7 +944,7 @@ WRITE_HANDLER( cgenie_track_w )
 	wd179x_track_w(offset, data);
 }
 
-WRITE_HANDLER( cgenie_sector_w )
+WRITE8_HANDLER( cgenie_sector_w )
 {
 	/* If the floppy isn't emulated, ignore the write */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1066,7 +952,7 @@ WRITE_HANDLER( cgenie_sector_w )
 	wd179x_sector_w(offset, data);
 }
 
-WRITE_HANDLER( cgenie_data_w )
+WRITE8_HANDLER( cgenie_data_w )
 {
 	/* If the floppy isn't emulated, ignore the write */
 	if( (readinputport(0) & 0x80) == 0 )
@@ -1074,7 +960,7 @@ WRITE_HANDLER( cgenie_data_w )
 	wd179x_data_w(offset, data);
 }
 
-READ_HANDLER( cgenie_irq_status_r )
+ READ8_HANDLER( cgenie_irq_status_r )
 {
 int result = irq_status;
 
@@ -1082,26 +968,22 @@ int result = irq_status;
 	return result;
 }
 
-int cgenie_timer_interrupt(void)
+INTERRUPT_GEN( cgenie_timer_interrupt )
 {
 	if( (irq_status & IRQ_TIMER) == 0 )
 	{
 		irq_status |= IRQ_TIMER;
-		cpu_cause_interrupt(0, 0);
-		return 0;
+		cpunum_set_input_line(0, 0, PULSE_LINE);
 	}
-	return ignore_interrupt();
 }
 
-int cgenie_fdc_interrupt(void)
+static INTERRUPT_GEN( cgenie_fdc_interrupt )
 {
 	if( (irq_status & IRQ_FDC) == 0 )
 	{
 		irq_status |= IRQ_FDC;
-		cpu_cause_interrupt(0, 0);
-		return 0;
+		cpunum_set_input_line(0, 0, PULSE_LINE);
 	}
-	return ignore_interrupt();
 }
 
 void cgenie_fdc_callback(int event)
@@ -1121,7 +1003,7 @@ void cgenie_fdc_callback(int event)
 	}
 }
 
-WRITE_HANDLER( cgenie_motor_w )
+WRITE8_HANDLER( cgenie_motor_w )
 {
 	UINT8 drive = 255;
 
@@ -1152,13 +1034,10 @@ WRITE_HANDLER( cgenie_motor_w )
 /*************************************
  *		Keyboard					 *
  *************************************/
-READ_HANDLER( cgenie_keyboard_r )
+ READ8_HANDLER( cgenie_keyboard_r )
 {
 	int result = 0;
 
-	if( setup_active() || onscrd_active() )
-		return result;
-
 	if( offset & 0x01 )
 		result |= input_port_1_r(0);
 	if( offset & 0x02 )
@@ -1188,7 +1067,7 @@ int cgenie_videoram_r( int offset )
 	return videoram[offset];
 }
 
-WRITE_HANDLER( cgenie_videoram_w )
+WRITE8_HANDLER( cgenie_videoram_w )
 {
 	/* write to video RAM */
 	if( data == videoram[offset] )
@@ -1197,12 +1076,12 @@ WRITE_HANDLER( cgenie_videoram_w )
 	dirtybuffer[offset] = 1;
 }
 
-READ_HANDLER( cgenie_colorram_r )
+ READ8_HANDLER( cgenie_colorram_r )
 {
 	return colorram[offset] | 0xf0;
 }
 
-WRITE_HANDLER( cgenie_colorram_w )
+WRITE8_HANDLER( cgenie_colorram_w )
 {
 	int a;
 
@@ -1221,12 +1100,12 @@ WRITE_HANDLER( cgenie_colorram_w )
 		dirtybuffer[a] = 1;
 }
 
-READ_HANDLER( cgenie_fontram_r )
+ READ8_HANDLER( cgenie_fontram_r )
 {
 	return cgenie_fontram[offset];
 }
 
-WRITE_HANDLER( cgenie_fontram_w )
+WRITE8_HANDLER( cgenie_fontram_w )
 {
 	UINT8 *dp;
 	int code;
@@ -1259,7 +1138,7 @@ WRITE_HANDLER( cgenie_fontram_w )
  *
  *************************************/
 
-int cgenie_frame_interrupt(void)
+INTERRUPT_GEN( cgenie_frame_interrupt )
 {
 	if( cgenie_tv_mode != (readinputport(0) & 0x10) )
 	{
@@ -1269,12 +1148,5 @@ int cgenie_frame_interrupt(void)
 		port_ff ^= FF_BGD0;
 		cgenie_port_ff_w(0, port_ff ^ FF_BGD0);
 	}
-
-	return 0;
-}
-
-void cgenie_nmi_generate(int param)
-{
-	cpu_set_nmi_line(0, PULSE_LINE);
 }
 
