@@ -7,9 +7,11 @@
 
 ******************************************************************************/
 #include "driver.h"
+#include "artwork.h"
 #include "cpu/m6502/m6502.h"
 #include "vidhrdw/generic.h"
 #include "includes/kim1.h"
+#include "sound/dac.h"
 
 typedef struct
 {
@@ -22,7 +24,7 @@ typedef struct
 	UINT8 irqen;	/* IRQ enabled ? */
 	UINT8 state;	/* current timer state (bit 7) */
 	double clock;	/* 100000/1(,8,64,1024) */
-	void *timer;	/* timer callback */
+	mame_timer *timer;	/* timer callback */
 }
 M6530;
 
@@ -30,12 +32,16 @@ static M6530 m6530[2];
 
 static void m6530_timer_cb(int chip);
 
-void init_kim1(void)
+OVERLAY_START( kim1_overlay )
+	OVERLAY_RECT( 0.0, 0.0, 1.0, 1.0, MAKE_ARGB(0x04,0x20,0xff,0xff) )
+OVERLAY_END
+
+DRIVER_INIT( kim1 )
 {
 	UINT8 *dst;
 	int x, y, i;
 
-	static char *seg7 =
+	static const char *seg7 =
 	"....aaaaaaaaaaaaa." \
 	"...f.aaaaaaaaaaa.b" \
 	"...ff.aaaaaaaaa.bb" \
@@ -62,7 +68,7 @@ void init_kim1(void)
 	"..................";
 
 
-	static char *keys[24] =
+	static const char *keys[24] =
 	{
 		"aaaaaaaaaaaaaaaaaaaaaaaa" \
 		"aaaaaaaaaaaaaaaaaaaaaaaa" \
@@ -591,9 +597,15 @@ void init_kim1(void)
 			}
 		}
 	}
+/*	artwork_set_overlay(kim1_overlay); */
 }
 
-void kim1_init_machine(void)
+static void set_chip_clock(int chip, int data)
+{
+	timer_adjust(m6530[chip].timer, 0, chip, TIME_IN_HZ((data + 1) * m6530[chip].clock / 256 / 256));
+}
+
+MACHINE_INIT( kim1 )
 {
 	UINT8 *RAM = memory_region(REGION_CPU1);
 
@@ -616,57 +628,47 @@ void kim1_init_machine(void)
 	m6530[0].dria = 0xff;
 	m6530[0].drib = 0xff;
 	m6530[0].clock = (double) 1000000 / 1;
-	m6530[0].timer = timer_pulse(TIME_IN_HZ(256 * m6530[0].clock / 256 / 256), 0, m6530_timer_cb);
+	m6530[0].timer = timer_alloc(m6530_timer_cb);
+	set_chip_clock(0, 255);
 
 	m6530[1].dria = 0xff;
 	m6530[1].drib = 0xff;
 	m6530[1].clock = (double) 1000000 / 1;
-	m6530[1].timer = timer_pulse(TIME_IN_HZ(256 * m6530[1].clock / 256 / 256), 1, m6530_timer_cb);
+	m6530[1].timer = timer_alloc(m6530_timer_cb);
+	set_chip_clock(1, 255);
 }
 
-int kim1_cassette_init(int id)
+static DEVICE_LOAD( kim1_cassette )
 {
 	const char magic[] = "KIM1";
 	char buff[4];
-	void *file;
+	UINT16 addr, size;
+	UINT8 ident, *RAM = memory_region(REGION_CPU1);
 
-	file = image_fopen(IO_CASSETTE, id, OSD_FILETYPE_IMAGE, 0);
-	if (file)
+	mame_fread(file, buff, sizeof (buff));
+	if (memcmp(buff, magic, sizeof (buff)))
 	{
-		UINT16 addr, size;
-		UINT8 ident, *RAM = memory_region(REGION_CPU1);
-
-		osd_fread(file, buff, sizeof (buff));
-		if (memcmp(buff, magic, sizeof (buff)))
-		{
-			logerror("kim1_rom_load: magic '%s' not found\n", magic);
-			return INIT_FAIL;
-		}
-		osd_fread_lsbfirst(file, &addr, 2);
-		osd_fread_lsbfirst(file, &size, 2);
-		osd_fread(file, &ident, 1);
-		logerror("kim1_rom_load: $%04X $%04X $%02X\n", addr, size, ident);
-		while (size-- > 0)
-			osd_fread(file, &RAM[addr++], 1);
-		osd_fclose(file);
+		logerror("kim1_rom_load: magic '%s' not found\n", magic);
+		return INIT_FAIL;
 	}
+	mame_fread_lsbfirst(file, &addr, 2);
+	mame_fread_lsbfirst(file, &size, 2);
+	mame_fread(file, &ident, 1);
+	logerror("kim1_rom_load: $%04X $%04X $%02X\n", addr, size, ident);
+	while (size-- > 0)
+		mame_fread(file, &RAM[addr++], 1);
 	return INIT_PASS;
 }
 
-void kim1_cassette_exit(int id)
-{
-	/* nothing yet */
-}
-
 static void m6530_timer_cb(int chip)
 {
 	logerror("m6530(%d) timer expired\n", chip);
 	m6530[chip].state |= 0x80;
 	if (m6530[chip].irqen)			   /* with IRQ? */
-		cpu_set_irq_line(0, 0, HOLD_LINE);
+		cpunum_set_input_line(0, 0, HOLD_LINE);
 }
 
-int kim1_interrupt(void)
+INTERRUPT_GEN( kim1_interrupt )
 {
 	int i;
 
@@ -676,7 +678,6 @@ int kim1_interrupt(void)
 		if (videoram[i * 2 + 1] > 0)
 			videoram[i * 2 + 1] -= 1;
 	}
-	return ignore_interrupt();
 }
 
 INLINE int m6530_r(int chip, int offset)
@@ -784,18 +785,18 @@ INLINE int m6530_r(int chip, int offset)
 	return data;
 }
 
-READ_HANDLER ( m6530_003_r )
+ READ8_HANDLER ( m6530_003_r )
 {
 	return m6530_r(0, offset);
 }
-READ_HANDLER ( m6530_002_r )
+ READ8_HANDLER ( m6530_002_r )
 {
 	return m6530_r(1, offset);
 }
 
-READ_HANDLER ( kim1_mirror_r )
+ READ8_HANDLER ( kim1_mirror_r )
 {
-	return cpu_readmem16(offset & 0x1fff);
+	return program_read_byte(offset & 0x1fff);
 }
 
 static void m6530_w(int chip, int offset, int data)
@@ -875,54 +876,60 @@ static void m6530_w(int chip, int offset, int data)
 		logerror("m6530(%d) TMR1  write: $%02x%s\n", chip, data, (offset & 8) ? " (IRQ)" : "");
 		m6530[chip].state &= ~0x80;
 		m6530[chip].irqen = (offset & 8) ? 1 : 0;
-		if (m6530[chip].timer)
-			timer_remove(m6530[chip].timer);
 		m6530[chip].clock = (double) 1000000 / 1;
-		m6530[chip].timer = timer_pulse(TIME_IN_HZ((data + 1) * m6530[chip].clock / 256 / 256), chip, m6530_timer_cb);
+		set_chip_clock(chip, data);
 		break;
 	case 0x05:
 	case 0x0d:						   /* Timer 8 start */
 		logerror("m6530(%d) TMR8  write: $%02x%s\n", chip, data, (offset & 8) ? " (IRQ)" : "");
 		m6530[chip].state &= ~0x80;
 		m6530[chip].irqen = (offset & 8) ? 1 : 0;
-		if (m6530[chip].timer)
-			timer_remove(m6530[chip].timer);
 		m6530[chip].clock = (double) 1000000 / 8;
-		m6530[chip].timer = timer_pulse(TIME_IN_HZ((data + 1) * m6530[chip].clock / 256 / 256), chip, m6530_timer_cb);
+		set_chip_clock(chip, data);
 		break;
 	case 0x06:
 	case 0x0e:						   /* Timer 64 start */
 		logerror("m6530(%d) TMR64 write: $%02x%s\n", chip, data, (offset & 8) ? " (IRQ)" : "");
 		m6530[chip].state &= ~0x80;
 		m6530[chip].irqen = (offset & 8) ? 1 : 0;
-		if (m6530[chip].timer)
-			timer_remove(m6530[chip].timer);
 		m6530[chip].clock = (double) 1000000 / 64;
-		m6530[chip].timer = timer_pulse(TIME_IN_HZ((data + 1) * m6530[chip].clock / 256 / 256), chip, m6530_timer_cb);
+		set_chip_clock(chip, data);
 		break;
 	case 0x07:
 	case 0x0f:						   /* Timer 1024 start */
 		logerror("m6530(%d) TMR1K write: $%02x%s\n", chip, data, (offset & 8) ? " (IRQ)" : "");
 		m6530[chip].state &= ~0x80;
 		m6530[chip].irqen = (offset & 8) ? 1 : 0;
-		if (m6530[chip].timer)
-			timer_remove(m6530[chip].timer);
 		m6530[chip].clock = (double) 1000000 / 1024;
-		m6530[chip].timer = timer_pulse(TIME_IN_HZ((data + 1) * m6530[chip].clock / 256 / 256), chip, m6530_timer_cb);
+		set_chip_clock(chip, data);
 		break;
 	}
 }
 
-WRITE_HANDLER ( m6530_003_w )
+WRITE8_HANDLER ( m6530_003_w )
 {
 	m6530_w(0, offset, data);
 }
-WRITE_HANDLER ( m6530_002_w )
+WRITE8_HANDLER ( m6530_002_w )
 {
 	m6530_w(1, offset, data);
 }
 
-WRITE_HANDLER ( kim1_mirror_w )
+WRITE8_HANDLER ( kim1_mirror_w )
 {
-	cpu_writemem16(offset & 0x1fff, data);
+	program_write_byte(offset & 0x1fff, data);
 }
+
+void kim1_cassette_getinfo(struct IODevice *dev)
+{
+	dev->type = IO_CASSETTE;
+	dev->count = 1;
+	dev->file_extensions = "kim1\0";
+	dev->reset_on_load = 1;
+	dev->readable = 1;
+	dev->writeable = 0;
+	dev->creatable = 0;
+	dev->load = device_load_kim1_cassette;
+}
+
+
