@@ -1,19 +1,22 @@
 /*
 	experimental LISA driver
 
-	Runs most ROM start-up code successfully, and loads the boot file, but it locks shortly
-	afterwards.
+	This driver runs most ROM test code successfully, and it loads the boot
+	file, but it locks when booting from Lisa OS (probably because of floppy
+	write bug).  MacWorks boots fine, though (I think it looks when we
+	attempt to write to floppy, too).
 
 	TODO :
 	* fix floppy write bug
 	* *** Boot and run LisaTest !!! ***
-	* finish MMU (does not switch to bank 0 on 68k trap)
+	* finish MMU (does not switch to bank 0 on 68k trap) --- fixed in June 2003???
 	* support hard disk to boot office system
 	* finish keyboard/mouse support
 	* finish clock support
 	* write SCC support
 	* finalize sound support (involves adding new features to the 6522 VIA core)
-	* fix warm-reset (I think I need to use a callback when 68k RESET instruction is called)
+	* fix warm-reset (I think I need to use a callback when 68k RESET
+	  instruction is called)
 	* write support for additionnal hardware (hard disk, etc...)
 	* emulate LISA1 (?)
 	* optimize MMU emulation !
@@ -24,20 +27,22 @@
 	* LISA2/Mac XL floppy hardware (november 2000)
 
 	Credits :
-	* the lisaemu project (<http://www.sundernet.com/>) has gathered much hardware information
-	(books, schematics...) without which this driver could never have been written
-	* The driver raised the interest of several MESS regulars (Paul Lunga, Dennis Munsie...)
-	who supported my feeble efforts
+	* the lisaemu project (<http://www.sundernet.com/>) has gathered much
+	   hardware information (books, schematics...) without which this driver
+	   could never have been written
+	* The driver raised the interest of several MESS regulars (Paul Lunga,
+	  Dennis Munsie...) who supported my feeble efforts
 
-	Raphael Nabet, 2000
+	Raphael Nabet, 2000-2003
 */
 
 #include "driver.h"
 #include "machine/6522via.h"
-#include "machine/iwm.h"
+#include "machine/applefdc.h"
 #include "machine/sonydriv.h"
 #include "machine/lisa.h"
-#include "m68k.h"
+#include "cpu/m68000/m68k.h"
+#include "sound/speaker.h"
 
 #include <time.h>
 
@@ -54,8 +59,9 @@ UINT8 *lisa_rom_ptr;
 #define RAM_OFFSET 0x004000
 #define ROM_OFFSET 0x000000
 
-/* 1kb of RAM for 6504 floppy disk controller (shared with 68000), and 4kb of ROM (8kb on some
-boards, but then only one 4kb bank is selected, according to the drive type (TWIGGY or 3.5'')) */
+/* 1kb of RAM for 6504 floppy disk controller (shared with 68000), and 4kb of
+ROM (8kb on some boards, but then only one 4kb bank is selected, according to
+the drive type (TWIGGY or 3.5'')) */
 static UINT8 *fdc_ram;
 static UINT8 *fdc_rom;
 
@@ -96,9 +102,10 @@ static mmu_entry mmu_regs[4][128];
 
 /*
 	parity logic - only hard errors are emulated for now, since
-	a) the ROMs only test these
-	b) most memory boards do not use soft errors (i.e. they only generate 1 parity bit to
-	detect errors, instead of generating several bits to fix errors)
+	a) the ROMs power-up test only tests hard errors
+	b) most memory boards do not use soft errors (i.e. they only generate 1
+	  parity bit to detect errors, instead of generating several ECC bits to
+	  fix errors)
 */
 
 static int diag2;			/* -> writes wrong parity data into RAM */
@@ -116,9 +123,7 @@ static UINT8 *bad_parity_table;	/* array : 1 bit set for each byte in RAM with w
 static int VTMSK;				/* VBI enable */
 static int VTIR;				/* VBI pending */
 static UINT16 video_address_latch;	/* register : MSBs of screen bitmap address (LSBs are 0s) */
-static UINT8 *videoram_ptr;		/* screen bitmap base address (derived from video_address_latch) */
-
-static UINT16 *old_display;		/* points to a copy of the screen, so that we can see which pixel change */
+static UINT16 *videoram_ptr;		/* screen bitmap base address (derived from video_address_latch) */
 
 
 /*
@@ -126,13 +131,13 @@ static UINT16 *old_display;		/* points to a copy of the screen, so that we can s
 	a hard disk
 */
 
-static READ_HANDLER(COPS_via_in_b);
-static WRITE_HANDLER(COPS_via_out_a);
-static WRITE_HANDLER(COPS_via_out_b);
-static WRITE_HANDLER(COPS_via_out_ca2);
-static WRITE_HANDLER(COPS_via_out_cb2);
+static  READ8_HANDLER(COPS_via_in_b);
+static WRITE8_HANDLER(COPS_via_out_a);
+static WRITE8_HANDLER(COPS_via_out_b);
+static WRITE8_HANDLER(COPS_via_out_ca2);
+static WRITE8_HANDLER(COPS_via_out_cb2);
 static void COPS_via_irq_func(int val);
-static READ_HANDLER(parallel_via_in_b);
+static  READ8_HANDLER(parallel_via_in_b);
 
 static int KBIR;	/* COPS VIA interrupt pending */
 
@@ -144,11 +149,8 @@ static struct via6522_interface lisa_via6522_intf[2] =
 		NULL, NULL,
 		COPS_via_out_a, COPS_via_out_b,
 		COPS_via_out_ca2, COPS_via_out_cb2,
+		NULL, NULL,
 		COPS_via_irq_func,
-		NULL,
-		NULL,
-		NULL,
-		NULL,
 	},
 	{	/* parallel interface via - incomplete */
 		NULL, parallel_via_in_b,
@@ -156,11 +158,7 @@ static struct via6522_interface lisa_via6522_intf[2] =
 		NULL, NULL,
 		NULL, NULL,
 		NULL, NULL,
-		NULL,
-		NULL,
-		NULL,
-		NULL,
-		NULL,
+		NULL, NULL,
 	}
 };
 
@@ -206,7 +204,7 @@ struct
 		twiggy,			/* twiggy drives (Lisa 1) */
 		sony_lisa2,		/* 3.5'' drive with LisaLite adapter (Lisa 2) */
 		sony_lisa210	/* 3.5'' drive with modified fdc hardware (Lisa 2/10, Mac XL) */
-	} floppy_hardware : 2;
+	} floppy_hardware;
 	unsigned int has_double_sided_floppy : 1;	/* true on lisa 1 and *hacked* lisa 2/10 / Mac XL */
 	unsigned int has_mac_xl_video : 1;	/* modified video for MacXL */
 } lisa_features;
@@ -230,25 +228,25 @@ static void lisa_field_interrupts(void)
 
 	/*if (RSIR)
 		// serial interrupt
-		cpu_set_irq_line(0, M68K_IRQ_6, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_6, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else if (int0)
 		// external interrupt
-		cpu_set_irq_line(0, M68K_IRQ_5, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_5, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else if (int1)
 		// external interrupt
-		cpu_set_irq_line(0, M68K_IRQ_4, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_4, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else if (int2)
 		// external interrupt
-		cpu_set_irq_line(0, M68K_IRQ_3, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_3, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else*/ if (KBIR)
 		/* COPS VIA interrupt */
-		cpu_set_irq_line(0, M68K_IRQ_2, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_2, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else if (FDIR || VTIR)
 		/* floppy disk or VBl */
-		cpu_set_irq_line(0, M68K_IRQ_1, ASSERT_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_1, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	else
 		/* clear all interrupts */
-		cpu_set_irq_line(0, M68K_IRQ_1, CLEAR_LINE);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_1, CLEAR_LINE, M68K_INT_ACK_AUTOVECTOR);
 }
 
 static void set_parity_error_pending(int value)
@@ -258,20 +256,18 @@ static void set_parity_error_pending(int value)
 	parity_error_pending = value;
 	if (parity_error_pending)
 	{
-		cpu_set_irq_line(0, M68K_IRQ_7, ASSERT_LINE);
-		cpu_irq_line_vector_w(0, M68K_IRQ_7, M68K_INT_ACK_AUTOVECTOR);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_7, ASSERT_LINE, M68K_INT_ACK_AUTOVECTOR);
 	}
 	else
 	{
-		cpu_set_irq_line(0, M68K_IRQ_7, CLEAR_LINE);
+		cpunum_set_input_line(0, M68K_IRQ_7, CLEAR_LINE);
 	}
 #else
 	/* work-around... */
 	if ((! parity_error_pending) && value)
 	{
 		parity_error_pending = TRUE;
-		cpu_set_irq_line(0, M68K_IRQ_7, PULSE_LINE);
-		cpu_irq_line_vector_w(0, M68K_IRQ_7, M68K_INT_ACK_AUTOVECTOR);
+		cpunum_set_input_line_and_vector(0, M68K_IRQ_7, PULSE_LINE, M68K_INT_ACK_AUTOVECTOR);
 	}
 	else if (parity_error_pending && (! value))
 	{
@@ -430,8 +426,8 @@ static void scan_keyboard( void )
 #if 0
 						if (keycode == NMIcode)
 						{	/* generate NMI interrupt */
-							cpu_set_irq_line(0, M68K_IRQ_7, PULSE_LINE);
-							cpu_irq_line_vector_w(0, M68K_IRQ_7, M68K_INT_ACK_AUTOVECTOR);
+							cpunum_set_input_line(0, M68K_IRQ_7, PULSE_LINE);
+							cpunum_set_input_line_vector(0, M68K_IRQ_7, M68K_INT_ACK_AUTOVECTOR);
 						}
 #endif
 						COPS_queue_data(& keycode, 1);
@@ -462,14 +458,10 @@ static void handle_mouse(int unused)
 		diff_x = new_mx - last_mx;
 
 		/* check for wrap */
-		/* does this code really work ??? */
-		if ((diff_x > 200) || (diff_x < -200))
-		{
-			if (diff_x < 0)
-				diff_x = - (256 + diff_x);
-			else
-				diff_x = 256 - diff_x;
-		}
+		if (diff_x > 0x80)
+			diff_x = 0x100-diff_x;
+		if  (diff_x < -0x80)
+			diff_x = -0x100-diff_x;
 
 		last_mx = new_mx;
 	}
@@ -479,14 +471,10 @@ static void handle_mouse(int unused)
 		diff_y = new_my - last_my;
 
 		/* check for wrap */
-		/* does this code really work ??? */
-		if (diff_y > 200 || diff_y < -200)
-		{
-			if (diff_y < 0)
-				diff_y = - (256 + diff_y);
-			else
-				diff_y = 256 - diff_y;
-		}
+		if (diff_y > 0x80)
+			diff_y = 0x100-diff_y;
+		if  (diff_y < -0x80)
+			diff_y = -0x100-diff_y;
 
 		last_my = new_my;
 	}
@@ -661,16 +649,10 @@ static void read_COPS_command(int unused)
 			break;
 
 		case 0x7:	/* send mouse command */
-			if (mouse_timer)
-			{
-				/* disable mouse timer -> disable mouse */
-				timer_remove(mouse_timer);
-				mouse_timer = NULL;
-			}
-
 			if (immediate & 0x8)
-				/* enable mouse */
-				mouse_timer = timer_pulse(TIME_IN_MSEC((immediate & 0x7)*4), 0, handle_mouse);
+				timer_adjust(mouse_timer, 0, 0, TIME_IN_MSEC((immediate & 0x7)*4)); /* enable mouse */
+			else
+				timer_reset(mouse_timer, TIME_NEVER);
 			break;
 		}
 	}
@@ -728,12 +710,7 @@ static void reset_COPS(void)
 	for (i=0; i<8; i++)
 		key_matrix[i] = 0;
 
-	if (mouse_timer)
-	{
-		/* disable mouse timer -> disable mouse */
-		timer_remove(mouse_timer);
-		mouse_timer = NULL;
-	}
+	timer_reset(mouse_timer, TIME_NEVER);
 }
 
 static void unplug_keyboard(void)
@@ -796,12 +773,12 @@ static void init_COPS(void)
 	CA1 (I) : COPS sending valid data
 	CA2 (O) : VIA -> COPS handshake
 */
-static WRITE_HANDLER(COPS_via_out_a)
+static WRITE8_HANDLER(COPS_via_out_a)
 {
 	COPS_command = data;
 }
 
-static WRITE_HANDLER(COPS_via_out_ca2)
+static WRITE8_HANDLER(COPS_via_out_ca2)
 {
 	hold_COPS_data = data;
 
@@ -825,7 +802,7 @@ static WRITE_HANDLER(COPS_via_out_ca2)
 	CB1 : not used
 	CB2 (O) : sound output
 */
-static READ_HANDLER(COPS_via_in_b)
+static  READ8_HANDLER(COPS_via_in_b)
 {
 	int val = 0;
 
@@ -838,7 +815,7 @@ static READ_HANDLER(COPS_via_in_b)
 	return val;
 }
 
-static WRITE_HANDLER(COPS_via_out_b)
+static WRITE8_HANDLER(COPS_via_out_b)
 {
 	/* pull-up */
 	data |= (~ via_read(0, VIA_DDRA)) & 0x01;
@@ -862,7 +839,7 @@ static WRITE_HANDLER(COPS_via_out_b)
 	}
 }
 
-static WRITE_HANDLER(COPS_via_out_cb2)
+static WRITE8_HANDLER(COPS_via_out_cb2)
 {
 	speaker_level_w(0, data);
 }
@@ -896,7 +873,7 @@ static void COPS_via_irq_func(int val)
 	CB1 : not used
 	CB2 (I) : current parity latch value
 */
-static READ_HANDLER(parallel_via_in_b)
+static  READ8_HANDLER(parallel_via_in_b)
 {
 	int val = 0;
 
@@ -906,6 +883,9 @@ static READ_HANDLER(parallel_via_in_b)
 	/* tell there is no hard disk : */
 	val |= 0x1;
 
+	/* keep busy high to work around a bug??? */
+	/*val |= 0x2; */
+
 	return val;
 }
 
@@ -918,70 +898,31 @@ static READ_HANDLER(parallel_via_in_b)
 	LISA video emulation
 */
 
-int lisa_vh_start(void)
+VIDEO_START( lisa )
 {
-	size_t videoram_size = 32760;	/*max(720*364, 608*431)/8*/
-
-	old_display = (UINT16 *) malloc(videoram_size);
-	if (! old_display)
-	{
-		return 1;
-	}
-	memset(old_display, 0, videoram_size);
-
 	return 0;
 }
 
-void lisa_vh_stop(void)
-{
-	free(old_display);
-}
-
 /*
-	TODO : use draw_scanline()...
-	"draw_scanline(bitmap, 0, y, (lisa_features.has_mac_xl_video) ? 608 : 720, buf, NULL, -1)"
+	Video update
 */
-void lisa_vh_screenrefresh(struct mame_bitmap *bitmap, int full_refresh)
+VIDEO_UPDATE( lisa )
 {
-	UINT16	data;
-	UINT16	*old;
-	UINT8	*v;
-	int		fg, bg, x, y;
-
+	UINT16 *v;
+	int x, y;
 	/* resolution is 720*364 on lisa, vs 608*431 on mac XL */
-	int resx = (lisa_features.has_mac_xl_video) ? 38 : 45;		/* width/16 */
+	int resx = (lisa_features.has_mac_xl_video) ? 608 : 720;	/* width */
 	int resy = (lisa_features.has_mac_xl_video) ? 431 : 364;	/* height */
 
-	v = videoram_ptr;
-	bg = Machine->pens[0];
-	fg = Machine->pens[1];
-	old = old_display;
+	UINT8 line_buffer[720];
 
-	for (y = 0; y < resy; y++) {
-		for ( x = 0; x < resx; x++ ) {
-			data = READ_WORD( v );
-			if (full_refresh || (data != *old)) {
-				plot_pixel( bitmap, ( x << 4 ) + 0x00, y, ( data & 0x8000 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x01, y, ( data & 0x4000 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x02, y, ( data & 0x2000 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x03, y, ( data & 0x1000 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x04, y, ( data & 0x0800 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x05, y, ( data & 0x0400 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x06, y, ( data & 0x0200 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x07, y, ( data & 0x0100 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x08, y, ( data & 0x0080 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x09, y, ( data & 0x0040 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0a, y, ( data & 0x0020 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0b, y, ( data & 0x0010 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0c, y, ( data & 0x0008 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0d, y, ( data & 0x0004 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0e, y, ( data & 0x0002 ) ? fg : bg );
-				plot_pixel( bitmap, ( x << 4 ) + 0x0f, y, ( data & 0x0001 ) ? fg : bg );
-				*old = data;
-			}
-			v += 2;
-			old++;
-		}
+	v = videoram_ptr;
+
+	for (y = 0; y < resy; y++)
+	{
+		for (x = 0; x < resx; x++)
+			line_buffer[x] = (v[(x+y*resx)>>4] & (0x8000 >> ((x+y*resx) & 0xf))) ? 1 : 0;
+		draw_scanline8(bitmap, 0, y, resx, line_buffer, Machine->pens, -1);
 	}
 }
 
@@ -1007,11 +948,14 @@ static OPBASE_HANDLER (lisa_OPbaseoverride)
 		{
 			if (address & 0x008000)
 			{	/* MMU register : BUS error ??? */
-				logerror("illegal opbase address%lX\n", address);
+				logerror("illegal opbase address%lX\n", (long) address);
 			}
 			else
 			{	/* system ROMs */
-				OP_ROM = OP_RAM = lisa_rom_ptr - (address & 0xffc000);
+				opcode_mask = 0xffffff;
+				opcode_base = opcode_arg_base = lisa_rom_ptr - (address & 0xffc000);
+				opcode_memory_min = (address & 0xffc000);
+				opcode_memory_max = (address & 0xffc000) + 0x003fff;
 				/*logerror("ROM (setup mode)\n");*/
 			}
 
@@ -1020,6 +964,9 @@ static OPBASE_HANDLER (lisa_OPbaseoverride)
 
 	}
 
+	if (cpunum_get_reg(0, M68K_SR) & 0x2000)
+		/* supervisor mode -> force register file 0 */
+		the_seg = 0;
 
 	{
 		int seg_offset = address & 0x01ffff;
@@ -1035,9 +982,12 @@ static OPBASE_HANDLER (lisa_OPbaseoverride)
 			if (seg_offset > mmu_regs[the_seg][segment].slim)
 			{
 				/* out of segment limits : bus error */
-				logerror("illegal opbase address%lX\n", address);
+				logerror("illegal opbase address%lX\n", (long) address);
 			}
-			OP_ROM = OP_RAM = lisa_ram_ptr + mapped_address - address;
+			opcode_mask = 0xffffff;
+			opcode_base = opcode_arg_base = lisa_ram_ptr + mapped_address - address;
+			opcode_memory_min = (address & 0xffc000);
+			opcode_memory_max = (address & 0xffc000) + 0x003fff;
 			/*logerror("RAM\n");*/
 			break;
 
@@ -1046,11 +996,14 @@ static OPBASE_HANDLER (lisa_OPbaseoverride)
 		case IO:			/* I/O : bus error ??? */
 		case invalid:		/* unmapped segment */
 			/* bus error */
-			logerror("illegal opbase address%lX\n", address);
+			logerror("illegal opbase address%lX\n", (long) address);
 			break;
 
 		case special_IO:
-			OP_ROM = OP_RAM = lisa_rom_ptr + (mapped_address & 0x003fff) - address;
+			opcode_mask = 0xffffff;
+			opcode_base = opcode_arg_base = lisa_rom_ptr + (mapped_address & 0x003fff) - address;
+			opcode_memory_min = (address & 0xffc000);
+			opcode_memory_max = (address & 0xffc000) + 0x003fff;
 			/*logerror("ROM\n");*/
 			break;
 		}
@@ -1069,27 +1022,9 @@ static OPBASE_HANDLER (lisa_fdc_OPbaseoverride)
 }
 
 
-int lisa_floppy_init(int id)
-{
-	/*if (lisa_features.lisa_floppy_hardware == twiggy)
-		return twiggy_floppy_init(id);
-	else*/
-		return sony_floppy_init(id, (lisa_features.has_double_sided_floppy)
-										? SONY_FLOPPY_ALLOW400K | SONY_FLOPPY_ALLOW800K
-										: SONY_FLOPPY_ALLOW400K | SONY_FLOPPY_EXT_SPEED_CONTROL);
-}
-
-void lisa_floppy_exit(int id)
-{
-	/*if (lisa_features.lisa_floppy_hardware == twiggy)
-		return twiggy_floppy_exit(id);
-	else*/
-		sony_floppy_exit(id);
-}
-
 /* should save PRAM to file */
 /* TODO : save time difference with host clock, set default date, etc */
-void lisa_nvram_handler(void *file, int read_or_write)
+NVRAM_HANDLER(lisa)
 {
 	UINT8 *l_fdc_ram = memory_region(REGION_CPU2) + FDC_RAM_OFFSET;	/* fdc_ram is not necessarily set yet */
 
@@ -1101,7 +1036,7 @@ void lisa_nvram_handler(void *file, int read_or_write)
 #if 0
 			logerror("Writing PRAM to file\n");
 #endif
-			osd_fwrite(file, l_fdc_ram, 1024);
+			mame_fwrite(file, l_fdc_ram, 1024);
 		}
 	}
 	else
@@ -1111,7 +1046,7 @@ void lisa_nvram_handler(void *file, int read_or_write)
 #if 0
 			logerror("Reading PRAM from file\n");
 #endif
-			osd_fread(file, l_fdc_ram, 1024);
+			mame_fread(file, l_fdc_ram, 1024);
 		}
 		else
 		{
@@ -1211,8 +1146,10 @@ static void lisa210_set_iwm_enable_lines(int enable_mask)
 	sony_set_enable_lines(enable_mask >> 1);
 }
 
-void lisa_init_machine(void)
+MACHINE_INIT( lisa )
 {
+	mouse_timer = timer_alloc(handle_mouse);
+
 	lisa_ram_ptr = memory_region(REGION_CPU1) + RAM_OFFSET;
 	lisa_rom_ptr = memory_region(REGION_CPU1) + ROM_OFFSET;
 
@@ -1224,6 +1161,8 @@ void lisa_init_machine(void)
 	memory_set_opbase_handler(0, lisa_OPbaseoverride);
 	memory_set_opbase_handler(1, lisa_fdc_OPbaseoverride);
 
+	cpunum_set_info_fct(0, CPUINFO_PTR_M68K_RESET_CALLBACK, (genf *)/*lisa_reset_instr_callback*/NULL);
+
 	/* init MMU */
 
 	setup = TRUE;
@@ -1246,7 +1185,7 @@ void lisa_init_machine(void)
 	set_VTIR(FALSE);
 
 	video_address_latch = 0;
-	videoram_ptr = lisa_ram_ptr;
+	videoram_ptr = (UINT16 *) lisa_ram_ptr;
 
 	/* reset COPS keyboard/mouse controller */
 	init_COPS();
@@ -1262,8 +1201,9 @@ void lisa_init_machine(void)
 
 	/* initialize floppy */
 	{
-		iwm_interface intf =
+		struct applefdc_interface intf =
 		{
+			APPLEFDC_IWM,
 			sony_set_lines,
 			sony_set_enable_lines,
 
@@ -1281,24 +1221,14 @@ void lisa_init_machine(void)
 			intf.set_enable_lines = lisa210_set_iwm_enable_lines;
 		}
 
-		iwm_init(& intf);
+		applefdc_init(& intf);
 
 		if (lisa_features.floppy_hardware == sony_lisa2)
 			sony_set_enable_lines(1);	/* on lisa2, drive unit 1 is always selected (?) */
 	}
 }
 
-void lisa_exit_machine(void)
-{
-	if (mouse_timer)
-	{
-		/* disable mouse timer -> disable mouse */
-		timer_remove(mouse_timer);
-		mouse_timer = NULL;
-	}
-}
-
-int lisa_interrupt(void)
+void lisa_interrupt(void)
 {
 	static int frame_count = 0;
 
@@ -1393,8 +1323,6 @@ int lisa_interrupt(void)
 
 	/* do keyboard scan */
 	scan_keyboard();
-
-	return 0;
 }
 
 /*
@@ -1425,7 +1353,7 @@ INLINE void lisa_fdc_ttl_glue_access(offs_t offset)
 			if (MT1 && ! oldMT1)
 			{
 				PWM_floppy_motor_speed = (PWM_floppy_motor_speed << 1) & 0xff;
-				if (iwm_get_lines() & IWM_PH0)
+				if (applefdc_get_lines() & APPLEFDC_PH0)
 					PWM_floppy_motor_speed |= 1;
 				sony_set_speed(((256-PWM_floppy_motor_speed) * 1.3) + 237);
 			}
@@ -1453,14 +1381,14 @@ INLINE void lisa_fdc_ttl_glue_access(offs_t offset)
 	}
 }
 
-READ_HANDLER ( lisa_fdc_io_r )
+ READ8_HANDLER ( lisa_fdc_io_r )
 {
 	int answer=0;
 
 	switch ((offset & 0x0030) >> 4)
 	{
 	case 0:	/* IWM */
-		answer = /*iwm_lisa_r*/iwm_r(offset);
+		answer = /*iwm_lisa_r*/applefdc_r(offset);
 		break;
 
 	case 1:	/* TTL glue */
@@ -1468,7 +1396,7 @@ READ_HANDLER ( lisa_fdc_io_r )
 		answer = 0;	/* ??? */
 		break;
 
-	case 2:	/* pulses the PWM LOAD line (mistake !) */
+	case 2:	/* pulses the PWM LOAD line (bug!) */
 		answer = 0;	/* ??? */
 		break;
 
@@ -1480,12 +1408,12 @@ READ_HANDLER ( lisa_fdc_io_r )
 	return answer;
 }
 
-WRITE_HANDLER ( lisa_fdc_io_w )
+WRITE8_HANDLER ( lisa_fdc_io_w )
 {
 	switch ((offset & 0x0030) >> 4)
 	{
 	case 0:	/* IWM */
-		/*iwm_lisa_w*/iwm_w(offset, data);
+		/*iwm_lisa_w*/applefdc_w(offset, data);
 		break;
 
 	case 1:	/* TTL glue */
@@ -1505,7 +1433,7 @@ WRITE_HANDLER ( lisa_fdc_io_w )
 	}
 }
 
-READ_HANDLER ( lisa_fdc_r )
+ READ8_HANDLER ( lisa_fdc_r )
 {
 	if (! (offset & 0x1000))
 	{
@@ -1521,7 +1449,7 @@ READ_HANDLER ( lisa_fdc_r )
 		return fdc_rom[offset & 0x0fff];
 }
 
-READ_HANDLER ( lisa210_fdc_r )
+ READ8_HANDLER ( lisa210_fdc_r )
 {
 	if (! (offset & 0x1000))
 	{
@@ -1537,7 +1465,7 @@ READ_HANDLER ( lisa210_fdc_r )
 		return fdc_rom[offset & 0x0fff];
 }
 
-WRITE_HANDLER ( lisa_fdc_w )
+WRITE8_HANDLER ( lisa_fdc_w )
 {
 	if (! (offset & 0x1000))
 	{
@@ -1551,7 +1479,7 @@ WRITE_HANDLER ( lisa_fdc_w )
 	}
 }
 
-WRITE_HANDLER ( lisa210_fdc_w )
+WRITE8_HANDLER ( lisa210_fdc_w )
 {
 	if (! (offset & 0x1000))
 	{
@@ -1611,6 +1539,10 @@ READ16_HANDLER ( lisa_r )
 		}
 	}
 
+	if (cpunum_get_reg(0, M68K_SR) & 0x2000)
+		/* supervisor mode -> force register file 0 */
+		the_seg = 0;
+
 	{
 		/* offset in segment */
 		int seg_offset = (offset & 0x00ffff) << 1;
@@ -1631,7 +1563,7 @@ READ16_HANDLER ( lisa_r )
 				/* out of segment limits : bus error */
 
 			}
-			answer = READ_WORD(lisa_ram_ptr + address);
+			answer = *(UINT16 *)(lisa_ram_ptr + address);
 
 			if (bad_parity_count && test_parity
 					&& (bad_parity_table[address >> 3] & (0x3 << (address & 0x7))))
@@ -1648,7 +1580,7 @@ READ16_HANDLER ( lisa_r )
 				/* out of segment limits : bus error */
 
 			}
-			answer = READ_WORD(lisa_ram_ptr + address);
+			answer = *(UINT16 *)(lisa_ram_ptr + address);
 
 			if (bad_parity_count && test_parity
 					&& (bad_parity_table[address >> 3] & (0x3 << (address & 0x7))))
@@ -1670,7 +1602,7 @@ READ16_HANDLER ( lisa_r )
 
 		case special_IO:
 			if (! (address & 0x008000))
-				answer = READ_WORD(lisa_rom_ptr + (address & 0x003fff));
+				answer = *(UINT16 *)(lisa_rom_ptr + (address & 0x003fff));
 			else
 			{	/* read serial number from ROM */
 				/* this has to be be the least efficient way to read a ROM :-) */
@@ -1838,6 +1770,10 @@ WRITE16_HANDLER ( lisa_w )
 		}
 	}
 
+	if (cpunum_get_reg(0, M68K_SR) & 0x2000)
+		/* supervisor mode -> force register file 0 */
+		the_seg = 0;
+
 	{
 		/* offset in segment */
 		int seg_offset = (offset & 0x00ffff) << 1;
@@ -2187,7 +2123,7 @@ static WRITE16_HANDLER ( lisa_IO_w )
 		case 0x1:	/* Video Address Latch */
 			/*logerror("video address latch write offs=%X, data=%X\n", offset, data);*/
 			COMBINE_DATA(& video_address_latch);
-			videoram_ptr = lisa_ram_ptr + ((video_address_latch << 7) & 0x1f8000);
+			videoram_ptr = ((UINT16 *)lisa_ram_ptr) + ((video_address_latch << 6) & 0xfc000);
 			/*logerror("video address latch %X -> base address %X\n", video_address_latch,
 							(video_address_latch << 7) & 0x1f8000);*/
 			break;
