@@ -7,26 +7,23 @@
 ***************************************************************************/
 #include <ctype.h>
 #include <stdio.h>
-#include "snprintf.h"
 
 #include "driver.h"
-#include "unzip.h"
+#include "image.h"
+#include "sound/dac.h"
 
 #define VERBOSE_DBG 0
 #include "includes/cbm.h"
 #include "includes/vc20tape.h"
 
-struct DACinterface vc20tape_sound_interface =
-{
-	1,
-	{25}
-};
-
 #define TONE_ON_VALUE 0xff
 
 /* write line high active, */
 /* read line low active!? */
 
+#define TAPE_WAV 1
+#define TAPE_PRG 2
+
 static struct
 {
 	int on, noise;
@@ -36,9 +33,6 @@ static struct
 	int motor;
 	void (*read_callback) (UINT32, UINT8);
 
-#define TAPE_WAV 1
-#define TAPE_PRG 2
-#define TAPE_ZIP 3
 	int type;						   /* 0 nothing */
 }
 tape;
@@ -48,21 +42,17 @@ struct
 {
 	int state;
 	void *timer;
-	int image_type;
-    int image_id;
 	int pos;
 	struct GameSample *sample;
+	mess_image *img;
 } wav;
 
 /* these are the values for prg files */
 struct
 {
 	int state;
-
-	/* these values are shared with the zip driver */
+	void *prg_timer;
 	void *timer;
-	int image_type;
-    int image_id;
 	int pos;
 
 #define VC20_SHORT		(176e-6)
@@ -82,29 +72,17 @@ struct
 	char name[16];					   /*name for cbm */
 	UINT8 chksum;
 	double lasttime;
+	mess_image *img;
 } prg;
 
-/* these are values for zip files */
-struct
-{
-	int image_type;
-    int image_id;
-	int state;
-	ZIP *zip;
-	struct zipent *zipentry;
-} zip;
-
 /* from sound/samples.c no changes (static declared) */
 /* readsamples not useable (loads files only from sample or game directory) */
 /* and doesn't search the rompath */
-#ifdef LSB_FIRST
-#define intelLong(x) (x)
-#else
-#define intelLong(x) (((x << 24) | (((unsigned long) x) >> 24) | \
-                       (( x & 0x0000ff00) << 8) | (( x & 0x00ff0000) >> 8)))
-#endif
-static struct GameSample *vc20_read_wav_sample (void *f)
+static struct GameSample *vc20_read_wav_sample (mame_file *f)
 {
+	/* NPW 28-Feb-2005 - this code sucks */
+	return NULL;
+#if 0
 	unsigned long offset = 0;
 	UINT32 length, rate, filesize, temp32;
 	UINT16 bits, temp16;
@@ -112,20 +90,20 @@ static struct GameSample *vc20_read_wav_sample (void *f)
 	struct GameSample *result;
 
 	/* read the core header and make sure it's a WAVE file */
-	offset += osd_fread (f, buf, 4);
+	offset += mame_fread (f, buf, 4);
 	if (offset < 4)
 		return NULL;
 	if (memcmp (&buf[0], "RIFF", 4) != 0)
 		return NULL;
 
 	/* get the total size */
-	offset += osd_fread (f, &filesize, 4);
+	offset += mame_fread (f, &filesize, 4);
 	if (offset < 8)
 		return NULL;
-	filesize = intelLong (filesize);
+	filesize = LITTLE_ENDIANIZE_INT32 (filesize);
 
 	/* read the RIFF file type and make sure it's a WAVE file */
-	offset += osd_fread (f, buf, 4);
+	offset += mame_fread (f, buf, 4);
 	if (offset < 12)
 		return NULL;
 	if (memcmp (&buf[0], "WAVE", 4) != 0)
@@ -134,56 +112,56 @@ static struct GameSample *vc20_read_wav_sample (void *f)
 	/* seek until we find a format tag */
 	while (1)
 	{
-		offset += osd_fread (f, buf, 4);
-		offset += osd_fread (f, &length, 4);
-		length = intelLong (length);
+		offset += mame_fread (f, buf, 4);
+		offset += mame_fread (f, &length, 4);
+		length = LITTLE_ENDIANIZE_INT32 (length);
 		if (memcmp (&buf[0], "fmt ", 4) == 0)
 			break;
 
 		/* seek to the next block */
-		osd_fseek (f, length, SEEK_CUR);
+		mame_fseek (f, length, SEEK_CUR);
 		offset += length;
 		if (offset >= filesize)
 			return NULL;
 	}
 
 	/* read the format -- make sure it is PCM */
-	offset += osd_fread_lsbfirst (f, &temp16, 2);
+	offset += mame_fread_lsbfirst (f, &temp16, 2);
 	if (temp16 != 1)
 		return NULL;
 
 	/* number of channels -- only mono is supported */
-	offset += osd_fread_lsbfirst (f, &temp16, 2);
+	offset += mame_fread_lsbfirst (f, &temp16, 2);
 	if (temp16 != 1)
 		return NULL;
 
 	/* sample rate */
-	offset += osd_fread (f, &rate, 4);
-	rate = intelLong (rate);
+	offset += mame_fread (f, &rate, 4);
+	rate = LITTLE_ENDIANIZE_INT32 (rate);
 
 	/* bytes/second and block alignment are ignored */
-	offset += osd_fread (f, buf, 6);
+	offset += mame_fread (f, buf, 6);
 
 	/* bits/sample */
-	offset += osd_fread_lsbfirst (f, &bits, 2);
+	offset += mame_fread_lsbfirst (f, &bits, 2);
 	if (bits != 8 && bits != 16)
 		return NULL;
 
 	/* seek past any extra data */
-	osd_fseek (f, length - 16, SEEK_CUR);
+	mame_fseek (f, length - 16, SEEK_CUR);
 	offset += length - 16;
 
 	/* seek until we find a data tag */
 	while (1)
 	{
-		offset += osd_fread (f, buf, 4);
-		offset += osd_fread (f, &length, 4);
-		length = intelLong (length);
+		offset += mame_fread (f, buf, 4);
+		offset += mame_fread (f, &length, 4);
+		length = LITTLE_ENDIANIZE_INT32 (length);
 		if (memcmp (&buf[0], "data", 4) == 0)
 			break;
 
 		/* seek to the next block */
-		osd_fseek (f, length, SEEK_CUR);
+		mame_fseek (f, length, SEEK_CUR);
 		offset += length;
 		if (offset >= filesize)
 			return NULL;
@@ -203,7 +181,7 @@ static struct GameSample *vc20_read_wav_sample (void *f)
 	/* read the data in */
 	if (bits == 8)
 	{
-		osd_fread (f, result->data, length);
+		mame_fread (f, result->data, length);
 
 		/* convert 8-bit data to signed samples */
 		for (temp32 = 0; temp32 < length; temp32++)
@@ -212,15 +190,17 @@ static struct GameSample *vc20_read_wav_sample (void *f)
 	else
 	{
 		/* 16-bit data is fine as-is */
-		osd_fread_lsbfirst (f, result->data, length);
+		mame_fread_lsbfirst (f, result->data, length);
 	}
-
 	return result;
+#endif
 }
 
 static void vc20_wav_timer (int data);
 static void vc20_wav_state (void)
 {
+	/* NPW 28-Feb-2005 - this code sucks */
+#if 0
 	switch (wav.state)
 	{
 	case 0:
@@ -244,7 +224,7 @@ static void vc20_wav_state (void)
 		if (tape.motor && tape.play)
 		{
 			wav.state = 3;
-			wav.timer = timer_pulse (1.0 / wav.sample->smpfreq, 0, vc20_wav_timer);
+			timer_adjust(wav.timer, 0, 0, 1.0 / wav.sample->smpfreq);
 			break;
 		}
 		if (tape.motor && tape.record)
@@ -260,15 +240,13 @@ static void vc20_wav_state (void)
 			tape.play = 0;
 			tape.record = 0;
 			DAC_data_w (0, 0);
-			if (wav.timer)
-				timer_remove (wav.timer);
+			timer_reset(wav.timer, TIME_NEVER);
 			break;
 		}
 		if (!tape.motor || !tape.play)
 		{
 			wav.state = 2;
-			if (wav.timer)
-				timer_remove (wav.timer);
+			timer_reset(wav.timer, TIME_NEVER);
 			DAC_data_w (0, 0);
 			break;
 		}
@@ -290,30 +268,20 @@ static void vc20_wav_state (void)
 		}
 		break;
 	}
+#endif
 }
 
-static void vc20_wav_open (int image_type, int image_id)
+static void vc20_wav_open(mess_image *img, mame_file *fp)
 {
-	FILE *fp;
-
-	fp = (FILE*)osd_fopen (Machine->gamedrv->name, device_filename(image_type,image_id), OSD_FILETYPE_IMAGE, 0);
-	if (!fp)
-	{
-		logerror("tape %s file not found\n", device_filename(image_type,image_id));
-		return;
-	}
 	if ((wav.sample = vc20_read_wav_sample (fp)) == NULL)
 	{
-		logerror("tape %s could not be loaded\n", device_filename(image_type,image_id));
-		osd_fclose (fp);
+		logerror("tape %s could not be loaded\n", image_filename(img));
 		return;
 	}
-	logerror("tape %s loaded\n", device_filename(image_type,image_id));
-	osd_fclose (fp);
+	logerror("tape %s loaded\n", image_filename(img));
 
-	wav.image_type = image_type;
-    wav.image_id = image_id;
 	tape.type = TAPE_WAV;
+	wav.img = img;
 	wav.pos = 0;
 	tape.on = 1;
 	wav.state = 2;
@@ -327,6 +295,8 @@ static void vc20_wav_write (int data)
 
 static void vc20_wav_timer (int data)
 {
+	/* NPW 28-Feb-2005 - this code sucks */
+#if 0
 	if (wav.sample->resolution == 8)
 	{
 		tape.data = wav.sample->data[wav.pos] > 0x0;
@@ -352,6 +322,7 @@ static void vc20_wav_timer (int data)
 	if (tape.read_callback)
 		tape.read_callback (0, tape.data);
 	/*    vc20_wav_state(); // removing timer in timer puls itself hangs */
+#endif
 }
 
 static void vc20_prg_timer (int data);
@@ -380,7 +351,7 @@ static void vc20_prg_state (void)
 		if (tape.motor && tape.play)
 		{
 			prg.state = 3;
-			prg.timer = timer_set (0.0, 0, vc20_prg_timer);
+			timer_reset(prg.timer, 0.0);
 			break;
 		}
 		if (tape.motor && tape.record)
@@ -396,15 +367,13 @@ static void vc20_prg_state (void)
 			tape.play = 0;
 			tape.record = 0;
 			DAC_data_w (0, 0);
-			if (prg.timer)
-				timer_remove (prg.timer);
+			timer_reset(prg.timer, TIME_NEVER);
 			break;
 		}
 		if (!tape.motor || !tape.play)
 		{
 			prg.state = 2;
-			if (prg.timer)
-				timer_remove (prg.timer);
+			timer_reset(prg.timer, TIME_NEVER);
 			DAC_data_w (0, 0);
 			break;
 		}
@@ -428,37 +397,27 @@ static void vc20_prg_state (void)
 	}
 }
 
-static void vc20_prg_open (int image_type, int image_id)
+static void vc20_prg_open(mess_image *img, mame_file *fp)
 {
 	const char *name;
-    FILE *fp;
 	int i;
 
-	fp = (FILE*)osd_fopen (Machine->gamedrv->name, device_filename(image_type,image_id), OSD_FILETYPE_IMAGE, 0);
-	if (!fp)
-	{
-		logerror("tape %s file not found\n", device_filename(image_type,image_id));
-		return;
-	}
-	prg.length = osd_fsize (fp);
+	prg.length = mame_fsize (fp);
 	if ((prg.prg = (UINT8 *) malloc (prg.length)) == NULL)
 	{
-		logerror("tape %s could not be loaded\n", device_filename(image_type,image_id));
-		osd_fclose (fp);
+		logerror("tape %s could not be loaded\n", image_filename(img));
 		return;
 	}
-	osd_fread (fp, prg.prg, prg.length);
-	logerror("tape %s loaded\n", device_filename(image_type,image_id));
-	osd_fclose (fp);
+	mame_fread (fp, prg.prg, prg.length);
+	logerror("tape %s loaded\n", image_filename(img));
 
-	name = device_filename(image_type,image_id);
+	name = image_filename(img);
     for (i = 0; name[i] != 0; i++)
 		prg.name[i] = toupper (name[i]);
 	for (; i < 16; i++)
 		prg.name[i] = ' ';
 
-	prg.image_type = image_type;
-    prg.image_id = image_id;
+	prg.img = img;
 	prg.stateblock = 0;
 	prg.stateheader = 0;
 	prg.statebyte = 0;
@@ -467,6 +426,7 @@ static void vc20_prg_open (int image_type, int image_id)
 	tape.on = 1;
 	prg.state = 2;
 	prg.pos = 0;
+	prg.timer = prg.prg_timer;
 }
 
 static void vc20_prg_write (int data)
@@ -911,7 +871,6 @@ static void vc20_prg_timer (int data)
 			vc20_tape_program ();
 			if (!prg.stateblock)
 			{
-				prg.timer = 0;
 				tape.play = 0;
 			}
 		}
@@ -921,187 +880,6 @@ static void vc20_prg_timer (int data)
 	vc20_prg_state ();
 }
 
-static void vc20_zip_timer (int data);
-static void vc20_zip_state (void)
-{
-	switch (zip.state)
-	{
-	case 0:
-		break;						   /* not inited */
-	case 1:						   /* off */
-		if (tape.on)
-		{
-			zip.state = 2;
-			break;
-		}
-		break;
-	case 2:						   /* on */
-		if (!tape.on)
-		{
-			zip.state = 1;
-			tape.play = 0;
-			tape.record = 0;
-			DAC_data_w (0, 0);
-			break;
-		}
-		if (tape.motor && tape.play)
-		{
-			zip.state = 3;
-			prg.timer = timer_set (0.0, 0, vc20_zip_timer);
-			break;
-		}
-		if (tape.motor && tape.record)
-		{
-			zip.state = 4;
-			break;
-		}
-		break;
-	case 3:						   /* reading */
-		if (!tape.on)
-		{
-			zip.state = 1;
-			tape.play = 0;
-			tape.record = 0;
-			DAC_data_w (0, 0);
-			if (prg.timer)
-				timer_remove (prg.timer);
-			break;
-		}
-		if (!tape.motor || !tape.play)
-		{
-			zip.state = 2;
-			if (prg.timer)
-				timer_remove (prg.timer);
-			DAC_data_w (0, 0);
-			break;
-		}
-		break;
-	case 4:						   /* saving */
-		if (!tape.on)
-		{
-			zip.state = 1;
-			tape.play = 0;
-			tape.record = 0;
-			DAC_data_w (0, 0);
-			timer_remove (prg.timer);
-			break;
-		}
-		if (!tape.motor || !tape.record)
-		{
-			zip.state = 2;
-			timer_remove (prg.timer);
-			DAC_data_w (0, 0);
-			break;
-		}
-		break;
-	}
-}
-
-static void vc20_zip_readfile (void)
-{
-	int i;
-	char *cp;
-
-	for (i = 0; i < 2; i++)
-	{
-		zip.zipentry = readzip (zip.zip);
-		if (zip.zipentry == NULL)
-		{
-			i++;
-			rewindzip (zip.zip);
-			continue;
-		}
-		if ((cp = strrchr (zip.zipentry->name, '.')) == NULL)
-			continue;
-		if (stricmp (cp, ".prg") == 0)
-			break;
-	}
-
-	if (i == 2)
-	{
-		zip.state = 0;
-		return;
-	}
-	for (i = 0; zip.zipentry->name[i] != 0; i++)
-		prg.name[i] = toupper (zip.zipentry->name[i]);
-	for (; i < 16; i++)
-		prg.name[i] = ' ';
-
-	prg.length = zip.zipentry->uncompressed_size;
-	if ((prg.prg = (UINT8 *) malloc (prg.length)) == NULL)
-	{
-		logerror("out of memory\n");
-		zip.state = 0;
-	}
-	readuncompresszip (zip.zip, zip.zipentry, (char *) prg.prg);
-}
-
-static void vc20_zip_open (int image_type, int image_id)
-{
-	if (!(zip.zip = openzip (device_filename(image_type,image_id))))
-	{
-		logerror("tape %s not found\n", device_filename(image_type,image_id));
-		return;
-	}
-
-	logerror("tape %s linked\n", device_filename(image_type,image_id));
-
-	tape.type = TAPE_ZIP;
-	tape.on = 1;
-	zip.image_type = image_type;
-    zip.image_id = image_id;
-	zip.state = 2;
-	prg.stateblock = 0;
-	prg.stateheader = 0;
-	prg.statebyte = 0;
-	prg.statebit = 0;
-	prg.pos = 0;
-	vc20_zip_readfile ();
-}
-
-static void vc20_zip_timer (int data)
-{
-	if (!tape.data)
-	{								   /* send the same low phase */
-		if (tape.noise)
-			DAC_data_w (0, 0);
-		tape.data = 1;
-		timer_reset (prg.timer, prg.lasttime);
-	}
-	else
-	{
-		if (tape.noise)
-			DAC_data_w (0, TONE_ON_VALUE);
-		tape.data = 0;
-		if (prg.statebit)
-		{
-			vc20_tape_bit (0);
-		}
-		else if (prg.statebyte)
-		{							   /* send the rest of the byte */
-			vc20_tape_byte ();
-		}
-		else if (prg.stateheader)
-		{
-			vc20_tape_prgheader ();
-		}
-		else
-		{
-			vc20_tape_program ();
-			if (!prg.stateblock)
-			{
-				/* loading next file of zip */
-				timer_reset (prg.timer, 0.0);
-				free (prg.prg);
-				vc20_zip_readfile ();
-			}
-		}
-	}
-	if (tape.read_callback)
-		tape.read_callback (0, tape.data);
-	vc20_prg_state ();
-}
-
 void vc20_tape_open (void (*read_callback) (UINT32, UINT8))
 {
 	tape.read_callback = read_callback;
@@ -1115,6 +893,8 @@ void vc20_tape_open (void (*read_callback) (UINT32, UINT8))
 	tape.data = 0;
 #endif
 	prg.c16 = 0;
+	wav.timer = timer_alloc(vc20_wav_timer);
+	prg.prg_timer = timer_alloc(vc20_prg_timer);
 }
 
 void c16_tape_open (void)
@@ -1123,9 +903,9 @@ void c16_tape_open (void)
 	prg.c16 = 1;
 }
 
-int vc20_tape_attach_image (int id)
+static DEVICE_LOAD(vc20_tape)
 {
-    char *cp;
+	const char *cp;
 
 	tape.type = 0;
 	tape.on = 0;
@@ -1135,29 +915,23 @@ int vc20_tape_attach_image (int id)
 	tape.motor = 0;
 	tape.data = 0;
 
-	if (device_filename(IO_CASSETTE,id) == NULL)
+	if (!file)
 		return INIT_PASS;
 
-	if ((cp = strrchr (device_filename(IO_CASSETTE,id), '.')) == NULL)
+	cp = image_filetype(image);
+	if (!cp)
 		return INIT_FAIL;
-	if (stricmp (cp, ".wav") == 0)
-	{
-		vc20_wav_open (IO_CASSETTE,id);
-	}
-	else if (stricmp (cp, ".prg") == 0)
-	{
-		vc20_prg_open (IO_CASSETTE,id);
-	}
-	else if (stricmp (cp, ".zip") == 0)
-	{
-		vc20_zip_open (IO_CASSETTE,id);
-	}
+
+	if (mame_stricmp (cp, "wav") == 0)
+		vc20_wav_open(image, file);
+	else if (mame_stricmp (cp, "prg") == 0)
+		vc20_prg_open(image, file);
 	else
 		return INIT_FAIL;
 	return INIT_PASS;
 }
 
-void vc20_tape_detach_image (int id)
+static DEVICE_UNLOAD(vc20_tape)
 {
 	vc20_tape_close();
 }
@@ -1172,10 +946,6 @@ void vc20_tape_close (void)
 	case TAPE_PRG:
 		free (prg.prg);
 		break;
-	case TAPE_ZIP:
-		free (prg.prg);
-		closezip (zip.zip);
-		break;
 	}
 	/* HJB reset so vc20_tape_close() can be called multiple times!? */
     tape.type = 0;
@@ -1191,9 +961,6 @@ static void vc20_state (void)
 	case TAPE_PRG:
 		vc20_prg_state ();
 		break;
-	case TAPE_ZIP:
-		vc20_zip_state ();
-		break;
 	}
 }
 
@@ -1209,9 +976,6 @@ int vc20_tape_switch (void)
 	case TAPE_PRG:
 		data = !((prg.state > 1) && (tape.play || tape.record));
 		break;
-	case TAPE_ZIP:
-		data = !((zip.state > 1) && (tape.play || tape.record));
-		break;
 	}
 	return data;
 }
@@ -1228,10 +992,6 @@ int vc20_tape_read (void)
 		if (prg.state == 3)
 			return tape.data;
 		break;
-	case TAPE_ZIP:
-		if (zip.state == 3)
-			return tape.data;
-		break;
 	}
 	return 0;
 }
@@ -1249,10 +1009,6 @@ void vc20_tape_write (int data)
 		if (prg.state == 4)
 			vc20_prg_write (data);
 		break;
-	case TAPE_ZIP:
-		if (zip.state == 4)
-			vc20_prg_write (data);
-		break;
 	}
 }
 
@@ -1266,9 +1022,6 @@ void vc20_tape_config (int on, int noise)
 	case TAPE_PRG:
 		tape.on = (prg.state != 0) && on;
 		break;
-	case TAPE_ZIP:
-		tape.on = (zip.state != 0) && on;
-		break;
 	}
 	tape.noise = tape.on && noise;
 	vc20_state ();
@@ -1309,10 +1062,13 @@ void vc20_tape_status (char *text, int size)
 			snprintf (text, size, "Tape saving");
 			break;
 		case 3:
+			/* NPW 28-Feb-2005 - this code sucks */
+#if 0
 			snprintf (text, size, "Tape (%s) loading %d/%dsec",
-					  device_filename(wav.image_type, wav.image_id),
+					  image_filename(wav.img),
 					  wav.pos / wav.sample->smpfreq,
 					  wav.sample->length / wav.sample->smpfreq);
+#endif
 			break;
 		}
 		break;
@@ -1324,21 +1080,21 @@ void vc20_tape_status (char *text, int size)
 			break;
 		case 3:
 			snprintf (text, size, "Tape (%s) loading %d",
-				device_filename(prg.image_type, prg.image_id), prg.pos);
-			break;
-		}
-		break;
-	case TAPE_ZIP:
-		switch (zip.state)
-		{
-		case 4:
-			snprintf (text, size, "Tape saving");
-			break;
-		case 3:
-			snprintf (text, size, "Tape (%s) File %s loading %d",
-				device_filename(zip.image_type,zip.image_id), zip.zipentry->name, prg.pos);
+				image_filename(prg.img), prg.pos);
 			break;
 		}
 		break;
 	}
 }
+
+
+
+void vc20tape_device_getinfo(struct IODevice *dev)
+{
+	dev->type = IO_CASSETTE;
+	dev->count = 1;
+	dev->file_extensions = "wav\0";
+	dev->load = device_load_vc20_tape;
+	dev->unload = device_unload_vc20_tape;
+}
+
