@@ -1,13 +1,13 @@
 /******************************************************************************
  PeT mess@utanet.at 2000,2001
 ******************************************************************************/
+
 #include <assert.h>
 #include <math.h>
 #include "osd_cpu.h"
 #include "sound/streams.h"
 #include "mame.h"
 #include "timer.h"
-#include "sound/mixer.h"
 
 #include "includes/lynx.h"
 
@@ -83,7 +83,7 @@ AUD_B_RIGHT	EQU %00000010
 AUD_A_RIGHT	EQU %00000001
 
  */
-static int mixer_channel;
+static sound_stream *mixer_channel;
 static int usec_per_sample;
 static int *shift_mask;
 static int *shift_xor;
@@ -129,22 +129,6 @@ void lynx_audio_count_down(int nr)
     This->count--;
 }
 
-void lynx_audio_debug(struct mame_bitmap *bitmap)
-{
-    char str[40];
-    sprintf(str,"%.2x %.2x %.2x %.2x %.2x %.2x %.2x %.2x",
-	    lynx_audio[0].reg.data[0],
-	    lynx_audio[0].reg.data[1],
-	    lynx_audio[0].reg.data[2],
-	    lynx_audio[0].reg.data[3],
-	    lynx_audio[0].reg.data[4],
-	    lynx_audio[0].reg.data[5],
-	    lynx_audio[0].reg.data[6],
-	    lynx_audio[0].reg.data[7]);
-
-/*    ui_text(bitmap, str, 0,0); */
-}
-
 static void lynx_audio_shift(LYNX_AUDIO *channel)
 {
     channel->shifter=((channel->shifter<<1)&0x3ff)
@@ -277,133 +261,128 @@ void lynx_audio_write(int offset, UINT8 data)
 /************************************/
 /* Sound handler update             */
 /************************************/
-void lynx_update (int param, INT16 *buffer, int length)
+static void lynx_update (void *param,stream_sample_t **inputs, stream_sample_t **_buffer,int length)
 {
-    int i, j;
-    LYNX_AUDIO *channel;
-    int v;
-    
-    for (i = 0; i < length; i++, buffer++)
-    {
-	*buffer = 0;
-	for (channel=lynx_audio, j=0; j<ARRAY_LENGTH(lynx_audio); j++, channel++) {
-	    lynx_audio_execute(channel);
-	    v=channel->reg.n.output;
-	    *buffer+=v*15;
+	int i, j;
+	LYNX_AUDIO *channel;
+	int v;
+	stream_sample_t *buffer = _buffer[0];
+
+	for (i = 0; i < length; i++, buffer++)
+	{
+		*buffer = 0;
+		for (channel=lynx_audio, j=0; j<ARRAY_LENGTH(lynx_audio); j++, channel++)
+		{
+			lynx_audio_execute(channel);
+			v=channel->reg.n.output;
+			*buffer+=v*15;
+		}
 	}
-    }
 }
 
-void lynx2_update (int param, INT16 **buffer, int length)
+static void lynx2_update (void *param,stream_sample_t **inputs, stream_sample_t **buffer,int length)
 {
-    INT16 *left=buffer[0], *right=buffer[1];
-    int i, j;
-    LYNX_AUDIO *channel;
-    int v;
-    
-    for (i = 0; i < length; i++, left++, right++)
-    {
-	*left = 0;
-	*right= 0;
-	for (channel=lynx_audio, j=0; j<ARRAY_LENGTH(lynx_audio); j++, channel++) {
-	    lynx_audio_execute(channel);
-	    v=channel->reg.n.output;
-	    if (!(master_enable&(0x10<<j))) {		    
-		if (attenuation_enable&(0x10<<j)) {
-		    *left+=v*(channel->attenuation>>4);
-		} else {
-		    *left+=v*15;
+	stream_sample_t *left=buffer[0], *right=buffer[1];
+	int i, j;
+	LYNX_AUDIO *channel;
+	int v;
+
+	for (i = 0; i < length; i++, left++, right++)
+	{
+		*left = 0;
+		*right= 0;
+		for (channel=lynx_audio, j=0; j<ARRAY_LENGTH(lynx_audio); j++, channel++)
+		{
+			lynx_audio_execute(channel);
+			v=channel->reg.n.output;
+			if (!(master_enable&(0x10<<j)))
+			{		    
+				if (attenuation_enable&(0x10<<j)) {
+					*left+=v*(channel->attenuation>>4);
+				} else {
+					*left+=v*15;
+				}
+			}
+			if (!(master_enable&(1<<j)))
+			{
+				if (attenuation_enable&(1<<j)) {
+					*right+=v*(channel->attenuation&0xf);
+				} else {
+					*right+=v*15;
+				}
+			}
 		}
-	    }
-	    if (!(master_enable&(1<<j))) {
-		if (attenuation_enable&(1<<j)) {
-		    *right+=v*(channel->attenuation&0xf);
-		} else {
-		    *right+=v*15;
-		}
-	    }
 	}
-    }
 }
 
 static void lynx_audio_init(void)
 {
-    int i;
-    shift_mask=(int*)malloc(512*sizeof(int));
-    assert(shift_mask!=0);
+	int i;
+	shift_mask = (int *) auto_malloc(512 * sizeof(int));
+	assert(shift_mask);
 
-    shift_xor=(int*)malloc(4096*sizeof(int));
-    assert(shift_xor!=0);
+	shift_xor = (int *) auto_malloc(4096 * sizeof(int));
+	assert(shift_xor);
 
-    for (i=0; i<512; i++) {
-	shift_mask[i]=0;
-	if (i&1) shift_mask[i]|=1;
-	if (i&2) shift_mask[i]|=2;
-	if (i&4) shift_mask[i]|=4;
-	if (i&8) shift_mask[i]|=8;
-	if (i&0x10) shift_mask[i]|=0x10;
-	if (i&0x20) shift_mask[i]|=0x20;
-	if (i&0x40) shift_mask[i]|=0x400;
-	if (i&0x80) shift_mask[i]|=0x800;
-	if (i&0x100) shift_mask[i]|=0x80;
-    }
-    for (i=0; i<4096; i++) {
-	int j;
-	shift_xor[i]=1;
-	for (j=4096/2; j>0; j>>=1) {
-	    if (i&j) shift_xor[i]^=1;
+	for (i=0; i<512; i++)
+	{
+		shift_mask[i]=0;
+		if (i&1) shift_mask[i]|=1;
+		if (i&2) shift_mask[i]|=2;
+		if (i&4) shift_mask[i]|=4;
+		if (i&8) shift_mask[i]|=8;
+		if (i&0x10) shift_mask[i]|=0x10;
+		if (i&0x20) shift_mask[i]|=0x20;
+		if (i&0x40) shift_mask[i]|=0x400;
+		if (i&0x80) shift_mask[i]|=0x800;
+		if (i&0x100) shift_mask[i]|=0x80;
+	}
+	for (i=0; i<4096; i++)
+	{
+		int j;
+		shift_xor[i]=1;
+		for (j=4096/2; j>0; j>>=1)
+		{
+			if (i & j)
+				shift_xor[i] ^= 1;
+		}
 	}
-    }
 }
 
 void lynx_audio_reset(void)
 {
-    int i;
-    for (i=0; i<ARRAY_LENGTH(lynx_audio); i++) {
-	lynx_audio_reset_channel(lynx_audio+i);
-    }
+	int i;
+	for (i=0; i<ARRAY_LENGTH(lynx_audio); i++)
+	{
+		lynx_audio_reset_channel(lynx_audio+i);
+	}
 }
 
+
+
 /************************************/
 /* Sound handler start              */
 /************************************/
-int lynx_custom_start (const struct MachineSound *driver)
+
+void *lynx_custom_start(int clock, const struct CustomSound_interface *config)
 {
-    if (!options.samplerate) return 0;
+	mixer_channel = stream_create(0, 1, options.samplerate, 0, lynx_update);
 
-    mixer_channel = stream_init("lynx", MIXER(50, MIXER_PAN_CENTER), 
-				options.samplerate, 0, lynx_update);
+	usec_per_sample = 1000000 / options.samplerate;
 
-    usec_per_sample=1000000/options.samplerate;
+	lynx_audio_init();
+	return (void *) ~0;
+}
+
+
+
+void *lynx2_custom_start(int clock, const struct CustomSound_interface *config)
+{
+    mixer_channel = stream_create(0, 2, options.samplerate, 0, lynx2_update);
+
+    usec_per_sample = 1000000 / options.samplerate;
     
     lynx_audio_init();
-    return 0;
+	return (void *) ~0;
 }
 
-int lynx2_custom_start (const struct MachineSound *driver)
-{
-    const int vol[2]={ MIXER(50, MIXER_PAN_LEFT), MIXER(50, MIXER_PAN_RIGHT) };
-    const char *names[2]= { "lynx", "lynx" };
-	
-    if (!options.samplerate) return 0;
-
-    mixer_channel = stream_init_multi(2, names, vol, options.samplerate, 0, lynx2_update);
-
-    usec_per_sample=1000000/options.samplerate;
-    
-    lynx_audio_init();
-    return 0;
-}
-
-/************************************/
-/* Sound handler stop               */
-/************************************/
-void lynx_custom_stop (void)
-{
-    free(shift_xor);
-    free(shift_mask);
-}
-
-void lynx_custom_update (void)
-{
-}
