@@ -9,73 +9,54 @@
 ***************************************************************************/
 
 #include "driver.h"
+#include "sound/sid6581.h"
 
 #define VERBOSE_DBG 0
 #include "includes/cbm.h"
 #include "includes/cia6526.h"
 #include "includes/vic4567.h"
-#include "includes/sid6581.h"
 #include "includes/cbmserb.h"
 #include "includes/vc1541.h"
 
 #include "includes/c65.h"
 
-static MEMORY_READ_START( c65_readmem )
-	{0x00000, 0x00001, c64_m6510_port_r},
-	{0x00002, 0x07fff, MRA_RAM},
-	{0x08000, 0x09fff, MRA_BANK1},
-	{0x0a000, 0x0bfff, MRA_BANK2},
-	{0x0c000, 0x0cfff, MRA_BANK3},
-	{0x0d000, 0x0d7ff, MRA_BANK4},
-	{0x0d800, 0x0dbff, MRA_BANK6},
-	{0x0dc00, 0x0dfff, MRA_BANK8},
-	{0x0e000, 0x0ffff, MRA_BANK10},
-	{0x10000, 0x1ffff, MRA_RAM},
-	{0x20000, 0x3ffff, MRA_ROM},
-	{0x40000, 0x7ffff, MRA_NOP},
-	{0x80000, 0xfffff, MRA_RAM},
-	/* 8 megabyte full address space! */
-MEMORY_END
+static ADDRESS_MAP_START( c65_mem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE(0x00000, 0x07fff) AM_RAMBANK(11)
+	AM_RANGE(0x08000, 0x09fff) AM_READWRITE(MRA8_BANK1, MWA8_BANK12)
+	AM_RANGE(0x0a000, 0x0bfff) AM_READWRITE(MRA8_BANK2, MWA8_BANK13)
+	AM_RANGE(0x0c000, 0x0cfff) AM_READWRITE(MRA8_BANK3, MWA8_BANK14)
+	AM_RANGE(0x0d000, 0x0d7ff) AM_READWRITE(MRA8_BANK4, MWA8_BANK5)
+	AM_RANGE(0x0d800, 0x0dbff) AM_READWRITE(MRA8_BANK6, MWA8_BANK7)
+	AM_RANGE(0x0dc00, 0x0dfff) AM_READWRITE(MRA8_BANK8, MWA8_BANK9)
+	AM_RANGE(0x0e000, 0x0ffff) AM_READWRITE(MRA8_BANK10, MWA8_BANK15)
+	AM_RANGE(0x10000, 0x1f7ff) AM_RAM
+	AM_RANGE(0x1f800, 0x1ffff) AM_RAM AM_BASE( &c64_colorram)
 
-static MEMORY_WRITE_START( c65_writemem )
-	{0x00000, 0x00001, c64_m6510_port_w, &c64_memory},
-	{0x00002, 0x07fff, MWA_RAM},
-	{0x08000, 0x09fff, MWA_RAM},
-	{0x0a000, 0x0cfff, MWA_RAM},
-	{0x0d000, 0x0d7ff, MWA_BANK5},
-	{0x0d800, 0x0dbff, MWA_BANK7},
-	{0x0dc00, 0x0dfff, MWA_BANK9},
-	{0x0e000, 0x0ffff, MWA_RAM},
-	{0x10000, 0x1f7ff, MWA_RAM},
-	{0x1f800, 0x1ffff, MWA_RAM, &c64_colorram},
-	{0x20000, 0x23fff, MWA_ROM}, /* &c65_dos},	   maps to 0x8000    */
-	{0x24000, 0x28fff, MWA_ROM}, /* reserved */
-	{0x29000, 0x29fff, MWA_ROM, &c65_chargen},
-	{0x2a000, 0x2bfff, MWA_ROM, &c64_basic},
-	{0x2c000, 0x2cfff, MWA_ROM, &c65_interface},
-	{0x2d000, 0x2dfff, MWA_ROM, &c64_chargen},
-	{0x2e000, 0x2ffff, MWA_ROM, &c64_kernal},
-	{0x30000, 0x31fff, MWA_ROM}, /*&c65_monitor},	  monitor maps to 0x6000    */
-	{0x32000, 0x37fff, MWA_ROM}, /*&c65_basic}, */
-	{0x38000, 0x3bfff, MWA_ROM}, /*&c65_graphics}, */
-	{0x3c000, 0x3dfff, MWA_ROM}, /* reserved */
-	{0x3e000, 0x3ffff, MWA_ROM}, /* &c65_kernal}, */
-	{0x40000, 0x7ffff, MWA_NOP},
-	{0x80000, 0xfffff, MWA_RAM},
-/*	{0x80000, 0xfffff, MWA_BANK16}, */
-MEMORY_END
+	AM_RANGE(0x20000, 0x23fff) AM_ROM /* &c65_dos,	   maps to 0x8000    */
+	AM_RANGE(0x24000, 0x28fff) AM_ROM /* reserved */
+	AM_RANGE(0x29000, 0x29fff) AM_ROM AM_BASE( &c65_chargen)
+	AM_RANGE(0x2a000, 0x2bfff) AM_ROM AM_BASE( &c64_basic)
+	AM_RANGE(0x2c000, 0x2cfff) AM_ROM AM_BASE( &c65_interface)
+	AM_RANGE(0x2d000, 0x2dfff) AM_ROM AM_BASE( &c64_chargen)
+	AM_RANGE(0x2e000, 0x2ffff) AM_ROM AM_BASE( &c64_kernal)
+
+	AM_RANGE(0x30000, 0x31fff) AM_ROM /*&c65_monitor,	  monitor maps to 0x6000    */
+	AM_RANGE(0x32000, 0x37fff) AM_ROM /*&c65_basic, */
+	AM_RANGE(0x38000, 0x3bfff) AM_ROM /*&c65_graphics, */
+	AM_RANGE(0x3c000, 0x3dfff) AM_ROM /* reserved */
+	AM_RANGE(0x3e000, 0x3ffff) AM_ROM /* &c65_kernal, */
+
+	AM_RANGE(0x40000, 0x7ffff) AM_NOP
+	/* 8 megabyte full address space! */
+ADDRESS_MAP_END
 
 #define DIPS_HELPER(bit, name, keycode) \
-   PORT_BITX(bit, IP_ACTIVE_HIGH, IPT_KEYBOARD, name, keycode, IP_JOY_NONE)
+   PORT_BIT(bit, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME(name) PORT_CODE(keycode)
 
 #define C65_DIPS \
      PORT_START \
 	 DIPS_HELPER( 0x8000, "Quickload", KEYCODE_SLASH_PAD)\
 	 PORT_BIT (0x7c00, 0x0, IPT_UNUSED) /* no tape */\
-     PORT_DIPNAME (0x300, 0x0, "Memory Expansion")\
-	 PORT_DIPSETTING (0, "None")\
-	 PORT_DIPSETTING (0x100, "512 KByte")\
-	 PORT_DIPSETTING (0x200, "4 MByte")\
 	PORT_DIPNAME   ( 0x80, 0x80, "Sid Chip Type")\
 	PORT_DIPSETTING(  0, "MOS6581" )\
 	PORT_DIPSETTING(0x80, "MOS8580" )\
@@ -86,10 +67,10 @@ MEMORY_END
 	 /*PORT_DIPSETTING (0x10, "CBM Supergames")*/\
 	 /*PORT_DIPSETTING (0x14, "Ocean Robocop2")*/\
 	 PORT_DIPNAME (0x02, 0x02, "Serial Bus/Device 10")\
-	 PORT_DIPSETTING (0, "None")\
+	 PORT_DIPSETTING (0, DEF_STR( None ))\
 	 PORT_DIPSETTING (2, "Floppy Drive Simulation")\
 	 PORT_DIPNAME (0x01, 0x01, "Serial Bus/Device 11")\
-	 PORT_DIPSETTING (0, "None")\
+	 PORT_DIPSETTING (0, DEF_STR( None ))\
 	 PORT_DIPSETTING (1, "Floppy Drive Simulation")
 
 
@@ -131,9 +112,7 @@ INPUT_PORTS_START (c65)
 	DIPS_HELPER (0x0002, "RESTORE", KEYCODE_PRTSCR)
 	DIPS_HELPER (0x0001, "CTRL", KEYCODE_RCONTROL)
 	PORT_START
-	PORT_BITX (0x8000, 0, IPT_DIPSWITCH_NAME|IPF_TOGGLE,
-			   "(Left-Shift) SHIFT-LOCK (switch)",
-			   KEYCODE_CAPSLOCK, IP_JOY_NONE)
+	PORT_DIPNAME (0x8000, 0, "(Left-Shift) SHIFT-LOCK (switch)") PORT_CODE(KEYCODE_CAPSLOCK)
 	PORT_DIPSETTING(  0, DEF_STR(Off) )
 	PORT_DIPSETTING(0x8000, DEF_STR(On) )
 	DIPS_HELPER (0x4000, "A", KEYCODE_A)
@@ -174,9 +153,7 @@ INPUT_PORTS_START (c65)
 	DIPS_HELPER (0x8000, "STOP RUN", KEYCODE_ESC)
 	DIPS_HELPER (0x4000, "(C65)ESC", KEYCODE_F1)
 	DIPS_HELPER (0x2000, "(C65)ALT", KEYCODE_F2)
-	PORT_BITX (0x1000, 0, IPT_DIPSWITCH_NAME|IPF_TOGGLE,
-			   "(C65)CAPSLOCK(switch)",
-			   KEYCODE_F3, IP_JOY_NONE)
+	PORT_DIPNAME (0x1000, 0, "(C65)CAPSLOCK(switch)") PORT_CODE(KEYCODE_F3)
 	PORT_DIPSETTING(  0, DEF_STR(Off) )
 	PORT_DIPSETTING(0x1000, DEF_STR(On) )
 	DIPS_HELPER (0x0800, "(C65)NO SCRL", KEYCODE_F4)
@@ -228,9 +205,7 @@ INPUT_PORTS_START (c65ger)
 	 DIPS_HELPER (0x0002, "RESTORE", KEYCODE_PRTSCR)
 	 DIPS_HELPER (0x0001, "CTRL", KEYCODE_RCONTROL)
 	 PORT_START
-     PORT_BITX (0x8000, 0, IPT_DIPSWITCH_NAME|IPF_TOGGLE,
-				"(Left-Shift)SHIFT-LOCK (switch)",
-				KEYCODE_CAPSLOCK, IP_JOY_NONE)
+     PORT_DIPNAME (0x8000, 0, "(Left-Shift)SHIFT-LOCK (switch)") PORT_CODE(KEYCODE_CAPSLOCK)
 	PORT_DIPSETTING(  0, DEF_STR(Off) )\
 	PORT_DIPSETTING(0x8000, DEF_STR(On) )\
 	 DIPS_HELPER (0x4000, "A", KEYCODE_A)
@@ -274,9 +249,7 @@ INPUT_PORTS_START (c65ger)
      DIPS_HELPER (0x8000, "STOP RUN", KEYCODE_ESC)
 	 DIPS_HELPER (0x4000, "(C65)ESC", KEYCODE_F1)
 	 DIPS_HELPER (0x2000, "(C65)ALT", KEYCODE_F2)
-	 PORT_BITX (0x1000, 0, IPT_DIPSWITCH_NAME|IPF_TOGGLE,
-				"(C65)DIN ASC(switch)",
-				KEYCODE_F3, IP_JOY_NONE)
+	 PORT_DIPNAME (0x1000, 0, "(C65)DIN ASC(switch)") PORT_CODE(KEYCODE_F3)
 	PORT_DIPSETTING(  0, "ASC" )
 	PORT_DIPSETTING(0x1000, "DIN" )
 	DIPS_HELPER (0x0800, "(C65)NO SCRL", KEYCODE_F4)
@@ -290,235 +263,130 @@ INPUT_PORTS_START (c65ger)
 	DIPS_HELPER (0x0008, "(C65)HELP", KEYCODE_F12)
 INPUT_PORTS_END
 
-static void c65_init_palette (unsigned char *sys_palette, unsigned short *sys_colortable, const unsigned char *color_prom)
+static PALETTE_INIT( c65 )
 {
-	memcpy (sys_palette, vic3_palette, sizeof (vic3_palette));
+	palette_set_colors(0, vic3_palette, sizeof(vic3_palette) / 3);
 }
 
 #if 0
 	/* caff */
 	/* dma routine alpha 1 (0x400000 reversed copy)*/
-	ROM_LOAD ("910111.bin", 0x20000, 0x20000, 0xc5d8d32e)
+	ROM_LOAD ("910111.bin", 0x20000, 0x20000, CRC(c5d8d32e) SHA1(71c05f098eff29d306b0170e2c1cdeadb1a5f206))
 	/* b96b */
 	/* dma routine alpha 2 */
-	ROM_LOAD ("910523.bin", 0x20000, 0x20000, 0xe8235dd4)
+	ROM_LOAD ("910523.bin", 0x20000, 0x20000, CRC(e8235dd4) SHA1(e453a8e7e5b95de65a70952e9d48012191e1b3e7))
 	/* 888c */
 	/* dma routine alpha 2 */
-	ROM_LOAD ("910626.bin", 0x20000, 0x20000, 0x12527742)
+	ROM_LOAD ("910626.bin", 0x20000, 0x20000, CRC(12527742) SHA1(07c185b3bc58410183422f7ac13a37ddd330881b))
 	/* c9cd */
 	/* dma routine alpha 2 */
-	ROM_LOAD ("910828.bin", 0x20000, 0x20000, 0x3ee40b06)
+	ROM_LOAD ("910828.bin", 0x20000, 0x20000, CRC(3ee40b06) SHA1(b63d970727a2b8da72a0a8e234f3c30a20cbcb26))
 	/* 4bcf loading demo disk??? */
 	/* basic program stored at 0x4000 ? */
 	/* dma routine alpha 2 */
-	ROM_LOAD ("911001.bin", 0x20000, 0x20000, 0x0888b50f)
+	ROM_LOAD ("911001.bin", 0x20000, 0x20000, CRC(0888b50f) SHA1(129b9a2611edaebaa028ac3e3f444927c8b1fc5d))
 	/* german e96a */
 	/* dma routine alpha 1 */
-	ROM_LOAD ("910429.bin", 0x20000, 0x20000, 0xb025805c)
+	ROM_LOAD ("910429.bin", 0x20000, 0x20000, CRC(b025805c) SHA1(c3b05665684f74adbe33052a2d10170a1063ee7d))
 #endif
 
 ROM_START (c65)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("911001.bin", 0x20000, 0x20000, 0x0888b50f)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("911001.bin", 0x20000, 0x20000, CRC(0888b50f) SHA1(129b9a2611edaebaa028ac3e3f444927c8b1fc5d))
 ROM_END
 
 ROM_START (c65e)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("910828.bin", 0x20000, 0x20000, 0x3ee40b06)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("910828.bin", 0x20000, 0x20000, CRC(3ee40b06) SHA1(b63d970727a2b8da72a0a8e234f3c30a20cbcb26))
 ROM_END
 
 ROM_START (c65d)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("910626.bin", 0x20000, 0x20000, 0x12527742)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("910626.bin", 0x20000, 0x20000, CRC(12527742) SHA1(07c185b3bc58410183422f7ac13a37ddd330881b))
 ROM_END
 
 ROM_START (c65c)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("910523.bin", 0x20000, 0x20000, 0xe8235dd4)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("910523.bin", 0x20000, 0x20000, CRC(e8235dd4) SHA1(e453a8e7e5b95de65a70952e9d48012191e1b3e7))
 ROM_END
 
 ROM_START (c65ger)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("910429.bin", 0x20000, 0x20000, 0xb025805c)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("910429.bin", 0x20000, 0x20000, CRC(b025805c) SHA1(c3b05665684f74adbe33052a2d10170a1063ee7d))
 ROM_END
 
 ROM_START (c65a)
-	ROM_REGION (0x800000, REGION_CPU1, 0)
-/*	ROM_REGION (0x100000, REGION_CPU1, 0) */
-	ROM_LOAD ("910111.bin", 0x20000, 0x20000, 0xc5d8d32e)
+	ROM_REGION (0x400000, REGION_CPU1, 0)
+	ROM_LOAD ("910111.bin", 0x20000, 0x20000, CRC(c5d8d32e) SHA1(71c05f098eff29d306b0170e2c1cdeadb1a5f206))
 ROM_END
 
 
-static SID6581_interface ntsc_sound_interface =
+static SID6581_interface c65_sound_interface =
 {
-	{
-		sid6581_custom_start,
-		sid6581_custom_stop,
-		sid6581_custom_update
-	},
-	2,
-	{
-		{
-			MIXER(50, MIXER_PAN_RIGHT),
-			MOS8580,
-			985248,
-			c64_paddle_read
-		},
-		{
-			MIXER(50, MIXER_PAN_LEFT),
-			MOS8580,
-			985248,
-			NULL
-		}
-	}
+	c64_paddle_read
 };
 
-static SID6581_interface pal_sound_interface =
-{
-	{
-		sid6581_custom_start,
-		sid6581_custom_stop,
-		sid6581_custom_update
-	},
-	2,
-	{
-		{
-			MIXER(50, MIXER_PAN_RIGHT),
-			MOS8580,
-			1022727,
-			c64_paddle_read
-		},
-		{
-			MIXER(50, MIXER_PAN_LEFT),
-			MOS8580,
-			1022727,
-			NULL
-		}
-	}
-};
+static MACHINE_DRIVER_START( c65 )
+	/* basic machine hardware */
+	MDRV_CPU_ADD_TAG("main", M4510, 3500000)  /* or VIC6567_CLOCK, */
+	MDRV_CPU_PROGRAM_MAP(c65_mem, 0)
+	MDRV_CPU_VBLANK_INT(c64_frame_interrupt, 1)
+	MDRV_CPU_PERIODIC_INT(vic3_raster_irq, TIME_IN_HZ(VIC2_HRETRACERATE))
+	MDRV_FRAMES_PER_SECOND(VIC6567_VRETRACERATE)
+	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
+	MDRV_INTERLEAVE(0)
 
-static struct MachineDriver machine_driver_c65 =
-{
-  /* basic machine hardware */
-	{
-		{
-			CPU_M4510,
-			3500000, /* or VIC6567_CLOCK, */
-			c65_readmem, c65_writemem,
-			0, 0,
-			c64_frame_interrupt, 1,
-			vic3_raster_irq, VIC2_HRETRACERATE,
-		},
-	},
-	VIC6567_VRETRACERATE, DEFAULT_REAL_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
-	0,
-	c65_init_machine,
-	c65_shutdown_machine,
+	MDRV_MACHINE_INIT( c65 )
 
-  /* video hardware */
-	656,							   /* screen width */
-	416,							   /* screen height */
-	{0, 656 - 1, 0, 416 - 1},		   /* visible_area */
-	0,								   /* graphics decode info */
-	sizeof (vic3_palette) / sizeof (vic3_palette[0]) / 3,
-	0,
-	c65_init_palette,				   /* convert color prom */
-	VIDEO_TYPE_RASTER,
-	0,
-	vic2_vh_start,
-	vic2_vh_stop,
-	vic2_vh_screenrefresh,
+	/* video hardware */
+	MDRV_IMPORT_FROM( vh_vic2 )
+	MDRV_SCREEN_SIZE(656, 416)
+	MDRV_VISIBLE_AREA(0, 656 - 1, 0, 416 - 1)
+	MDRV_PALETTE_INIT( c65 )
 
-  /* sound hardware */
-	SOUND_SUPPORTS_STEREO, 0, 0, 0,
-	{
-		{ SOUND_CUSTOM, &ntsc_sound_interface },
-		{ 0 }
-	}
-};
+	/* sound hardware */
+	MDRV_SPEAKER_STANDARD_STEREO("left", "right")
+	MDRV_SOUND_ADD_TAG("sid_r", SID8580, 985248)
+	MDRV_SOUND_CONFIG(c65_sound_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "right", 0.50)
+	MDRV_SOUND_ADD_TAG("sid_l", SID8580, 985248)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "left", 0.50)
+MACHINE_DRIVER_END
 
-static struct MachineDriver machine_driver_c65pal =
-{
-  /* basic machine hardware */
-	{
-		{
-			CPU_M4510,
-			3500000, /* or VIC6569_CLOCK,*/
-			c65_readmem, c65_writemem,
-			0, 0,
-			c64_frame_interrupt, 1,
-			vic3_raster_irq, VIC2_HRETRACERATE,
-		},
-	},
-	VIC6569_VRETRACERATE,
-	DEFAULT_REAL_60HZ_VBLANK_DURATION, /* frames per second, vblank duration */
-	0,
-	c65_init_machine,
-	c65_shutdown_machine,
 
-  /* video hardware */
-	656,							   /* screen width */
-	416,							   /* screen height */
-	{0, 656 - 1, 0, 416 - 1},		   /* visible_area */
-	0,								   /* graphics decode info */
-	sizeof (vic3_palette) / sizeof (vic3_palette[0]) / 3,
-	0,
-	c65_init_palette,				   /* convert color prom */
-	VIDEO_TYPE_RASTER,
-	0,
-	vic2_vh_start,
-	vic2_vh_stop,
-	vic2_vh_screenrefresh,
+static MACHINE_DRIVER_START( c65pal )
+	MDRV_IMPORT_FROM( c65 )
+	MDRV_FRAMES_PER_SECOND(VIC6569_VRETRACERATE)
 
-  /* sound hardware */
-	SOUND_SUPPORTS_STEREO, 0, 0, 0,
-	{
-		{ SOUND_CUSTOM, &pal_sound_interface },
-		{ 0 }
-	}
-};
-
-static const struct IODevice io_c65[] =
-{
-	IODEVICE_CBM_C65_QUICK,
-	IODEVICE_CBM_DRIVE,
-	{IO_END}
-};
+	MDRV_SOUND_REPLACE("sid_r", SID8580, 1022727)
+	MDRV_SOUND_CONFIG(c65_sound_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "right", 0.50)
+	MDRV_SOUND_ADD_TAG("sid_l", SID8580, 1022727)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "left", 0.50)
+MACHINE_DRIVER_END
 
 #define init_c65 c65_driver_init
 #define init_c65_alpha1 c65_driver_init
 #define init_c65pal c65pal_driver_init
 
-#define io_c65ger io_c65
-#define io_c65e io_c65
-#define io_c65d io_c65
-#define io_c65c io_c65
-#define io_c65a io_c65
-
-/*		YEAR	NAME	PARENT	MACHINE INPUT	INIT		COMPANY 							FULLNAME */
-COMPX ( 199?,	c65,	0,		c65,	c65,	c65,		"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 911001)",        GAME_NOT_WORKING)
-COMPX ( 199?,	c65e,	c65,	c65,	c65,	c65,		"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910828)",        GAME_NOT_WORKING)
-COMPX ( 199?,	c65d,	c65,	c65,	c65,	c65,		"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910626)",        GAME_NOT_WORKING)
-COMPX ( 199?,	c65c,	c65,	c65,	c65,	c65,		"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910523)",        GAME_NOT_WORKING)
-COMPX ( 199?,	c65ger, c65,	c65pal, c65ger, c65pal, 	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, German PAL, 910429)",  GAME_NOT_WORKING)
-COMPX ( 199?,	c65a,	c65,	c65,	c65,	c65_alpha1, "Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910111)",        GAME_NOT_WORKING)
-
-#ifdef RUNTIME_LOADER
-extern void c65_runtime_loader_init(void)
+static void c65_quickload_getinfo(struct IODevice *dev)
 {
-	int i;
-	for (i=0; drivers[i]; i++) {
-		if ( strcmp(drivers[i]->name,"c65")==0) drivers[i]=&driver_c65;
-		if ( strcmp(drivers[i]->name,"c65e")==0) drivers[i]=&driver_c65e;
-		if ( strcmp(drivers[i]->name,"c65d")==0) drivers[i]=&driver_c65d;
-		if ( strcmp(drivers[i]->name,"c65c")==0) drivers[i]=&driver_c65c;
-		if ( strcmp(drivers[i]->name,"c65ger")==0) drivers[i]=&driver_c65ger;
-		if ( strcmp(drivers[i]->name,"c65a")==0) drivers[i]=&driver_c65a;
-	}
+	quickload_device_getinfo(dev, quickload_load_cbm_c65, CBM_QUICKLOAD_DELAY);
+	dev->file_extensions = "p00\0prg\0";
 }
-#endif
+
+SYSTEM_CONFIG_START(c65)
+	CONFIG_DEVICE(cbmfloppy_device_getinfo)
+	CONFIG_DEVICE(c65_quickload_getinfo)
+	CONFIG_RAM_DEFAULT(128 * 1024)
+	CONFIG_RAM((128 + 512) * 1024)
+	CONFIG_RAM((128 + 4096) * 1024)
+SYSTEM_CONFIG_END
+
+/*		YEAR	NAME	PARENT	COMPAT	MACHINE INPUT	INIT		CONFIG  COMPANY 							FULLNAME */
+COMP ( 1991,	c65,	0,		0,		c65,	c65,	c65,		c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 911001)",        GAME_NOT_WORKING)
+COMP ( 1991,	c65e,	c65,	0,		c65,	c65,	c65,		c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910828)",        GAME_NOT_WORKING)
+COMP ( 1991,	c65d,	c65,	0,		c65,	c65,	c65,		c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910626)",        GAME_NOT_WORKING)
+COMP ( 1991,	c65c,	c65,	0,		c65,	c65,	c65,		c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910523)",        GAME_NOT_WORKING)
+COMP ( 1991,	c65ger, c65,	0,		c65pal, c65ger, c65pal, 	c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, German PAL, 910429)",  GAME_NOT_WORKING)
+COMP ( 1991,	c65a,	c65,	0,		c65,	c65,	c65_alpha1, c65,	"Commodore Business Machines Co.",  "C65 / C64DX (Prototype, NTSC, 910111)",        GAME_NOT_WORKING)
