@@ -419,6 +419,7 @@ ROM_START (cgenie)
 	ROM_REGION(0x13000,REGION_CPU1,0)
 	ROM_LOAD ("cgenie.rom",  0x00000, 0x4000, CRC(d359ead7) SHA1(d8c2fc389ad38c45fba0ed556a7d91abac5463f4))
 	ROM_LOAD ("cgdos.rom",   0x10000, 0x2000, CRC(2a96cf74) SHA1(6dcac110f87897e1ee7521aefbb3d77a14815893))
+	ROM_CART_LOAD(0, "rom\0", 0x12000, 0x1000, ROM_NOMIRROR | ROM_OPTIONAL)
 
 	ROM_REGION(0x0c00,REGION_GFX1,0)
 	ROM_LOAD ("cgenie1.fnt", 0x0000, 0x0800, CRC(4fed774a) SHA1(d53df8212b521892cc56be690db0bb474627d2ff))
@@ -428,15 +429,6 @@ ROM_START (cgenie)
 
 ROM_END
 
-static void cgenie_cartslot_getinfo(struct IODevice *dev)
-{
-	/* cartslot */
-	cartslot_device_getinfo(dev);
-	dev->count = 1;
-	dev->file_extensions = "rom\0";
-	dev->load = device_load_cgenie_cart;
-}
-
 static void cgenie_floppy_getinfo(struct IODevice *dev)
 {
 	/* floppy */
@@ -459,7 +451,7 @@ static void cgenie_cassette_getinfo(struct IODevice *dev)
 }
 
 SYSTEM_CONFIG_START(cgenie)
-	CONFIG_DEVICE(cgenie_cartslot_getinfo)
+	CONFIG_DEVICE(cartslot_device_getinfo)
 	CONFIG_DEVICE(cgenie_floppy_getinfo)
 	CONFIG_DEVICE(cgenie_cassette_getinfo)
 SYSTEM_CONFIG_END
