@@ -28,7 +28,7 @@
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
-
+#include "devices/cartslot.h"
 #include "includes/mekd2.h"
 
 #ifndef VERBOSE
@@ -41,17 +41,17 @@
 #define LOG(x)	/* x */
 #endif
 
-static READ_HANDLER(mekd2_pia_r) { return 0xff; }
-static READ_HANDLER(mekd2_cas_r) { return 0xff; }
-static READ_HANDLER(mekd2_kbd_r) { return 0xff; }
-static READ_HANDLER(mekd2_mirror_r) { UINT8 *mem = memory_region(REGION_CPU1); return mem[0xe000+(offset&0x3ff)]; }
+static  READ8_HANDLER(mekd2_pia_r) { return 0xff; }
+static  READ8_HANDLER(mekd2_cas_r) { return 0xff; }
+static  READ8_HANDLER(mekd2_kbd_r) { return 0xff; }
+static  READ8_HANDLER(mekd2_mirror_r) { UINT8 *mem = memory_region(REGION_CPU1); return mem[0xe000+(offset&0x3ff)]; }
 
 UINT8 pia[8];
 
-static WRITE_HANDLER(mekd2_pia_w) { }
-static WRITE_HANDLER(mekd2_cas_w) { }
+static WRITE8_HANDLER(mekd2_pia_w) { }
+static WRITE8_HANDLER(mekd2_cas_w) { }
 
-static WRITE_HANDLER(mekd2_kbd_w)
+static WRITE8_HANDLER(mekd2_kbd_w)
 {
 	pia[offset] = data;
 	switch( offset )
@@ -92,36 +92,36 @@ static WRITE_HANDLER(mekd2_kbd_w)
 }
 
 
-static MEMORY_READ_START( readmem )
-	{ 0x0000, 0x00ff, MRA_RAM },
-/*	{ 0x0100, 0x01ff, MRA_RAM },	   optional, set up in mekd2_init_machine    */
-/*	{ 0x6000, 0x67ff, MRA_ROM },	   -"-    */
+static ADDRESS_MAP_START( readmem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x00ff) AM_READ( MRA8_RAM )
+/*	{ 0x0100, 0x01ff, MRA8_RAM },	   optional, set up in mekd2_init_machine    */
+/*	{ 0x6000, 0x67ff, MRA8_ROM },	   -"-    */
 /*	  { 0x8004, 0x8007, mekd2_pia_r }, */
 /*	  { 0x8008, 0x8008, mekd2_cas_r }, */
 /*	  { 0x8020, 0x8023, mekd2_kbd_r }, */
-	{ 0xa000, 0xa07f, MRA_RAM },
-/*	{ 0xc000, 0xc7ff, MRA_RAM },	   optional, set up in mekd2_init_machine    */
-	{ 0xe000, 0xe3ff, MRA_ROM },	/* JBUG ROM */
-	{ 0xe400, 0xffff, mekd2_mirror_r },
-MEMORY_END
+	AM_RANGE( 0xa000, 0xa07f) AM_READ( MRA8_RAM )
+/*	{ 0xc000, 0xc7ff, MRA8_RAM },	   optional, set up in mekd2_init_machine    */
+	AM_RANGE( 0xe000, 0xe3ff) AM_READ( MRA8_ROM )	/* JBUG ROM */
+	AM_RANGE( 0xe400, 0xffff) AM_READ( mekd2_mirror_r )
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( writemem )
-	{ 0x0000, 0x00ff, MWA_RAM },
-/*	{ 0x0100, 0x01ff, MWA_RAM },	   optional, set up in mekd2_init_machine    */
-/*	{ 0x6000, 0x67ff, MWA_ROM },	   -"-    */
+static ADDRESS_MAP_START( writemem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x00ff) AM_WRITE( MWA8_RAM )
+/*	{ 0x0100, 0x01ff, MWA8_RAM },	   optional, set up in mekd2_init_machine    */
+/*	{ 0x6000, 0x67ff, MWA8_ROM },	   -"-    */
 /*	  { 0x8004, 0x8007, mekd2_pia_w }, */
 /*	  { 0x8008, 0x8008, mekd2_cas_w }, */
-	{ 0x8020, 0x8023, mekd2_kbd_w },
-	{ 0xa000, 0xa07f, MWA_RAM },
-/*	{ 0xc000, 0xc7ff, MWA_RAM },	   optional, set up in mekd2_init_machine    */
-	{ 0xe000, 0xe3ff, MWA_ROM },	/* JBUG ROM */
-MEMORY_END
+	AM_RANGE( 0x8020, 0x8023) AM_WRITE( mekd2_kbd_w )
+	AM_RANGE( 0xa000, 0xa07f) AM_WRITE( MWA8_RAM )
+/*	{ 0xc000, 0xc7ff, MWA8_RAM },	   optional, set up in mekd2_init_machine    */
+	AM_RANGE( 0xe000, 0xe3ff) AM_WRITE( MWA8_ROM )	/* JBUG ROM */
+ADDRESS_MAP_END
 
 INPUT_PORTS_START( mekd2 )
 	PORT_START			/* IN0 keys row 0 */
 INPUT_PORTS_END
 
-static struct GfxLayout led_layout =
+static gfx_layout led_layout =
 {
 	18, 24, 	/* 16 x 24 LED 7segment displays */
 	128,		/* 128 codes */
@@ -141,7 +141,7 @@ static struct GfxLayout led_layout =
 	24 * 24,	/* every LED code takes 32 times 18 (aligned 24) bit words */
 };
 
-static struct GfxLayout key_layout =
+static gfx_layout key_layout =
 {
 	24, 18, 	/* 24 * 18 keyboard icons */
 	24, 		/* 24  codes */
@@ -156,102 +156,70 @@ static struct GfxLayout key_layout =
 	18 * 24 * 2,	/* every icon takes 18 rows of 24 * 2 bits */
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static gfx_decode gfxdecodeinfo[] =
 {
 	{ 1, 0, &led_layout, 0, 16 },
 	{ 2, 0, &key_layout, 16*2, 2 },
 	{ -1 } /* end of array */
 };
 
-static struct DACinterface dac_interface =
-{
-	1,			/* number of DACs */
-	{ 100 } 	/* volume */
-};
 
-static struct MachineDriver machine_driver_mekd2 =
-{
+
+static MACHINE_DRIVER_START( mekd2 )
 	/* basic machine hardware */
-	{
-		{
-			CPU_M6800,
-			614400, /* 614.4 kHz */
-			readmem,writemem,0,0,
-			mekd2_interrupt, 1
-		}
-	},
-	/* frames per second, VBL duration */
-	60, DEFAULT_60HZ_VBLANK_DURATION,
-	1,				/* single CPU */
-	mekd2_init_machine,
-	NULL,			/* stop machine */
+	MDRV_CPU_ADD(M6800, 614400)        /* 614.4 kHz */
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)
+	MDRV_FRAMES_PER_SECOND(60)
+	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
+	MDRV_INTERLEAVE(1)
 
-	/* video hardware (well, actually there was no video ;) */
-	600, 768, { 0, 600 - 1, 0, 768 - 1},
-	gfxdecodeinfo,
-	21 + 32768,
-	256,
-	mekd2_init_colors,		 /* convert color prom */
+    /* video hardware */
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(600, 768)
+	MDRV_VISIBLE_AREA(0, 600-1, 0, 768-1)
+	MDRV_GFXDECODE( gfxdecodeinfo )
+	MDRV_PALETTE_LENGTH(21 + 32768)
+	MDRV_COLORTABLE_LENGTH(256)
+	MDRV_PALETTE_INIT( mekd2 )
 
-	VIDEO_TYPE_RASTER | VIDEO_SUPPORTS_DIRTY,	/* video flags */
-	0,						/* obsolete */
-	mekd2_vh_start,
-	mekd2_vh_stop,
-	mekd2_vh_screenrefresh,
+	MDRV_VIDEO_START( mekd2 )
+	MDRV_VIDEO_UPDATE( mekd2 )
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_DAC,
-			&dac_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(DAC, 0)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.00)
+MACHINE_DRIVER_END
+
 
 ROM_START(mekd2)
 	ROM_REGION(0x10000,REGION_CPU1,0)
-		ROM_LOAD("jbug.rom",    0xe000, 0x0400, 0xa2a56502)
+		ROM_LOAD("jbug.rom",    0xe000, 0x0400, CRC(a2a56502) SHA1(60b6e48f35fe4899e29166641bac3e81e3b9d220))
 	ROM_REGION(128 * 24 * 3,REGION_GFX1,0)
 		/* space filled with 7segement graphics by mekd2_init_driver */
 	ROM_REGION( 24 * 18 * 3 * 2,REGION_GFX2,0)
 		/* space filled with key icons by mekd2_init_driver */
 ROM_END
 
-
-
-static const struct IODevice io_mekd2[] = {
-	{
-		IO_CARTSLOT,		/* type */
-		1,					/* count */
-		"d2\0",             /* file extensions */
-		IO_RESET_ALL,		/* reset if file changed */
-		0,
-		mekd2_rom_load, 	/* init */
-		NULL,				/* exit */
-		NULL,				/* info */
-		NULL,				/* open */
-		NULL,				/* close */
-		NULL,				/* status */
-		NULL,				/* seek */
-		NULL,				/* tell */
-		NULL,				/* input */
-		NULL,				/* output */
-		NULL,				/* input_chunk */
-		NULL				/* output_chunk */
-	},
-	{ IO_END }
-};
-
-/*	  YEAR	NAME	  PARENT	MACHINE   INPUT 	INIT	  COMPANY	  FULLNAME */
-CONS( 1977, mekd2,	   0,		mekd2,	  mekd2,	mekd2,	  "Motorola", "MEK6800D2" )
-
-#ifdef RUNTIME_LOADER
-extern void mekd2_runtime_loader_init(void)
+static void mekd2_cartslot_getinfo(struct IODevice *dev)
 {
-	int i;
-	for (i=0; drivers[i]; i++) {
-		if ( strcmp(drivers[i]->name,"mekd2")==0) drivers[i]=&driver_mekd2;
-	}
+	/* cartslot */
+	cartslot_device_getinfo(dev);
+	dev->count = 1;
+	dev->file_extensions = "d2\0";
+	dev->load = device_load_mekd2_cart;
 }
-#endif
+
+SYSTEM_CONFIG_START(mekd2)
+	CONFIG_DEVICE(mekd2_cartslot_getinfo)
+SYSTEM_CONFIG_END
+
+/***************************************************************************
+
+  Game driver(s)
+
+***************************************************************************/
+
+/*	  YEAR	NAME	PARENT	COMPAT	MACHINE   INPUT 	INIT	CONFIG	COMPANY		FULLNAME */
+CONS( 1977, mekd2,	0,		0,		mekd2,	  mekd2,	mekd2,	mekd2,	"Motorola",	"MEK6800D2" , 0)
