@@ -80,113 +80,78 @@
 /* special keys
    red c-ce and reset; warm boot, program NOT lost*/
 
-static MEMORY_READ_START( pc1401_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-	{ 0x2000, 0x47ff, MRA_RAM },
+static ADDRESS_MAP_START( pc1401_mem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_ROM
+	AM_RANGE( 0x2000, 0x47ff) AM_READ( MRA8_RAM )
+/*	{ 0x2000, 0x3fff, MWA8_RAM }, // done in pc1401_machine_init */
 /*	{ 0x5000, 0x57ff, ? }, */
-	{ 0x6000, 0x67ff, pc1401_lcd_read },
-	{ 0x7000, 0x77ff, pc1401_lcd_read },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
+	AM_RANGE( 0x6000, 0x67ff) AM_READWRITE( pc1401_lcd_read, pc1401_lcd_write ) AM_MIRROR(0x1000)
+	AM_RANGE( 0x8000, 0xffff) AM_ROM
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( pc1401_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-/*	{ 0x2000, 0x3fff, MWA_RAM }, // done in pc1401_machine_init */
-	{ 0x4000, 0x47ff, MWA_RAM },
-/*	{ 0x5000, 0x57ff, ? }, */
-	{ 0x6000, 0x67ff, pc1401_lcd_write },
-	{ 0x7000, 0x77ff, pc1401_lcd_write },
-	{ 0x8000, 0xffff, MWA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( pc1251_mem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_ROM
+/*	{ 0x2000, 0x3fff, MRA8_RAM }, */
+	AM_RANGE( 0x4000, 0x7fff) AM_ROM
+	AM_RANGE( 0xa000, 0xcbff) AM_ROM
+	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+ADDRESS_MAP_END
 
-static MEMORY_READ_START( pc1251_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-/*	{ 0x2000, 0x3fff, MRA_RAM }, */
-	{ 0x4000, 0x7fff, MRA_ROM },
-	{ 0xa000, 0xcbff, MRA_ROM },
-	{ 0xf800, 0xf8ff, pc1251_lcd_read },
-MEMORY_END
-
-static MEMORY_WRITE_START( pc1251_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-	{ 0x4000, 0x7fff, MWA_ROM },
-/*	{ 0xa000, 0xcbff, MWA_ROM }, // c600 b800 b000 a000 tested */
-	{ 0xf800, 0xf8ff, pc1251_lcd_write },
-MEMORY_END
-
-static MEMORY_READ_START( pc1350_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-	{ 0x2000, 0x3fff, MRA_RAM },
-	{ 0x4000, 0x5fff, MRA_RAM },
-	{ 0x6000, 0x6fff, MRA_RAM },
-	{ 0x7000, 0x7eff, pc1350_lcd_read },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
-
-static MEMORY_WRITE_START( pc1350_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-	{ 0x2000, 0x3fff, MWA_RAM }, /*ram card 16k */
-	{ 0x4000, 0x5fff, MWA_RAM }, /*ram card 16k oder 8k */
-	{ 0x6000, 0x6fff, MWA_RAM },
-	{ 0x7000, 0x7eff, pc1350_lcd_write },
-	{ 0x8000, 0xffff, MWA_ROM },
-MEMORY_END
-
-static MEMORY_READ_START( pc1403_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-{ 0x3000, 0x30bf, pc1403_lcd_read },    
-{ 0x3800, 0x3fff, pc1403_asic_read },    
-{ 0x4000, 0x7fff, MRA_BANK1 },
-{ 0xe000,0xffff, MRA_RAM },
-MEMORY_END
-
-static MEMORY_WRITE_START( pc1403_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-{ 0x3000, 0x30bf, pc1403_lcd_write },    
-{ 0x3800, 0x3fff, pc1403_asic_write },    
-{ 0xe000,0xffff, MWA_RAM },
-MEMORY_END
+static ADDRESS_MAP_START( pc1350_mem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_ROM
+	AM_RANGE( 0x2000, 0x3fff) AM_RAM	/* ram card 16k */
+	AM_RANGE( 0x4000, 0x5fff) AM_RAM	/* ram card 16k oder 8k */
+	AM_RANGE( 0x6000, 0x6fff) AM_RAM
+	AM_RANGE( 0x7000, 0x7eff) AM_READWRITE( pc1350_lcd_read, pc1350_lcd_write )
+	AM_RANGE( 0x8000, 0xffff) AM_ROM
+ADDRESS_MAP_END
 
+static ADDRESS_MAP_START( pc1403_mem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_ROM
+	AM_RANGE( 0x3000, 0x30bf) AM_READWRITE( pc1403_lcd_read, pc1403_lcd_write )  
+	AM_RANGE( 0x3800, 0x3fff) AM_READWRITE( pc1403_asic_read, pc1403_asic_write )    
+	AM_RANGE( 0x4000, 0x7fff) AM_ROMBANK(1)
+	AM_RANGE( 0xe000, 0xffff) AM_RAM
+ADDRESS_MAP_END
 
 
 #if 0
-static MEMORY_READ_START( pc1421_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-	{ 0x3800, 0x47ff, MRA_RAM },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( pc1421_readmem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_READ( MRA8_ROM )
+	AM_RANGE( 0x3800, 0x47ff) AM_READ( MRA8_RAM )
+	AM_RANGE( 0x8000, 0xffff) AM_READ( MRA8_ROM )
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( pc1421_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-	{ 0x2000, 0x37ff, MWA_RAM },
-	{ 0x3800, 0x47ff, MWA_RAM },
-	{ 0x8000, 0xffff, MWA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( pc1421_writemem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_WRITE( MWA8_ROM )
+	AM_RANGE( 0x2000, 0x37ff) AM_WRITE( MWA8_RAM )
+	AM_RANGE( 0x3800, 0x47ff) AM_WRITE( MWA8_RAM )
+	AM_RANGE( 0x8000, 0xffff) AM_WRITE( MWA8_ROM )
+ADDRESS_MAP_END
 
-static MEMORY_READ_START( pc1260_readmem )
-	{ 0x0000, 0x1fff, MRA_ROM },
-	{ 0x4000, 0x5fff, MRA_RAM },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( pc1260_readmem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_READ( MRA8_ROM )
+	AM_RANGE( 0x4000, 0x5fff) AM_READ( MRA8_RAM )
+	AM_RANGE( 0x8000, 0xffff) AM_READ( MRA8_ROM )
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( pc1260_writemem )
-	{ 0x0000, 0x1fff, MWA_ROM },
-	{ 0x4000, 0x57ff, MWA_RAM }, /* 1261 */
-	{ 0x5800, 0x67ff, MWA_RAM },
-	{ 0x6000, 0x6fff, MWA_RAM },
+static ADDRESS_MAP_START( pc1260_writemem , ADDRESS_SPACE_PROGRAM, 8)
+	AM_RANGE( 0x0000, 0x1fff) AM_WRITE( MWA8_ROM )
+	AM_RANGE( 0x4000, 0x57ff) AM_WRITE( MWA8_RAM ) /* 1261 */
+	AM_RANGE( 0x5800, 0x67ff) AM_WRITE( MWA8_RAM )
+	AM_RANGE( 0x6000, 0x6fff) AM_WRITE( MWA8_RAM )
 
-	{ 0x8000, 0xffff, MWA_ROM },
-MEMORY_END
+	AM_RANGE( 0x8000, 0xffff) AM_WRITE( MWA8_ROM )
+ADDRESS_MAP_END
 #endif
 
 #define DIPS_HELPER(bit, name, keycode, r) \
-   PORT_BITX(bit, IP_ACTIVE_HIGH, IPT_KEYBOARD, name, keycode, r)
+   PORT_BIT(bit, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME(name) PORT_CODE(keycode) PORT_CODE(r)
 
 
 #define PC1401_HELPER1 \
 	PORT_START \
-	PORT_BITX (0x80, 0x00, IPT_DIPSWITCH_NAME|IPF_TOGGLE, \
-			   "Power",CODE_DEFAULT, CODE_NONE) \
+	PORT_DIPNAME (0x80, 0x00,  "Power")  PORT_CODE(CODE_DEFAULT)\
 	PORT_DIPSETTING( 0x80, DEF_STR( Off ) ) \
 	PORT_DIPSETTING( 0x00, DEF_STR( On ) ) \
 	DIPS_HELPER( 0x40, "CAL", KEYCODE_F1, CODE_NONE) \
@@ -406,8 +371,7 @@ INPUT_PORTS_END
 
 INPUT_PORTS_START( pc1350 )
 	PORT_START
-    PORT_BITX (0x80, 0x00, IPT_DIPSWITCH_NAME|IPF_TOGGLE,
-			   "Power",CODE_DEFAULT, CODE_NONE)
+    PORT_DIPNAME (0x80, 0x00, "Power") PORT_CODE(CODE_DEFAULT)
 	PORT_DIPSETTING( 0x80, DEF_STR( Off ) )
 	PORT_DIPSETTING( 0x00, DEF_STR( On ) )
 	DIPS_HELPER( 0x40, "DOWN", KEYCODE_DOWN, CODE_NONE)
@@ -495,97 +459,78 @@ INPUT_PORTS_START( pc1350 )
 	PORT_DIPSETTING( 7, "7/High" )
 INPUT_PORTS_END
 
-static struct GfxLayout pc1401_charlayout =
+static gfx_layout pc1401_charlayout =
 {
-        2,21,
-        128,                                    /* 256 characters */
-        1,                      /* 1 bits per pixel */
-        { 0,0 },                  /* no bitplanes; 1 bit per pixel */
-        /* x offsets */
-        { 0,0 },
-        /* y offsets */
-        {
-			7, 7, 7,
-			6, 6, 6,
-			5, 5, 5,
-			4, 4, 4,
-			3, 3, 3,
-			2, 2, 2,
-			1, 1, 1
-        },
-        1*8
+	2,21,
+	128,                                    /* 256 characters */
+	1,                      /* 1 bits per pixel */
+	{ 0,0 },                  /* no bitplanes; 1 bit per pixel */
+	/* x offsets */
+	{ 0,0 },
+	/* y offsets */
+	{
+		7, 7, 7,
+		6, 6, 6,
+		5, 5, 5,
+		4, 4, 4,
+		3, 3, 3,
+		2, 2, 2,
+		1, 1, 1
+	},
+	1*8
 };
 
-static struct GfxLayout pc1251_charlayout =
+static gfx_layout pc1251_charlayout =
 {
-        3,21,
-        128,                                    /* 256 characters */
-        1,                      /* 1 bits per pixel */
-        { 0, },                  /* no bitplanes; 1 bit per pixel */
-        /* x offsets */
-        { 0,0,0 },
-        /* y offsets */
-        {
-			7, 7, 7,
-			6, 6, 6,
-			5, 5, 5,
-			4, 4, 4,
-			3, 3, 3,
-			2, 2, 2,
-			1, 1, 1
-        },
-        1*8
+	3,21,
+	128,                                    /* 256 characters */
+	1,                      /* 1 bits per pixel */
+	{ 0, },                  /* no bitplanes; 1 bit per pixel */
+	/* x offsets */
+	{ 0,0,0 },
+	/* y offsets */
+	{
+		7, 7, 7,
+		6, 6, 6,
+		5, 5, 5,
+		4, 4, 4,
+		3, 3, 3,
+		2, 2, 2,
+		1, 1, 1
+	},
+	1*8
 };
 
-static struct GfxLayout pc1350_charlayout =
+static gfx_layout pc1350_charlayout =
 {
-        2,16,
-        256,                                    /* 256 characters */
-        1,                      /* 1 bits per pixel */
-        { 0,0 },                  /* no bitplanes; 1 bit per pixel */
-        /* x offsets */
-        { 0 },
-        /* y offsets */
-        {
-			7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1, 1, 0, 0
-        },
-        1*8
+	2,16,
+	256,                                    /* 256 characters */
+	1,                      /* 1 bits per pixel */
+	{ 0,0 },                  /* no bitplanes; 1 bit per pixel */
+	/* x offsets */
+	{ 0 },
+	/* y offsets */
+	{
+		7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1, 1, 0, 0
+	},
+	1*8
 };
 
-static struct GfxDecodeInfo pc1401_gfxdecodeinfo[] = {
+static gfx_decode pc1401_gfxdecodeinfo[] = {
 	{ REGION_GFX1, 0x0000, &pc1401_charlayout,                     0, 8 },
     { -1 } /* end of array */
 };
 
-static struct GfxDecodeInfo pc1251_gfxdecodeinfo[] = {
+static gfx_decode pc1251_gfxdecodeinfo[] = {
 	{ REGION_GFX1, 0x0000, &pc1251_charlayout,                     0, 8 },
     { -1 } /* end of array */
 };
 
-static struct GfxDecodeInfo pc1350_gfxdecodeinfo[] = {
+static gfx_decode pc1350_gfxdecodeinfo[] = {
 	{ REGION_GFX1, 0x0000, &pc1350_charlayout,                     0, 8 },
     { -1 } /* end of array */
 };
 
-#if 0
-static struct DACinterface dac_interface =
-{
-	1,			/* number of DACs */
-	{ 100 } 	/* volume */
-};
-#endif
-
-static int pocketc_frame_int(void)
-{
-	return 0;
-}
-
-struct DACinterface pocketc_sound_interface =
-{
-        1,
-        {25}
-};
-
 static SC61860_CONFIG config={
     pc1401_reset, pc1401_brk, NULL,
     pc1401_ina, pc1401_outa,
@@ -593,211 +538,139 @@ static SC61860_CONFIG config={
     pc1401_outc
 };
 
-static struct MachineDriver machine_driver_pc1401 =
-{
+static MACHINE_DRIVER_START( pc1401 )
 	/* basic machine hardware */
-	{
-		{
-			CPU_SC61860,
-			192000,
-			pc1401_readmem,pc1401_writemem,0,0,
-			pocketc_frame_int, 1,
-			0,0,
-			&config
-        }
-	},
-	/* frames per second, VBL duration */
-	20, DEFAULT_60HZ_VBLANK_DURATION, /* very early and slow lcd */
-	1,				/* single CPU */
-	pc1401_machine_init,
-	pc1401_machine_stop,
+	MDRV_CPU_ADD_TAG("main", SC61860, 192000)        /* 7.8336 Mhz */
+	MDRV_CPU_PROGRAM_MAP(pc1401_mem, 0)
+	MDRV_CPU_CONFIG(config)
+
+	MDRV_FRAMES_PER_SECOND(20)	/* very early and slow lcd */
+	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
+	MDRV_INTERLEAVE(1)
+
+	MDRV_NVRAM_HANDLER( pc1401 )
 
 	/*
 	   aim: show sharp with keyboard
 	   resolution depends on the dots of the lcd
 	   (lcd dot displayed as 2x3 pixel)
 	   it seams to have 3/4 ratio in the real pc1401 */
+    /* video hardware */
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(594, 273)
+	MDRV_VISIBLE_AREA(0, 594-1, 0, 273-1)
+/*	MDRV_SCREEN_SIZE(640, 273) */
+/*	MDRV_VISIBLE_AREA(0, 640-1, 0, 273-1) */
+	MDRV_GFXDECODE( pc1401_gfxdecodeinfo )
+	MDRV_PALETTE_LENGTH(248 + 32768)
+	MDRV_COLORTABLE_LENGTH( 16 )
+	MDRV_PALETTE_INIT( pocketc )
 
-	594, 273, { 0, 594 - 1, 0, 273 - 1},
-/*	640, 273, { 0, 640 - 1, 0, 273 - 1}, */
-	pc1401_gfxdecodeinfo,			   /* graphics decode info */
-	sizeof (pocketc_palette) / sizeof (pocketc_palette[0]) + 32768,
-	sizeof (pocketc_colortable) / sizeof(pocketc_colortable[0][0]),
-	pocketc_init_colors,		/* convert color prom */
-
-	VIDEO_TYPE_RASTER| VIDEO_SUPPORTS_DIRTY,	/* video flags */
-	0,						/* obsolete */
-    pocketc_vh_start,
-	pocketc_vh_stop,
-	pc1401_vh_screenrefresh,
+	MDRV_VIDEO_START( pocketc )
+	MDRV_VIDEO_UPDATE( pc1401 )
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-/*		{SOUND_DAC, &pocketc_sound_interface}, */
-        { 0 }
-    }
-};
+	/*MDRV_SOUND_ADD(DAC, pocketc_sound_interface)*/
+MACHINE_DRIVER_END
 
-static SC61860_CONFIG pc1251_config={
+
+static SC61860_CONFIG pc1251_config =
+{
     NULL, pc1251_brk, NULL,
     pc1251_ina, pc1251_outa,
     pc1251_inb, pc1251_outb,
     pc1251_outc
 };
 
-static struct MachineDriver machine_driver_pc1251 =
+static MACHINE_DRIVER_START( pc1251 )
+	MDRV_IMPORT_FROM( pc1401 )
+	MDRV_CPU_MODIFY( "main" )
+	MDRV_CPU_PROGRAM_MAP( pc1251_mem, 0 )
+	MDRV_CPU_CONFIG( pc1251_config )
+
+	MDRV_NVRAM_HANDLER( pc1251 )
+
+	/* video hardware */
+	MDRV_SCREEN_SIZE(608, 300)
+	MDRV_VISIBLE_AREA(0, 608-1, 0, 300-1)
+/*	MDRV_SCREEN_SIZE(640, 334) */
+/*	MDRV_VISIBLE_AREA(0, 640-1, 0, 334-1) */
+	MDRV_GFXDECODE( pc1251_gfxdecodeinfo )
+
+	MDRV_VIDEO_UPDATE( pc1251 )
+MACHINE_DRIVER_END
+
+
+static SC61860_CONFIG pc1350_config =
 {
-	/* basic machine hardware */
-	{
-		{
-			CPU_SC61860,
-			192000,
-			pc1251_readmem,pc1251_writemem,0,0,
-			pocketc_frame_int, 1,
-			0,0,
-			&pc1251_config
-        }
-	},
-	/* frames per second, VBL duration */
-	20, DEFAULT_60HZ_VBLANK_DURATION, /* very early and slow lcd */
-	1,				/* single CPU */
-	pc1251_machine_init,
-	pc1251_machine_stop,
-
-	/*
-	   aim: show sharp with keyboard
-	   resolution depends on the dots of the lcd
-	   (lcd dot displayed as 3x3 pixel) */
-
-	608, 300, { 0, 608 - 1, 0, 300 - 1},
-/*	640, 334, { 0, 640 - 1, 0, 334 - 1}, */
-	pc1251_gfxdecodeinfo,			   /* graphics decode info */
-	sizeof (pocketc_palette) / sizeof (pocketc_palette[0]) + 32768 ,
-	sizeof (pocketc_colortable) / sizeof(pocketc_colortable[0][0]),
-	pocketc_init_colors,		/* convert color prom */
-
-	VIDEO_TYPE_RASTER| VIDEO_SUPPORTS_DIRTY,	/* video flags */
-	0,						/* obsolete */
-    pocketc_vh_start,
-	pocketc_vh_stop,
-	pc1251_vh_screenrefresh,
-
-	/* sound hardware */
-	0,0,0,0,
-	{
-        { 0 }
-    }
-};
-
-static SC61860_CONFIG pc1350_config={
     NULL, pc1350_brk,NULL,
     pc1350_ina, pc1350_outa,
     pc1350_inb, pc1350_outb,
     pc1350_outc
 };
 
-static struct MachineDriver machine_driver_pc1350 =
-{
-	/* basic machine hardware */
-	{
-		{
-			CPU_SC61860,
-			192000,
-			pc1350_readmem,pc1350_writemem,0,0,
-			pocketc_frame_int, 1,
-			0,0,
-			&pc1350_config
-        }
-	},
-	/* frames per second, VBL duration */
-	20, DEFAULT_60HZ_VBLANK_DURATION, /* very early and slow lcd */
-	1,				/* single CPU */
-	pc1350_machine_init,
-	pc1350_machine_stop,
+static MACHINE_DRIVER_START( pc1350 )
+	MDRV_IMPORT_FROM( pc1401 )
+	MDRV_CPU_MODIFY( "main" )
+	MDRV_CPU_PROGRAM_MAP( pc1350_mem, 0 )
+	MDRV_CPU_CONFIG( pc1350_config )
+
+	MDRV_MACHINE_INIT( pc1350 )
+	MDRV_NVRAM_HANDLER( pc1350 )
 
 	/*
 	   aim: show sharp with keyboard
 	   resolution depends on the dots of the lcd
 	   (lcd dot displayed as 2x2 pixel) */
+	/* video hardware */
+	MDRV_SCREEN_SIZE(640, 252)
+	MDRV_VISIBLE_AREA(0, 640-1, 0, 252-1)
+/*	MDRV_SCREEN_SIZE(640, 255) */
+/*	MDRV_VISIBLE_AREA(0, 640-1, 0, 255-1) */
+	MDRV_GFXDECODE( pc1350_gfxdecodeinfo )
 
-	640, 252, { 0, 640 - 1, 0, 252 - 1},
-/*	640, 255, { 0, 640 - 1, 0, 255 - 1}, */
-	pc1350_gfxdecodeinfo,			   /* graphics decode info */
-	sizeof (pocketc_palette) / sizeof (pocketc_palette[0]) + 32768,
-	sizeof (pocketc_colortable) / sizeof(pocketc_colortable[0][0]),
-	pocketc_init_colors,		/* convert color prom */
+	MDRV_VIDEO_UPDATE( pc1350 )
+MACHINE_DRIVER_END
 
-	VIDEO_TYPE_RASTER| VIDEO_SUPPORTS_DIRTY,	/* video flags */
-	0,						/* obsolete */
-    pocketc_vh_start,
-	pocketc_vh_stop,
-	pc1350_vh_screenrefresh,
 
-	/* sound hardware */
-	0,0,0,0,
-	{
-        { 0 }
-    }
-};
-static SC61860_CONFIG pc1403_config={
+static SC61860_CONFIG pc1403_config =
+{
     NULL, pc1403_brk, NULL,
     pc1403_ina, pc1403_outa,
     NULL,NULL,
     pc1403_outc
 };
 
-static struct MachineDriver machine_driver_pc1403 =
-{
-	/* basic machine hardware */
-	{
-		{
-			CPU_SC61860,
-			256000,
-			pc1403_readmem,pc1403_writemem,0,0,
-			pocketc_frame_int, 1,
-			0,0,
-			&pc1403_config
-        }
-	},
-	/* frames per second, VBL duration */
-	20, DEFAULT_60HZ_VBLANK_DURATION, /* very early and slow lcd */
-	1,				/* single CPU */
-	pc1403_machine_init,
-	pc1403_machine_stop,
+static MACHINE_DRIVER_START( pc1403 )
+	MDRV_IMPORT_FROM( pc1401 )
+	MDRV_CPU_REPLACE( "main", SC61860, 256000 )
+	MDRV_CPU_PROGRAM_MAP( pc1403_mem, 0 )
+	MDRV_CPU_CONFIG( pc1403_config )
+
+	MDRV_NVRAM_HANDLER( pc1403 )
 
 	/*
 	   aim: show sharp with keyboard
 	   resolution depends on the dots of the lcd
 	   (lcd dot displayed as 2x2 pixel) */
+	/* video hardware */
+	MDRV_SCREEN_SIZE(848, 320)
+	MDRV_VISIBLE_AREA(0, 848-1, 0, 320-1)
+/*	MDRV_SCREEN_SIZE(848, 361) */
+/*	MDRV_VISIBLE_AREA(0, 848-1, 0, 361-1) */
 
-	848, 320, { 0, 848 - 1, 0, 320 - 1},
-/*	848, 361, { 0, 848 - 1, 0, 361 - 1}, */
-	pc1401_gfxdecodeinfo,			   /* graphics decode info */
-	sizeof (pocketc_palette) / sizeof (pocketc_palette[0])  + 32768,
-	sizeof (pocketc_colortable) / sizeof(pocketc_colortable[0][0]),
-	pocketc_init_colors,		/* convert color prom */
+	MDRV_VIDEO_START( pc1403 )
+	MDRV_VIDEO_UPDATE( pc1403 )
+MACHINE_DRIVER_END
 
-	VIDEO_TYPE_RASTER| VIDEO_SUPPORTS_DIRTY,	/* video flags */
-	0,						/* obsolete */
-    pc1403_vh_start,
-	pocketc_vh_stop,
-	pc1403_vh_screenrefresh,
-
-	/* sound hardware */
-	0,0,0,0,
-	{
-        { 0 }
-    }
-};
 
 ROM_START(pc1401)
 	ROM_REGION(0x10000,REGION_CPU1,0)
 	/* SC61860A08 5H 13LD cpu with integrated rom*/
-	ROM_LOAD("sc61860.a08", 0x0000, 0x2000, 0x44bee438)
+	ROM_LOAD("sc61860.a08", 0x0000, 0x2000, CRC(44bee438))
 /* 5S1 SC613256 D30
    or SC43536LD 5G 13 (LCD chip?) */
-	ROM_LOAD("sc613256.d30", 0x8000, 0x8000, 0x69b9d587)
+	ROM_LOAD("sc613256.d30", 0x8000, 0x8000, CRC(69b9d587))
 	ROM_REGION(0x80,REGION_GFX1,0)
 ROM_END
 
@@ -806,8 +679,8 @@ ROM_END
 ROM_START(pc1251)
 	ROM_REGION(0x10000,REGION_CPU1,0)
 	/* sc61860a13 6c 13 ld */
-	ROM_LOAD("cpu1251.rom", 0x0000, 0x2000, 0xf7287aca)
-	ROM_LOAD("bas1251.rom", 0x4000, 0x4000, 0x93ecb629)
+	ROM_LOAD("cpu1251.rom", 0x0000, 0x2000, CRC(f7287aca))
+	ROM_LOAD("bas1251.rom", 0x4000, 0x4000, CRC(93ecb629))
 	ROM_REGION(0x80,REGION_GFX1,0)
 ROM_END
 
@@ -816,38 +689,32 @@ ROM_END
 ROM_START(pc1350)
 	ROM_REGION(0x10000,REGION_CPU1,0)
 	/* sc61860a13 6c 13 ld */
-	ROM_LOAD("cpu.rom", 0x0000, 0x2000, 0x79a924bc)
-	ROM_LOAD("basic.rom", 0x8000, 0x8000, 0x158b28e2)
+	ROM_LOAD("cpu.rom", 0x0000, 0x2000, CRC(79a924bc))
+	ROM_LOAD("basic.rom", 0x8000, 0x8000, CRC(158b28e2))
 	ROM_REGION(0x100,REGION_GFX1,0)
 ROM_END
 
 ROM_START(pc1403)
 	ROM_REGION(0x10000,REGION_CPU1,0)
-    ROM_LOAD("introm.bin", 0x0000, 0x2000, 0x588c500b )
+    ROM_LOAD("introm.bin", 0x0000, 0x2000, CRC(588c500b ))
 	ROM_REGION(0x10000,REGION_USER1,0)
-    ROM_LOAD("extrom08.bin", 0x0000, 0x4000, 0x1fa65140 )
-    ROM_LOAD("extrom09.bin", 0x4000, 0x4000, 0x4a7da6ab )
-    ROM_LOAD("extrom0a.bin", 0x8000, 0x4000, 0x9925174f )
-    ROM_LOAD("extrom0b.bin", 0xc000, 0x4000, 0xfa5df9ec )
+    ROM_LOAD("extrom08.bin", 0x0000, 0x4000, CRC(1fa65140 ))
+    ROM_LOAD("extrom09.bin", 0x4000, 0x4000, CRC(4a7da6ab ))
+    ROM_LOAD("extrom0a.bin", 0x8000, 0x4000, CRC(9925174f ))
+    ROM_LOAD("extrom0b.bin", 0xc000, 0x4000, CRC(fa5df9ec ))
 	ROM_REGION(0x100,REGION_GFX1,0)
 ROM_END
 
 #define rom_pc1403h rom_pc1403
 
-static const struct IODevice io_pc1401[] = {
-/*	IO_CASSETTE_WAVE(1,"wav\0",mycas_id,mycas_init,mycas_exit), */
-    { IO_END }
-};
-
-#define io_pc1402 io_pc1401
-#define io_pc1350 io_pc1401
-#define io_pc1251 io_pc1401
-#define io_trs80pc3 io_pc1401
-
 /* disk drive support */
 #define io_pc1403 io_pc1401
 #define io_pc1403h io_pc1403
 
+SYSTEM_CONFIG_START(pocketc)
+/*	CONFIG_DEVICE_CASSETTE(1, "", mycas_init) */
+SYSTEM_CONFIG_END
+
 /*    YEAR  NAME      PARENT    MACHINE   INPUT     INIT      MONITOR	COMPANY   FULLNAME */
 
 /* cpu sc43177, sc43178 (4bit!)
@@ -862,44 +729,16 @@ static const struct IODevice io_pc1401[] = {
    pc1600
 */
 
+/*    YEAR  NAME		PARENT	COMPAT	MACHINE			INPUT		INIT	CONFIG		COMPANY		FULLNAME */
 /* cpu sc61860 */
-COMPX( 198?, pc1251,	  0, 		pc1251,  pc1251, 	pc1251,	  "Sharp",  "Pocket Computer 1251", GAME_NOT_WORKING)
-COMPX( 198?, trs80pc3,	  pc1251, 	pc1251,  pc1251, 	pc1251,	  "Tandy",  "TRS80 PC-3", GAME_ALIAS|GAME_NOT_WORKING)
+COMP( 1982, pc1251,	0, 		0,		pc1251,	pc1251,	pc1251,	pocketc,	"Sharp",  "Pocket Computer 1251", GAME_NOT_WORKING)
+COMP( 198?, trs80pc3,	pc1251,	0,		pc1251,	pc1251,	pc1251,	pocketc,	"Tandy",  "TRS80 PC-3", GAME_NOT_WORKING)
 
 /* pc1261/pc1262 */
+COMP( 1984, pc1350,	0, 		0,		pc1350,	pc1350,	pc1350,	pocketc,	"Sharp",  "Pocket Computer 1350", GAME_NOT_WORKING)
+COMP( 1983, pc1401,	0, 		0,		pc1401,	pc1401,	pc1401,	pocketc,	"Sharp",  "Pocket Computer 1401", GAME_NOT_WORKING)
+COMP( 1984, pc1402,	pc1401,	0,		pc1401,	pc1401,	pc1401,	pocketc,	"Sharp",  "Pocket Computer 1402", GAME_NOT_WORKING)
 
-COMPX( 198?, pc1350,	  0, 		pc1350,  pc1350, 	pc1350,	  "Sharp",  "Pocket Computer 1350", GAME_NOT_WORKING)
-
-COMPX( 1983, pc1401,	  0, 		pc1401,  pc1401, 	pc1401,	  "Sharp",  "Pocket Computer 1401", GAME_NOT_WORKING)
-COMPX( 198?, pc1402,	  pc1401, 	pc1401,  pc1401, 	pc1401,	  "Sharp",  "Pocket Computer 1402", GAME_ALIAS|GAME_NOT_WORKING)
-
-/* 72kb rom, 32kb ram, cpu? 
-   pc1360
-*/
-COMPX( 198?, pc1403,	0,	pc1403,		pc1403,		pc1403,	"Sharp", "Pocket Computer 1403", GAME_NOT_WORKING)
-COMPX( 198?, pc1403h,	pc1403,	pc1403,		pc1403,		pc1403,	"Sharp", "Pocket Computer 1403H", GAME_ALIAS|GAME_NOT_WORKING)
-
-/* cpu sc62015 esr-l
-   pc-e500
- */
-
-/* cpu hd61747 ???
-   tandy pocket scientific computer pc-6
-   clone of ? sharp pb1000
-*/
-#ifdef RUNTIME_LOADER
-extern void pocketc_runtime_loader_init(void)
-{
-	int i;
-	for (i=0; drivers[i]; i++) {
-		if ( strcmp(drivers[i]->name,"pc1251")==0) drivers[i]=&driver_pc1251;
-		if ( strcmp(drivers[i]->name,"trs80pc3")==0) drivers[i]=&driver_trs80pc3;
-		if ( strcmp(drivers[i]->name,"pc1401")==0) drivers[i]=&driver_pc1401;
-		if ( strcmp(drivers[i]->name,"pc1402")==0) drivers[i]=&driver_pc1402;
-		if ( strcmp(drivers[i]->name,"pc1350")==0) drivers[i]=&driver_pc1350;
-		if ( strcmp(drivers[i]->name,"pc1403")==0) drivers[i]=&driver_pc1403;
-		if ( strcmp(drivers[i]->name,"pc1403h")==0) drivers[i]=&driver_pc1403h;
-	}
-}
-#endif
-
+/* 72kb rom, 32kb ram, cpu? pc1360 */
+COMP( 198?, pc1403,	0,		0,		pc1403,	pc1403,	pc1403,	pocketc,	"Sharp", "Pocket Computer 1403", GAME_NOT_WORKING)
+COMP( 198?, pc1403h,	pc1403,	0,		pc1403,	pc1403,	pc1403,	pocketc,	"Sharp", "Pocket Computer 1403H", GAME_NOT_WORKING)
