@@ -28,6 +28,31 @@
 #ifndef INVALID_SET_FILE_POINTER
 #define INVALID_SET_FILE_POINTER 0xffffffff
 #endif
+
+#else
+
+#ifdef HAVE_UNISTD_H
+#include <sys/types.h>
+#include <unistd.h>
+#endif
+
+#ifdef _POSIX_VERSION
+#define OFF_T off_t
+#define FSEEK fseeko
+#define FTELL ftello
+#else
+#define OFF_T long
+#define FSEEK fseek
+#define FTELL ftell
+#endif
+
+typedef struct
+{
+	FILE	*file;
+	char	*buf;
+	OFF_T	size;
+} iso_file_t;
+
 #endif
 
 
@@ -1459,7 +1484,7 @@ static int handle_custom_chomp(const char *name, chd_file *chd, UINT32 *maxhunk)
 	{
 		UINT32 sectors[4];
 		UINT8 *data;
-		int i;
+		int i, maxdiff;
 
 		if (!chd_read(chd, 0x200 / header->hunkbytes, 1, temp))
 			goto error;
@@ -1469,9 +1494,12 @@ static int handle_custom_chomp(const char *name, chd_file *chd, UINT32 *maxhunk)
 			goto error;
 		for (i = 0; i < 4; i++)
 			sectors[i] = data[i*4+0x40] | (data[i*4+0x41] << 8) | (data[i*4+0x42] << 16) | (data[i*4+0x43] << 24);
-		if (sectors[0] != 8 || sectors[2] - sectors[1] != sectors[3] - sectors[2])
+		maxdiff = sectors[2] - sectors[1];
+		if (sectors[3] - sectors[2] > maxdiff)
+			maxdiff = sectors[3] - sectors[2];
+		if (sectors[0] != 8)
 			goto error;
-		*maxhunk = (sectors[3] + (sectors[3] - sectors[2]) + sectors_per_hunk - 1) / sectors_per_hunk;
+		*maxhunk = (sectors[3] + maxdiff + sectors_per_hunk - 1) / sectors_per_hunk;
 		printf("Maximum hunk: %d\n", *maxhunk);
 		if (*maxhunk >= header->totalhunks)
 		{
@@ -1861,7 +1889,7 @@ static UINT64 get_file_size(const char *file)
 	CloseHandle(handle);
 	return filesize;
 #else
-	size_t filesize;
+	OFF_T filesize;
 	FILE *f;
 
 	/* attempt to open the file */
@@ -1870,8 +1898,8 @@ static UINT64 get_file_size(const char *file)
 		return 0;
 
 	/* get the size */
-	fseek(f, 0, SEEK_END);
-	filesize = ftell(f);
+	FSEEK(f, 0, SEEK_END);
+	filesize = FTELL(f);
 	fclose(f);
 
 	return filesize;
@@ -1920,7 +1948,35 @@ static chd_interface_file *chdman_open(const char *filename, const char *mode)
 			return NULL;
 		return (chd_interface_file *)handle;
 #else
-		return (chd_interface_file *)fopen(filename, mode);
+		iso_file_t *iso_file = NULL;
+		FILE *file = fopen(filename, mode);
+		if (file)
+		{
+			OFF_T size;
+			char *buf = (char *)malloc(BUFSIZ);
+			if (buf)
+			{
+				/*
+				 * Calling setbuf makes -createhd and -merge run 5 - 10x faster 
+				 * on * Linux with the GNU C Library, and it shouldn't hurt 
+				 * performance on other platforms (verified on Mac OS X).
+				 */
+				setbuf(file, buf);
+			}
+
+			FSEEK(file, 0, SEEK_END);
+			size = FTELL(file);
+			rewind(file);
+
+			iso_file = (iso_file_t *)malloc(sizeof(iso_file_t));
+			if (iso_file)
+			{
+				iso_file->file = file;
+				iso_file->buf = buf;
+				iso_file->size = size;
+			}
+		}
+		return (chd_interface_file *)iso_file;
 #endif
 	}
 }
@@ -1940,7 +1996,9 @@ static void chdman_close(chd_interface_file *file)
 #ifdef _WIN32
 	CloseHandle((HANDLE)file);
 #else
-	fclose((FILE *)file);
+	fclose(((iso_file_t *)file)->file);
+	free(((iso_file_t *)file)->buf);
+	free(file);
 #endif
 }
 
@@ -2023,8 +2081,11 @@ static UINT32 chdman_read(chd_interface_file *file, UINT64 offset, UINT32 count,
 		else
 			return 0;
 #else
-		fseek((FILE *)file, offset, SEEK_SET);
-		return fread(buffer, 1, count, (FILE *)file);
+		iso_file_t *iso_file = (iso_file_t *)file;
+		if (FSEEK(iso_file->file, offset, SEEK_SET) == 0)
+			return fread(buffer, 1, count, iso_file->file);
+		else
+			return 0;
 #endif
 	}
 }
@@ -2062,8 +2123,15 @@ static UINT32 chdman_write(chd_interface_file *file, UINT64 offset, UINT32 count
 		else
 			return 0;
 #else
-		fseek((FILE *)file, offset, SEEK_SET);
-		return fwrite(buffer, 1, count, (FILE *)file);
+		size_t bytes_written = 0;
+		iso_file_t *iso_file = (iso_file_t *)file;
+		if (FSEEK(iso_file->file, offset, SEEK_SET) == 0)
+		{
+			bytes_written = fwrite(buffer, 1, count, iso_file->file);
+			if (offset + bytes_written > iso_file->size)
+				iso_file->size = offset + bytes_written;
+		}
+		return bytes_written;
 #endif
 	}
 }
@@ -2094,15 +2162,7 @@ static UINT64 chdman_length(chd_interface_file *file)
 			filesize = 0;
 		return filesize;
 #else
-		size_t oldpos = ftell((FILE *)file);
-		size_t filesize;
-
-		/* get the size */
-		fseek((FILE *)file, 0, SEEK_END);
-		filesize = ftell((FILE *)file);
-		fseek((FILE *)file, oldpos, SEEK_SET);
-
-		return filesize;
+		return ((iso_file_t *)file)->size;
 #endif
 	}
 }
@@ -2128,9 +2188,9 @@ int main(int argc, char **argv)
 	/* handle the appropriate command */
 	if (!mame_stricmp(argv[1], "-createhd"))
 		do_createhd(argc, argv);
-	if (!mame_stricmp(argv[1], "-createblankhd"))
+	else if (!mame_stricmp(argv[1], "-createblankhd"))
 		do_createblankhd(argc, argv);
-	if (!mame_stricmp(argv[1], "-copydata"))
+	else if (!mame_stricmp(argv[1], "-copydata"))
 		do_copydata(argc, argv);
 	else if (!mame_stricmp(argv[1], "-createcd"))
 		do_createcd(argc, argv);
