@@ -1,5 +1,5 @@
 /*===========================================================================
-	ASTAT -- ALU/MAC status register
+    ASTAT -- ALU/MAC status register
 ===========================================================================*/
 
 /* flag definitions */
@@ -56,10 +56,19 @@
 #define CALC_NZVC(s,d,r) CLR_FLAGS; CALC_N(r); CALC_Z(r); CALC_V(s,d,r); CALC_C(r)
 #define CALC_NZVC_SUB(s,d,r) CLR_FLAGS; CALC_N(r); CALC_Z(r); CALC_V(s,d,r); CALC_C_SUB(r)
 
+/* ADSP-218x constants */
+static const INT32 constants[] =
+{
+	0x0001, 0xfffe, 0x0002, 0xfffd, 0x0004, 0xfffb, 0x0008, 0xfff7,
+	0x0010, 0xffef, 0x0020, 0xffdf, 0x0040, 0xffbf, 0x0080, 0xff7f,
+	0x0100, 0xfeff, 0x0200, 0xfdff, 0x0400, 0xfbff, 0x0800, 0xf7ff,
+	0x1000, 0xefff, 0x2000, 0xdfff, 0x4000, 0xbfff, 0x8000, 0x7fff
+};
+
 
 
 /*===========================================================================
-	MSTAT -- ALU/MAC control register
+    MSTAT -- ALU/MAC control register
 ===========================================================================*/
 
 /* flag definitions */
@@ -67,12 +76,9 @@
 #define MSTAT_REVERSE	0x02			/* bit-reverse addressing enable (DAG1) */
 #define MSTAT_STICKYV	0x04			/* sticky ALU overflow enable */
 #define MSTAT_SATURATE	0x08			/* AR saturation mode enable */
-
-#if SUPPORT_2101_EXTENSIONS
 #define MSTAT_INTEGER	0x10			/* MAC result placement; 0=fractional, 1=integer */
 #define MSTAT_TIMER		0x20			/* timer enable */
 #define MSTAT_GOMODE	0x40			/* go mode enable */
-#endif
 
 /* you must call this in order to change MSTAT */
 INLINE void set_mstat(int new_value)
@@ -92,7 +98,7 @@ INLINE void set_mstat(int new_value)
 
 
 /*===========================================================================
-	SSTAT -- stack status register
+    SSTAT -- stack status register
 ===========================================================================*/
 
 /* flag definitions */
@@ -108,7 +114,7 @@ INLINE void set_mstat(int new_value)
 
 
 /*===========================================================================
-	PC stack handlers
+    PC stack handlers
 ===========================================================================*/
 
 INLINE UINT32 pc_stack_top(void)
@@ -160,6 +166,7 @@ INLINE void pc_stack_pop(void)
 			adsp2100.sstat |= PC_EMPTY;
 	}
 	adsp2100.pc = adsp2100.pc_stack[adsp2100.pc_sp];
+	CHANGEPC();
 }
 
 INLINE UINT32 pc_stack_pop_val(void)
@@ -175,7 +182,7 @@ INLINE UINT32 pc_stack_pop_val(void)
 
 
 /*===========================================================================
-	CNTR stack handlers
+    CNTR stack handlers
 ===========================================================================*/
 
 INLINE UINT32 cntr_stack_top(void)
@@ -211,7 +218,7 @@ INLINE void cntr_stack_pop(void)
 
 
 /*===========================================================================
-	LOOP stack handlers
+    LOOP stack handlers
 ===========================================================================*/
 
 INLINE UINT32 loop_stack_top(void)
@@ -257,7 +264,7 @@ INLINE void loop_stack_pop(void)
 
 
 /*===========================================================================
-	STAT stack handlers
+    STAT stack handlers
 ===========================================================================*/
 
 INLINE void stat_stack_push(void)
@@ -283,21 +290,22 @@ INLINE void stat_stack_pop(void)
 			adsp2100.sstat |= STATUS_EMPTY;
 	}
 	set_mstat(adsp2100.stat_stack[adsp2100.stat_sp][0]);
-	adsp2100.imask = adsp2100.stat_stack[adsp2100.stat_sp][1]; check_irqs();
+	adsp2100.imask = adsp2100.stat_stack[adsp2100.stat_sp][1];
 	adsp2100.astat = adsp2100.stat_stack[adsp2100.stat_sp][2];
+ 	check_irqs();
 }
 
 
 
 /*===========================================================================
-	condition code checking
+    condition code checking
 ===========================================================================*/
 
 INLINE int CONDITION(int c)
 {
 	if (c != 14)
 		return condition_table[((c) << 8) | adsp2100.astat];
-	else if (--adsp2100.cntr != 0)
+	else if ((INT32)--adsp2100.cntr > 0)
 		return 1;
 	else
 	{
@@ -309,10 +317,10 @@ INLINE int CONDITION(int c)
 
 
 /*===========================================================================
-	register writing
+    register writing
 ===========================================================================*/
 
-static void wr_inval(INT32 val) { logerror( "ADSP %04x: Writing to an invalid register!", adsp2100.ppc ); }
+static void wr_inval(INT32 val) { logerror( "ADSP %04x: Writing to an invalid register!\n", adsp2100.ppc ); }
 static void wr_ax0(INT32 val)   { adsp2100.core.ax0.s = val; }
 static void wr_ax1(INT32 val)   { adsp2100.core.ax1.s = val; }
 static void wr_mx0(INT32 val)   { adsp2100.core.mx0.s = val; }
@@ -354,27 +362,55 @@ static void wr_l5(INT32 val)    { adsp2100.l[5] = val & 0x3fff; adsp2100.lmask[5
 static void wr_l6(INT32 val)    { adsp2100.l[6] = val & 0x3fff; adsp2100.lmask[6] = mask_table[val & 0x3fff]; adsp2100.base[6] = adsp2100.i[6] & adsp2100.lmask[6]; }
 static void wr_l7(INT32 val)    { adsp2100.l[7] = val & 0x3fff; adsp2100.lmask[7] = mask_table[val & 0x3fff]; adsp2100.base[7] = adsp2100.i[7] & adsp2100.lmask[7]; }
 static void wr_astat(INT32 val) { adsp2100.astat = val & 0x00ff; }
-#if SUPPORT_2101_EXTENSIONS
-static void wr_mstat(INT32 val) { set_mstat(val & 0x007f); }
-#else
-static void wr_mstat(INT32 val) { set_mstat(val & 0x000f); }
-#endif
+static void wr_mstat(INT32 val) { set_mstat(val & mstat_mask); }
 static void wr_sstat(INT32 val) { adsp2100.sstat = val & 0x00ff; }
-#if SUPPORT_2101_EXTENSIONS
-static void wr_imask(INT32 val) { adsp2100.imask = val & 0x003f; check_irqs(); }
-#else
-static void wr_imask(INT32 val) { adsp2100.imask = val & 0x000f; check_irqs(); }
-#endif
+static void wr_imask(INT32 val) { adsp2100.imask = val & imask_mask; check_irqs(); }
 static void wr_icntl(INT32 val) { adsp2100.icntl = val & 0x001f; check_irqs(); }
 static void wr_cntr(INT32 val)  { cntr_stack_push(); adsp2100.cntr = val & 0x3fff; }
 static void wr_sb(INT32 val)    { adsp2100.core.sb.s = (INT32)(val << 27) >> 27; }
 static void wr_px(INT32 val)    { adsp2100.px = val; }
-#if SUPPORT_2101_EXTENSIONS
-static void wr_ifc(INT32 val)	{ adsp2100.ifc = val; }
-static void wr_tx0(INT32 val)	{ if ( adsp2105_tx_callback ) (*adsp2105_tx_callback)( 0, val ); }
-static void wr_tx1(INT32 val)	{ if ( adsp2105_tx_callback ) (*adsp2105_tx_callback)( 1, val ); }
+static void wr_ifc(INT32 val)
+{
+	adsp2100.ifc = val;
+	if (chip_type >= CHIP_TYPE_ADSP2181)
+	{
+		/* clear timer */
+		if (val & 0x0002) adsp2100.irq_latch[ADSP2181_IRQ0] = 0;
+		if (val & 0x0004) adsp2100.irq_latch[ADSP2181_IRQ1] = 0;
+		/* clear BDMA */
+		if (val & 0x0010) adsp2100.irq_latch[ADSP2181_IRQE] = 0;
+		if (val & 0x0020) adsp2100.irq_latch[ADSP2181_SPORT0_RX] = 0;
+		if (val & 0x0040) adsp2100.irq_latch[ADSP2181_SPORT0_TX] = 0;
+		if (val & 0x0080) adsp2100.irq_latch[ADSP2181_IRQ2] = 0;
+		/* force timer */
+		if (val & 0x0200) adsp2100.irq_latch[ADSP2181_IRQ0] = 1;
+		if (val & 0x0400) adsp2100.irq_latch[ADSP2181_IRQ1] = 1;
+		/* force BDMA */
+		if (val & 0x1000) adsp2100.irq_latch[ADSP2181_IRQE] = 1;
+		if (val & 0x2000) adsp2100.irq_latch[ADSP2181_SPORT0_RX] = 1;
+		if (val & 0x4000) adsp2100.irq_latch[ADSP2181_SPORT0_TX] = 1;
+		if (val & 0x8000) adsp2100.irq_latch[ADSP2181_IRQ2] = 1;
+	}
+	else
+	{
+		/* clear timer */
+		if (val & 0x002) adsp2100.irq_latch[ADSP2101_IRQ0] = 0;
+		if (val & 0x004) adsp2100.irq_latch[ADSP2101_IRQ1] = 0;
+		if (val & 0x008) adsp2100.irq_latch[ADSP2101_SPORT0_RX] = 0;
+		if (val & 0x010) adsp2100.irq_latch[ADSP2101_SPORT0_TX] = 0;
+		if (val & 0x020) adsp2100.irq_latch[ADSP2101_IRQ2] = 0;
+		/* set timer */
+		if (val & 0x080) adsp2100.irq_latch[ADSP2101_IRQ0] = 1;
+		if (val & 0x100) adsp2100.irq_latch[ADSP2101_IRQ1] = 1;
+		if (val & 0x200) adsp2100.irq_latch[ADSP2101_SPORT0_RX] = 1;
+		if (val & 0x400) adsp2100.irq_latch[ADSP2101_SPORT0_TX] = 1;
+		if (val & 0x800) adsp2100.irq_latch[ADSP2101_IRQ2] = 1;
+	}
+	check_irqs();
+}
+static void wr_tx0(INT32 val)	{ if (adsp2100.sport_tx_callback) (*adsp2100.sport_tx_callback)(0, val); }
+static void wr_tx1(INT32 val)	{ if (adsp2100.sport_tx_callback) (*adsp2100.sport_tx_callback)(1, val); }
 static void wr_owrctr(INT32 val) { adsp2100.cntr = val & 0x3fff; }
-#endif
 static void wr_topstack(INT32 val) { pc_stack_push_val(val & 0x3fff); }
 
 #define WRITE_REG(grp,reg,val) ((*wr_reg[grp][reg])(val))
@@ -395,21 +431,17 @@ static void (*wr_reg[4][16])(INT32) =
 	},
 	{
 		wr_astat, wr_mstat, wr_inval, wr_imask, wr_icntl, wr_cntr, wr_sb, wr_px,
-#if SUPPORT_2101_EXTENSIONS
 		wr_inval, wr_tx0, wr_inval, wr_tx1, wr_ifc, wr_owrctr, wr_inval, wr_topstack
-#else
-		wr_inval, wr_inval, wr_inval, wr_inval, wr_inval, wr_inval, wr_inval, wr_topstack
-#endif
 	}
 };
 
 
 
 /*===========================================================================
-	register reading
+    register reading
 ===========================================================================*/
 
-static INT32 rd_inval(void) { logerror( "ADSP %04x: Writing to an invalid register!", adsp2100.ppc ); return 0; }
+static INT32 rd_inval(void) { logerror( "ADSP %04x: Writing to an invalid register!\n", adsp2100.ppc ); return 0; }
 static INT32 rd_ax0(void)   { return adsp2100.core.ax0.s; }
 static INT32 rd_ax1(void)   { return adsp2100.core.ax1.s; }
 static INT32 rd_mx0(void)   { return adsp2100.core.mx0.s; }
@@ -458,10 +490,8 @@ static INT32 rd_icntl(void) { return adsp2100.icntl; }
 static INT32 rd_cntr(void)  { return adsp2100.cntr; }
 static INT32 rd_sb(void)    { return adsp2100.core.sb.s; }
 static INT32 rd_px(void)    { return adsp2100.px; }
-#if SUPPORT_2101_EXTENSIONS
-static INT32 rd_rx0(void)	{ if ( adsp2105_rx_callback ) return (*adsp2105_rx_callback)( 0 ); else return 0; }
-static INT32 rd_rx1(void)	{ if ( adsp2105_rx_callback ) return (*adsp2105_rx_callback)( 1 ); else return 0; }
-#endif
+static INT32 rd_rx0(void)	{ if (adsp2100.sport_rx_callback) return (*adsp2100.sport_rx_callback)(0); else return 0; }
+static INT32 rd_rx1(void)	{ if (adsp2100.sport_rx_callback) return (*adsp2100.sport_rx_callback)(1); else return 0; }
 static INT32 rd_stacktop(void)	{ return pc_stack_pop_val(); }
 
 #define READ_REG(grp,reg) ((*rd_reg[grp][reg])())
@@ -482,18 +512,14 @@ static INT32 (*rd_reg[4][16])(void) =
 	},
 	{
 		rd_astat, rd_mstat, rd_sstat, rd_imask, rd_icntl, rd_cntr, rd_sb, rd_px,
-#if SUPPORT_2101_EXTENSIONS
 		rd_rx0, rd_inval, rd_rx1, rd_inval, rd_inval, rd_inval, rd_inval, rd_stacktop
-#else
-		rd_inval, rd_inval, rd_inval, rd_inval, rd_inval, rd_inval, rd_inval, rd_stacktop
-#endif
 	}
 };
 
 
 
 /*===========================================================================
-	Modulus addressing logic
+    Modulus addressing logic
 ===========================================================================*/
 
 INLINE void modify_address(UINT32 ireg, UINT32 mreg)
@@ -511,7 +537,7 @@ INLINE void modify_address(UINT32 ireg, UINT32 mreg)
 
 
 /*===========================================================================
-	Data memory accessors
+    Data memory accessors
 ===========================================================================*/
 
 INLINE void data_write_dag1(UINT32 op, INT32 val)
@@ -598,7 +624,7 @@ INLINE UINT32 data_read_dag2(UINT32 op)
 }
 
 /*===========================================================================
-	Program memory accessors
+    Program memory accessors
 ===========================================================================*/
 
 INLINE void pgm_write_dag2(UINT32 op, INT32 val)
@@ -642,7 +668,7 @@ INLINE UINT32 pgm_read_dag2(UINT32 op)
 
 
 /*===========================================================================
-	ALU register reading
+    ALU register reading
 ===========================================================================*/
 
 #define ALU_GETXREG_UNSIGNED(x) (*(UINT16 *)alu_xregs[x])
@@ -673,7 +699,7 @@ static const void *alu_yregs[4] =
 
 
 /*===========================================================================
-	MAC register reading
+    MAC register reading
 ===========================================================================*/
 
 #define MAC_GETXREG_UNSIGNED(x) (*(UINT16 *)mac_xregs[x])
@@ -704,7 +730,7 @@ static const void *mac_yregs[4] =
 
 
 /*===========================================================================
-	SHIFT register reading
+    SHIFT register reading
 ===========================================================================*/
 
 #define SHIFT_GETXREG_UNSIGNED(x) (*(UINT16 *)shift_xregs[x])
@@ -725,7 +751,7 @@ static const void *shift_xregs[8] =
 
 
 /*===========================================================================
-	ALU operations (result in AR)
+    ALU operations (result in AR)
 ===========================================================================*/
 
 void alu_op_ar(int op)
@@ -734,16 +760,15 @@ void alu_op_ar(int op)
 	INT32 yop = (op >> 11) & 3;
 	INT32 res;
 
-	/*switch ((op >> 13) & 15) */
 	switch (op & (15<<13))  /*JB*/
 	{
 		case 0x00<<13:
-			/* Y				Clear when y = 0 */
+			/* Y                Clear when y = 0 */
 			res = ALU_GETYREG_UNSIGNED(yop);
 			CALC_NZ(res);
 			break;
 		case 0x01<<13:
-			/* Y + 1			PASS 1 when y = 0 */
+			/* Y + 1            PASS 1 when y = 0 */
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop + 1;
 			CALC_NZ(res);
@@ -759,7 +784,7 @@ void alu_op_ar(int op)
 			CALC_NZVC(xop, yop, res);
 			break;
 		case 0x03<<13:
-			/* X + Y			X when y = 0 */
+			/* X + Y            X when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = xop + yop;
@@ -779,7 +804,7 @@ void alu_op_ar(int op)
 			if (yop == 0x0000) SET_C;
 			break;
 		case 0x06<<13:
-			/* X - Y + C - 1	X + C - 1 when y = 0 */
+			/* X - Y + C - 1    X + C - 1 when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = xop - yop + (GET_C >> 3) - 1;
@@ -793,7 +818,7 @@ void alu_op_ar(int op)
 			CALC_NZVC_SUB(xop, yop, res);
 			break;
 		case 0x08<<13:
-			/* Y - 1			PASS -1 when y = 0 */
+			/* Y - 1            PASS -1 when y = 0 */
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - 1;
 			CALC_NZ(res);
@@ -801,14 +826,14 @@ void alu_op_ar(int op)
 			else if (yop == 0x0000) SET_C;
 			break;
 		case 0x09<<13:
-			/* Y - X			-X when y = 0 */
+			/* Y - X            -X when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - xop;
 			CALC_NZVC_SUB(yop, xop, res);
 			break;
 		case 0x0a<<13:
-			/* Y - X + C - 1	-X + C - 1 when y = 0 */
+			/* Y - X + C - 1    -X + C - 1 when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - xop + (GET_C >> 3) - 1;
@@ -864,7 +889,133 @@ void alu_op_ar(int op)
 
 
 /*===========================================================================
-	ALU operations (result in AF)
+    ALU operations (result in AR, constant yop)
+===========================================================================*/
+
+void alu_op_ar_const(int op)
+{
+	INT32 xop = (op >> 8) & 7;
+	INT32 yop = constants[((op >> 5) & 0x07) | ((op >> 8) & 0x18)];
+	INT32 res;
+
+	switch (op & (15<<13))  /*JB*/
+	{
+		case 0x00<<13:
+			/* Y                Clear when y = 0 */
+			res = yop;
+			CALC_NZ(res);
+			break;
+		case 0x01<<13:
+			/* Y + 1            PASS 1 when y = 0 */
+			res = yop + 1;
+			CALC_NZ(res);
+			if (yop == 0x7fff) SET_V;
+			else if (yop == 0xffff) SET_C;
+			break;
+		case 0x02<<13:
+			/* X + Y + C */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop += GET_C >> 3;
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x03<<13:
+			/* X + Y            X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x04<<13:
+			/* NOT Y */
+			res = yop ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x05<<13:
+			/* -Y */
+			res = -yop;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			if (yop == 0x0000) SET_C;
+			break;
+		case 0x06<<13:
+			/* X - Y + C - 1    X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop - yop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x07<<13:
+			/* X - Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop - yop;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x08<<13:
+			/* Y - 1            PASS -1 when y = 0 */
+			res = yop - 1;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			else if (yop == 0x0000) SET_C;
+			break;
+		case 0x09<<13:
+			/* Y - X            -X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = yop - xop;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0a<<13:
+			/* Y - X + C - 1    -X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = yop - xop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0b<<13:
+			/* NOT X */
+			res = ALU_GETXREG_UNSIGNED(xop) ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x0c<<13:
+			/* X AND Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop & yop;
+			CALC_NZ(res);
+			break;
+		case 0x0d<<13:
+			/* X OR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop | yop;
+			CALC_NZ(res);
+			break;
+		case 0x0e<<13:
+			/* X XOR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop ^ yop;
+			CALC_NZ(res);
+			break;
+		case 0x0f<<13:
+			/* ABS X */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = (xop & 0x8000) ? -xop : xop;
+			if (xop == 0) SET_Z;
+			if (xop == 0x8000) SET_N, SET_V;
+			CLR_S;
+			if (xop & 0x8000) SET_S;
+			break;
+		default:
+			res = 0;	/* just to keep the compiler happy */
+			break;
+	}
+
+	/* saturate */
+	if ((adsp2100.mstat & MSTAT_SATURATE) && GET_V) res = GET_C ? -32768 : 32767;
+
+	/* set the final value */
+	adsp2100.core.ar.u = res;
+}
+
+
+
+/*===========================================================================
+    ALU operations (result in AF)
 ===========================================================================*/
 
 void alu_op_af(int op)
@@ -873,16 +1024,15 @@ void alu_op_af(int op)
 	INT32 yop = (op >> 11) & 3;
 	INT32 res;
 
-/*	switch ((op >> 13) & 15) */
 	switch (op & (15<<13))  /*JB*/
 	{
 		case 0x00<<13:
-			/* Y				Clear when y = 0 */
+			/* Y                Clear when y = 0 */
 			res = ALU_GETYREG_UNSIGNED(yop);
 			CALC_NZ(res);
 			break;
 		case 0x01<<13:
-			/* Y + 1			PASS 1 when y = 0 */
+			/* Y + 1            PASS 1 when y = 0 */
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop + 1;
 			CALC_NZ(res);
@@ -898,7 +1048,7 @@ void alu_op_af(int op)
 			CALC_NZVC(xop, yop, res);
 			break;
 		case 0x03<<13:
-			/* X + Y			X when y = 0 */
+			/* X + Y            X when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = xop + yop;
@@ -918,7 +1068,7 @@ void alu_op_af(int op)
 			if (yop == 0x0000) SET_C;
 			break;
 		case 0x06<<13:
-			/* X - Y + C - 1	X + C - 1 when y = 0 */
+			/* X - Y + C - 1    X + C - 1 when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = xop - yop + (GET_C >> 3) - 1;
@@ -932,7 +1082,7 @@ void alu_op_af(int op)
 			CALC_NZVC_SUB(xop, yop, res);
 			break;
 		case 0x08<<13:
-			/* Y - 1			PASS -1 when y = 0 */
+			/* Y - 1            PASS -1 when y = 0 */
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - 1;
 			CALC_NZ(res);
@@ -940,14 +1090,14 @@ void alu_op_af(int op)
 			else if (yop == 0x0000) SET_C;
 			break;
 		case 0x09<<13:
-			/* Y - X			-X when y = 0 */
+			/* Y - X            -X when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - xop;
 			CALC_NZVC_SUB(yop, xop, res);
 			break;
 		case 0x0a<<13:
-			/* Y - X + C - 1	-X + C - 1 when y = 0 */
+			/* Y - X + C - 1    -X + C - 1 when y = 0 */
 			xop = ALU_GETXREG_UNSIGNED(xop);
 			yop = ALU_GETYREG_UNSIGNED(yop);
 			res = yop - xop + (GET_C >> 3) - 1;
@@ -1000,22 +1150,269 @@ void alu_op_af(int op)
 
 
 /*===========================================================================
-	MAC operations (result in MR)
+    ALU operations (result in AF, constant yop)
+===========================================================================*/
+
+void alu_op_af_const(int op)
+{
+	INT32 xop = (op >> 8) & 7;
+	INT32 yop = constants[((op >> 5) & 0x07) | ((op >> 8) & 0x18)];
+	INT32 res;
+
+	switch (op & (15<<13))  /*JB*/
+	{
+		case 0x00<<13:
+			/* Y                Clear when y = 0 */
+			res = yop;
+			CALC_NZ(res);
+			break;
+		case 0x01<<13:
+			/* Y + 1            PASS 1 when y = 0 */
+			res = yop + 1;
+			CALC_NZ(res);
+			if (yop == 0x7fff) SET_V;
+			else if (yop == 0xffff) SET_C;
+			break;
+		case 0x02<<13:
+			/* X + Y + C */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop += GET_C >> 3;
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x03<<13:
+			/* X + Y            X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x04<<13:
+			/* NOT Y */
+			res = yop ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x05<<13:
+			/* -Y */
+			res = -yop;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			if (yop == 0x0000) SET_C;
+			break;
+		case 0x06<<13:
+			/* X - Y + C - 1    X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop - yop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x07<<13:
+			/* X - Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop - yop;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x08<<13:
+			/* Y - 1            PASS -1 when y = 0 */
+			res = yop - 1;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			else if (yop == 0x0000) SET_C;
+			break;
+		case 0x09<<13:
+			/* Y - X            -X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = yop - xop;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0a<<13:
+			/* Y - X + C - 1    -X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = yop - xop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0b<<13:
+			/* NOT X */
+			res = ALU_GETXREG_UNSIGNED(xop) ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x0c<<13:
+			/* X AND Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop & yop;
+			CALC_NZ(res);
+			break;
+		case 0x0d<<13:
+			/* X OR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop | yop;
+			CALC_NZ(res);
+			break;
+		case 0x0e<<13:
+			/* X XOR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = xop ^ yop;
+			CALC_NZ(res);
+			break;
+		case 0x0f<<13:
+			/* ABS X */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = (xop & 0x8000) ? -xop : xop;
+			if (xop == 0) SET_Z;
+			if (xop == 0x8000) SET_N, SET_V;
+			CLR_S;
+			if (xop & 0x8000) SET_S;
+			break;
+		default:
+			res = 0;	/* just to keep the compiler happy */
+			break;
+	}
+
+	/* set the final value */
+	adsp2100.core.af.u = res;
+}
+
+
+
+/*===========================================================================
+    ALU operations (no result)
+===========================================================================*/
+
+void alu_op_none(int op)
+{
+	INT32 xop = (op >> 8) & 7;
+	INT32 yop = (op >> 11) & 3;
+	INT32 res;
+
+	switch (op & (15<<13))  /*JB*/
+	{
+		case 0x00<<13:
+			/* Y                Clear when y = 0 */
+			res = ALU_GETYREG_UNSIGNED(yop);
+			CALC_NZ(res);
+			break;
+		case 0x01<<13:
+			/* Y + 1            PASS 1 when y = 0 */
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = yop + 1;
+			CALC_NZ(res);
+			if (yop == 0x7fff) SET_V;
+			else if (yop == 0xffff) SET_C;
+			break;
+		case 0x02<<13:
+			/* X + Y + C */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			yop += GET_C >> 3;
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x03<<13:
+			/* X + Y            X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop + yop;
+			CALC_NZVC(xop, yop, res);
+			break;
+		case 0x04<<13:
+			/* NOT Y */
+			res = ALU_GETYREG_UNSIGNED(yop) ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x05<<13:
+			/* -Y */
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = -yop;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			if (yop == 0x0000) SET_C;
+			break;
+		case 0x06<<13:
+			/* X - Y + C - 1    X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop - yop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x07<<13:
+			/* X - Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop - yop;
+			CALC_NZVC_SUB(xop, yop, res);
+			break;
+		case 0x08<<13:
+			/* Y - 1            PASS -1 when y = 0 */
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = yop - 1;
+			CALC_NZ(res);
+			if (yop == 0x8000) SET_V;
+			else if (yop == 0x0000) SET_C;
+			break;
+		case 0x09<<13:
+			/* Y - X            -X when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = yop - xop;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0a<<13:
+			/* Y - X + C - 1    -X + C - 1 when y = 0 */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = yop - xop + (GET_C >> 3) - 1;
+			CALC_NZVC_SUB(yop, xop, res);
+			break;
+		case 0x0b<<13:
+			/* NOT X */
+			res = ALU_GETXREG_UNSIGNED(xop) ^ 0xffff;
+			CALC_NZ(res);
+			break;
+		case 0x0c<<13:
+			/* X AND Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop & yop;
+			CALC_NZ(res);
+			break;
+		case 0x0d<<13:
+			/* X OR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop | yop;
+			CALC_NZ(res);
+			break;
+		case 0x0e<<13:
+			/* X XOR Y */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			yop = ALU_GETYREG_UNSIGNED(yop);
+			res = xop ^ yop;
+			CALC_NZ(res);
+			break;
+		case 0x0f<<13:
+			/* ABS X */
+			xop = ALU_GETXREG_UNSIGNED(xop);
+			res = (xop & 0x8000) ? -xop : xop;
+			if (xop == 0) SET_Z;
+			if (xop == 0x8000) SET_N, SET_V;
+			CLR_S;
+			if (xop & 0x8000) SET_S;
+			break;
+	}
+}
+
+
+
+/*===========================================================================
+    MAC operations (result in MR)
 ===========================================================================*/
 
 void mac_op_mr(int op)
 {
-#if SUPPORT_2101_EXTENSIONS
 	INT8 shift = ((adsp2100.mstat & MSTAT_INTEGER) >> 4) ^ 1;
-#else
-	INT8 shift = 1;
-#endif
 	INT32 xop = (op >> 8) & 7;
 	INT32 yop = (op >> 11) & 3;
 	INT32 temp;
 	INT64 res;
 
-/*	switch ((op >> 13) & 15) */
 	switch (op & (15<<13))	/*JB*/
 	{
 		case 0x00<<13:
@@ -1070,7 +1467,7 @@ void mac_op_mr(int op)
 #endif
 			break;
 		case 0x04<<13:
-			/* X * Y (SS)		Clear when y = 0 */
+			/* X * Y (SS)       Clear when y = 0 */
 			xop = MAC_GETXREG_SIGNED(xop);
 			yop = MAC_GETYREG_SIGNED(yop);
 			temp = (xop * yop) << shift;
@@ -1168,22 +1565,164 @@ void mac_op_mr(int op)
 
 
 /*===========================================================================
-	MAC operations (result in MF)
+    MAC operations (result in MR, yop == xop)
+===========================================================================*/
+
+void mac_op_mr_xop(int op)
+{
+	INT8 shift = ((adsp2100.mstat & MSTAT_INTEGER) >> 4) ^ 1;
+	INT32 xop = (op >> 8) & 7;
+	INT32 temp;
+	INT64 res;
+
+	switch (op & (15<<13))	/*JB*/
+	{
+		case 0x00<<13:
+			/* no-op */
+			return;
+		case 0x01<<13:
+			/* X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x02<<13:
+			/* MR + X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x03<<13:
+			/* MR - X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x04<<13:
+			/* X * Y (SS)       Clear when y = 0 */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x05<<13:
+			/* X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x06<<13:
+			/* X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x07<<13:
+			/* X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x08<<13:
+			/* MR + X * Y (SS) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x09<<13:
+			/* MR + X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0a<<13:
+			/* MR + X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0b<<13:
+			/* MR + X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0c<<13:
+			/* MR - X * Y (SS) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0d<<13:
+			/* MR - X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0e<<13:
+			/* MR - X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0f<<13:
+			/* MR - X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		default:
+			res = 0;	/* just to keep the compiler happy */
+			break;
+	}
+
+	/* set the final value */
+	temp = (res >> 31) & 0x1ff;
+	CLR_MV;
+	if (temp != 0x000 && temp != 0x1ff) SET_MV;
+	adsp2100.core.mr.mr = res;
+}
+
+
+
+/*===========================================================================
+    MAC operations (result in MF)
 ===========================================================================*/
 
 void mac_op_mf(int op)
 {
-#if SUPPORT_2101_EXTENSIONS
 	INT8 shift = ((adsp2100.mstat & MSTAT_INTEGER) >> 4) ^ 1;
-#else
-	INT8 shift = 1;
-#endif
 	INT32 xop = (op >> 8) & 7;
 	INT32 yop = (op >> 11) & 3;
 	INT32 temp;
 	INT64 res;
 
-/*	switch ((op >> 13) & 15) */
 	switch (op & (15<<13))	/*JB*/
 	{
 		case 0x00<<13:
@@ -1238,7 +1777,7 @@ void mac_op_mf(int op)
 #endif
 			break;
 		case 0x04<<13:
-			/* X * Y (SS)		Clear when y = 0 */
+			/* X * Y (SS)       Clear when y = 0 */
 			xop = MAC_GETXREG_SIGNED(xop);
 			yop = MAC_GETYREG_SIGNED(yop);
 			temp = (xop * yop) << shift;
@@ -1333,7 +1872,151 @@ void mac_op_mf(int op)
 
 
 /*===========================================================================
-	SHIFT operations (result in SR/SE/SB)
+    MAC operations (result in MF, yop == xop)
+===========================================================================*/
+
+void mac_op_mf_xop(int op)
+{
+	INT8 shift = ((adsp2100.mstat & MSTAT_INTEGER) >> 4) ^ 1;
+	INT32 xop = (op >> 8) & 7;
+	INT32 temp;
+	INT64 res;
+
+	switch (op & (15<<13))	/*JB*/
+	{
+		case 0x00<<13:
+			/* no-op */
+			return;
+		case 0x01<<13:
+			/* X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x02<<13:
+			/* MR + X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x03<<13:
+			/* MR - X * Y (RND) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+#if 0
+			if ((res & 0xffff) == 0x8000) res &= ~((UINT64)0x10000);
+			else res += (res & 0x8000) << 1;
+#else
+			temp &= 0xffff;
+			res += 0x8000;
+			if ( temp == 0x8000 )
+				res &= ~((UINT64)0x10000);
+#endif
+			break;
+		case 0x04<<13:
+			/* X * Y (SS)       Clear when y = 0 */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x05<<13:
+			/* X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x06<<13:
+			/* X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x07<<13:
+			/* X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = (INT64)temp;
+			break;
+		case 0x08<<13:
+			/* MR + X * Y (SS) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x09<<13:
+			/* MR + X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0a<<13:
+			/* MR + X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0b<<13:
+			/* MR + X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr + (INT64)temp;
+			break;
+		case 0x0c<<13:
+			/* MR - X * Y (SS) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0d<<13:
+			/* MR - X * Y (SU) */
+			xop = MAC_GETXREG_SIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0e<<13:
+			/* MR - X * Y (US) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		case 0x0f<<13:
+			/* MR - X * Y (UU) */
+			xop = MAC_GETXREG_UNSIGNED(xop);
+			temp = (xop * xop) << shift;
+			res = adsp2100.core.mr.mr - (INT64)temp;
+			break;
+		default:
+			res = 0;	/* just to keep the compiler happy */
+			break;
+	}
+
+	/* set the final value */
+	adsp2100.core.mf.u = (UINT32)res >> 16;
+}
+
+
+
+/*===========================================================================
+    SHIFT operations (result in SR/SE/SB)
 ===========================================================================*/
 
 void shift_op(int op)
@@ -1342,7 +2025,6 @@ void shift_op(int op)
 	INT32 xop = (op >> 8) & 7;
 	UINT32 res;
 
-/*	switch ((op >> 11) & 15) */
 	switch (op & (15<<11))	/*JB*/
 	{
 		case 0x00<<11:
@@ -1516,7 +2198,7 @@ void shift_op(int op)
 
 
 /*===========================================================================
-	Immediate SHIFT operations (result in SR/SE/SB)
+    Immediate SHIFT operations (result in SR/SE/SB)
 ===========================================================================*/
 
 void shift_op_imm(int op)
@@ -1525,7 +2207,6 @@ void shift_op_imm(int op)
 	INT32 xop = (op >> 8) & 7;
 	UINT32 res;
 
-/*	switch ((op >> 11) & 15) */
 	switch (op & (15<<11))	/*JB*/
 	{
 		case 0x00<<11:
