@@ -20,32 +20,6 @@
 
 
 
-/*###################################################################################################
-**  DEBUGGING
-**#################################################################################################*/
-
-#define DEBUG			0
-
-
-
-/*###################################################################################################
-**  CONSTANTS
-**#################################################################################################*/
-
-
-
-/*###################################################################################################
-**  TYPE DEFINITIONS
-**#################################################################################################*/
-
-
-
-/*###################################################################################################
-**  LOCAL VARIABLES
-**#################################################################################################*/
-
-
-
 /*###################################################################################################
 **  PROTOTYPES
 **#################################################################################################*/
@@ -87,6 +61,7 @@ static void execute_trace(int ref, int params, const char **param);
 static void execute_traceover(int ref, int params, const char **param);
 static void execute_snap(int ref, int params, const char **param);
 static void execute_source(int ref, int params, const char **param);
+static void execute_map(int ref, int params, const char **param);
 static void execute_memdump(int ref, int params, const char **param);
 
 
@@ -175,6 +150,9 @@ void debug_command_init(void)
 
 	debug_console_register_command("source",    CMDFLAG_NONE, 0, 1, 1, execute_source);
 
+	debug_console_register_command("map",		CMDFLAG_NONE, ADDRESS_SPACE_PROGRAM, 1, 1, execute_map);
+	debug_console_register_command("mapd",		CMDFLAG_NONE, ADDRESS_SPACE_DATA, 1, 1, execute_map);
+	debug_console_register_command("mapi",		CMDFLAG_NONE, ADDRESS_SPACE_IO, 1, 1, execute_map);
 	debug_console_register_command("memdump",	CMDFLAG_NONE, 0, 0, 1, execute_memdump);
 }
 
@@ -1736,6 +1714,43 @@ static void execute_source(int ref, int params, const char *param[])
 }
 
 
+/*-------------------------------------------------
+    execute_map - execute the map command
+-------------------------------------------------*/
+
+static void execute_map(int ref, int params, const char *param[])
+{
+	UINT64 address, cpunum = cpu_getactivecpu();
+	const struct debug_cpu_info *info;
+	int spacenum = ref;
+	offs_t taddress;
+
+	/* validate parameters */
+	if (!validate_parameter_number(param[0], &address))
+		return;
+	info = debug_get_cpu_info(cpunum);
+
+	/* do the translation first */
+	taddress = ADDR2BYTE(address, info, spacenum);
+	if (info->translate)
+	{
+		if ((*info->translate)(spacenum, &taddress))
+			debug_console_printf("%08X logical -> %08X physical\n", (UINT32)address, BYTE2ADDR(taddress, info, spacenum));
+		else
+		{
+			debug_console_printf("%08X logical -> unmapped\n", (UINT32)address);
+			return;
+		}
+	}
+	else
+		debug_console_printf("%08X physical\n", BYTE2ADDR(taddress, info, spacenum));
+
+	/* now do the mapping */
+	debug_console_printf("  -> read: %s\n", memory_get_handler_string(0, cpunum, spacenum, taddress));
+	debug_console_printf("  -> write: %s\n", memory_get_handler_string(1, cpunum, spacenum, taddress));
+}
+
+
 /*-------------------------------------------------
     execute_memdump - execute the memdump command
 -------------------------------------------------*/
