@@ -18,14 +18,6 @@
 
 
 
-/*###################################################################################################
-**  DEBUGGING
-**#################################################################################################*/
-
-#define DEBUG			0
-
-
-
 /*###################################################################################################
 **  CONSTANTS
 **#################################################################################################*/
