@@ -527,7 +527,7 @@ INPUT_PORTS_END
 
 /**********************************************************************************/
 
-static gfx_layout charlayout =
+static const gfx_layout charlayout =
 {
 	8,8,
 	RGN_FRAC(1,2),
@@ -538,7 +538,7 @@ static gfx_layout charlayout =
 	16*8
 };
 
-static gfx_layout tilelayout =
+static const gfx_layout tilelayout =
 {
 	16,16,
 	RGN_FRAC(1,2),
@@ -551,7 +551,7 @@ static gfx_layout tilelayout =
 	64*8
 };
 
-static gfx_decode gfxdecodeinfo[] =
+static const gfx_decode gfxdecodeinfo[] =
 {
 	/* REGION_GFX1 is copied to REGION_GFX2 at runtime */
 	{ REGION_GFX2, 0, &charlayout,     0,  32 },	/* Characters 8x8 */
