@@ -1351,7 +1351,7 @@ INPUT_PORTS_END
 /*****************************************************************************/
 
 
-static gfx_layout legionna_charlayout =
+static const gfx_layout legionna_charlayout =
 {
 	8,8,
 	RGN_FRAC(1,4),	/* other half is BK3, decoded in char2layout */
@@ -1362,7 +1362,7 @@ static gfx_layout legionna_charlayout =
 	16*8
 };
 
-static gfx_layout heatbrl_charlayout =
+static const gfx_layout heatbrl_charlayout =
 {
 	8,8,
 	RGN_FRAC(1,2),	/* second half is junk, like legionna we may need a different decode */
@@ -1374,7 +1374,7 @@ static gfx_layout heatbrl_charlayout =
 };
 
 
-static gfx_layout legionna_char2layout =
+static const gfx_layout legionna_char2layout =
 {
 	16,16,
 	256,	/* Can't use RGN_FRAC as (1,16) not supported */
@@ -1389,7 +1389,7 @@ static gfx_layout legionna_char2layout =
 	16*8
 };
 
-static gfx_layout legionna_tilelayout =
+static const gfx_layout legionna_tilelayout =
 {
 	16,16,
 	RGN_FRAC(1,1),
@@ -1402,7 +1402,7 @@ static gfx_layout legionna_tilelayout =
 	128*8
 };
 
-static gfx_layout legionna_spritelayout =
+static const gfx_layout legionna_spritelayout =
 {
 	16,16,
 	RGN_FRAC(1,1),
@@ -1415,7 +1415,7 @@ static gfx_layout legionna_spritelayout =
 	128*8
 };
 
-static gfx_decode legionna_gfxdecodeinfo[] =
+static const gfx_decode legionna_gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &legionna_charlayout,   48*16, 16 },
 	{ REGION_GFX3, 0, &legionna_tilelayout,    0*16, 16 },
@@ -1426,7 +1426,7 @@ static gfx_decode legionna_gfxdecodeinfo[] =
 	{ -1 } /* end of array */
 };
 
-static gfx_decode heatbrl_gfxdecodeinfo[] =
+static const gfx_decode heatbrl_gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &heatbrl_charlayout,    48*16, 16 },
 	{ REGION_GFX3, 0, &legionna_tilelayout,    0*16, 16 },
@@ -1819,7 +1819,7 @@ Notes:
 */
 
 ROM_START( godzilla )
-	ROM_REGION( 0x80000, REGION_CPU1, 0 )	/* 68000 code */
+	ROM_REGION( 0x100000, REGION_CPU1, 0 )	/* 68000 code */
 	ROM_LOAD32_BYTE( "2.025",        0x000000, 0x020000, CRC(be9c6e5a) SHA1(9a7e49ac9cdbcc02b13b3448544cee5fe398ec16) )
 	ROM_LOAD32_BYTE( "1.024",        0x000001, 0x020000, CRC(0d6b663d) SHA1(01e02999cffd2642f7a37e492fe7f83770cddd67) )
 	ROM_LOAD32_BYTE( "4.026",        0x000002, 0x020000, CRC(bb8c0132) SHA1(fa8b049f590be710b3cf82f27deade63656db730) )
@@ -2191,17 +2191,17 @@ static DRIVER_INIT( legionna )
 
 
 
-GAMEX( 1992, legionna, 0,        legionna, legionna, legionna, ROT0, "Tad", "Legionnaire (World)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1992, legionnu, legionna, legionna, legionna, legionna, ROT0, "Tad (Fabtek license)", "Legionnaire (US)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, legionna, 0,        legionna, legionna, legionna, ROT0, "Tad", "Legionnaire (World)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, legionnu, legionna, legionna, legionna, legionna, ROT0, "Tad (Fabtek license)", "Legionnaire (US)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
 
-GAMEX( 1992, heatbrl,  0,        heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (World)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1992, heatbrlo, heatbrl,  heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (World old version)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1992, heatbrlu, heatbrl,  heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (US)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, heatbrl,  0,        heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (World)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, heatbrlo, heatbrl,  heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (World old version)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, heatbrlu, heatbrl,  heatbrl,  heatbrl,  0,        ROT0, "Tad", "Heated Barrel (US)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
 
-GAMEX( 1993, godzilla, 0,        godzilla, godzilla, 0,        ROT0, "Banpresto", "Godzilla", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1993, denjinmk, 0,        godzilla, godzilla, 0,        ROT0, "Banpresto", "Denjin Makai", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1993, sdgndmrb, 0,        sdgndmrb, sdgndmrb, 0,	       ROT0, "Banpresto", "SD Gundam Sangokushi Rainbow Tairiku Senki", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1993, godzilla, 0,        godzilla, godzilla, 0,        ROT0, "Banpresto", "Godzilla", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1993, denjinmk, 0,        godzilla, godzilla, 0,        ROT0, "Banpresto", "Denjin Makai", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1993, sdgndmrb, 0,        sdgndmrb, sdgndmrb, 0,	       ROT0, "Banpresto", "SD Gundam Sangokushi Rainbow Tairiku Senki", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
 
-GAMEX( 1992, cupsoc,   0,        cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Seibu Cup Soccer", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1992, cupsoc2,  cupsoc,   cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Seibu Cup Soccer (set 2)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
-GAMEX( 1992, olysoc92, cupsoc,   cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Olympic Soccer '92", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, cupsoc,   0,        cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Seibu Cup Soccer", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, cupsoc2,  cupsoc,   cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Seibu Cup Soccer (set 2)", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
+GAME( 1992, olysoc92, cupsoc,   cupsoc,  cupsoc,    0,        ROT0, "Seibu", "Olympic Soccer '92", GAME_UNEMULATED_PROTECTION | GAME_NOT_WORKING )
