@@ -233,7 +233,7 @@ INPUT_PORTS_START( mwarr )
 	PORT_SERVICE_NO_TOGGLE( 0x8000, IP_ACTIVE_LOW )
 INPUT_PORTS_END
 
-static gfx_layout mwarr_tile8_layout =
+static const gfx_layout mwarr_tile8_layout =
 {
 	8,8,
 	RGN_FRAC(1,2),
@@ -244,7 +244,7 @@ static gfx_layout mwarr_tile8_layout =
 	8*16
 };
 
-static gfx_layout mwarr_tile16_layout =
+static const gfx_layout mwarr_tile16_layout =
 {
 	16,16,
 	RGN_FRAC(1,2),
@@ -256,7 +256,7 @@ static gfx_layout mwarr_tile16_layout =
 	32*16
 };
 
-static gfx_layout mwarr_6bpp_sprites =
+static const gfx_layout mwarr_6bpp_sprites =
 {
 	16,16,
 	RGN_FRAC(1,6),
@@ -267,7 +267,7 @@ static gfx_layout mwarr_6bpp_sprites =
 	32*8
 };
 
-static gfx_decode gfxdecodeinfo[] =
+static const gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &mwarr_6bpp_sprites,  1024, 16 },
 	{ REGION_GFX2, 0, &mwarr_tile8_layout,	 384,  8 },
@@ -553,4 +553,4 @@ ROM_START( mwarr )
 	ROM_COPY( REGION_USER1, 0x060000, 0x0e0000, 0x020000)
 ROM_END
 
-GAME( 199?, mwarr, 0, mwarr, mwarr, 0, ROT0,  "Elettronica Video-Games S.R.L.", "Mighty Warriors" )
+GAME( 199?, mwarr, 0, mwarr, mwarr, 0, ROT0,  "Elettronica Video-Games S.R.L.", "Mighty Warriors", 0 )
