@@ -1,7 +1,7 @@
 /***************************************************************************
 
  *****************************
- *** NINJA KID II hardware ***		(by Roberto Ventura)
+ *** NINJA KID II hardware ***      (by Roberto Ventura)
  *****************************
 
 Game authors:
@@ -44,8 +44,8 @@ c800-cdff       Color RAM
 d000-d7ff       "FRONT" tile map
 d800-dfff       "BACK" tile map
 e000-efff       Main RAM
-f400-f7ff	??? screen frame ???
-f800-f9ff	CPU Stack
+f400-f7ff   ??? screen frame ???
+f800-f9ff   CPU Stack
 fa00-ffff       Sprite registers (misc RAM)
 
 
@@ -61,11 +61,8 @@ frame.
 Game level maps,additional code and data are available to main
 program via CPU banking at lacations 8000-bf00
 
-The encryption scheme for ROM 6(alternate version) is very simple,
-I was surprised by such a large ROM (64k) for sound.
-The first half contains opcodes only (see machine 1 pin)
-while the second 32k chunk is for immediate data.
-Opcode and Data keep their relative positions as well.
+In two of the sets, the encrypted sound program ROM is replaced with a
+double-sized decrypted version. I don0t know if they are bootlegs or originals.
 
 
 
@@ -82,8 +79,8 @@ c000    "KEYCOIN" button
         ^--------START 2
 
 
-c001	"PAD1"
-c002	"PAD2"
+c001    "PAD1"
+c002    "PAD2"
 
         76543210
           ||||||
@@ -122,12 +119,12 @@ c004    "DIPSW2"
         |^------->>
         ^-------->
 
-c200	Sound command?
-	This byte is written when game plays sound effects...
-	it is set when music or sound effects (both pcm and fm) are triggered;
-	I guess it is read by another CPU,then.
+c200    Sound command?
+    This byte is written when game plays sound effects...
+    it is set when music or sound effects (both pcm and fm) are triggered;
+    I guess it is read by another CPU,then.
 
-c201	Unknown,but used.
+c201    Unknown,but used.
 
 c202    Bank selector (0 to 7)
 
@@ -139,7 +136,7 @@ c203    Sprite 'overdraw'
         when an extra weapon is collected.
         Appears to work like a xor mask,a sprite removes older
         sprite 'bitmap' when present;other memory locations connected to
-	this function may be f400-f7ff...should be investigated more.
+    this function may be f400-f7ff...should be investigated more.
         -mmmh... I believe this is sci-fiction for a non-bitmap game...
 
 C208    Scroll X  0-7
@@ -223,14 +220,15 @@ The first sprite data is located at fa0b,then fa1b and so on.
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
+#include "sound/2203intf.h"
+#include "sound/samples.h"
 
-WRITE_HANDLER( ninjakd2_bgvideoram_w );
-WRITE_HANDLER( ninjakd2_fgvideoram_w );
-WRITE_HANDLER( ninjakd2_sprite_overdraw_w );
-WRITE_HANDLER( ninjakd2_background_enable_w );
-int  ninjakd2_vh_start(void);
-void ninjakd2_vh_stop(void);
-void ninjakd2_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
+WRITE8_HANDLER( ninjakd2_bgvideoram_w );
+WRITE8_HANDLER( ninjakd2_fgvideoram_w );
+WRITE8_HANDLER( ninjakd2_sprite_overdraw_w );
+WRITE8_HANDLER( ninjakd2_background_enable_w );
+VIDEO_START( ninjakd2 );
+VIDEO_UPDATE( ninjakd2 );
 
 extern unsigned char 	*ninjakd2_scrolly_ram;
 extern unsigned char 	*ninjakd2_scrollx_ram;
@@ -243,51 +241,37 @@ extern size_t ninjakd2_foregroundram_size;
 
 static int ninjakd2_bank_latch = 255;
 
+static INT16 *sampledata[8];
+static int samplelen[8];
 
-
-int ninjakd2_init_samples(const struct MachineSound *msound)
+void ninjakd2_init_samples(void)
 {
 	int i,n;
 	unsigned char *source = memory_region(REGION_SOUND1);
-	struct GameSamples *samples;
 	int sample_info [9][2] = { {0x0000,0x0A00},{0x0A00,0x1D00},{0x2700,0x1700},
 	{0x3E00,0x1500},{0x5300,0x0B00},{0x5E00,0x0A00},{0x6800,0x0E00},{0x7600,0x1E00},{0xF000,0x0400} };
 
-	if ((Machine->samples = malloc(sizeof(struct GameSamples) + 9 * sizeof(struct GameSample *))) == NULL)
-		return 1;
-
-	samples = Machine->samples;
-	samples->total = 8;
-
 	for (i=0;i<8;i++)
 	{
-		if ((samples->sample[i] = malloc(sizeof(struct GameSample) + (sample_info[i][1]))) == NULL)
-			return 1;
-
-		samples->sample[i]->length = sample_info[i][1];
-		samples->sample[i]->smpfreq = 16000;	/* 16 kHz */
-		samples->sample[i]->resolution = 8;
+		sampledata[i] = auto_malloc(sample_info[i][1] * sizeof(sampledata[0]));
+		samplelen[i] = sample_info[i][1];
 		for (n=0; n<sample_info[i][1]; n++)
-			samples->sample[i]->data[n] = source[sample_info[i][0]+n] ^ 0x80;
+			sampledata[i][n] = (INT8)(source[sample_info[i][0]+n] ^ 0x80) * 256;
 	}
-
-	/*	The samples are now ready to be used.  They are a 8 bit, 16 kHz samples. 	 */
-
-	return 0;
 }
 
 
-int ninjakd2_interrupt(void)
+INTERRUPT_GEN( ninjakd2_interrupt )
 {
-	return 0x00d7;	/* RST 10h */
+	cpunum_set_input_line_and_vector(0, 0, HOLD_LINE, 0xd7);	/* RST 10h */
 }
 
-READ_HANDLER( ninjakd2_bankselect_r )
+READ8_HANDLER( ninjakd2_bankselect_r )
 {
 	return ninjakd2_bank_latch;
 }
 
-WRITE_HANDLER( ninjakd2_bankselect_w )
+WRITE8_HANDLER( ninjakd2_bankselect_w )
 {
 	unsigned char *RAM = memory_region(REGION_CPU1);
 	int bankaddress;
@@ -297,11 +281,11 @@ WRITE_HANDLER( ninjakd2_bankselect_w )
 		ninjakd2_bank_latch = data;
 
 		bankaddress = 0x10000 + ((data & 0x7) * 0x4000);
-		cpu_setbank(1,&RAM[bankaddress]);	 /* Select 8 banks of 16k */
+		memory_set_bankptr(1,&RAM[bankaddress]);	 /* Select 8 banks of 16k */
 	}
 }
 
-WRITE_HANDLER( ninjakd2_pcm_play_w )
+WRITE8_HANDLER( ninjakd2_pcm_play_w )
 {
 	int i;
 	int sample_no[9] = { 0x00,0x0A,0x27,0x3E,0x53,0x5E,0x68,0x76,0xF0 };
@@ -312,67 +296,69 @@ WRITE_HANDLER( ninjakd2_pcm_play_w )
 	if (i==8)
 		sample_stop(0);
 	else
-		sample_start(0,i,0);
+		sample_start_raw(0,sampledata[i],samplelen[i],16000,0);
 }
 
-static MEMORY_READ_START( readmem )
-	{ 0x0000, 0x7fff, MRA_ROM },
-	{ 0x8000, 0xbfff, MRA_BANK1 },
-	{ 0xc000, 0xc000, input_port_2_r },
-	{ 0xc001, 0xc001, input_port_0_r },
-	{ 0xc002, 0xc002, input_port_1_r },
-	{ 0xc003, 0xc003, input_port_3_r },
-	{ 0xc004, 0xc004, input_port_4_r },
-	{ 0xc200, 0xc200, MRA_RAM },
-	{ 0xc201, 0xc201, MRA_RAM },		/* unknown but used */
-	{ 0xc202, 0xc202, ninjakd2_bankselect_r },
-	{ 0xc203, 0xc203, MRA_RAM },
-	{ 0xc208, 0xc209, MRA_RAM },
-	{ 0xc20a, 0xc20b, MRA_RAM },
-	{ 0xc20c, 0xc20c, MRA_RAM },
-	{ 0xc800, 0xffff, MRA_RAM },
-MEMORY_END
+static ADDRESS_MAP_START( readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0x8000, 0xbfff) AM_READ(MRA8_BANK1)
+	AM_RANGE(0xc000, 0xc000) AM_READ(input_port_2_r)
+	AM_RANGE(0xc001, 0xc001) AM_READ(input_port_0_r)
+	AM_RANGE(0xc002, 0xc002) AM_READ(input_port_1_r)
+	AM_RANGE(0xc003, 0xc003) AM_READ(input_port_3_r)
+	AM_RANGE(0xc004, 0xc004) AM_READ(input_port_4_r)
+	AM_RANGE(0xc200, 0xc200) AM_READ(MRA8_RAM)
+	AM_RANGE(0xc201, 0xc201) AM_READ(MRA8_RAM)		/* unknown but used */
+	AM_RANGE(0xc202, 0xc202) AM_READ(ninjakd2_bankselect_r)
+	AM_RANGE(0xc203, 0xc203) AM_READ(MRA8_RAM)
+	AM_RANGE(0xc208, 0xc209) AM_READ(MRA8_RAM)
+	AM_RANGE(0xc20a, 0xc20b) AM_READ(MRA8_RAM)
+	AM_RANGE(0xc20c, 0xc20c) AM_READ(MRA8_RAM)
+	AM_RANGE(0xc800, 0xffff) AM_READ(MRA8_RAM)
+ADDRESS_MAP_END
 
 
-static MEMORY_WRITE_START( writemem )
-	{ 0x0000, 0xbfff, MWA_ROM },
-	{ 0xc200, 0xc200, soundlatch_w },
-	{ 0xc201, 0xc201, MWA_RAM },		/* unknown but used */
-	{ 0xc202, 0xc202, ninjakd2_bankselect_w },
-	{ 0xc203, 0xc203, ninjakd2_sprite_overdraw_w, &ninjakd2_spoverdraw_ram },
-	{ 0xc208, 0xc209, MWA_RAM, &ninjakd2_scrollx_ram },
-	{ 0xc20a, 0xc20b, MWA_RAM, &ninjakd2_scrolly_ram },
-	{ 0xc20c, 0xc20c, ninjakd2_background_enable_w, &ninjakd2_bgenable_ram },
-	{ 0xc800, 0xcdff, paletteram_RRRRGGGGBBBBxxxx_swap_w, &paletteram },
-	{ 0xd000, 0xd7ff, ninjakd2_fgvideoram_w, &ninjakd2_foreground_videoram, &ninjakd2_foregroundram_size },
-	{ 0xd800, 0xdfff, ninjakd2_bgvideoram_w, &ninjakd2_background_videoram, &ninjakd2_backgroundram_size },
-	{ 0xe000, 0xf9ff, MWA_RAM },
-	{ 0xfa00, 0xffff, MWA_RAM, &spriteram, &spriteram_size },
-MEMORY_END
+static ADDRESS_MAP_START( writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0xbfff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0xc200, 0xc200) AM_WRITE(soundlatch_w)
+	AM_RANGE(0xc201, 0xc201) AM_WRITE(MWA8_RAM)		/* unknown but used */
+	AM_RANGE(0xc202, 0xc202) AM_WRITE(ninjakd2_bankselect_w)
+	AM_RANGE(0xc203, 0xc203) AM_WRITE(ninjakd2_sprite_overdraw_w) AM_BASE(&ninjakd2_spoverdraw_ram)
+	AM_RANGE(0xc208, 0xc209) AM_WRITE(MWA8_RAM) AM_BASE(&ninjakd2_scrollx_ram)
+	AM_RANGE(0xc20a, 0xc20b) AM_WRITE(MWA8_RAM) AM_BASE(&ninjakd2_scrolly_ram)
+	AM_RANGE(0xc20c, 0xc20c) AM_WRITE(ninjakd2_background_enable_w) AM_BASE(&ninjakd2_bgenable_ram)
+	AM_RANGE(0xc800, 0xcdff) AM_WRITE(paletteram_RRRRGGGGBBBBxxxx_swap_w) AM_BASE(&paletteram)
+	AM_RANGE(0xd000, 0xd7ff) AM_WRITE(ninjakd2_fgvideoram_w) AM_BASE(&ninjakd2_foreground_videoram) AM_SIZE(&ninjakd2_foregroundram_size)
+	AM_RANGE(0xd800, 0xdfff) AM_WRITE(ninjakd2_bgvideoram_w) AM_BASE(&ninjakd2_background_videoram) AM_SIZE(&ninjakd2_backgroundram_size)
+	AM_RANGE(0xe000, 0xf9ff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0xfa00, 0xffff) AM_WRITE(MWA8_RAM) AM_BASE(&spriteram) AM_SIZE(&spriteram_size)
+ADDRESS_MAP_END
 
 
-static MEMORY_READ_START( snd_readmem )
-	{ 0x0000, 0xbfff, MRA_ROM },
-	{ 0xc000, 0xc7ff, MRA_RAM },
-	{ 0xe000, 0xe000, soundlatch_r },
-	{ 0xefee, 0xefee, MRA_NOP },
-MEMORY_END
+static ADDRESS_MAP_START( snd_readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0x8000, 0xbfff) AM_READ(MRA8_ROM)
+	AM_RANGE(0xc000, 0xc7ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xe000, 0xe000) AM_READ(soundlatch_r)
+	AM_RANGE(0xefee, 0xefee) AM_READ(MRA8_NOP)
+ADDRESS_MAP_END
 
 
-static MEMORY_WRITE_START( snd_writemem )
-	{ 0x0000, 0xbfff, MWA_ROM },
-	{ 0xc000, 0xc7ff, MWA_RAM },
-	{ 0xf000, 0xf000, ninjakd2_pcm_play_w },	/* PCM SAMPLE OFFSET*256 */
-	{ 0xeff5, 0xeff6, MWA_NOP },			/* SAMPLE FREQUENCY ??? */
-	{ 0xefee, 0xefee, MWA_NOP },			/* CHIP COMMAND ?? */
-MEMORY_END
+static ADDRESS_MAP_START( snd_writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0xbfff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0xc000, 0xc7ff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0xf000, 0xf000) AM_WRITE(ninjakd2_pcm_play_w)	/* PCM SAMPLE OFFSET*256 */
+	AM_RANGE(0xeff5, 0xeff6) AM_WRITE(MWA8_NOP)			/* SAMPLE FREQUENCY ??? */
+	AM_RANGE(0xefee, 0xefee) AM_WRITE(MWA8_NOP)			/* CHIP COMMAND ?? */
+ADDRESS_MAP_END
 
-static PORT_WRITE_START( snd_writeport )
-	{ 0x0000, 0x0000, YM2203_control_port_0_w },
-	{ 0x0001, 0x0001, YM2203_write_port_0_w },
-	{ 0x0080, 0x0080, YM2203_control_port_1_w },
-	{ 0x0081, 0x0081, YM2203_write_port_1_w },
-PORT_END
+static ADDRESS_MAP_START( snd_writeport, ADDRESS_SPACE_IO, 8 )
+	ADDRESS_MAP_FLAGS( AMEF_ABITS(8) )
+	AM_RANGE(0x00, 0x00) AM_WRITE(YM2203_control_port_0_w)
+	AM_RANGE(0x01, 0x01) AM_WRITE(YM2203_write_port_0_w)
+	AM_RANGE(0x80, 0x80) AM_WRITE(YM2203_control_port_1_w)
+	AM_RANGE(0x81, 0x81) AM_WRITE(YM2203_write_port_1_w)
+ADDRESS_MAP_END
 
 
 
@@ -388,12 +374,12 @@ INPUT_PORTS_START( ninjakd2 )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
     PORT_START    /* player 2 controls */
-    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_COCKTAIL )
-    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_COCKTAIL )
-    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_COCKTAIL )
-    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_COCKTAIL )
-    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_COCKTAIL )
-    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_COCKTAIL )
+    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_COCKTAIL
+    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_COCKTAIL
+    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_COCKTAIL
+    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_COCKTAIL
+    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_COCKTAIL
+    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_COCKTAIL
     PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
@@ -415,22 +401,22 @@ INPUT_PORTS_START( ninjakd2 )
     PORT_DIPSETTING(    0x04, "20000 50000" )
     PORT_DIPSETTING(    0x06, "30000 50000" )
     PORT_DIPSETTING(    0x02, "50000 100000" )
-    PORT_DIPSETTING(    0x00, "None" )
-    PORT_DIPNAME( 0x08, 0x08, "Allow Continue" )
+    PORT_DIPSETTING(    0x00, DEF_STR( None ) )
+    PORT_DIPNAME( 0x08, 0x08, DEF_STR( Allow_Continue ) )
     PORT_DIPSETTING(    0x00, DEF_STR( No ) )
     PORT_DIPSETTING(    0x08, DEF_STR( Yes )  )
     PORT_DIPNAME( 0x10, 0x00, DEF_STR( Demo_Sounds ) )
     PORT_DIPSETTING(    0x10, DEF_STR( Off ) )
     PORT_DIPSETTING(    0x00, DEF_STR( On )  )
     PORT_DIPNAME( 0x20, 0x20, DEF_STR( Difficulty ) )
-    PORT_DIPSETTING(    0x20, "Normal" )
-    PORT_DIPSETTING(    0x00, "Hard" )
+    PORT_DIPSETTING(    0x20, DEF_STR( Normal ) )
+    PORT_DIPSETTING(    0x00, DEF_STR( Hard ) )
     PORT_DIPNAME( 0x40, 0x40, DEF_STR( Lives ) )
     PORT_DIPSETTING(    0x40, "3" )
     PORT_DIPSETTING(    0x00, "4" )
-    PORT_DIPNAME( 0x80, 0x00, "Language" )
-    PORT_DIPSETTING(    0x00, "English" )
-    PORT_DIPSETTING(    0x80, "Japanese" )
+    PORT_DIPNAME( 0x80, 0x00, DEF_STR( Language ) )
+    PORT_DIPSETTING(    0x00, DEF_STR( English ) )
+    PORT_DIPSETTING(    0x80, DEF_STR( Japanese ) )
 
     PORT_START  /* dsw1 */
 	PORT_SERVICE( 0x01, IP_ACTIVE_LOW )
@@ -458,7 +444,7 @@ INPUT_PORTS_END
 
 
 
-static struct GfxLayout charlayout =
+static const gfx_layout charlayout =
 {
 	8,8,     /* 8*8 characters */
 	1024,    /* 1024 characters */
@@ -469,7 +455,7 @@ static struct GfxLayout charlayout =
 	8*16
 };
 
-static struct GfxLayout spritelayout =
+static const gfx_layout spritelayout =
 {
 	16,16,   /* 16*16 characters */
 	1024,    /* 1024 sprites */
@@ -482,7 +468,7 @@ static struct GfxLayout spritelayout =
 	8*64
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static const gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &spritelayout,  0*16, 16 },
 	{ REGION_GFX2, 0, &spritelayout, 16*16, 16 },
@@ -493,301 +479,247 @@ static struct GfxDecodeInfo gfxdecodeinfo[] =
 static struct Samplesinterface samples_interface =
 {
 	1,	/* 1 channel */
-	25	/* volume */
-};
-
-static struct CustomSound_interface custom_interface =
-{
-	ninjakd2_init_samples,
-	0,
-	0
+	NULL,
+	ninjakd2_init_samples
 };
 
 /* handler called by the 2203 emulator when the internal timers cause an IRQ */
 static void irqhandler(int irq)
 {
-	cpu_set_irq_line(1,0,irq ? ASSERT_LINE : CLEAR_LINE);
+	cpunum_set_input_line(1,0,irq ? ASSERT_LINE : CLEAR_LINE);
 }
 
 static struct YM2203interface ym2203_interface =
 {
-	2, 	 /* 2 chips */
-	1500000, /* 12000000/8 MHz */
-	{ YM2203_VOL(25,25), YM2203_VOL(25,25) },
-	{ 0 },
-	{ 0 },
-	{ 0 },
-	{ 0 },
-	{ irqhandler }
+	0,0,0,0,irqhandler
 };
 
 
-static const struct MachineDriver machine_driver_ninjakd2 =
-{
-	{
-		{
-			CPU_Z80,
-			6000000,		/* 12000000/2 ??? */
-			readmem,writemem,0,0,	/* very sensitive to these settings */
-			ninjakd2_interrupt,1
-		}
-	},
-	60, 10000,			/* frames per second, vblank duration */
-	10,				/* single CPU, no need for interleaving */
-	0,
-	32*8, 32*8,
-	{ 0*8, 32*8-1, 4*8, 28*8-1 },
-	gfxdecodeinfo,
-	768, 0,
-	0,
+static MACHINE_DRIVER_START( ninjakd2 )
 
-	VIDEO_TYPE_RASTER,
-	0,
-	ninjakd2_vh_start,
-	ninjakd2_vh_stop,
-	ninjakd2_vh_screenrefresh,
+	/* basic machine hardware */
+	MDRV_CPU_ADD(Z80, 12000000/2)		/* 12000000/2 ??? */
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)	/* very sensitive to these settings */
+	MDRV_CPU_VBLANK_INT(ninjakd2_interrupt,1)
+
+	MDRV_CPU_ADD(Z80, 5000000)
+	/* audio CPU */		/* 5mhz crystal ??? */
+	MDRV_CPU_PROGRAM_MAP(snd_readmem,snd_writemem)
+	MDRV_CPU_IO_MAP(0,snd_writeport)
+
+	MDRV_FRAMES_PER_SECOND(60)
+	MDRV_VBLANK_DURATION(10000)
+
+	/* video hardware */
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(32*8, 32*8)
+	MDRV_VISIBLE_AREA(0*8, 32*8-1, 4*8, 28*8-1)
+	MDRV_GFXDECODE(gfxdecodeinfo)
+	MDRV_PALETTE_LENGTH(768)
+
+	MDRV_VIDEO_START(ninjakd2)
+	MDRV_VIDEO_UPDATE(ninjakd2)
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_YM2203,
-			&ym2203_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
 
-static const struct MachineDriver machine_driver_ninjak2a =
-{
-	{
-		{
-			CPU_Z80,
-			6000000,		/* 12000000/2 ??? */
-			readmem,writemem,0,0,	/* very sensitive to these settings */
-			ninjakd2_interrupt,1
-		},
-		{
-			CPU_Z80 | CPU_AUDIO_CPU,
-			4000000,		/* 12000000/3 ??? */
-			snd_readmem,snd_writemem,0,snd_writeport,
-			ignore_interrupt,0,
-		}
-	},
-	60, 10000,	/* frames per second, vblank duration */
-	10,
-	0,
-	32*8, 32*8,
-	{ 0*8, 32*8-1, 4*8, 28*8-1 },
-	gfxdecodeinfo,
-	768, 0,
-	0,
+	MDRV_SOUND_ADD(YM2203, 1500000)
+	MDRV_SOUND_CONFIG(ym2203_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.30)
 
-	VIDEO_TYPE_RASTER,
-	0,
-	ninjakd2_vh_start,
-	ninjakd2_vh_stop,
-	ninjakd2_vh_screenrefresh,
+	MDRV_SOUND_ADD(YM2203, 1500000)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.30)
 
-	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_YM2203,
-			&ym2203_interface
-		},
-		{
-			SOUND_SAMPLES,
-			&samples_interface
-		},
-		{
-			SOUND_CUSTOM,	/* actually initializes the samples */
-			&custom_interface
-		}
-	}
-};
+	MDRV_SOUND_ADD(SAMPLES, 0)
+	MDRV_SOUND_CONFIG(samples_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.30)
+MACHINE_DRIVER_END
 
 
 
 ROM_START( ninjakd2 )
 	ROM_REGION( 0x30000, REGION_CPU1, 0 )
-	ROM_LOAD( "nk2_01.rom",   0x00000, 0x8000, 0x3cdbb906 )
-	ROM_LOAD( "nk2_02.rom",   0x10000, 0x8000, 0xb5ce9a1a )
-	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, 0xad275654 )
-	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, 0xe7692a77 )
-	ROM_LOAD( "nk2_05.rom",   0x28000, 0x8000, 0x5dac9426 )
+	ROM_LOAD( "nk2_01.rom",   0x00000, 0x8000, CRC(3cdbb906) SHA1(f48f82528b5fc581ee3b1ccd0ef9cdecc7249bb3) )
+	ROM_LOAD( "nk2_02.rom",   0x10000, 0x8000, CRC(b5ce9a1a) SHA1(295a7e1d41e1a8ee45f1250086a0c9314837eded) )
+	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, CRC(ad275654) SHA1(7d29a17132adb19aeee9b98be5b76bd6e91f308e) )
+	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, CRC(e7692a77) SHA1(84beb8b02c564bffa9cc00313214e8f109bd40f9) )
+	ROM_LOAD( "nk2_05.rom",   0x28000, 0x8000, CRC(5dac9426) SHA1(0916cddbbe1e93c32b96fe28e145d34b2a892e80) )
 
 	ROM_REGION( 0x10000, REGION_CPU2, 0 )
-	ROM_LOAD( "nk2_06.rom",   0x0000, 0x10000, 0xd3a18a79 )  /* sound z80 code encrypted */
+	ROM_LOAD( "nk2_06.rom",   0x0000, 0x10000, CRC(d3a18a79) SHA1(e4df713f89d8a8b43ef831b14864c50ec9b53f0b) )  /* sound z80 code encrypted */
 
 	ROM_REGION( 0x20000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, 0x41a714b3 )	/* background tiles */
+	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, CRC(41a714b3) SHA1(b05f48d71a9837914c12c13e0b479c8a6dc8c25e) )	/* background tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, 0xc913c4ab )
+	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, CRC(c913c4ab) SHA1(f822c5621b3e32c1a284f6367bdcace81c1c74b3) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x20000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, 0x1b79c50a )	/* sprites tiles */
+	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, CRC(1b79c50a) SHA1(8954bc51cb9fbbe16b09381f35c84ccc56a803f3) )	/* sprites tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, 0x0be5cd13 )
+	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, CRC(0be5cd13) SHA1(8f94a8fef6668aaf13329715fee81302dbd6c685) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x08000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, 0xdb5657a9 )	/* foreground tiles */
+	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, CRC(db5657a9) SHA1(abbb033edb9a5a0c66ee5981d1e4df1ab334a82d) )	/* foreground tiles */
 	ROM_CONTINUE(             0x04000, 0x02000)
 	ROM_CONTINUE(             0x02000, 0x02000)
 	ROM_CONTINUE(             0x06000, 0x02000)
 
 	ROM_REGION( 0x10000, REGION_SOUND1, 0 )
-	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, 0xc1d2d170 )	/* raw pcm samples */
+	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, CRC(c1d2d170) SHA1(0f325815086fde90fd85360d3660042b0b68ba96) )	/* raw pcm samples */
 ROM_END
 
 ROM_START( ninjak2a )
 	ROM_REGION( 0x30000, REGION_CPU1, 0 )
-	ROM_LOAD( "nk2_01.bin",   0x00000, 0x8000, 0xe6adca65 )
-	ROM_LOAD( "nk2_02.bin",   0x10000, 0x8000, 0xd9284bd1 )
-	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, 0xad275654 )
-	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, 0xe7692a77 )
-	ROM_LOAD( "nk2_05.bin",   0x28000, 0x8000, 0x960725fb )
+	ROM_LOAD( "nk2_01.bin",   0x00000, 0x8000, CRC(e6adca65) SHA1(33d483dde0853f37455cde32b461f4e919601b4b) )
+	ROM_LOAD( "nk2_02.bin",   0x10000, 0x8000, CRC(d9284bd1) SHA1(e790fb1a718a1f7997931f2f390fe053655f231d) )
+	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, CRC(ad275654) SHA1(7d29a17132adb19aeee9b98be5b76bd6e91f308e) )
+	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, CRC(e7692a77) SHA1(84beb8b02c564bffa9cc00313214e8f109bd40f9) )
+	ROM_LOAD( "nk2_05.bin",   0x28000, 0x8000, CRC(960725fb) SHA1(160c8bfaf089cbeeef2023f12379793079bff93b) )
 
 	ROM_REGION( 2*0x10000, REGION_CPU2, 0 )	/* 64k for code + 64k for decrypted opcodes */
-	ROM_LOAD( "nk2_06.bin",   0x10000, 0x8000, 0x7bfe6c9e )	/* decrypted opcodes */
+	ROM_LOAD( "nk2_06.bin",   0x10000, 0x8000, CRC(7bfe6c9e) SHA1(aef8cbeb0024939bf65f77113a5cf777f6613722) )	/* decrypted opcodes */
 	ROM_CONTINUE(             0x00000, 0x8000 )				/* decrypted data */
 
 	ROM_REGION( 0x20000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, 0x41a714b3 )	/* background tiles */
+	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, CRC(41a714b3) SHA1(b05f48d71a9837914c12c13e0b479c8a6dc8c25e) )	/* background tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, 0xc913c4ab )
+	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, CRC(c913c4ab) SHA1(f822c5621b3e32c1a284f6367bdcace81c1c74b3) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x20000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, 0x1b79c50a )	/* sprites tiles */
+	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, CRC(1b79c50a) SHA1(8954bc51cb9fbbe16b09381f35c84ccc56a803f3) )	/* sprites tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, 0x0be5cd13 )
+	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, CRC(0be5cd13) SHA1(8f94a8fef6668aaf13329715fee81302dbd6c685) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x08000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, 0xdb5657a9 )	/* foreground tiles */
+	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, CRC(db5657a9) SHA1(abbb033edb9a5a0c66ee5981d1e4df1ab334a82d) )	/* foreground tiles */
 	ROM_CONTINUE(             0x04000, 0x02000)
 	ROM_CONTINUE(             0x02000, 0x02000)
 	ROM_CONTINUE(             0x06000, 0x02000)
 
 	ROM_REGION( 0x10000, REGION_SOUND1, 0 )
-	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, 0xc1d2d170 )	/* raw pcm samples */
+	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, CRC(c1d2d170) SHA1(0f325815086fde90fd85360d3660042b0b68ba96) )	/* raw pcm samples */
 ROM_END
 
 ROM_START( ninjak2b )
 	ROM_REGION( 0x30000, REGION_CPU1, 0 )
-	ROM_LOAD( "1.3s",         0x00000, 0x8000, 0xcb4f4624 )
-	ROM_LOAD( "2.3q",         0x10000, 0x8000, 0x0ad0c100 )
-	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, 0xad275654 )
-	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, 0xe7692a77 )
-	ROM_LOAD( "nk2_05.rom",   0x28000, 0x8000, 0x5dac9426 )
+	ROM_LOAD( "1.3s",         0x00000, 0x8000, CRC(cb4f4624) SHA1(4fc66641adc0a2c0eca332f27c5777df62fa507b) )
+	ROM_LOAD( "2.3q",         0x10000, 0x8000, CRC(0ad0c100) SHA1(c5bbc107ba07bd6950bb4d7377e827c084b8229b) )
+	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, CRC(ad275654) SHA1(7d29a17132adb19aeee9b98be5b76bd6e91f308e) )
+	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, CRC(e7692a77) SHA1(84beb8b02c564bffa9cc00313214e8f109bd40f9) )
+	ROM_LOAD( "nk2_05.rom",   0x28000, 0x8000, CRC(5dac9426) SHA1(0916cddbbe1e93c32b96fe28e145d34b2a892e80) )
 
 	ROM_REGION( 2*0x10000, REGION_CPU2, 0 )	/* 64k for code + 64k for decrypted opcodes */
-	ROM_LOAD( "nk2_06.bin",   0x10000, 0x8000, 0x7bfe6c9e )	/* decrypted opcodes */
+	ROM_LOAD( "nk2_06.bin",   0x10000, 0x8000, CRC(7bfe6c9e) SHA1(aef8cbeb0024939bf65f77113a5cf777f6613722) )	/* decrypted opcodes */
 	ROM_CONTINUE(             0x00000, 0x8000 )				/* decrypted data */
 
 	ROM_REGION( 0x20000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, 0x41a714b3 )	/* background tiles */
+	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, CRC(41a714b3) SHA1(b05f48d71a9837914c12c13e0b479c8a6dc8c25e) )	/* background tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, 0xc913c4ab )
+	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, CRC(c913c4ab) SHA1(f822c5621b3e32c1a284f6367bdcace81c1c74b3) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x20000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, 0x1b79c50a )	/* sprites tiles */
+	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, CRC(1b79c50a) SHA1(8954bc51cb9fbbe16b09381f35c84ccc56a803f3) )	/* sprites tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, 0x0be5cd13 )
+	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, CRC(0be5cd13) SHA1(8f94a8fef6668aaf13329715fee81302dbd6c685) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x08000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, 0xdb5657a9 )	/* foreground tiles */
+	ROM_LOAD( "nk2_12.rom",   0x00000, 0x02000, CRC(db5657a9) SHA1(abbb033edb9a5a0c66ee5981d1e4df1ab334a82d) )	/* foreground tiles */
 	ROM_CONTINUE(             0x04000, 0x02000)
 	ROM_CONTINUE(             0x02000, 0x02000)
 	ROM_CONTINUE(             0x06000, 0x02000)
 
 	ROM_REGION( 0x10000, REGION_SOUND1, 0 )
-	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, 0xc1d2d170 )	/* raw pcm samples */
+	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, CRC(c1d2d170) SHA1(0f325815086fde90fd85360d3660042b0b68ba96) )	/* raw pcm samples */
 ROM_END
 
 ROM_START( rdaction )
 	ROM_REGION( 0x30000, REGION_CPU1, 0 )
-	ROM_LOAD( "1.3u",  	      0x00000, 0x8000, 0x5c475611 )
-	ROM_LOAD( "2.3s",         0x10000, 0x8000, 0xa1e23bd2 )
-	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, 0xad275654 )
-	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, 0xe7692a77 )
-	ROM_LOAD( "nk2_05.bin",   0x28000, 0x8000, 0x960725fb )
+	ROM_LOAD( "1.3u",  	      0x00000, 0x8000, CRC(5c475611) SHA1(2da88a95b5d68b259c8ae48af1438a82a1d601c1) )
+	ROM_LOAD( "2.3s",         0x10000, 0x8000, CRC(a1e23bd2) SHA1(c3b6574dc9fa66b4f41c37754a0d20a865f8bc28) )
+	ROM_LOAD( "nk2_03.rom",   0x18000, 0x8000, CRC(ad275654) SHA1(7d29a17132adb19aeee9b98be5b76bd6e91f308e) )
+	ROM_LOAD( "nk2_04.rom",   0x20000, 0x8000, CRC(e7692a77) SHA1(84beb8b02c564bffa9cc00313214e8f109bd40f9) )
+	ROM_LOAD( "nk2_05.bin",   0x28000, 0x8000, CRC(960725fb) SHA1(160c8bfaf089cbeeef2023f12379793079bff93b) )
 
-	ROM_REGION( 2*0x10000, REGION_CPU2, 0 )	/* 64k for code + 64k for decrypted opcodes */
-	ROM_LOAD( "nk2_06.bin",   0x10000, 0x8000, 0x7bfe6c9e )	/* decrypted opcodes */
-	ROM_CONTINUE(             0x00000, 0x8000 )				/* decrypted data */
+	ROM_REGION( 0x10000, REGION_CPU2, 0 )	/* 64k for code + 64k for decrypted opcodes */
+	ROM_LOAD( "nk2_06.rom",   0x0000, 0x10000, CRC(d3a18a79) SHA1(e4df713f89d8a8b43ef831b14864c50ec9b53f0b) )  /* sound z80 code encrypted */
 
 	ROM_REGION( 0x20000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, 0x41a714b3 )	/* background tiles */
+	ROM_LOAD( "nk2_11.rom",   0x00000, 0x4000, CRC(41a714b3) SHA1(b05f48d71a9837914c12c13e0b479c8a6dc8c25e) )	/* background tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, 0xc913c4ab )
+	ROM_LOAD( "nk2_10.rom",   0x08000, 0x4000, CRC(c913c4ab) SHA1(f822c5621b3e32c1a284f6367bdcace81c1c74b3) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x20000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, 0x1b79c50a )	/* sprites tiles */
+	ROM_LOAD( "nk2_08.rom",   0x00000, 0x4000, CRC(1b79c50a) SHA1(8954bc51cb9fbbe16b09381f35c84ccc56a803f3) )	/* sprites tiles */
 	ROM_CONTINUE(             0x10000, 0x4000)
 	ROM_CONTINUE(             0x04000, 0x4000)
 	ROM_CONTINUE(             0x14000, 0x4000)
-	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, 0x0be5cd13 )
+	ROM_LOAD( "nk2_07.rom",   0x08000, 0x4000, CRC(0be5cd13) SHA1(8f94a8fef6668aaf13329715fee81302dbd6c685) )
 	ROM_CONTINUE(             0x18000, 0x4000)
 	ROM_CONTINUE(             0x0c000, 0x4000)
 	ROM_CONTINUE(             0x1c000, 0x4000)
 
 	ROM_REGION( 0x08000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "12.5n",        0x00000, 0x02000, 0x0936b365 )	/* foreground tiles */
+	ROM_LOAD( "12.5n",        0x00000, 0x02000, CRC(0936b365) SHA1(3705f42b76ab474357e77c1a9b8e3755c7ab2c0c) )	/* foreground tiles */
 	ROM_CONTINUE(             0x04000, 0x02000)
 	ROM_CONTINUE(             0x02000, 0x02000)
 	ROM_CONTINUE(             0x06000, 0x02000)
 
 	ROM_REGION( 0x10000, REGION_SOUND1, 0 )
-	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, 0xc1d2d170 )	/* raw pcm samples */
+	ROM_LOAD( "nk2_09.rom",   0x0000, 0x10000, CRC(c1d2d170) SHA1(0f325815086fde90fd85360d3660042b0b68ba96) )	/* raw pcm samples */
 ROM_END
 
 
 
-void init_ninjak2a(void)
-{
-	unsigned char *rom = memory_region(REGION_CPU2);
-	int diff = memory_region_length(REGION_CPU2) / 2;
+void mc8123_decrypt_ninjakid2(void);
 
-	memory_set_opcode_base(1,rom+diff);
+static DRIVER_INIT( ninjakd2 )
+{
+	mc8123_decrypt_ninjakid2();
+}
+
+static DRIVER_INIT( bootleg )
+{
+	memory_set_decrypted_region(1, 0x0000, 0x7fff, memory_region(REGION_CPU2) + 0x10000);
 }
 
 
 
-GAMEX(1987, ninjakd2, 0,        ninjakd2, ninjakd2, 0,        ROT0, "UPL", "Ninja Kid II (set 1)", GAME_NO_SOUND )	/* sound program is encrypted */
-GAME( 1987, ninjak2a, ninjakd2, ninjak2a, ninjakd2, ninjak2a, ROT0, "UPL", "Ninja Kid II (set 2)" )
-GAME( 1987, ninjak2b, ninjakd2, ninjak2a, ninjakd2, ninjak2a, ROT0, "UPL", "Ninja Kid II (set 3)" )
-GAME( 1987, rdaction, ninjakd2, ninjak2a, ninjakd2, ninjak2a, ROT0, "UPL (World Games license)", "Rad Action" )
+GAME( 1987, ninjakd2, 0,        ninjakd2, ninjakd2, ninjakd2, ROT0, "UPL", "Ninja-Kid II (set 1)", 0 )
+GAME( 1987, ninjak2a, ninjakd2, ninjakd2, ninjakd2, bootleg,  ROT0, "UPL", "Ninja-Kid II (set 2)", 0 )
+GAME( 1987, ninjak2b, ninjakd2, ninjakd2, ninjakd2, bootleg,  ROT0, "UPL", "Ninja-Kid II (set 3)", 0 )
+GAME( 1987, rdaction, ninjakd2, ninjakd2, ninjakd2, ninjakd2, ROT0, "UPL (World Games license)", "Rad Action", 0 )
