@@ -2,11 +2,13 @@
 
  Title                          Copyright            PCB - ID
  ----------------------------------------------------------------------------
- Multi 5 / New Multi Game 5     (c) 1998 Yun Sung    YS-1301
+ Multi 5 / New Multi Game 5     (c) 1998 Yun Sung    YS-1300
+ Multi 5 / New Multi Game 5                          YS-1301 undumped alt. version ?
  Search Eye                     (c) 1999 Yun Sung    YS-1905
  Puzzle Club (set 1)            (c) 2000 Yun Sung    YS-2113
  Puzzle Club (set 2)            (c) 2000 Yun Sung    YS-2111
  Wonder Stick                   (c) ???? Yun Sung    YS-2320
+ Garogun Seroyang               (c) 2000 Yun Sung    YS-2111
 
  driver by Pierpaolo Prazzoli
 
@@ -16,29 +18,29 @@
 */
 
 #include "driver.h"
+#include "state.h"
 #include "vidhrdw/generic.h"
 #include "sound/okim6295.h"
 #include "sound/3812intf.h"
 
 static tilemap *fg_tilemap,*bg_tilemap;
-UINT16 *nmg5_bitmap;
-static UINT16 *fgvideoram,*bgvideoram,*scroll_ram,*sprite_pri;
-static UINT16 gfx_bank = 0;
-static UINT16 input_data;
+static UINT16 *nmg5_bitmap;
+static UINT16 *fg_videoram,*bg_videoram,*scroll_ram;
+static UINT8 prot_val, input_data, priority_reg, gfx_bank;
 
-static WRITE16_HANDLER( fgvideoram_w )
+static WRITE16_HANDLER( fg_videoram_w )
 {
-	int oldword = fgvideoram[offset];
-	COMBINE_DATA(&fgvideoram[offset]);
-	if (oldword != fgvideoram[offset])
+	int oldword = fg_videoram[offset];
+	COMBINE_DATA(&fg_videoram[offset]);
+	if (oldword != fg_videoram[offset])
 		tilemap_mark_tile_dirty(fg_tilemap,offset);
 }
 
-static WRITE16_HANDLER( bgvideoram_w )
+static WRITE16_HANDLER( bg_videoram_w )
 {
-	int oldword = bgvideoram[offset];
-	COMBINE_DATA(&bgvideoram[offset]);
-	if (oldword != bgvideoram[offset])
+	int oldword = bg_videoram[offset];
+	COMBINE_DATA(&bg_videoram[offset]);
+	if (oldword != bg_videoram[offset])
 		tilemap_mark_tile_dirty(bg_tilemap,offset);
 }
 
@@ -53,12 +55,7 @@ static WRITE16_HANDLER( nmg5_soundlatch_w )
 
 static READ16_HANDLER( prot_r )
 {
-	return 0x10 | input_data;
-}
-
-static READ16_HANDLER( wondstck_prot_r )
-{
-	return input_data;
+	return prot_val | input_data;
 }
 
 static WRITE16_HANDLER( prot_w )
@@ -75,6 +72,14 @@ static WRITE16_HANDLER( gfx_bank_w )
 	}
 }
 
+static WRITE16_HANDLER( priority_reg_w )
+{
+	priority_reg = data & 7;
+
+	if(priority_reg == 4 || priority_reg == 5 || priority_reg == 6)
+		ui_popup("unknown priority_reg value = %d\n",priority_reg);
+}
+
 static WRITE8_HANDLER( oki_banking_w )
 {
 	OKIM6295_set_bank_base(0, (data & 1) ? 0x40000 : 0 );
@@ -98,11 +103,11 @@ static ADDRESS_MAP_START( nmg5_map, ADDRESS_SPACE_PROGRAM, 16 )
 	AM_RANGE(0x180008, 0x180009) AM_READ(input_port_0_word_r)
 	AM_RANGE(0x18000a, 0x18000b) AM_READ(input_port_1_word_r)
 	AM_RANGE(0x18000c, 0x18000d) AM_READ(input_port_2_word_r)
-	AM_RANGE(0x18000e, 0x18000f) AM_WRITE(MWA16_RAM) AM_BASE(&sprite_pri)
+	AM_RANGE(0x18000e, 0x18000f) AM_WRITE(priority_reg_w)
 	AM_RANGE(0x300002, 0x300009) AM_WRITE(MWA16_RAM) AM_BASE(&scroll_ram)
 	AM_RANGE(0x30000a, 0x30000f) AM_WRITENOP
-	AM_RANGE(0x320000, 0x321fff) AM_RAM AM_WRITE(bgvideoram_w) AM_BASE(&bgvideoram)
-	AM_RANGE(0x322000, 0x323fff) AM_RAM AM_WRITE(fgvideoram_w) AM_BASE(&fgvideoram)
+	AM_RANGE(0x320000, 0x321fff) AM_RAM AM_WRITE(bg_videoram_w) AM_BASE(&bg_videoram)
+	AM_RANGE(0x322000, 0x323fff) AM_RAM AM_WRITE(fg_videoram_w) AM_BASE(&fg_videoram)
 	AM_RANGE(0x800000, 0x80ffff) AM_RAM AM_BASE(&nmg5_bitmap)
 ADDRESS_MAP_END
 
@@ -118,10 +123,10 @@ static ADDRESS_MAP_START( pclubys_map, ADDRESS_SPACE_PROGRAM, 16 )
 	AM_RANGE(0x480008, 0x480009) AM_READ(input_port_0_word_r)
 	AM_RANGE(0x48000a, 0x48000b) AM_READ(input_port_1_word_r)
 	AM_RANGE(0x48000c, 0x48000d) AM_READ(input_port_2_word_r)
-	AM_RANGE(0x48000e, 0x48000f) AM_WRITE(MWA16_RAM) AM_BASE(&sprite_pri)
+	AM_RANGE(0x48000e, 0x48000f) AM_WRITE(priority_reg_w)
 	AM_RANGE(0x500002, 0x500009) AM_WRITE(MWA16_RAM) AM_BASE(&scroll_ram)
-	AM_RANGE(0x520000, 0x521fff) AM_RAM AM_WRITE(bgvideoram_w) AM_BASE(&bgvideoram)
-	AM_RANGE(0x522000, 0x523fff) AM_RAM AM_WRITE(fgvideoram_w) AM_BASE(&fgvideoram)
+	AM_RANGE(0x520000, 0x521fff) AM_RAM AM_WRITE(bg_videoram_w) AM_BASE(&bg_videoram)
+	AM_RANGE(0x522000, 0x523fff) AM_RAM AM_WRITE(fg_videoram_w) AM_BASE(&fg_videoram)
 	AM_RANGE(0x800000, 0x80ffff) AM_RAM AM_BASE(&nmg5_bitmap)
 ADDRESS_MAP_END
 
@@ -235,7 +240,7 @@ INPUT_PORTS_START( searchey )
 	PORT_DIPSETTING(      0x0001, "Fast" )
 	PORT_DIPSETTING(      0x0002, "Slow" )
 	PORT_DIPSETTING(      0x0003, "Slowest" )
-	PORT_DIPNAME( 0x000c, 0x000c, "Number of Helps" )
+	PORT_DIPNAME( 0x000c, 0x0000, "Number of Helps" )
 	PORT_DIPSETTING(      0x000c, "1" )
 	PORT_DIPSETTING(      0x0008, "2" )
 	PORT_DIPSETTING(      0x0004, "3" )
@@ -284,7 +289,7 @@ INPUT_PORTS_START( searchey )
 	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(1) PORT_8WAY
 	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
 	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(1)
-	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(1)
+	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(1) /* not used */
 	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START1 )
 	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_JOYSTICK_UP )    PORT_PLAYER(2) PORT_8WAY
 	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN )  PORT_PLAYER(2) PORT_8WAY
@@ -292,7 +297,7 @@ INPUT_PORTS_START( searchey )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(2) PORT_8WAY
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
 	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(2)
-	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(2)
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(2) /* not used */
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_START2 )
 INPUT_PORTS_END
 
@@ -437,24 +442,93 @@ INPUT_PORTS_START( wondstck )
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_START2 )
 INPUT_PORTS_END
 
+INPUT_PORTS_START( garogun )
+	PORT_START
+	PORT_DIPNAME( 0x0003, 0x0003, "Number of Helps" )
+	PORT_DIPSETTING(      0x0000, "1" )
+	PORT_DIPSETTING(      0x0001, "2" )
+	PORT_DIPSETTING(      0x0002, "3" )
+	PORT_DIPSETTING(      0x0003, "4" )
+	PORT_DIPNAME( 0x0004, 0x0004, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0004, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x0008, 0x0008, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0008, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x0010, 0x0010, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0010, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x0020, 0x0020, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0020, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x00c0, 0x00c0, DEF_STR( Coinage ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( 3C_1C ) )
+	PORT_DIPSETTING(      0x0040, DEF_STR( 2C_1C ) )
+	PORT_DIPSETTING(      0x00c0, DEF_STR( 1C_1C ) )
+	PORT_DIPSETTING(      0x0080, DEF_STR( 1C_2C ) )
+	PORT_SERVICE( 0x0100, IP_ACTIVE_LOW )
+	PORT_DIPNAME( 0x0200, 0x0000, DEF_STR( Demo_Sounds ) )
+	PORT_DIPSETTING(      0x0200, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x0400, 0x0400, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0400, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x0800, 0x0800, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x0800, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x1000, 0x1000, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x1000, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x2000, 0x2000, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x2000, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x4000, 0x4000, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x4000, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+	PORT_DIPNAME( 0x8000, 0x8000, DEF_STR( Unknown ) )
+	PORT_DIPSETTING(      0x8000, DEF_STR( Off ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
+
+	PORT_START	/* Coins */
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW,  IPT_COIN1 )
+	PORT_BIT( 0xfffe, IP_ACTIVE_LOW,  IPT_UNUSED )
+
+	PORT_START
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP )    PORT_PLAYER(1) PORT_8WAY
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN )  PORT_PLAYER(1) PORT_8WAY
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT )  PORT_PLAYER(1) PORT_8WAY
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(1) PORT_8WAY
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
+	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(1)
+	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(1) /* not used */
+	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START1 )
+	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_JOYSTICK_UP )    PORT_PLAYER(2) PORT_8WAY
+	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN )  PORT_PLAYER(2) PORT_8WAY
+	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT )  PORT_PLAYER(2) PORT_8WAY
+	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(2) PORT_8WAY
+	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(2)
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(2) /* not used */
+	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_START2 )
+INPUT_PORTS_END
+
 
 INLINE void get_tile_info(int tile_index,UINT16 *vram,int color)
 {
 	SET_TILE_INFO(0,vram[tile_index] | (gfx_bank << 16),color,0)
 }
 
-static void fg_get_tile_info(int tile_index) { get_tile_info(tile_index,fgvideoram, 0); }
-static void bg_get_tile_info(int tile_index) { get_tile_info(tile_index,bgvideoram, 1); }
+static void fg_get_tile_info(int tile_index) { get_tile_info(tile_index,fg_videoram, 0); }
+static void bg_get_tile_info(int tile_index) { get_tile_info(tile_index,bg_videoram, 1); }
 
 VIDEO_START( nmg5 )
 {
-	bg_tilemap = tilemap_create(bg_get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,64);
+	bg_tilemap = tilemap_create(bg_get_tile_info,tilemap_scan_rows,TILEMAP_OPAQUE,     8,8,64,64);
 	fg_tilemap = tilemap_create(fg_get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,64);
 
 	if (!bg_tilemap || !fg_tilemap)
 		return 1;
 
-	tilemap_set_transparent_pen(bg_tilemap,0);
 	tilemap_set_transparent_pen(fg_tilemap,0);
 
 	return 0;
@@ -471,7 +545,7 @@ static void draw_sprites(mame_bitmap *bitmap,const rectangle *cliprect)
 		sx = spriteram16[offs + 2];
 		sy = spriteram16[offs + 0];
 		code = spriteram16[offs + 1];
-		color = (spriteram16[offs + 2] >> 9)&0xf;
+		color = (spriteram16[offs + 2] >> 9) & 0xf;
 		height = 1 << ((spriteram16[offs + 0] & 0x0600) >> 9);
 		flipx = spriteram16[offs + 0] & 0x2000;
 		flipy = spriteram16[offs + 0] & 0x4000;
@@ -532,26 +606,38 @@ VIDEO_UPDATE( nmg5 )
 	tilemap_set_scrolly(fg_tilemap,0,scroll_ram[1]+9);
 	tilemap_set_scrollx(fg_tilemap,0,scroll_ram[0]-1);
 
-	fillbitmap(bitmap,Machine->pens[0x100],cliprect);
-
-	if(*sprite_pri == 0)
-		draw_sprites(bitmap,cliprect);
-
 	tilemap_draw(bitmap,cliprect,bg_tilemap,0,0);
 
-	if(*sprite_pri == 1)
+	if(priority_reg == 0)
+	{
 		draw_sprites(bitmap,cliprect);
-
-	tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
-
-	if(*sprite_pri == 3)
+		tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+		draw_bitmap(bitmap);
+	}
+	else if(priority_reg == 1)
+	{
+		draw_bitmap(bitmap);
 		draw_sprites(bitmap,cliprect);
-
-	draw_bitmap(bitmap);
-
-	if(*sprite_pri == 7)
+		tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+	}
+	else if(priority_reg == 2)
+	{
 		draw_sprites(bitmap,cliprect);
-
+		draw_bitmap(bitmap);
+		tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+	}
+	else if(priority_reg == 3)
+	{
+		tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+		draw_sprites(bitmap,cliprect);
+		draw_bitmap(bitmap);
+	}
+	else if(priority_reg == 7)
+	{
+		tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+		draw_bitmap(bitmap);
+		draw_sprites(bitmap,cliprect);
+	}
 }
 
 
@@ -613,6 +699,12 @@ static struct YM3812interface ym3812_intf =
 	soundirq	/* IRQ Line */
 };
 
+static MACHINE_INIT( nmg5 )
+{
+	/* some games don't set the priority register so it should be hard-coded to a normal layout */
+	priority_reg = 7;
+}
+
 static MACHINE_DRIVER_START( nmg5 )
 
 	/* basic machine hardware */
@@ -628,6 +720,8 @@ static MACHINE_DRIVER_START( nmg5 )
 	MDRV_FRAMES_PER_SECOND(60)
 	MDRV_VBLANK_DURATION(DEFAULT_60HZ_VBLANK_DURATION)
 
+	MDRV_MACHINE_INIT(nmg5)
+
 	/* video hardware */
 	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
 	MDRV_SCREEN_SIZE(320, 256)
@@ -650,8 +744,19 @@ static MACHINE_DRIVER_START( nmg5 )
 	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
 MACHINE_DRIVER_END
 
-static MACHINE_DRIVER_START( pclubys )
+static MACHINE_DRIVER_START( garogun )
+	/* basic machine hardware */
+	MDRV_IMPORT_FROM(nmg5)
 
+	MDRV_CPU_MODIFY("main")
+	MDRV_CPU_PROGRAM_MAP(pclubys_map,0)
+
+	MDRV_CPU_MODIFY("sound")
+	MDRV_CPU_PROGRAM_MAP(pclubys_sound_map,0)
+MACHINE_DRIVER_END
+
+
+static MACHINE_DRIVER_START( pclubys )
 	/* basic machine hardware */
 	MDRV_IMPORT_FROM(nmg5)
 
@@ -840,23 +945,64 @@ ROM_START( wondstck )
 	ROM_LOAD( "u137.bin", 0x000000, 0x40000, CRC(294b6cbd) SHA1(1498a3298c53d62f56f9c85c82035d09a5bb8b4a) )
 ROM_END
 
-DRIVER_INIT( nmg5 )
+ROM_START( garogun )
+	ROM_REGION( 0x100000, REGION_CPU1, 0 )		/* 68000 Code */
+	ROM_LOAD16_BYTE( "p1.u7", 0x000000, 0x80000, CRC(9b5627f8) SHA1(d336d4f34de7fdf5ba16bc76223e701369d24a5e) )
+	ROM_LOAD16_BYTE( "p2.u2", 0x000001, 0x80000, CRC(1d2ff271) SHA1(6b875be42f945b5793ba41ff20e23dacf8eb6a9a) )
+
+	ROM_REGION( 0x10000, REGION_CPU2, 0 )		/* Z80 Code */
+	ROM_LOAD( "rom.u128", 0x00000, 0x10000,  CRC(117b31ce) SHA1(1681aea60111274599c86b7050d46ea497878f9e) )
+
+	ROM_REGION( 0x400000, REGION_GFX1, ROMREGION_DISPOSE )	/* 8x8x8 */
+	ROM_LOAD( "8.u63",  0x000000, 0x80000, CRC(2d152d32) SHA1(7dd3b0bb9db8cec8ff8a75099375fbad51ee5676) )
+	ROM_LOAD( "11.u68", 0x080000, 0x80000, CRC(60ec7f67) SHA1(9c6c3f5a5be244fe5ac24da3642fd666def11120) )
+	ROM_LOAD( "9.u73",  0x100000, 0x80000, CRC(a4b16319) SHA1(8f7976f58ecbccd728cf1a01d0fcd1cef6b90d47) )
+	ROM_LOAD( "13.u79", 0x180000, 0x80000, CRC(2dc14fb6) SHA1(d6a01e8bb0ce2f94c1562d8302f62fbbb86e5fa0) )
+	ROM_LOAD( "6.u64",  0x200000, 0x80000, CRC(a0fc7547) SHA1(91249226b9491085d15216c11e00f39b03f9128a) )
+	ROM_LOAD( "10.u69", 0x280000, 0x80000, CRC(e5dc36c3) SHA1(370b5f93d2f425fe59a519dafce9cb859bd7b609) )
+	ROM_LOAD( "7.u74",  0x300000, 0x80000, CRC(a0574f8d) SHA1(3016dd5ee2c78eb2e16b396cedcdc69151312a06) )
+	ROM_LOAD( "12.u80", 0x380000, 0x80000, CRC(94d66169) SHA1(4a84d46caa7da98ac376965d6e1ebe1d26fda542) )
+
+	ROM_REGION( 0x280000, REGION_GFX2, ROMREGION_DISPOSE )	/* 16x16x5 */
+	ROM_LOAD( "4.u83", 0x000000, 0x40000, CRC(3d1d46ff) SHA1(713beb8cb5b105900d59380f760d759e94f4b0b2) )
+	ROM_LOAD( "5.u82", 0x080000, 0x40000, CRC(2a7b2fb5) SHA1(f860047d78f625592605f425e9e066c3e595be62) )
+	ROM_LOAD( "3.u105",0x100000, 0x40000, CRC(cd20e39c) SHA1(beb129a44223cc542906f96e5bb17aabfe4c4c49) )
+	ROM_LOAD( "2.u96", 0x180000, 0x40000, CRC(4df3b502) SHA1(638138e09d69390c899f48ae59dcd116c1b338c7) )
+	ROM_LOAD( "1.u97", 0x200000, 0x40000, CRC(591b3efe) SHA1(ea7d2f2802effa6895e02f50cc9f7c189a720ef5) )
+
+	ROM_REGION( 0x80000, REGION_SOUND1, 0 )	/* Samples */
+	ROM_LOAD( "s.u137", 0x00000, 0x80000, CRC(3eadc21a) SHA1(b1c131c3f59adbc370696b277f8f04681212761d) )
+ROM_END
+
+static void set_save_states(void)
 {
-	/* hardcoded sprites priority */
-	*sprite_pri = 7;
+	state_save_register_UINT8("nmg5", 0, "Gfx Bank", &gfx_bank, 1);
+	state_save_register_UINT8("nmg5", 0, "Priority Reg", &priority_reg, 1);
+	state_save_register_UINT8("nmg5", 0, "Input Data", &input_data, 1);
+	state_save_register_item_array("nmg5", 0, nmg5_bitmap);
 }
 
-DRIVER_INIT( wondstck )
+DRIVER_INIT( prot_val_10 )
 {
-	/* hardcoded sprites priority */
-	*sprite_pri = 7;
-
-	memory_install_read16_handler(0, ADDRESS_SPACE_PROGRAM, 0x180004, 0x180005, 0, 0, wondstck_prot_r );
+	set_save_states();
+	prot_val = 0x10;
 }
 
+DRIVER_INIT( prot_val_00 )
+{
+	set_save_states();
+	prot_val = 0x00;
+}
 
-GAME( 1998, nmg5,     0,       nmg5,    nmg5,      nmg5,     ROT0, "Yun Sung", "Multi 5 / New Multi Game 5", 0 )
-GAME( 1999, searchey, 0,       nmg5,    searchey,  0,        ROT0, "Yun Sung", "Search Eye", 0 )
-GAME( 2000, pclubys,  0,       pclubys, pclubys,   0,        ROT0, "Yun Sung", "Puzzle Club (Yun Sung - set 1)", 0 )
-GAME( 2000, pclubysa, pclubys, pclubys, pclubys,   0,        ROT0, "Yun Sung", "Puzzle Club (Yun Sung - set 2)", 0 )
-GAME( ????, wondstck, 0,       nmg5,    wondstck,  wondstck, ROT0, "Yun Sung", "Wonder Stick", 0 )
+DRIVER_INIT( prot_val_40 )
+{
+	set_save_states();
+	prot_val = 0x40;
+}
+
+GAME( 1998, nmg5,     0,       nmg5,    nmg5,      prot_val_10, ROT0, "Yun Sung", "Multi 5 / New Multi Game 5", GAME_SUPPORTS_SAVE )
+GAME( 1999, searchey, 0,       nmg5,    searchey,  prot_val_10, ROT0, "Yun Sung", "Search Eye", GAME_SUPPORTS_SAVE )
+GAME( 2000, pclubys,  0,       pclubys, pclubys,   prot_val_10, ROT0, "Yun Sung", "Puzzle Club (Yun Sung - set 1)", GAME_SUPPORTS_SAVE )
+GAME( 2000, pclubysa, pclubys, pclubys, pclubys,   prot_val_10, ROT0, "Yun Sung", "Puzzle Club (Yun Sung - set 2)", GAME_SUPPORTS_SAVE )
+GAME( 2000, garogun,  0,       garogun, garogun,   prot_val_40, ROT0, "Yun Sung", "Garogun Seroyang (Korea)", GAME_SUPPORTS_SAVE )
+GAME( ????, wondstck, 0,       nmg5,    wondstck,  prot_val_00, ROT0, "Yun Sung", "Wonder Stick", GAME_SUPPORTS_SAVE )
