@@ -226,8 +226,8 @@ READ32_HANDLER(K001604_reg_r)
 
 VIDEO_START( nwktr )
 {
-	video_start_voodoo_2x4mb();
-	voodoo_reset();
+	if (voodoo_start(0, VOODOO_1, 2, 4, 0))
+		return 1;
 
 	return K001604_vh_start();
 }
@@ -237,7 +237,7 @@ VIDEO_UPDATE( nwktr )
 {
 	fillbitmap(bitmap, Machine->remapped_colortable[0], cliprect);
 
-	video_update_voodoo(screen, bitmap, cliprect);
+	voodoo_update(0, bitmap, cliprect);
 
 	K001604_tile_update();
 	K001604_tile_draw(bitmap, cliprect);
@@ -267,7 +267,7 @@ static READ32_HANDLER( sysreg_r )
 		}
 		if (!(mem_mask & 0x000000ff))
 		{
-			r |= 0xff;
+			r |= 0xf7;
 		}
 	}
 	else if (offset == 1)
@@ -399,9 +399,7 @@ static ADDRESS_MAP_START( sharc_map, ADDRESS_SPACE_DATA, 32 )
 	AM_RANGE(0x0400000, 0x041ffff) AM_READWRITE(cgboard_dsp_shared_r_sharc, cgboard_dsp_shared_w_sharc)
 	AM_RANGE(0x0500000, 0x05fffff) AM_READWRITE(dsp_dataram_r, dsp_dataram_w)
 	AM_RANGE(0x1400000, 0x14fffff) AM_RAM
-	AM_RANGE(0x2400000, 0x24fffff) AM_READWRITE(voodoo_regs_r, voodoo_regs_w)
-	AM_RANGE(0x2500000, 0x25fffff) AM_READWRITE(voodoo_framebuf_r, voodoo_framebuf_w)
-	AM_RANGE(0x2600000, 0x27fffff) AM_WRITE(voodoo_textureram_w)
+	AM_RANGE(0x2400000, 0x27fffff) AM_READWRITE(voodoo_0_r, voodoo_0_w)
 	AM_RANGE(0x3400000, 0x34000ff) AM_READWRITE(cgboard_dsp_comm_r_sharc, cgboard_dsp_comm_w_sharc)
 	AM_RANGE(0x3500000, 0x35000ff) AM_READWRITE(pci_3dfx_r, pci_3dfx_w)
 ADDRESS_MAP_END
