@@ -1,15 +1,13 @@
 /*##########################################################################
 
-	atarigen.h
+    atarigen.h
 
-	General functions for Atari raster games.
+    General functions for Atari raster games.
 
 ##########################################################################*/
 
 #include "driver.h"
-#include "vidhrdw/ataripf.h"
 #include "vidhrdw/atarimo.h"
-#include "vidhrdw/atarian.h"
 #include "vidhrdw/atarirle.h"
 
 #ifndef __MACHINE_ATARIGEN__
@@ -17,7 +15,7 @@
 
 
 /*##########################################################################
-	CONSTANTS
+    CONSTANTS
 ##########################################################################*/
 
 #define ATARI_CLOCK_14MHz	14318180
@@ -28,7 +26,7 @@
 
 
 /*##########################################################################
-	TYPES & STRUCTURES
+    TYPES & STRUCTURES
 ##########################################################################*/
 
 typedef void (*atarigen_int_callback)(void);
@@ -54,53 +52,69 @@ struct atarivc_state_desc
 
 
 /*##########################################################################
-	GLOBALS
+    GLOBALS
 ##########################################################################*/
 
-extern int 				atarigen_scanline_int_state;
-extern int 				atarigen_sound_int_state;
-extern int 				atarigen_video_int_state;
+extern UINT8			atarigen_scanline_int_state;
+extern UINT8			atarigen_sound_int_state;
+extern UINT8			atarigen_video_int_state;
 
-extern const data16_t *	atarigen_eeprom_default;
-extern data16_t *		atarigen_eeprom;
+extern const UINT16 *	atarigen_eeprom_default;
+extern UINT16 *		atarigen_eeprom;
 extern size_t 			atarigen_eeprom_size;
 
-extern int 				atarigen_cpu_to_sound_ready;
-extern int 				atarigen_sound_to_cpu_ready;
+extern UINT8			atarigen_cpu_to_sound_ready;
+extern UINT8			atarigen_sound_to_cpu_ready;
 
-extern data16_t *		atarivc_data;
-extern data16_t *		atarivc_eof_data;
+extern UINT16 *		atarigen_playfield;
+extern UINT16 *		atarigen_playfield2;
+extern UINT16 *		atarigen_playfield_upper;
+extern UINT16 *		atarigen_alpha;
+extern UINT16 *		atarigen_alpha2;
+extern UINT16 *		atarigen_xscroll;
+extern UINT16 *		atarigen_yscroll;
+
+extern UINT32 *		atarigen_playfield32;
+extern UINT32 *		atarigen_alpha32;
+
+extern tilemap *		atarigen_playfield_tilemap;
+extern tilemap *		atarigen_playfield2_tilemap;
+extern tilemap *		atarigen_alpha_tilemap;
+extern tilemap *		atarigen_alpha2_tilemap;
+
+extern UINT16 *		atarivc_data;
+extern UINT16 *		atarivc_eof_data;
 extern struct atarivc_state_desc atarivc_state;
 
 
 
 /*##########################################################################
-	FUNCTION PROTOTYPES
+    FUNCTION PROTOTYPES
 ##########################################################################*/
 
 /*---------------------------------------------------------------
-	INTERRUPT HANDLING
+    INTERRUPT HANDLING
 ---------------------------------------------------------------*/
 
 void atarigen_interrupt_reset(atarigen_int_callback update_int);
 void atarigen_update_interrupts(void);
 
 void atarigen_scanline_int_set(int scanline);
-int atarigen_scanline_int_gen(void);
+INTERRUPT_GEN( atarigen_scanline_int_gen );
 WRITE16_HANDLER( atarigen_scanline_int_ack_w );
 WRITE32_HANDLER( atarigen_scanline_int_ack32_w );
 
-int atarigen_sound_int_gen(void);
+INTERRUPT_GEN( atarigen_sound_int_gen );
 WRITE16_HANDLER( atarigen_sound_int_ack_w );
 WRITE32_HANDLER( atarigen_sound_int_ack32_w );
 
-int atarigen_video_int_gen(void);
+INTERRUPT_GEN( atarigen_video_int_gen );
 WRITE16_HANDLER( atarigen_video_int_ack_w );
 WRITE32_HANDLER( atarigen_video_int_ack32_w );
 
 
 /*---------------------------------------------------------------
-	EEPROM HANDLING
+    EEPROM HANDLING
 ---------------------------------------------------------------*/
 
 void atarigen_eeprom_reset(void);
@@ -114,12 +128,12 @@ WRITE32_HANDLER( atarigen_eeprom_enable32_w );
 WRITE32_HANDLER( atarigen_eeprom32_w );
 READ32_HANDLER( atarigen_eeprom_upper32_r );
 
-void atarigen_nvram_handler(void *file,int read_or_write);
+NVRAM_HANDLER( atarigen );
 void atarigen_hisave(void);
 
 
 /*---------------------------------------------------------------
-	SLAPSTIC HANDLING
+    SLAPSTIC HANDLING
 ---------------------------------------------------------------*/
 
 void atarigen_slapstic_init(int cpunum, int base, int chipnum);
@@ -130,14 +144,14 @@ READ16_HANDLER( atarigen_slapstic_r );
 
 
 /*---------------------------------------------------------------
-	SOUND I/O
+    SOUND I/O
 ---------------------------------------------------------------*/
 
 void atarigen_sound_io_reset(int cpu_num);
 
-int atarigen_6502_irq_gen(void);
-READ_HANDLER( atarigen_6502_irq_ack_r );
-WRITE_HANDLER( atarigen_6502_irq_ack_w );
+INTERRUPT_GEN( atarigen_6502_irq_gen );
+READ8_HANDLER( atarigen_6502_irq_ack_r );
+WRITE8_HANDLER( atarigen_6502_irq_ack_w );
 
 void atarigen_ym2151_irq_gen(int irq);
 
@@ -151,15 +165,14 @@ READ32_HANDLER( atarigen_sound_upper32_r );
 
 void atarigen_sound_reset(void);
 WRITE16_HANDLER( atarigen_sound_reset_w );
-WRITE_HANDLER( atarigen_6502_sound_w );
-READ_HANDLER( atarigen_6502_sound_r );
+WRITE8_HANDLER( atarigen_6502_sound_w );
+READ8_HANDLER( atarigen_6502_sound_r );
 
 
 /*---------------------------------------------------------------
-	SOUND HELPERS
+    SOUND HELPERS
 ---------------------------------------------------------------*/
 
-void atarigen_init_6502_speedup(int cpunum, int compare_pc1, int compare_pc2);
 void atarigen_set_ym2151_vol(int volume);
 void atarigen_set_ym2413_vol(int volume);
 void atarigen_set_pokey_vol(int volume);
@@ -168,11 +181,11 @@ void atarigen_set_oki6295_vol(int volume);
 
 
 /*---------------------------------------------------------------
-	VIDEO CONTROLLER
+    VIDEO CONTROLLER
 ---------------------------------------------------------------*/
 
-void atarivc_reset(data16_t *eof_data);
-void atarivc_update(const data16_t *data);
+void atarivc_reset(UINT16 *eof_data, int playfields);
+void atarivc_update(const UINT16 *data);
 
 WRITE16_HANDLER( atarivc_w );
 READ16_HANDLER( atarivc_r );
@@ -185,7 +198,27 @@ INLINE void atarivc_update_pf_xscrolls(void)
 
 
 /*---------------------------------------------------------------
-	VIDEO HELPERS
+    PLAYFIELD/ALPHA MAP HELPERS
+---------------------------------------------------------------*/
+
+WRITE16_HANDLER( atarigen_alpha_w );
+WRITE32_HANDLER( atarigen_alpha32_w );
+WRITE16_HANDLER( atarigen_alpha2_w );
+void atarigen_set_playfield_latch(int data);
+void atarigen_set_playfield2_latch(int data);
+WRITE16_HANDLER( atarigen_playfield_w );
+WRITE32_HANDLER( atarigen_playfield32_w );
+WRITE16_HANDLER( atarigen_playfield_large_w );
+WRITE16_HANDLER( atarigen_playfield_upper_w );
+WRITE16_HANDLER( atarigen_playfield_dual_upper_w );
+WRITE16_HANDLER( atarigen_playfield_latched_lsb_w );
+WRITE16_HANDLER( atarigen_playfield_latched_msb_w );
+WRITE16_HANDLER( atarigen_playfield2_w );
+WRITE16_HANDLER( atarigen_playfield2_latched_msb_w );
+
+
+/*---------------------------------------------------------------
+    VIDEO HELPERS
 ---------------------------------------------------------------*/
 
 void atarigen_scanline_timer_reset(atarigen_scanline_callback update_graphics, int frequency);
@@ -197,10 +230,58 @@ WRITE32_HANDLER( atarigen_666_paletteram32_w );
 
 
 /*---------------------------------------------------------------
-	MISC HELPERS
+    MISC HELPERS
 ---------------------------------------------------------------*/
 
-void atarigen_invert_region(int region);
 void atarigen_swap_mem(void *ptr1, void *ptr2, int bytes);
+void atarigen_blend_gfx(int gfx0, int gfx1, int mask0, int mask1);
+
+
+
+/*##########################################################################
+    GENERAL ATARI NOTES
+############################################################################
+
+    Atari 68000 list:
+
+    Driver      Pr? Up? VC? PF? P2? MO? AL? BM? PH?
+    ----------  --- --- --- --- --- --- --- --- ---
+    arcadecl.c       *               *       *
+    atarig1.c        *       *      rle  *
+    atarig42.c       *       *      rle  *
+    atarigt.c                *      rle  *
+    atarigx2.c               *      rle  *
+    atarisy1.c   *   *       *       *   *              270->260
+    atarisy2.c   *   *       *       *   *              150->120
+    badlands.c       *       *       *                  250->260
+    batman.c     *   *   *   *   *   *   *       *      200->160 ?
+    blstroid.c       *       *       *                  240->230
+    cyberbal.c       *       *       *   *              125->105 ?
+    eprom.c          *       *       *   *              170->170
+    gauntlet.c   *   *       *       *   *       *      220->250
+    klax.c       *   *       *       *                  480->440 ?
+    offtwall.c       *   *   *       *                  260->260
+    rampart.c        *               *       *          280->280
+    relief.c     *   *   *   *   *   *                  240->240
+    shuuz.c          *   *   *       *                  410->290 fix!
+    skullxbo.c       *       *       *   *              150->145
+    thunderj.c       *   *   *   *   *   *       *      180->180
+    toobin.c         *       *       *   *              140->115 fix!
+    vindictr.c   *   *       *       *   *       *      200->210
+    xybots.c     *   *       *       *   *              235->238
+    ----------  --- --- --- --- --- --- --- --- ---
+
+    Pr? - do we have verifiable proof on priorities?
+    Up? - have we updated to use new MO's & tilemaps?
+    VC? - does it use the video controller?
+    PF? - does it have a playfield?
+    P2? - does it have a dual playfield?
+    MO? - does it have MO's?
+    AL? - does it have an alpha layer?
+    BM? - does it have a bitmap layer?
+    PH? - does it use the palette hack?
+
+##########################################################################*/
+
 
 #endif
