@@ -1,12 +1,12 @@
 /***************************************************************************
 
-	Cyberball 68000 sound simulator
+    Cyberball 68000 sound simulator
 
 ****************************************************************************/
 
 #include "driver.h"
 #include "machine/atarigen.h"
-
+#include "sound/dac.h"
 #include <math.h>
 
 
@@ -24,7 +24,7 @@ void cyberbal_sound_reset(void)
 {
 	/* reset the sound system */
 	bank_base = &memory_region(REGION_CPU2)[0x10000];
-	cpu_setbank(8, &bank_base[0x0000]);
+	memory_set_bankptr(8, &bank_base[0x0000]);
 	fast_68k_int = io_68k_int = 0;
 	sound_data_from_68k = sound_data_from_6502 = 0;
 	sound_data_from_68k_ready = sound_data_from_6502_ready = 0;
@@ -34,11 +34,11 @@ void cyberbal_sound_reset(void)
 
 /*************************************
  *
- *	6502 Sound Interface
+ *  6502 Sound Interface
  *
  *************************************/
 
-READ_HANDLER( cyberbal_special_port3_r )
+READ8_HANDLER( cyberbal_special_port3_r )
 {
 	int temp = readinputport(3);
 	if (!(readinputport(0) & 0x8000)) temp ^= 0x80;
@@ -48,7 +48,7 @@ READ_HANDLER( cyberbal_special_port3_r )
 }
 
 
-READ_HANDLER( cyberbal_sound_6502_stat_r )
+READ8_HANDLER( cyberbal_sound_6502_stat_r )
 {
 	int temp = 0xff;
 	if (sound_data_from_6502_ready) temp ^= 0x80;
@@ -57,25 +57,25 @@ READ_HANDLER( cyberbal_sound_6502_stat_r )
 }
 
 
-WRITE_HANDLER( cyberbal_sound_bank_select_w )
+WRITE8_HANDLER( cyberbal_sound_bank_select_w )
 {
-	cpu_setbank(8, &bank_base[0x1000 * ((data >> 6) & 3)]);
+	memory_set_bankptr(8, &bank_base[0x1000 * ((data >> 6) & 3)]);
 	coin_counter_w(1, (data >> 5) & 1);
 	coin_counter_w(0, (data >> 4) & 1);
-	cpu_set_reset_line(3, (data & 0x08) ? CLEAR_LINE : ASSERT_LINE);
-	if (!(data & 0x01)) YM2151_sh_reset();
+	cpunum_set_input_line(3, INPUT_LINE_RESET, (data & 0x08) ? CLEAR_LINE : ASSERT_LINE);
+	if (!(data & 0x01)) sndti_reset(SOUND_YM2151, 0);
 
 }
 
 
-READ_HANDLER( cyberbal_sound_68k_6502_r )
+READ8_HANDLER( cyberbal_sound_68k_6502_r )
 {
 	sound_data_from_68k_ready = 0;
 	return sound_data_from_68k;
 }
 
 
-WRITE_HANDLER( cyberbal_sound_68k_6502_w )
+WRITE8_HANDLER( cyberbal_sound_68k_6502_w )
 {
 	sound_data_from_6502 = data;
 	sound_data_from_6502_ready = 1;
@@ -91,7 +91,7 @@ WRITE_HANDLER( cyberbal_sound_68k_6502_w )
 
 /*************************************
  *
- *	68000 Sound Interface
+ *  68000 Sound Interface
  *
  *************************************/
 
@@ -105,20 +105,19 @@ static void update_sound_68k_interrupts(void)
 		newstate |= 2;
 
 	if (newstate)
-		cpu_set_irq_line(3, newstate, ASSERT_LINE);
+		cpunum_set_input_line(3, newstate, ASSERT_LINE);
 	else
-		cpu_set_irq_line(3, 7, CLEAR_LINE);
+		cpunum_set_input_line(3, 7, CLEAR_LINE);
 }
 
 
-int cyberbal_sound_68k_irq_gen(void)
+INTERRUPT_GEN( cyberbal_sound_68k_irq_gen )
 {
 	if (!fast_68k_int)
 	{
 		fast_68k_int = 1;
 		update_sound_68k_interrupts();
 	}
-	return 0;
 }
 
 
