@@ -164,6 +164,7 @@ void c352_get_info(void *token, UINT32 state, union sndinfo *info);
 void vrender0_get_info(void *token, UINT32 state, union sndinfo *info);
 void votrax_get_info(void *token, UINT32 state, union sndinfo *info);
 void es8712_get_info(void *token, UINT32 state, union sndinfo *info);
+void rf5c400_get_info(void *token, UINT32 state, union sndinfo *info);
 
 #ifdef MESS
 void beep_get_info(void *token, UINT32 state, union sndinfo *info);
@@ -413,6 +414,9 @@ const struct
 #if (HAS_ES8712)
 	{ SOUND_ES8712, es8712_get_info },
 #endif
+#if (HAS_RF5C400)
+	{ SOUND_RF5C400, rf5c400_get_info },
+#endif
 
 #ifdef MESS
 #if (HAS_BEEP)
