@@ -11,16 +11,24 @@
 #include <math.h>
 
 #include "driver.h"
-#include "adpcm.h"
 #include "state.h"
+#include "ymz280b.h"
 
 
 #define MAX_SAMPLE_CHUNK	10000
+#define MAKE_WAVS			0
 
 #define FRAC_BITS			14
 #define FRAC_ONE			(1 << FRAC_BITS)
 #define FRAC_MASK			(FRAC_ONE - 1)
 
+#define INTERNAL_BUFFER_SIZE 	(1 << 15)
+#define INTERNAL_SAMPLE_RATE 	(chip->master_clock * 2.0)
+
+#if MAKE_WAVS
+#include "wavwrite.h"
+#endif
+
 
 /* struct describing a single playing ADPCM voice */
 struct YMZ280BVoice
@@ -53,11 +61,12 @@ struct YMZ280BVoice
 	INT32 output_pos;		/* current fractional position */
 	INT16 last_sample;		/* last sample output */
 	INT16 curr_sample;		/* current sample target */
+	UINT8 irq_schedule;		/* 1 if the IRQ state is updated by timer */
 };
 
 struct YMZ280BChip
 {
-	int stream;						/* which stream are we using */
+	sound_stream * stream;			/* which stream are we using */
 	UINT8 *region_base;				/* pointer to the base of the region */
 	UINT8 current_register;			/* currently accessible register */
 	UINT8 status_register;			/* current status register */
@@ -68,18 +77,55 @@ struct YMZ280BChip
 	double master_clock;			/* master clock frequency */
 	void (*irq_callback)(int);		/* IRQ callback */
 	struct YMZ280BVoice	voice[8];	/* the 8 voices */
+	void *update_timer;				/* timer for ymz280b_force_update */
+	UINT32 rom_readback_addr;		/* where the CPU can read the ROM */
+	read8_handler ext_ram_read;		/* external RAM read handler */
+	write8_handler ext_ram_write;	/* external RAM write handler */
+
+	INT16 *ibuffer[2];				/* internal buffer */
+	UINT32 ibuffer_pos_in;
+	UINT32 ibuffer_pos_out;			/* (32-FRAC_BITS).FRAC_BITS fix point */
+	UINT32 ibuffer_samples;
+	int ibuffer_step;				/* (32-FRAC_BITS).FRAC_BITS fix point */
+	int ibuffer_samples_per_frame;	/* (32-FRAC_BITS).FRAC_BITS fix point */
+	double samples_left_over;
+
+#if MAKE_WAVS
+	void *		wavresample;			/* resampled waveform */
+#endif
+
+	INT32 *accumulator;
+	INT16 *scratch;
 };
 
-static struct YMZ280BChip ymz280b[MAX_YMZ280B];
-static INT32 *accumulator;
-static INT16 *scratch;
-
 /* step size index shift table */
-static int index_scale[8] = { 0x0e6, 0x0e6, 0x0e6, 0x0e6, 0x133, 0x199, 0x200, 0x266 };
+static const int index_scale[8] = { 0x0e6, 0x0e6, 0x0e6, 0x0e6, 0x133, 0x199, 0x200, 0x266 };
 
 /* lookup table for the precomputed difference */
 static int diff_lookup[16];
 
+/* timer callback */
+static void update_irq_state_timer_0(void *param);
+static void update_irq_state_timer_1(void *param);
+static void update_irq_state_timer_2(void *param);
+static void update_irq_state_timer_3(void *param);
+static void update_irq_state_timer_4(void *param);
+static void update_irq_state_timer_5(void *param);
+static void update_irq_state_timer_6(void *param);
+static void update_irq_state_timer_7(void *param);
+
+static void (*update_irq_state_cb[])(void *) =
+{
+	update_irq_state_timer_0,
+	update_irq_state_timer_1,
+	update_irq_state_timer_2,
+	update_irq_state_timer_3,
+	update_irq_state_timer_4,
+	update_irq_state_timer_5,
+	update_irq_state_timer_6,
+	update_irq_state_timer_7
+};
+
 
 
 INLINE void update_irq_state(struct YMZ280BChip *chip)
@@ -96,12 +142,14 @@ INLINE void update_irq_state(struct YMZ280BChip *chip)
 		chip->irq_state = 1;
 		if (chip->irq_callback)
 			(*chip->irq_callback)(1);
+		else logerror("YMZ280B: IRQ generated, but no callback specified!");
 	}
 	else if (!irq_bits && chip->irq_state)
 	{
 		chip->irq_state = 0;
 		if (chip->irq_callback)
 			(*chip->irq_callback)(0);
+		else logerror("YMZ280B: IRQ generated, but no callback specified!");
 	}
 }
 
@@ -122,7 +170,7 @@ INLINE void update_step(struct YMZ280BChip *chip, struct YMZ280BVoice *voice)
 		frequency = chip->master_clock * (double)((voice->fnum & 0x0ff) + 1) * (1.0 / 256.0);
 	else
 		frequency = chip->master_clock * (double)((voice->fnum & 0x1ff) + 1) * (1.0 / 256.0);
-	voice->output_step = (UINT32)(frequency * (double)FRAC_ONE / (double)Machine->sample_rate);
+	voice->output_step = (UINT32)(frequency * (double)FRAC_ONE / INTERNAL_SAMPLE_RATE);
 }
 
 
@@ -146,6 +194,44 @@ INLINE void update_volumes(struct YMZ280BVoice *voice)
 }
 
 
+static void YMZ280B_state_save_update_step(void *param)
+{
+	struct YMZ280BChip *chip = param;
+	int j;
+	for (j = 0; j < 8; j++)
+	{
+		struct YMZ280BVoice *voice = &chip->voice[j];
+		update_step(chip, voice);
+		if(voice->irq_schedule)
+			timer_set_ptr(0, chip, update_irq_state_cb[j]);
+	}
+	timer_adjust(chip->update_timer, TIME_NEVER, 0, 0);
+}
+
+
+static void update_irq_state_timer_common(void *param, int voicenum)
+{
+	struct YMZ280BChip *chip = param;
+	struct YMZ280BVoice *voice = &chip->voice[voicenum];
+
+	if(!voice->irq_schedule) return;
+
+	voice->playing = 0;
+	chip->status_register |= 1 << voicenum;
+	update_irq_state(chip);
+	voice->irq_schedule = 0;
+}
+
+static void update_irq_state_timer_0(void *param) { update_irq_state_timer_common(param, 0); }
+static void update_irq_state_timer_1(void *param) { update_irq_state_timer_common(param, 1); }
+static void update_irq_state_timer_2(void *param) { update_irq_state_timer_common(param, 2); }
+static void update_irq_state_timer_3(void *param) { update_irq_state_timer_common(param, 3); }
+static void update_irq_state_timer_4(void *param) { update_irq_state_timer_common(param, 4); }
+static void update_irq_state_timer_5(void *param) { update_irq_state_timer_common(param, 5); }
+static void update_irq_state_timer_6(void *param) { update_irq_state_timer_common(param, 6); }
+static void update_irq_state_timer_7(void *param) { update_irq_state_timer_common(param, 7); }
+
+
 /**********************************************************************************************
 
      compute_tables -- compute the difference tables
@@ -406,15 +492,30 @@ static int generate_pcm16(struct YMZ280BVoice *voice, UINT8 *base, INT16 *buffer
 
 ***********************************************************************************************/
 
-static void ymz280b_update(int num, INT16 **buffer, int length)
+static void ymz280b_force_update(struct YMZ280BChip *chip)
 {
-	struct YMZ280BChip *chip = &ymz280b[num];
-	INT32 *lacc = accumulator;
-	INT32 *racc = accumulator + length;
+	int length;
+
+	INT32 *lacc;
+	INT32 *racc;
 	int v;
+	double time_last_update;
+
+	if (Machine->sample_rate == 0) return;
+
+	/* elapsed time since the last callback */
+	time_last_update = timer_timeelapsed(chip->update_timer);
+	if(time_last_update <= 0) return;
+
+	/* compute how many samples to generate */
+	length = (int)(chip->samples_left_over + time_last_update * INTERNAL_SAMPLE_RATE) - chip->ibuffer_samples;
+	if(length <= 0) return;
+
+	lacc = chip->accumulator;
+	racc = chip->accumulator + length;
 
 	/* clear out the accumulator */
-	memset(accumulator, 0, 2 * length * sizeof(accumulator[0]));
+	memset(chip->accumulator, 0, 2 * length * sizeof(chip->accumulator[0]));
 
 	/* loop over voices */
 	for (v = 0; v < 8; v++)
@@ -422,7 +523,7 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 		struct YMZ280BVoice *voice = &chip->voice[v];
 		INT16 prev = voice->last_sample;
 		INT16 curr = voice->curr_sample;
-		INT16 *curr_data = scratch;
+		INT16 *curr_data = chip->scratch;
 		INT32 *ldest = lacc;
 		INT32 *rdest = racc;
 		UINT32 new_samples, samples_left;
@@ -436,7 +537,7 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 			continue;
 
 		/* finish off the current sample */
-		if (voice->output_pos > 0)
+/*      if (voice->output_pos > 0) */
 		{
 			/* interpolate */
 			while (remaining > 0 && voice->output_pos < FRAC_ONE)
@@ -457,7 +558,7 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 
 		/* compute how many new samples we need */
 		final_pos = voice->output_pos + remaining * voice->output_step;
-		new_samples = (final_pos + FRAC_ONE - 1) >> FRAC_BITS;
+		new_samples = (final_pos + FRAC_ONE) >> FRAC_BITS;
 		if (new_samples > MAX_SAMPLE_CHUNK)
 			new_samples = MAX_SAMPLE_CHUNK;
 		samples_left = new_samples;
@@ -467,11 +568,11 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 		{
 			switch (voice->mode)
 			{
-				case 1:	samples_left = generate_adpcm(voice, chip->region_base, scratch, new_samples);	break;
-				case 2:	samples_left = generate_pcm8(voice, chip->region_base, scratch, new_samples);	break;
-				case 3:	samples_left = generate_pcm16(voice, chip->region_base, scratch, new_samples);	break;
+				case 1:	samples_left = generate_adpcm(voice, chip->region_base, chip->scratch, new_samples);	break;
+				case 2:	samples_left = generate_pcm8(voice, chip->region_base, chip->scratch, new_samples);	break;
+				case 3:	samples_left = generate_pcm16(voice, chip->region_base, chip->scratch, new_samples);	break;
 				default:
-				case 0:	samples_left = 0; memset(scratch, 0, new_samples * sizeof(scratch[0]));			break;
+				case 0:	samples_left = 0; memset(chip->scratch, 0, new_samples * sizeof(chip->scratch[0]));			break;
 			}
 		}
 
@@ -479,20 +580,22 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 		if (samples_left)
 		{
 			int base = new_samples - samples_left;
-			int i, t = (base == 0) ? curr : scratch[base - 1];
+			int i, t = (base == 0) ? curr : chip->scratch[base - 1];
 			for (i = 0; i < samples_left; i++)
 			{
 				if (t < 0) t = -((-t * 15) >> 4);
 				else if (t > 0) t = (t * 15) >> 4;
-				scratch[base + i] = t;
+				chip->scratch[base + i] = t;
 			}
 
 			/* if we hit the end and IRQs are enabled, signal it */
 			if (base != 0)
 			{
 				voice->playing = 0;
-				chip->status_register |= 1 << v;
-				update_irq_state(chip);
+
+				/* set update_irq_state_timer. IRQ is signaled on next CPU execution. */
+				timer_set_ptr(0, chip, update_irq_state_cb[v]);
+				voice->irq_schedule = 1;
 			}
 		}
 
@@ -538,119 +641,172 @@ static void ymz280b_update(int num, INT16 **buffer, int length)
 		if (rsamp < -32768) rsamp = -32768;
 		else if (rsamp > 32767) rsamp = 32767;
 
-		buffer[0][v] = lsamp;
-		buffer[1][v] = rsamp;
+		chip->ibuffer[0][chip->ibuffer_pos_in] = lsamp;
+		chip->ibuffer[1][chip->ibuffer_pos_in] = rsamp;
+		chip->ibuffer_pos_in++;
+		chip->ibuffer_pos_in &= INTERNAL_BUFFER_SIZE - 1;
 	}
+
+	chip->ibuffer_samples += length;
 }
 
 
-/*ks s */
-static int chip_num;
-static void YMZ280B_state_save_update_step(void)
+static void ymz280b_update(void *param, stream_sample_t **inputs, stream_sample_t **buffer, int length)
 {
-	int i,j;
-	for (i = 0; i < chip_num; i++)
+	struct YMZ280BChip *chip = param;
+	int step = chip->ibuffer_step;
+	int samples_per_frame = chip->ibuffer_samples_per_frame;
+	UINT32 pos1,pos2,internal_samples;
+	int pos_dif;
+	double time_last_update;
+
+	ymz280b_force_update(chip);
+
+	/* elapsed time since the last callback */
+	time_last_update = timer_timeelapsed(chip->update_timer);
+	if(time_last_update <= 0)
+		return;
+
+	/* compute how many samples */
+	chip->samples_left_over += time_last_update * INTERNAL_SAMPLE_RATE;
+	internal_samples = (UINT32)chip->samples_left_over;
+	chip->samples_left_over -= (double)internal_samples;
+
+	pos2 = chip->ibuffer_pos_out;
+	for (pos1 = 0; pos1 < length ; pos1++)
 	{
-		for (j = 0; j < 8; j++)
-		{
-			struct YMZ280BChip *chip = &ymz280b[i];
-			struct YMZ280BVoice *voice = &chip->voice[j];
-			update_step(chip, voice);
-		}
+		buffer[0][pos1] = chip->ibuffer[0][pos2>>FRAC_BITS];
+		buffer[1][pos1] = chip->ibuffer[1][pos2>>FRAC_BITS];
+		pos2+=step;
+		pos2 &= (INTERNAL_BUFFER_SIZE<<FRAC_BITS)-1;
 	}
+	chip->ibuffer_pos_out = pos2;
+
+	pos_dif = (INTERNAL_BUFFER_SIZE<<FRAC_BITS) + chip->ibuffer_pos_out - (chip->ibuffer_pos_in<<FRAC_BITS);
+	pos_dif &= (INTERNAL_BUFFER_SIZE<<FRAC_BITS)-1;
+	if(pos_dif > (INTERNAL_BUFFER_SIZE<<FRAC_BITS)/2) pos_dif-=INTERNAL_BUFFER_SIZE<<FRAC_BITS;
+
+	if(pos_dif < -step * 32)
+		{
+		pos_dif += step/4;
+		chip->ibuffer_pos_out += step/4;
+		}
+	else if(pos_dif > -step * 16)
+	{
+		pos_dif -= step/4;
+		chip->ibuffer_pos_out -= step/4;
+	}
+
+	if(pos_dif > -step * 16)
+		chip->ibuffer_pos_out = (chip->ibuffer_pos_in<<FRAC_BITS) - samples_per_frame;
+	else if(pos_dif < -samples_per_frame)
+		chip->ibuffer_pos_out = (chip->ibuffer_pos_in<<FRAC_BITS) - step * 24;
+
+	chip->ibuffer_pos_out &= (INTERNAL_BUFFER_SIZE<<FRAC_BITS)-1;
+	chip->ibuffer_samples = 0;
+	timer_adjust(chip->update_timer, TIME_NEVER, 0, 0);
+
+#if MAKE_WAVS
+	/* log the resampled data */
+	if (ymz280b[num].wavresample)
+		wav_add_data_16lr(ymz280b[num].wavresample, buffer[0], buffer[1], length);
+#endif
 }
-/*ks e */
+
+
+
 /**********************************************************************************************
 
      YMZ280B_sh_start -- start emulation of the YMZ280B
 
 ***********************************************************************************************/
 
-int YMZ280B_sh_start(const struct MachineSound *msound)
+static void *ymz280b_start(int sndindex, int clock, const void *config)
 {
-	const struct YMZ280Binterface *intf = msound->sound_interface;
-	char stream_name[2][40];
-	const char *stream_name_ptrs[2];
-	int vol[2];
-	int i;
+	const struct YMZ280Binterface *intf = config;
+	struct YMZ280BChip *chip;
+
+	chip = auto_malloc(sizeof(*chip));
+	memset(chip, 0, sizeof(*chip));
+
+	chip->ext_ram_read = intf->ext_read;
+	chip->ext_ram_write = intf->ext_write;
 
 	/* compute ADPCM tables */
 	compute_tables();
 
-	/* initialize the voices */
-	memset(&ymz280b, 0, sizeof(ymz280b));
-	for (i = 0; i < intf->num; i++)
-	{
-		/* generate the name and create the stream */
-		sprintf(stream_name[0], "%s #%d Ch1", sound_name(msound), i);
-		sprintf(stream_name[1], "%s #%d Ch2", sound_name(msound), i);
-		stream_name_ptrs[0] = stream_name[0];
-		stream_name_ptrs[1] = stream_name[1];
+	/* create the stream */
+	chip->stream = stream_create(0, 2, Machine->sample_rate, chip, ymz280b_update);
 
-		/* set the volumes */
-		vol[0] = intf->mixing_level[i] & 0xffff;
-		vol[1] = intf->mixing_level[i] >> 16;
+	/* initialize the rest of the structure */
+	chip->master_clock = (double)clock / 384.0;
+	chip->region_base = memory_region(intf->region);
+	chip->irq_callback = intf->irq_callback;
 
-		/* create the stream */
-		ymz280b[i].stream = stream_init_multi(2, stream_name_ptrs, vol, Machine->sample_rate, i, ymz280b_update);
-		if (ymz280b[i].stream == -1)
-			return 1;
+	chip->update_timer = timer_alloc(NULL);
+	timer_adjust(chip->update_timer, TIME_NEVER, 0, 0);
 
-		/* initialize the rest of the structure */
-		ymz280b[i].master_clock = (double)intf->baseclock[i] / 384.0;
-		ymz280b[i].region_base = memory_region(intf->region[i]);
-		ymz280b[i].irq_callback = intf->irq_callback[i];
-	}
+	chip->ibuffer[0] = auto_malloc(sizeof(chip->ibuffer[0][0]) * 2 * INTERNAL_BUFFER_SIZE);
+	if (!chip->ibuffer[0]) return NULL;
+	chip->ibuffer[1] = chip->ibuffer[0] + INTERNAL_BUFFER_SIZE;
+
+	if (Machine->sample_rate == 0) chip->ibuffer_step = 0;
+	else chip->ibuffer_step = (UINT32)(INTERNAL_SAMPLE_RATE * (double)FRAC_ONE / (double)Machine->sample_rate + 0.5);
+	chip->ibuffer_samples_per_frame = (UINT32)(INTERNAL_SAMPLE_RATE * (double)FRAC_ONE / Machine->drv->frames_per_second + 0.5);
+	chip->ibuffer_pos_in = (chip->ibuffer_step * 24)>>FRAC_BITS;
+	chip->ibuffer_pos_out = 0;
 
 	/* allocate memory */
-	accumulator = malloc(sizeof(accumulator[0]) * 2 * MAX_SAMPLE_CHUNK);
-	scratch = malloc(sizeof(scratch[0]) * MAX_SAMPLE_CHUNK);
-	if (!accumulator || !scratch)
-		return 1;
+	chip->accumulator = auto_malloc(sizeof(chip->accumulator[0]) * 2 * MAX_SAMPLE_CHUNK);
+	chip->scratch = auto_malloc(sizeof(chip->scratch[0]) * MAX_SAMPLE_CHUNK);
 
-/*ks s */
 	/* state save */
-	for (i = 0; i < intf->num; i++)
 	{
 		int j;
-		state_save_register_UINT8("YMZ280B", i, "current_register", &ymz280b[i].current_register,1);
-		state_save_register_UINT8("YMZ280B", i, "status_register", &ymz280b[i].status_register,1);
-		state_save_register_UINT8("YMZ280B", i, "irq_state", &ymz280b[i].irq_state,1);
-		state_save_register_UINT8("YMZ280B", i, "irq_mask", &ymz280b[i].irq_mask,1);
-		state_save_register_UINT8("YMZ280B", i, "irq_enable", &ymz280b[i].irq_enable,1);
-		state_save_register_UINT8("YMZ280B", i, "keyon_enable", &ymz280b[i].keyon_enable,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "current_register", &chip->current_register,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "status_register", &chip->status_register,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "irq_state", &chip->irq_state,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "irq_mask", &chip->irq_mask,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "irq_enable", &chip->irq_enable,1);
+		state_save_register_UINT8("YMZ280B", sndindex, "keyon_enable", &chip->keyon_enable,1);
+		state_save_register_UINT32("YMZ280B", sndindex, "rom_readback", &chip->rom_readback_addr,1);
+		state_save_register_double("YMZ280B", sndindex, "samples_left_over", &chip->samples_left_over,1);
 		for (j = 0; j < 8; j++)
 		{
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "playing", &ymz280b[i].voice[j].playing,1);
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "keyon", &ymz280b[i].voice[j].keyon,1);
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "looping", &ymz280b[i].voice[j].looping,1);
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "mode", &ymz280b[i].voice[j].mode,1);
-			state_save_register_UINT16 ("YMZ280B.voice", i*8+j, "fnum", &ymz280b[i].voice[j].fnum,1);
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "level", &ymz280b[i].voice[j].level,1);
-			state_save_register_UINT8 ("YMZ280B.voice", i*8+j, "pan", &ymz280b[i].voice[j].pan,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "start", &ymz280b[i].voice[j].start,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "stop", &ymz280b[i].voice[j].stop,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "loop_start", &ymz280b[i].voice[j].loop_start,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "loop_end", &ymz280b[i].voice[j].loop_end,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "position", &ymz280b[i].voice[j].position,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "signal", &ymz280b[i].voice[j].signal,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "step", &ymz280b[i].voice[j].step,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "loop_signal", &ymz280b[i].voice[j].loop_signal,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "loop_step", &ymz280b[i].voice[j].loop_step,1);
-			state_save_register_UINT32 ("YMZ280B.voice", i*8+j, "loop_count", &ymz280b[i].voice[j].loop_count,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "output_left", &ymz280b[i].voice[j].output_left,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "output_right", &ymz280b[i].voice[j].output_right,1);
-			state_save_register_INT32 ("YMZ280B.voice", i*8+j, "output_pos", &ymz280b[i].voice[j].output_pos,1);
-			state_save_register_INT16 ("YMZ280B.voice", i*8+j, "last_sample", &ymz280b[i].voice[j].last_sample,1);
-			state_save_register_INT16 ("YMZ280B.voice", i*8+j, "curr_sample", &ymz280b[i].voice[j].curr_sample,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "playing", &chip->voice[j].playing,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "keyon", &chip->voice[j].keyon,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "looping", &chip->voice[j].looping,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "mode", &chip->voice[j].mode,1);
+			state_save_register_UINT16 ("YMZ280B.voice", sndindex*8+j, "fnum", &chip->voice[j].fnum,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "level", &chip->voice[j].level,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "pan", &chip->voice[j].pan,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "start", &chip->voice[j].start,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "stop", &chip->voice[j].stop,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "loop_start", &chip->voice[j].loop_start,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "loop_end", &chip->voice[j].loop_end,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "position", &chip->voice[j].position,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "signal", &chip->voice[j].signal,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "step", &chip->voice[j].step,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "loop_signal", &chip->voice[j].loop_signal,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "loop_step", &chip->voice[j].loop_step,1);
+			state_save_register_UINT32 ("YMZ280B.voice", sndindex*8+j, "loop_count", &chip->voice[j].loop_count,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "output_left", &chip->voice[j].output_left,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "output_right", &chip->voice[j].output_right,1);
+			state_save_register_INT32 ("YMZ280B.voice", sndindex*8+j, "output_pos", &chip->voice[j].output_pos,1);
+			state_save_register_INT16 ("YMZ280B.voice", sndindex*8+j, "last_sample", &chip->voice[j].last_sample,1);
+			state_save_register_INT16 ("YMZ280B.voice", sndindex*8+j, "curr_sample", &chip->voice[j].curr_sample,1);
+			state_save_register_UINT8 ("YMZ280B.voice", sndindex*8+j, "irq_schedule", &chip->voice[j].irq_schedule,1);
 		}
 	}
-	state_save_register_func_postload(YMZ280B_state_save_update_step);
-	chip_num = intf->num;
-/*ks e */
+
+	state_save_register_func_postload_ptr(YMZ280B_state_save_update_step, chip);
+
+#if MAKE_WAVS
+	chip->wavresample = wav_open("resamp.wav", Machine->sample_rate, 2);
+#endif
 
 	/* success */
-	return 0;
+	return chip;
 }
 
 
@@ -663,14 +819,17 @@ int YMZ280B_sh_start(const struct MachineSound *msound)
 
 void YMZ280B_sh_stop(void)
 {
-	/* free memory */
-	if (accumulator)
-		free(accumulator);
-	accumulator = NULL;
+#if MAKE_WAVS
+{
+	int i;
 
-	if (scratch)
-		free(scratch);
-	scratch = NULL;
+	for (i = 0; i < MAX_BSMT2000; i++)
+	{
+		if (ymz280b[i].wavresample)
+			wav_close(ymz280b[i].wavresample);
+	}
+}
+#endif
 }
 
 
@@ -687,7 +846,7 @@ static void write_to_register(struct YMZ280BChip *chip, int data)
 	int i;
 
 	/* force an update */
-	stream_update(chip->stream, 0);
+	ymz280b_force_update(chip);
 
 	/* lower registers follow a pattern */
 	if (chip->current_register < 0x80)
@@ -712,14 +871,17 @@ static void write_to_register(struct YMZ280BChip *chip, int data)
 					voice->signal = voice->loop_signal = 0;
 					voice->step = voice->loop_step = 0x7f;
 					voice->loop_count = 0;
+
+					/* if update_irq_state_timer is set, cancel it. */
+					voice->irq_schedule = 0;
 				}
 				if (voice->keyon && !(data & 0x80) && !voice->looping)
-/*ks start */
 				{
 					voice->playing = 0;
-					chip->status_register &= ~(1 << ((chip->current_register >> 2) & 7));
+
+					/* if update_irq_state_timer is set, cancel it. */
+					voice->irq_schedule = 0;
 				}
-/*ks end */
 				voice->keyon = (data & 0x80) >> 7;
 				update_step(chip, voice);
 				break;
@@ -793,6 +955,28 @@ static void write_to_register(struct YMZ280BChip *chip, int data)
 	{
 		switch (chip->current_register)
 		{
+			case 0x84:		/* ROM readback / RAM write (high) */
+				chip->rom_readback_addr &= 0xffff;
+				chip->rom_readback_addr |= (data<<16);
+				break;
+
+			case 0x85:		/* ROM readback / RAM write (med) */
+				chip->rom_readback_addr &= 0xff00ff;
+				chip->rom_readback_addr |= (data<<8);
+				break;
+
+			case 0x86:		/* ROM readback / RAM write (low) */
+				chip->rom_readback_addr &= 0xffff00;
+				chip->rom_readback_addr |= data;
+				break;
+
+			case 0x87:		/* RAM write */
+				if (chip->ext_ram_write)
+					chip->ext_ram_write(chip->rom_readback_addr, data);
+				else
+					logerror("YMZ280B attempted RAM write to %X\n", chip->rom_readback_addr);
+				break;
+
 			case 0xfe:		/* IRQ mask */
 				chip->irq_mask = data;
 				update_irq_state(chip);
@@ -801,16 +985,26 @@ static void write_to_register(struct YMZ280BChip *chip, int data)
 			case 0xff:		/* IRQ enable, test, etc */
 				chip->irq_enable = (data & 0x10) >> 4;
 				update_irq_state(chip);
-/*ks start */
+
 				if (chip->keyon_enable && !(data & 0x80))
+				{
 					for (i = 0; i < 8; i++)
+					{
 						chip->voice[i].playing = 0;
-				if (!chip->keyon_enable && (data & 0x80))
+
+						/* if update_irq_state_timer is set, cancel it. */
+						chip->voice[i].irq_schedule = 0;
+					}
+				}
+				else if (!chip->keyon_enable && (data & 0x80))
+				{
 					for (i = 0; i < 8; i++)
+					{
 						if (chip->voice[i].keyon && chip->voice[i].looping)
 							chip->voice[i].playing = 1;
+					}
+				}
 				chip->keyon_enable = (data & 0x80) >> 7;
-/*ks end */
 				break;
 
 			default:
@@ -830,10 +1024,18 @@ static void write_to_register(struct YMZ280BChip *chip, int data)
 
 static int compute_status(struct YMZ280BChip *chip)
 {
-	UINT8 result = chip->status_register;
+	UINT8 result;
+
+	/* ROM/RAM readback? */
+	if (chip->current_register == 0x86)
+	{
+		return chip->region_base[chip->rom_readback_addr];
+	}
 
 	/* force an update */
-	stream_update(chip->stream, 0);
+	ymz280b_force_update(chip);
+
+	result = chip->status_register;
 
 	/* clear the IRQ state */
 	chip->status_register = 0;
@@ -850,17 +1052,41 @@ static int compute_status(struct YMZ280BChip *chip)
 
 ***********************************************************************************************/
 
-READ_HANDLER( YMZ280B_status_0_r )
+READ8_HANDLER( YMZ280B_status_0_r )
 {
-	return compute_status(&ymz280b[0]);
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	return compute_status(chip);
 }
 
-READ_HANDLER( YMZ280B_status_1_r )
+READ8_HANDLER( YMZ280B_status_1_r )
 {
-	return compute_status(&ymz280b[1]);
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	return compute_status(chip);
 }
 
+READ16_HANDLER( YMZ280B_status_0_lsb_r )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	return compute_status(chip);
+}
 
+READ16_HANDLER( YMZ280B_status_0_msb_r )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	return compute_status(chip) << 8;
+}
+
+READ16_HANDLER( YMZ280B_status_1_lsb_r )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	return compute_status(chip);
+}
+
+READ16_HANDLER( YMZ280B_status_1_msb_r )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	return compute_status(chip) << 8;
+}
 
 /**********************************************************************************************
 
@@ -868,17 +1094,41 @@ READ_HANDLER( YMZ280B_status_1_r )
 
 ***********************************************************************************************/
 
-WRITE_HANDLER( YMZ280B_register_0_w )
+WRITE8_HANDLER( YMZ280B_register_0_w )
 {
-	ymz280b[0].current_register = data;
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	chip->current_register = data;
 }
 
-WRITE_HANDLER( YMZ280B_register_1_w )
+WRITE8_HANDLER( YMZ280B_register_1_w )
 {
-	ymz280b[1].current_register = data;
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	chip->current_register = data;
 }
 
+WRITE16_HANDLER( YMZ280B_register_0_lsb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	if (ACCESSING_LSB)	chip->current_register = data & 0xff;
+}
 
+WRITE16_HANDLER( YMZ280B_register_0_msb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	if (ACCESSING_MSB)	chip->current_register = (data >> 8) & 0xff;
+}
+
+WRITE16_HANDLER( YMZ280B_register_1_lsb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	if (ACCESSING_LSB)	chip->current_register = data & 0xff;
+}
+
+WRITE16_HANDLER( YMZ280B_register_1_msb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	if (ACCESSING_MSB)	chip->current_register = (data >> 8) & 0xff;
+}
 
 /**********************************************************************************************
 
@@ -886,12 +1136,100 @@ WRITE_HANDLER( YMZ280B_register_1_w )
 
 ***********************************************************************************************/
 
-WRITE_HANDLER( YMZ280B_data_0_w )
+WRITE8_HANDLER( YMZ280B_data_0_w )
 {
-	write_to_register(&ymz280b[0], data);
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	write_to_register(chip, data);
 }
 
-WRITE_HANDLER( YMZ280B_data_1_w )
+WRITE8_HANDLER( YMZ280B_data_1_w )
 {
-	write_to_register(&ymz280b[1], data);
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	write_to_register(chip, data);
 }
+
+WRITE16_HANDLER( YMZ280B_data_0_lsb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	if (ACCESSING_LSB)	write_to_register(chip, data & 0xff);
+}
+
+WRITE16_HANDLER( YMZ280B_data_0_msb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	if (ACCESSING_MSB)	write_to_register(chip, (data >> 8) & 0xff);
+}
+
+WRITE16_HANDLER( YMZ280B_data_1_lsb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	if (ACCESSING_LSB)	write_to_register(chip, data & 0xff);
+}
+
+WRITE16_HANDLER( YMZ280B_data_1_msb_w )
+{
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	if (ACCESSING_MSB)	write_to_register(chip, (data >> 8) & 0xff);
+}
+
+/**********************************************************************************************
+
+     YMZ280B_data_0_r/YMZ280B_data_1_r -- handle an external RAM read
+
+***********************************************************************************************/
+
+READ8_HANDLER( YMZ280B_data_0_r )
+{
+	UINT8 data;
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 0);
+	data = chip->ext_ram_read(chip->rom_readback_addr - 1);
+	chip->rom_readback_addr++;
+	return data;
+}
+
+READ8_HANDLER( YMZ280B_data_1_r )
+{
+	UINT8 data;
+	struct YMZ280BChip *chip = sndti_token(SOUND_YMZ280B, 1);
+	data = chip->ext_ram_read(chip->rom_readback_addr - 1);
+	chip->rom_readback_addr++;
+	return data;
+}
+
+
+
+
+/**************************************************************************
+ * Generic get_info
+ **************************************************************************/
+
+static void ymz280b_set_info(void *token, UINT32 state, union sndinfo *info)
+{
+	switch (state)
+	{
+		/* no parameters to set */
+	}
+}
+
+
+void ymz280b_get_info(void *token, UINT32 state, union sndinfo *info)
+{
+	switch (state)
+	{
+		/* --- the following bits of info are returned as 64-bit signed integers --- */
+
+		/* --- the following bits of info are returned as pointers to data or functions --- */
+		case SNDINFO_PTR_SET_INFO:						info->set_info = ymz280b_set_info;		break;
+		case SNDINFO_PTR_START:							info->start = ymz280b_start;			break;
+		case SNDINFO_PTR_STOP:							/* Nothing */							break;
+		case SNDINFO_PTR_RESET:							/* Nothing */							break;
+
+		/* --- the following bits of info are returned as NULL-terminated strings --- */
+		case SNDINFO_STR_NAME:							info->s = "YMZ280B";					break;
+		case SNDINFO_STR_CORE_FAMILY:					info->s = "Yamaha Wavetable";			break;
+		case SNDINFO_STR_CORE_VERSION:					info->s = "1.0";						break;
+		case SNDINFO_STR_CORE_FILE:						info->s = __FILE__;						break;
+		case SNDINFO_STR_CORE_CREDITS:					info->s = "Copyright (c) 2004, The MAME Team"; break;
+	}
+}
+
