@@ -1,48 +1,49 @@
 /*****************************************************************************
 
-	Irem M92 video hardware, Bryan McPhail, mish@tendril.co.uk
+    Irem M92 video hardware, Bryan McPhail, mish@tendril.co.uk
 
-	Brief Overview:
+    Brief Overview:
 
-		3 scrolling playfields, 512 by 512.
-		Each playfield can enable rowscroll, change shape (to 1024 by 512),
-		be enabled/disabled and change position in VRAM.
+        3 scrolling playfields, 512 by 512.
+        Each playfield can enable rowscroll, change shape (to 1024 by 512),
+        be enabled/disabled and change position in VRAM.
 
-		Tiles can have several priority values:
-			0 = standard
-			1 = Top 8 pens appear over sprites (split tilemap)
-			2 = Whole tile appears over sprites
-			3 = ?  Seems to be the whole tile is over sprites (as 2).
+        Tiles can have several priority values:
+            0 = standard
+            1 = Top 8 pens appear over sprites (split tilemap)
+            2 = Whole tile appears over sprites
+            3 = ?  Seems to be the whole tile is over sprites (as 2).
 
-		Sprites have 2 priority values:
-			0 = standard
-			1 = Sprite appears over all tiles, including high priority pf1
+        Sprites have 2 priority values:
+            0 = standard
+            1 = Sprite appears over all tiles, including high priority pf1
 
-		Raster interrupts can be triggered at any line of the screen redraw,
-		typically used in games like R-Type Leo to multiplex the top playfield.
+        Raster interrupts can be triggered at any line of the screen redraw,
+        typically used in games like R-Type Leo to multiplex the top playfield.
 
 *****************************************************************************
 
-	Master Control registers:
+    Master Control registers:
 
-		Word 0:	Playfield 1 control
-			Bit  0x40:  1 = Rowscroll enable, 0 = disable
-			Bit  0x10:	0 = Playfield enable, 1 = disable
-			Bit  0x04:  0 = 512 x 512 playfield, 1 = 1024 x 512 playfield
-			Bits 0x03:  Playfield location in VRAM (0, 0x4000, 0x8000, 0xc000)
-		Word 1: Playfield 2 control (as above)
-		Word 2: Playfield 3 control (as above)
-		Word 3: Raster IRQ position.
+        Word 0: Playfield 1 control
+            Bit  0x40:  1 = Rowscroll enable, 0 = disable
+            Bit  0x10:  0 = Playfield enable, 1 = disable
+            Bit  0x04:  0 = 512 x 512 playfield, 1 = 1024 x 512 playfield
+            Bits 0x03:  Playfield location in VRAM (0, 0x4000, 0x8000, 0xc000)
+        Word 1: Playfield 2 control (as above)
+        Word 2: Playfield 3 control (as above)
+        Word 3: Raster IRQ position.
 
-	The raster IRQ position is offset by 128+8 from the first visible line,
-	suggesting there are 8 lines before the first visible one.
+    The raster IRQ position is offset by 128+8 from the first visible line,
+    suggesting there are 8 lines before the first visible one.
 
 *****************************************************************************/
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
+#include "state.h"
 
-static struct tilemap *pf3_wide_layer,*pf3_layer,*pf2_layer,*pf1_wide_layer,*pf1_layer,*pf1_hlayer;
+static tilemap *pf3_wide_layer,*pf3_layer,*pf2_layer,*pf1_wide_layer,*pf1_layer;
 static int pf1_control[8],pf2_control[8],pf3_control[8],pf4_control[8];
 static int pf1_vram_ptr,pf2_vram_ptr,pf3_vram_ptr;
 static int pf1_enable,pf2_enable,pf3_enable;
@@ -54,11 +55,6 @@ int m92_raster_irq_position,m92_raster_enable;
 unsigned char *m92_vram_data,*m92_spritecontrol;
 int m92_game_kludge;
 
-/* This is a kludgey speedup that I consider to be pretty ugly..  But -
-it gets a massive speed increase (~40fps -> ~65fps).  It works by avoiding
-the need to dirty the top playfield twice a frame */
-#define RYPELEO_SPEEDUP m92_game_kludge==1
-
 extern void m92_sprite_interrupt(void);
 int m92_sprite_buffer_busy;
 static int m92_palette_bank;
@@ -72,7 +68,7 @@ static void spritebuffer_callback (int dummy)
 		m92_sprite_interrupt();
 }
 
-WRITE_HANDLER( m92_spritecontrol_w )
+WRITE8_HANDLER( m92_spritecontrol_w )
 {
 	static int sprite_extent;
 
@@ -98,48 +94,48 @@ WRITE_HANDLER( m92_spritecontrol_w )
 		m92_sprite_buffer_busy=0;
 
 		/* Pixel clock is 26.6666 MHz, we have 0x800 bytes, or 0x400 words
-		to copy from spriteram to the buffer.  It seems safe to assume 1
-		word can be copied per clock.  So:
+        to copy from spriteram to the buffer.  It seems safe to assume 1
+        word can be copied per clock.  So:
 
-			1 MHz clock would be 1 word every 0.000,001s = 1000ns
-			26.6666MHz clock would be 1 word every 0.000,000,037 = 37 ns
-			Buffer should copy in about 37888 ns.
-		*/
+            1 MHz clock would be 1 word every 0.000,001s = 1000ns
+            26.6666MHz clock would be 1 word every 0.000,000,037 = 37 ns
+            Buffer should copy in about 37888 ns.
+        */
 		timer_set (TIME_IN_NSEC(37 * 0x400), 0, spritebuffer_callback);
 	}
-/*	logerror("%04x: m92_spritecontrol_w %08x %08x\n",cpu_get_pc(),offset,data); */
+/*  logerror("%04x: m92_spritecontrol_w %08x %08x\n",activecpu_get_pc(),offset,data); */
 }
 
-WRITE_HANDLER( m92_videocontrol_w )
+WRITE8_HANDLER( m92_videocontrol_w )
 {
 	/*
-		Many games write:
-			0x2000
-			0x201b in alternate frames.
+        Many games write:
+            0x2000
+            0x201b in alternate frames.
 
-		Some games write to this both before and after the sprite buffer
-		register - perhaps some kind of acknowledge bit is in there?
+        Some games write to this both before and after the sprite buffer
+        register - perhaps some kind of acknowledge bit is in there?
 
-		Lethal Thunder fails it's RAM test with the upper palette bank
-		enabled.  This was one of the earlier games and could actually
-		be a different motherboard revision (most games use M92-A-B top
-		pcb, a M92-A-A revision could exist...).
-	*/
+        Lethal Thunder fails it's RAM test with the upper palette bank
+        enabled.  This was one of the earlier games and could actually
+        be a different motherboard revision (most games use M92-A-B top
+        pcb, a M92-A-A revision could exist...).
+    */
 	if (offset==0)
 	{
 		/* Access to upper palette bank */
 		if ((data & 0x2) == 0x2 && m92_game_kludge!=3) m92_palette_bank = 1;
 		else                     m92_palette_bank = 0;
 	}
-/*	logerror("%04x: m92_videocontrol_w %d = %02x\n",cpu_get_pc(),offset,data); */
+/*  logerror("%04x: m92_videocontrol_w %d = %02x\n",activecpu_get_pc(),offset,data); */
 }
 
-READ_HANDLER( m92_paletteram_r )
+READ8_HANDLER( m92_paletteram_r )
 {
 	return paletteram_r(offset + 0x800*m92_palette_bank);
 }
 
-WRITE_HANDLER( m92_paletteram_w )
+WRITE8_HANDLER( m92_paletteram_w )
 {
 	paletteram_xBBBBBGGGGGRRRRR_w(offset + 0x800*m92_palette_bank,data);
 }
@@ -164,44 +160,6 @@ static void get_pf1_tile_info(int tile_index)
 			TILE_FLIPYX((m92_vram_data[tile_index+3] & 0x6)>>1) | TILE_SPLIT(pri))
 }
 
-static void get_pf1_htile_info(int tile_index)
-{
-	int tile,color,pri;
-	tile_index = 4*tile_index + 0xc000;
-
-	tile=m92_vram_data[tile_index]+(m92_vram_data[tile_index+1]<<8)+((m92_vram_data[tile_index+3]&0x80)<<9);
-	color=m92_vram_data[tile_index+2];
-	if (m92_vram_data[tile_index+3]&1) pri = 2;
-	else if (color&0x80) pri = 1;
-	else pri = 0;
-
-	SET_TILE_INFO(
-			0,
-			tile,
-			color&0x7f,
-			TILE_FLIPYX((m92_vram_data[tile_index+3] & 0x6)>>1) | TILE_SPLIT(pri))
-}
-
-static void get_pf1_ltile_info(int tile_index)
-{
-	int tile,color,pri;
-	tile_index *= 4;
-	if (pf1_vram_ptr==0x4000) tile_index += 0x4000;
-	else if (pf1_vram_ptr==0x8000) tile_index += 0x8000;
-
-	tile=m92_vram_data[tile_index]+(m92_vram_data[tile_index+1]<<8)+((m92_vram_data[tile_index+3]&0x80)<<9);
-	color=m92_vram_data[tile_index+2];
-	if (m92_vram_data[tile_index+3]&1) pri = 2;
-	else if (color&0x80) pri = 1;
-	else pri = 0;
-
-	SET_TILE_INFO(
-			0,
-			tile,
-			color&0x7f,
-			TILE_FLIPYX((m92_vram_data[tile_index+3] & 0x6)>>1) | TILE_SPLIT(pri))
-}
-
 static void get_pf2_tile_info(int tile_index)
 {
 	int tile,color,pri;
@@ -275,12 +233,12 @@ static void get_pf3_wide_tile_info(int tile_index)
 
 /*****************************************************************************/
 
-READ_HANDLER( m92_vram_r )
+READ8_HANDLER( m92_vram_r )
 {
 	return m92_vram_data[offset];
 }
 
-WRITE_HANDLER( m92_vram_w )
+WRITE8_HANDLER( m92_vram_w )
 {
 	int a,wide;
 
@@ -291,14 +249,6 @@ WRITE_HANDLER( m92_vram_w )
 	wide=offset&0x7fff;
 	offset&=0x3fff;
 
-	if (RYPELEO_SPEEDUP) {
-		if (a==0xc000) {
-			tilemap_mark_tile_dirty(pf1_hlayer,offset/4);
-			return;
-		}
-		tilemap_mark_tile_dirty(pf1_layer,offset/4);
-	}
-	else
 	if (a==pf1_vram_ptr || (a==pf1_vram_ptr+0x4000)) {
 		tilemap_mark_tile_dirty(pf1_layer,offset/4);
 		tilemap_mark_tile_dirty(pf1_wide_layer,wide/4);
@@ -315,22 +265,22 @@ WRITE_HANDLER( m92_vram_w )
 
 /*****************************************************************************/
 
-WRITE_HANDLER( m92_pf1_control_w )
+WRITE8_HANDLER( m92_pf1_control_w )
 {
 	pf1_control[offset]=data;
 }
 
-WRITE_HANDLER( m92_pf2_control_w )
+WRITE8_HANDLER( m92_pf2_control_w )
 {
 	pf2_control[offset]=data;
 }
 
-WRITE_HANDLER( m92_pf3_control_w )
+WRITE8_HANDLER( m92_pf3_control_w )
 {
 	pf3_control[offset]=data;
 }
 
-WRITE_HANDLER( m92_master_control_w )
+WRITE8_HANDLER( m92_master_control_w )
 {
 	static int last_pf1_ptr,last_pf2_ptr,last_pf3_ptr;
 
@@ -343,7 +293,6 @@ WRITE_HANDLER( m92_master_control_w )
 			pf1_vram_ptr=(pf4_control[0]&3)*0x4000;
 			pf1_shape=(pf4_control[0]&4)>>2;
 
-			if (RYPELEO_SPEEDUP) tilemap_set_enable(pf1_hlayer,pf1_enable);
 			if (pf1_shape) {
 				tilemap_set_enable(pf1_layer,0);
 				tilemap_set_enable(pf1_wide_layer,pf1_enable);
@@ -391,44 +340,27 @@ WRITE_HANDLER( m92_master_control_w )
 			break;
 		case 6:
 		case 7:
-/*			if (flip_screen) */
-/*				m92_raster_irq_position=256-(((pf4_control[7]<<8) | pf4_control[6])-128); */
-/*			else */
+/*          if (flip_screen) */
+/*              m92_raster_irq_position=256-(((pf4_control[7]<<8) | pf4_control[6])-128); */
+/*          else */
 				m92_raster_irq_position=((pf4_control[7]<<8) | pf4_control[6])-128;
-/*			if (offset==7) */
-/*				logerror("%06x: Raster %d %d\n",cpu_get_pc(),offset, m92_raster_irq_position); */
+/*          if (offset==7) */
+/*              logerror("%06x: Raster %d %d\n",activecpu_get_pc(),offset, m92_raster_irq_position); */
 			break;
 	}
 }
 
 /*****************************************************************************/
 
-int m92_vh_start(void)
+VIDEO_START( m92 )
 {
-	if (RYPELEO_SPEEDUP) {
-		pf1_hlayer = tilemap_create(
-			get_pf1_htile_info,tilemap_scan_rows,
-			TILEMAP_SPLIT,
-			8,8,
-			64,64
-		);
-		pf1_layer = tilemap_create(
-				get_pf1_ltile_info,tilemap_scan_rows,
-				TILEMAP_SPLIT,
-				8,8,
-				64,64
-			);
-		tilemap_set_transmask(pf1_hlayer,0,0xffff,0x0001);
-		tilemap_set_transmask(pf1_hlayer,1,0x00ff,0xff01);
-		tilemap_set_transmask(pf1_hlayer,2,0x0001,0xffff);
-	}
-	else
-		pf1_layer = tilemap_create(
-			get_pf1_tile_info,tilemap_scan_rows,
-			TILEMAP_SPLIT,
-			8,8,
-			64,64
-		);
+
+	pf1_layer = tilemap_create(
+		get_pf1_tile_info,tilemap_scan_rows,
+		TILEMAP_SPLIT,
+		8,8,
+		64,64
+	);
 
 	pf2_layer = tilemap_create(
 		get_pf2_tile_info,tilemap_scan_rows,
@@ -458,7 +390,7 @@ int m92_vh_start(void)
 		128,64
 	);
 
-	paletteram = malloc(0x1000);
+	paletteram = auto_malloc(0x1000);
 
 	if (!pf1_layer || !pf2_layer || !pf3_layer || !pf1_wide_layer || !pf3_wide_layer || !paletteram)
 		return 1;
@@ -490,17 +422,39 @@ int m92_vh_start(void)
 	memset(spriteram,0,0x800);
 	memset(buffered_spriteram,0,0x800);
 
-	return 0;
-}
+	state_save_register_int  ("video", 0, "pf1_vram_ptr",            &pf1_vram_ptr);
+	state_save_register_int  ("video", 0, "pf1_shape",               &pf1_shape);
+	state_save_register_int  ("video", 0, "pf1_enable",              &pf1_enable);
+	state_save_register_int  ("video", 0, "pf1_rowscroll",           &pf1_rowscroll);
+	state_save_register_UINT8("video", 0, "pf1_control",    (UINT8*) pf1_control, sizeof(pf1_control));
 
-void m92_vh_stop(void)
-{
-	free(paletteram);
+	state_save_register_int  ("video", 0, "pf2_vram_ptr",            &pf2_vram_ptr);
+	state_save_register_int  ("video", 0, "pf2_shape",               &pf2_shape);
+	state_save_register_int  ("video", 0, "pf2_enable",              &pf2_enable);
+	state_save_register_int  ("video", 0, "pf2_rowscroll",           &pf2_rowscroll);
+	state_save_register_UINT8("video", 0, "pf2_control",    (UINT8*) pf2_control, sizeof(pf2_control));
+
+	state_save_register_int  ("video", 0, "pf3_vram_ptr",            &pf3_vram_ptr);
+	state_save_register_int  ("video", 0, "pf3_shape",               &pf3_shape);
+	state_save_register_int  ("video", 0, "pf3_enable",              &pf3_enable);
+	state_save_register_int  ("video", 0, "pf3_rowscroll",           &pf3_rowscroll);
+	state_save_register_UINT8("video", 0, "pf3_control",    (UINT8*) pf3_control, sizeof(pf3_control));
+
+	state_save_register_UINT8("video", 0, "pf4_control",    (UINT8*) pf4_control, sizeof(pf4_control));
+
+	state_save_register_int  ("video", 0, "m92_sprite_list",         &m92_sprite_list);
+	state_save_register_int  ("video", 0, "m92_raster_irq_position", &m92_raster_irq_position);
+	state_save_register_int  ("video", 0, "m92_sprite_buffer_busy",  &m92_sprite_buffer_busy);
+	state_save_register_int  ("video", 0, "m92_palette_bank",        &m92_palette_bank);
+
+	state_save_register_UINT8("video", 0, "paletteram",              paletteram, 0x1000);
+
+	return 0;
 }
 
 /*****************************************************************************/
 
-static void m92_drawsprites(struct mame_bitmap *bitmap, const struct rectangle *clip)
+static void m92_drawsprites(mame_bitmap *bitmap, const rectangle *cliprect)
 {
 	int offs=0;
 
@@ -526,7 +480,8 @@ static void m92_drawsprites(struct mame_bitmap *bitmap, const struct rectangle *
 		y_multi=1 << y_multi; /* 1, 2, 4 or 8 */
 		x_multi=1 << x_multi; /* 1, 2, 4 or 8 */
 
-		if (fx && x_multi>1) x+=16;
+		if (fx) x+=16 * (x_multi - 1);
+
 		for (j=0; j<x_multi; j++)
 		{
 			s_ptr=8 * j;
@@ -543,14 +498,14 @@ static void m92_drawsprites(struct mame_bitmap *bitmap, const struct rectangle *
 							colour,
 							ffx,ffy,
 							496-x,496-(y-i*16),
-							clip,TRANSPARENCY_PEN,0,pri);
+							cliprect,TRANSPARENCY_PEN,0,pri);
 				} else {
 					pdrawgfx(bitmap,Machine->gfx[1],
 							sprite + s_ptr,
 							colour,
 							fx,fy,
 							x,y-i*16,
-							clip,TRANSPARENCY_PEN,0,pri);
+							cliprect,TRANSPARENCY_PEN,0,pri);
 				}
 				if (fy) s_ptr++; else s_ptr--;
 			}
@@ -562,39 +517,20 @@ static void m92_drawsprites(struct mame_bitmap *bitmap, const struct rectangle *
 
 /*****************************************************************************/
 
-void m92_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
-{
-	/* Screen refresh is handled by raster interrupt routine, here
-		we just check the keyboard */
-	if (keyboard_pressed_memory(KEYCODE_F1)) {
-		m92_raster_enable ^= 1;
-		if (m92_raster_enable)
-			usrintf_showmessage("Raster IRQ enabled");
-		else
-			usrintf_showmessage("Raster IRQ disabled");
-	}
-
-	/* Flipscreen appears hardwired to the dipswitch - strange */
-	if (readinputport(5)&1)
-		flip_screen_set(0);
-	else
-		flip_screen_set(1);
-}
-
 static void m92_update_scroll_positions(void)
 {
 	int i,pf1_off,pf2_off,pf3_off;
 
-	/*	Playfield 3 rowscroll data is 0xdfc00 - 0xdffff
-		Playfield 2 rowscroll data is 0xdf800 - 0xdfbff
-		Playfield 1 rowscroll data is 0xdf400 - 0xdf7ff
+	/*  Playfield 3 rowscroll data is 0xdfc00 - 0xdffff
+        Playfield 2 rowscroll data is 0xdf800 - 0xdfbff
+        Playfield 1 rowscroll data is 0xdf400 - 0xdf7ff
 
-		It appears to be hardwired to those locations.
+        It appears to be hardwired to those locations.
 
-		In addition, each playfield is staggered 2 pixels horizontally from the
-		previous one.  This is most obvious in Hook & Blademaster.
+        In addition, each playfield is staggered 2 pixels horizontally from the
+        previous one.  This is most obvious in Hook & Blademaster.
 
-	*/
+    */
 
 	if (flip_screen) {
 		pf1_off=-25;
@@ -648,68 +584,53 @@ static void m92_update_scroll_positions(void)
 	tilemap_set_scrolly( pf3_layer,0, (pf3_control[1]<<8)+pf3_control[0] );
 	tilemap_set_scrolly( pf1_wide_layer,0, (pf1_control[1]<<8)+pf1_control[0] );
 	tilemap_set_scrolly( pf3_wide_layer,0, (pf3_control[1]<<8)+pf3_control[0] );
-
-	if (RYPELEO_SPEEDUP) {
-		tilemap_set_scroll_rows(pf1_hlayer,1);
-		tilemap_set_scrollx( pf1_hlayer,0, (pf1_control[5]<<8)+pf1_control[4] );
-		tilemap_set_scrolly( pf1_hlayer,0, (pf1_control[1]<<8)+pf1_control[0] );
-	}
 }
 
 /*****************************************************************************/
 
-static void m92_screenrefresh(struct mame_bitmap *bitmap,const struct rectangle *clip)
+static void m92_screenrefresh(mame_bitmap *bitmap,const rectangle *cliprect)
 {
-	fillbitmap(priority_bitmap,0,NULL);
+	fillbitmap(priority_bitmap,0,cliprect);
 
 	if (pf3_enable) {
-		tilemap_draw(bitmap,pf3_wide_layer,TILEMAP_BACK,0);
-		tilemap_draw(bitmap,pf3_layer,TILEMAP_BACK,0);
+		tilemap_draw(bitmap,cliprect,pf3_wide_layer,TILEMAP_BACK,0);
+		tilemap_draw(bitmap,cliprect,pf3_layer,		TILEMAP_BACK,0);
 	}
 	else
-		fillbitmap(bitmap,Machine->pens[0],clip);
+		fillbitmap(bitmap,Machine->pens[0],cliprect);
 
-	tilemap_draw(bitmap,pf2_layer,TILEMAP_BACK,0);
-	tilemap_draw(bitmap,pf1_wide_layer,TILEMAP_BACK,0);
-	if (RYPELEO_SPEEDUP && pf1_vram_ptr==0xc000)
-		tilemap_draw(bitmap,pf1_hlayer,TILEMAP_BACK,0);
-	else
-		tilemap_draw(bitmap,pf1_layer,TILEMAP_BACK,0);
+	tilemap_draw(bitmap,cliprect,pf3_wide_layer,TILEMAP_FRONT,1);
+	tilemap_draw(bitmap,cliprect,pf3_layer,		TILEMAP_FRONT,1);
 
-	tilemap_draw(bitmap,pf3_wide_layer,TILEMAP_FRONT,1);
-	tilemap_draw(bitmap,pf3_layer,TILEMAP_FRONT,1);
-	tilemap_draw(bitmap,pf2_layer,TILEMAP_FRONT,1);
-	tilemap_draw(bitmap,pf1_wide_layer,TILEMAP_FRONT,1);
-	if (RYPELEO_SPEEDUP && pf1_vram_ptr==0xc000)
-		tilemap_draw(bitmap,pf1_hlayer,TILEMAP_FRONT,1);
-	else
-		tilemap_draw(bitmap,pf1_layer,TILEMAP_FRONT,1);
+	tilemap_draw(bitmap,cliprect,pf2_layer,		TILEMAP_BACK, 0);
+	tilemap_draw(bitmap,cliprect,pf2_layer,		TILEMAP_FRONT,1);
 
-	m92_drawsprites(bitmap,clip);
+	tilemap_draw(bitmap,cliprect,pf1_wide_layer,TILEMAP_BACK, 0);
+	tilemap_draw(bitmap,cliprect,pf1_layer,		TILEMAP_BACK, 0);
+	tilemap_draw(bitmap,cliprect,pf1_wide_layer,TILEMAP_FRONT,1);
+	tilemap_draw(bitmap,cliprect,pf1_layer,		TILEMAP_FRONT,1);
+
+	m92_drawsprites(bitmap,cliprect);
 }
 
-void m92_vh_raster_partial_refresh(struct mame_bitmap *bitmap,int start_line,int end_line)
+VIDEO_UPDATE( m92 )
 {
-	struct rectangle clip;
+	m92_update_scroll_positions();
+	m92_screenrefresh(bitmap,cliprect);
 
-	clip.min_x = 0;/*Machine->visible_area.min_x; */
-	clip.max_x = 511;/*Machine->visible_area.max_x; */
-	clip.min_y = start_line+128;
-	clip.max_y = end_line+128;
-	if (clip.min_y < Machine->visible_area.min_y)
-		clip.min_y = Machine->visible_area.min_y;
-	if (clip.max_y > Machine->visible_area.max_y)
-		clip.max_y = Machine->visible_area.max_y;
-
-	if (clip.max_y > clip.min_y)
-	{
-		m92_update_scroll_positions();
-		if (RYPELEO_SPEEDUP) tilemap_set_clip(pf1_hlayer,&clip);
-		tilemap_set_clip(pf1_layer,&clip);
-		tilemap_set_clip(pf2_layer,&clip);
-		tilemap_set_clip(pf3_layer,&clip);
-		tilemap_set_clip(pf1_wide_layer,&clip);
-		tilemap_set_clip(pf3_wide_layer,&clip);
-		m92_screenrefresh(bitmap,&clip);
+	/* check the keyboard */
+	if (code_pressed_memory(KEYCODE_F1)) {
+		m92_raster_enable ^= 1;
+		if (m92_raster_enable)
+			ui_popup("Raster IRQ enabled");
+		else
+			ui_popup("Raster IRQ disabled");
 	}
+
+	/* Flipscreen appears hardwired to the dipswitch - strange */
+	if (readinputport(5)&1)
+		flip_screen_set(0);
+	else
+		flip_screen_set(1);
 }
+
