@@ -7,20 +7,21 @@
 ***************************************************************************/
 
 #include "driver.h"
+#include "phoenix.h"
 
 
 /* in sndhrdw/pleiads.c */
-WRITE_HANDLER( pleiads_sound_control_c_w );
+WRITE8_HANDLER( pleiads_sound_control_c_w );
 
-static unsigned char *videoram_pg1;
-static unsigned char *videoram_pg2;
-static unsigned char *current_videoram_pg;
+static UINT8 *videoram_pg1;
+static UINT8 *videoram_pg2;
+static UINT8 *current_videoram_pg;
 static int current_videoram_pg_index;
 static int palette_bank;
 static int cocktail_mode;
 static int pleiads_protection_question;
 static int survival_protection_value;
-static struct tilemap *fg_tilemap, *bg_tilemap;
+static tilemap *fg_tilemap, *bg_tilemap;
 
 
 /***************************************************************************
@@ -45,7 +46,7 @@ static struct tilemap *fg_tilemap, *bg_tilemap;
 
 ***************************************************************************/
 
-void phoenix_vh_convert_color_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *color_prom)
+PALETTE_INIT( phoenix )
 {
 	int i;
 	#define COLOR(gfxn,offs) (colortable[Machine->drv->gfxdecodeinfo[gfxn].color_codes_start + offs])
@@ -53,19 +54,20 @@ void phoenix_vh_convert_color_prom(unsigned char *palette, unsigned short *color
 
 	for (i = 0;i < Machine->drv->total_colors;i++)
 	{
-		int bit0,bit1;
+		int bit0,bit1,r,g,b;
 
 
 		bit0 = (color_prom[0] >> 0) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 0) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		r = 0x55 * bit0 + 0xaa * bit1;
 		bit0 = (color_prom[0] >> 2) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 2) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		g = 0x55 * bit0 + 0xaa * bit1;
 		bit0 = (color_prom[0] >> 1) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 1) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		b = 0x55 * bit0 + 0xaa * bit1;
 
+		palette_set_color(i,r,g,b);
 		color_prom++;
 	}
 
@@ -83,7 +85,7 @@ void phoenix_vh_convert_color_prom(unsigned char *palette, unsigned short *color
 	}
 }
 
-void pleiads_vh_convert_color_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *color_prom)
+PALETTE_INIT( pleiads )
 {
 	int i;
 	#define TOTAL_COLORS(gfxn) (Machine->gfx[gfxn]->total_colors * Machine->gfx[gfxn]->color_granularity)
@@ -92,19 +94,20 @@ void pleiads_vh_convert_color_prom(unsigned char *palette, unsigned short *color
 
 	for (i = 0;i < Machine->drv->total_colors;i++)
 	{
-		int bit0,bit1;
+		int bit0,bit1,r,g,b;
 
 
 		bit0 = (color_prom[0] >> 0) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 0) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		r = 0x55 * bit0 + 0xaa * bit1;
 		bit0 = (color_prom[0] >> 2) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 2) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		g = 0x55 * bit0 + 0xaa * bit1;
 		bit0 = (color_prom[0] >> 1) & 0x01;
 		bit1 = (color_prom[Machine->drv->total_colors] >> 1) & 0x01;
-		*(palette++) = 0x55 * bit0 + 0xaa * bit1;
+		b = 0x55 * bit0 + 0xaa * bit1;
 
+		palette_set_color(i,r,g,b);
 		color_prom++;
 	}
 
@@ -159,14 +162,18 @@ static void get_bg_tile_info(int tile_index)
 
 ***************************************************************************/
 
-int phoenix_vh_start(void)
+VIDEO_START( phoenix )
 {
-	if ((videoram_pg1 = malloc(0x1000)) == 0)
+	if ((videoram_pg1 = auto_malloc(0x1000)) == 0)
 		return 1;
 
-	if ((videoram_pg2 = malloc(0x1000)) == 0)
+	if ((videoram_pg2 = auto_malloc(0x1000)) == 0)
 		return 1;
 
+	memory_configure_bank(1, 0, 1, videoram_pg1, 0);
+	memory_configure_bank(1, 1, 1, videoram_pg2, 0);
+	memory_set_bank(1, 0);
+
     current_videoram_pg_index = -1;
 	current_videoram_pg = videoram_pg1;		/* otherwise, hiscore loading crashes */
 
@@ -178,41 +185,24 @@ int phoenix_vh_start(void)
 		return 1;
 
 	tilemap_set_transparent_pen(fg_tilemap,0);
+	tilemap_set_scrolldx(fg_tilemap,0,8);
+	tilemap_set_scrolldy(fg_tilemap,0,48);
+	tilemap_set_scrolldx(bg_tilemap,0,8);
+	tilemap_set_scrolldy(bg_tilemap,0,48);
 
 	return 0;
 }
 
 
-/***************************************************************************
-
-  Stop the video hardware emulation.
-
-***************************************************************************/
-
-void phoenix_vh_stop(void)
-{
-	free(videoram_pg1);
-	free(videoram_pg2);
-
-	videoram_pg1 = 0;
-	videoram_pg2 = 0;
-}
-
-
 /***************************************************************************
 
   Memory handlers
 
 ***************************************************************************/
 
-READ_HANDLER( phoenix_videoram_r )
+WRITE8_HANDLER( phoenix_videoram_w )
 {
-	return current_videoram_pg[offset];
-}
-
-WRITE_HANDLER( phoenix_videoram_w )
-{
-	unsigned char *rom = memory_region(REGION_CPU1);
+	UINT8 *rom = memory_region(REGION_CPU1);
 
 	current_videoram_pg[offset] = data;
 
@@ -229,12 +219,13 @@ WRITE_HANDLER( phoenix_videoram_w )
 }
 
 
-WRITE_HANDLER( phoenix_videoreg_w )
+WRITE8_HANDLER( phoenix_videoreg_w )
 {
     if (current_videoram_pg_index != (data & 1))
 	{
 		/* set memory bank */
 		current_videoram_pg_index = data & 1;
+		memory_set_bank(1, current_videoram_pg_index);
 		current_videoram_pg = current_videoram_pg_index ? videoram_pg2 : videoram_pg1;
 
 		cocktail_mode = current_videoram_pg_index && (input_port_3_r(0) & 0x01);
@@ -252,7 +243,7 @@ WRITE_HANDLER( phoenix_videoreg_w )
 	}
 }
 
-WRITE_HANDLER( pleiads_videoreg_w )
+WRITE8_HANDLER( pleiads_videoreg_w )
 {
     if (current_videoram_pg_index != (data & 1))
 	{
@@ -268,12 +259,12 @@ WRITE_HANDLER( pleiads_videoreg_w )
 
 
 	/* the palette table is at $0420-$042f and is set by $06bc.
-	   Four palette changes by level.  The palette selection is
-	   wrong, but the same paletter is used for both layers. */
+       Four palette changes by level.  The palette selection is
+       wrong, but the same paletter is used for both layers. */
 
-    if (palette_bank != ((~data >> 1) & 3))
+    if (palette_bank != ((data >> 1) & 3))
 	{
-		palette_bank = ((~data >> 1) & 3);
+		palette_bank = ((data >> 1) & 3);
 
 		tilemap_mark_all_tiles_dirty(ALL_TILEMAPS);
 
@@ -287,13 +278,13 @@ WRITE_HANDLER( pleiads_videoreg_w )
 }
 
 
-WRITE_HANDLER( phoenix_scroll_w )
+WRITE8_HANDLER( phoenix_scroll_w )
 {
 	tilemap_set_scrollx(bg_tilemap,0,data);
 }
 
 
-READ_HANDLER( phoenix_input_port_0_r )
+READ8_HANDLER( phoenix_input_port_0_r )
 {
 	if (cocktail_mode)
 		return (input_port_0_r(0) & 0x07) | (input_port_1_r(0) & 0xf8);
@@ -301,7 +292,7 @@ READ_HANDLER( phoenix_input_port_0_r )
 		return input_port_0_r(0);
 }
 
-READ_HANDLER( pleiads_input_port_0_r )
+READ8_HANDLER( pleiads_input_port_0_r )
 {
 	int ret = phoenix_input_port_0_r(0) & 0xf7;
 
@@ -318,13 +309,13 @@ READ_HANDLER( pleiads_input_port_0_r )
 		ret	|= 0x08;
 		break;
 	default:
-		logerror("Unknown protection question %02X at %04X\n", pleiads_protection_question, cpu_get_pc());
+		logerror("Unknown protection question %02X at %04X\n", pleiads_protection_question, activecpu_get_pc());
 	}
 
 	return ret;
 }
 
-READ_HANDLER( survival_input_port_0_r )
+READ8_HANDLER( survival_input_port_0_r )
 {
 	int ret = phoenix_input_port_0_r(0);
 
@@ -336,9 +327,9 @@ READ_HANDLER( survival_input_port_0_r )
 	return ret;
 }
 
-READ_HANDLER( survival_protection_r )
+READ8_HANDLER( survival_protection_r )
 {
-	if (cpu_get_pc() == 0x2017)
+	if (activecpu_get_pc() == 0x2017)
 	{
 		survival_protection_value ^= 1;
 	}
@@ -352,8 +343,8 @@ READ_HANDLER( survival_protection_r )
 
 ***************************************************************************/
 
-void phoenix_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+VIDEO_UPDATE( phoenix )
 {
-	tilemap_draw(bitmap,bg_tilemap,0,0);
-	tilemap_draw(bitmap,fg_tilemap,0,0);
+	tilemap_draw(bitmap,cliprect,bg_tilemap,0,0);
+	tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
 }
