@@ -1,9 +1,9 @@
 /******************************************************************************
 
-	Nintendo 2C03B PPU emulation.
+    Nintendo 2C03B PPU emulation.
 
-	Written by Ernesto Corvi.
-	This code is heavily based on Brad Oliver's MESS implementation.
+    Written by Ernesto Corvi.
+    This code is heavily based on Brad Oliver's MESS implementation.
 
 ******************************************************************************/
 
@@ -12,46 +12,14 @@
 
 /* constant definitions */
 #define BOTTOM_VISIBLE_SCANLINE	239		/* The bottommost visible scanline */
-#define NMI_SCANLINE     		244		/* 244 times Bayou Billy perfectly */
-#define SCANLINES_PER_FRAME		262		/* Total scanlines to account for per frame */
+#define NMI_SCANLINE     		243		/* Triggered at the beginning of this scanline */
 #define VISIBLE_SCREEN_WIDTH	(32*8)	/* Visible screen width */
 #define VISIBLE_SCREEN_HEIGHT	(30*8)	/* Visible screen height */
 #define VIDEORAM_SIZE			0x4000	/* videoram size */
 #define SPRITERAM_SIZE			0x100	/* spriteram size */
+#define SPRITERAM_MASK			(0x100-1)	/* spriteram size */
 #define CHARGEN_NUM_CHARS		512		/* max number of characters handled by the chargen */
 
-/* registers definition */
-enum {
-	PPU_CONTROL0 = 0,
-	PPU_CONTROL1,
-	PPU_STATUS,
-	PPU_SPRITE_ADDRESS,
-	PPU_SPRITE_DATA,
-	PPU_SCROLL,
-	PPU_ADDRESS,
-	PPU_DATA,
-	PPU_MAX_REG
-};
-
-/* bit definitions for (some of) the registers */
-enum {
-	PPU_CONTROL0_INC			= 0x04,
-	PPU_CONTROL0_SPR_SELECT		= 0x08,
-	PPU_CONTROL0_CHR_SELECT		= 0x10,
-	PPU_CONTROL0_SPRITE_SIZE	= 0x20,
-	PPU_CONTROL0_NMI			= 0x80,
-
-	PPU_CONTROL1_DISPLAY_MONO	= 0x01,
-	PPU_CONTROL1_BACKGROUND_L8	= 0x02,
-	PPU_CONTROL1_SPRITES_L8		= 0x04,
-	PPU_CONTROL1_BACKGROUND		= 0x08,
-	PPU_CONTROL1_SPRITES		= 0x10,
-
-	PPU_STATUS_8SPRITES			= 0x20,
-	PPU_STATUS_SPRITE0_HIT		= 0x40,
-	PPU_STATUS_VBLANK			= 0x80
-};
-
 /* default monochromatic colortable */
 pen_t default_colortable_mono[] =
 {
@@ -67,7 +35,7 @@ pen_t default_colortable_mono[] =
 
 /* our chip state */
 typedef struct {
-	struct mame_bitmap		*bitmap;				/* target bitmap */
+	mame_bitmap		*bitmap;				/* target bitmap */
 	UINT8					*videoram;				/* video ram */
 	UINT8					*spriteram;				/* sprite ram */
 	pen_t					*colortable_mono;		/* monochromatic color table modified at run time */
@@ -75,9 +43,9 @@ typedef struct {
 	int						chars_are_dirty;		/* master flag to check if theres any dirty character */
 	void 					*scanline_timer;		/* scanline timer */
 	int						scanline;				/* scanline count */
-	ppu2c03b_scanline_cb	scanline_callback_proc;	/* optinal scanline callback */
-	ppu2c03b_vidaccess_cb	vidaccess_callback_proc;/* optinal video access callback */
-	int						has_videorom;			/* wether we access a video rom or not */
+	ppu2c03b_scanline_cb	scanline_callback_proc;	/* optional scanline callback */
+	ppu2c03b_vidaccess_cb	vidaccess_callback_proc;/* optional video access callback */
+	int						has_videorom;			/* whether we access a video rom or not */
 	int						videorom_banks;			/* number of banks in the videorom (if available) */
 	int						regs[PPU_MAX_REG];		/* registers */
 	int						refresh_data;			/* refresh-related */
@@ -94,6 +62,7 @@ typedef struct {
 	UINT8					*ppu_page[4];			/* ppu pages */
 	int						nes_vram[8];			/* keep track of 8 .5k vram pages to speed things up */
 	int						scan_scale;				/* scan scale */
+	int						scanlines_per_frame;	/* number of scanlines per frame */
 } ppu2c03b_chip;
 
 /* our local copy of the interface */
@@ -102,17 +71,17 @@ static struct ppu2c03b_interface *intf;
 /* chips state - allocated at init time */
 static ppu2c03b_chip *chips = 0;
 
+static void scanline_callback( int num );
+
+void (*ppu_latch)( offs_t offset );
+
 
 /*************************************
  *
- *	PPU Palette Initialization
+ *  PPU Palette Initialization
  *
  *************************************/
-void ppu2c03b_init_palette( unsigned char *palette ) {
-
-#ifndef M_PI
-#define M_PI 			(3.14159265358979323846L)
-#endif
+void ppu2c03b_init_palette( int first_entry ) {
 
 	/* This routine builds a palette using a transformation from */
 	/* the YUV (Y, B-Y, R-Y) to the RGB color space */
@@ -195,9 +164,7 @@ void ppu2c03b_init_palette( unsigned char *palette ) {
 				B = 255;
 
 			/* Round, and set the value */
-			*palette++ = floor(R+.5);
-			*palette++ = floor(G+.5);
-			*palette++ = floor(B+.5);
+			palette_set_color(first_entry++, floor(R+.5), floor(G+.5), floor(B+.5));
 		}
 	}
 
@@ -205,7 +172,7 @@ void ppu2c03b_init_palette( unsigned char *palette ) {
 }
 
 /* the charlayout we use for the chargen */
-static struct GfxLayout ppu_charlayout =
+static gfx_layout ppu_charlayout =
 {
 	8,8,	/* 8*8 characters */
 	512,	/* 512 characters - modified at runtime */
@@ -218,7 +185,7 @@ static struct GfxLayout ppu_charlayout =
 
 /*************************************
  *
- *	PPU Initialization and Disposal
+ *  PPU Initialization and Disposal
  *
  *************************************/
 int ppu2c03b_init( struct ppu2c03b_interface *interface )
@@ -232,7 +199,7 @@ int ppu2c03b_init( struct ppu2c03b_interface *interface )
 	if ( intf->num <= 0 )
 		return -1;
 
-	chips = malloc( intf->num * sizeof( ppu2c03b_chip ) );
+	chips = auto_malloc( intf->num * sizeof( ppu2c03b_chip ) );
 
 	if ( chips == 0 )
 		return -1;
@@ -241,24 +208,20 @@ int ppu2c03b_init( struct ppu2c03b_interface *interface )
 	for( i = 0; i < intf->num; i++ )
 	{
 		/* initialize the scanline handling portion */
-		chips[i].scanline_timer = 0;
+		chips[i].scanline_timer = timer_alloc(scanline_callback);
 		chips[i].scanline = 0;
 		chips[i].scan_scale = 1;
 
 		/* allocate a screen bitmap, videoram and spriteram, a dirtychar array and the monochromatic colortable */
-		chips[i].bitmap = bitmap_alloc( VISIBLE_SCREEN_WIDTH, VISIBLE_SCREEN_HEIGHT );
-		chips[i].videoram = malloc( VIDEORAM_SIZE );
-		chips[i].spriteram = malloc( SPRITERAM_SIZE );
-		chips[i].dirtychar = malloc( CHARGEN_NUM_CHARS );
-		chips[i].colortable_mono = malloc( sizeof( default_colortable_mono ) );
+		chips[i].bitmap = auto_bitmap_alloc( VISIBLE_SCREEN_WIDTH, VISIBLE_SCREEN_HEIGHT );
+		chips[i].videoram = auto_malloc( VIDEORAM_SIZE );
+		chips[i].spriteram = auto_malloc( SPRITERAM_SIZE );
+		chips[i].dirtychar = auto_malloc( CHARGEN_NUM_CHARS );
+		chips[i].colortable_mono = auto_malloc( sizeof( default_colortable_mono ) );
 
 		/* see if it failed */
 		if ( !chips[i].bitmap || !chips[i].videoram || !chips[i].spriteram || !chips[i].dirtychar || !chips[i].colortable_mono )
-		{
-			/* if we did fail, release everything allocated so far and return */
-			ppu2c03b_dispose();
 			return -1;
-		}
 
 		/* clear videoram & spriteram */
 		memset( chips[i].videoram, 0, VIDEORAM_SIZE );
@@ -293,11 +256,7 @@ int ppu2c03b_init( struct ppu2c03b_interface *interface )
 			Machine->gfx[intf->gfx_layout_number[i]] = decodegfx( src, &ppu_charlayout );
 
 			if ( Machine->gfx[intf->gfx_layout_number[i]] == 0 )
-			{
-				/* failed */
-				ppu2c03b_dispose();
 				return -1;
-			}
 
 			if ( Machine->remapped_colortable )
 				Machine->gfx[intf->gfx_layout_number[i]]->colortable = &Machine->remapped_colortable[intf->color_base[i]];
@@ -313,57 +272,10 @@ int ppu2c03b_init( struct ppu2c03b_interface *interface )
 	return 0;
 }
 
-void ppu2c03b_dispose( void )
-{
-	int i;
-
-	/* clean up */
-	if ( chips ) {
-
-		/* iterate through the virtual chips and free storage */
-		for( i = 0; i < intf->num; i++ )
-		{
-			/* release the bitmap(s) */
-			if ( chips[i].bitmap )
-				bitmap_free( chips[i].bitmap );
-			chips[i].bitmap = 0;
-
-			/* release the videoram */
-			if ( chips[i].videoram )
-				free( chips[i].videoram );
-			chips[i].videoram = 0;
-
-			/* release the spriteram */
-			if ( chips[i].spriteram )
-				free( chips[i].spriteram );
-			chips[i].spriteram = 0;
-
-			/* release the dirtychar array */
-			if ( chips[i].dirtychar )
-				free( chips[i].dirtychar );
-			chips[i].dirtychar = 0;
-
-			/* release the colortable_mono array */
-			if ( chips[i].colortable_mono )
-				free( chips[i].colortable_mono );
-			chips[i].colortable_mono = 0;
-
-			/* release the timer */
-			if ( chips[i].scanline_timer )
-				timer_remove( chips[i].scanline_timer );
-			chips[i].scanline_timer = 0;
-		}
-
-		/* dispose our chips states */
-		free( chips );
-		chips = 0;
-	}
-}
-
 static void draw_background( const int num, UINT8 *line_priority )
 {
 	/* cache some values locally */
-	struct mame_bitmap *bitmap = chips[num].bitmap;
+	mame_bitmap *bitmap = chips[num].bitmap;
 	const int *ppu_regs = &chips[num].regs[0];
 	const int scanline = chips[num].scanline;
 	const int refresh_data = chips[num].refresh_data;
@@ -376,7 +288,8 @@ static void draw_background( const int num, UINT8 *line_priority )
 	UINT8 *gfx_data = Machine->gfx[gfx_bank]->gfxdata;
 	UINT8 **ppu_page = chips[num].ppu_page;
 	int	start_x = ( chips[num].x_fine ^ 0x07 ) - 7;
-	UINT32 back_pen;
+	UINT16 back_pen;
+	UINT16 *dest;
 
 	UINT8 scroll_x_coarse, scroll_y_coarse, scroll_y_fine, color_mask;
 	int x, tile_index, start, i;
@@ -385,6 +298,8 @@ static void draw_background( const int num, UINT8 *line_priority )
 	const pen_t *paldata;
 	const UINT8 *sd;
 
+	int tilecount=0;
+
 	/* setup the color mask and colortable to use */
 	if ( ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_DISPLAY_MONO )
 	{
@@ -411,8 +326,11 @@ static void draw_background( const int num, UINT8 *line_priority )
 	/* get the tile index */
 	tile_index = ( ( refresh_data & 0xc00 ) | 0x2000 ) + scroll_y_coarse * 32;
 
+	/* set up dest */
+	dest = ((UINT16 *) bitmap->base) + (bitmap->rowpixels * scanline) + start_x;
+
 	/* draw the 32 or 33 tiles that make up a line */
-	while ( start_x < VISIBLE_SCREEN_WIDTH )
+	while ( tilecount <34)
 	{
 		int color_byte;
 		int color_bits;
@@ -420,6 +338,7 @@ static void draw_background( const int num, UINT8 *line_priority )
 		int index1;
 		int page, page2, address;
 		int index2;
+		UINT16 pen;
 
 		index1 = tile_index + x;
 
@@ -436,52 +355,66 @@ static void draw_background( const int num, UINT8 *line_priority )
 		page2 = ppu_page[page][address];
 		index2 = nes_vram[ ( page2 >> 6 ) | tile_page ] + ( page2 & 0x3f );
 
-		paldata = &color_table[ 4 * ( ( ( color_byte >> color_bits ) & 0x03 ) ) ];
-		start = ( index2 % total_elements ) * char_modulo + scroll_y_fine * line_modulo;
-		sd = &gfx_data[start];
-
-		/* render the pixel */
-		for( i = 0; i < 8; i++ )
+		/*27/12/2002 */
+		if( ppu_latch )
 		{
-			if ( ( start_x+i ) >= 0 && ( start_x+i ) < VISIBLE_SCREEN_WIDTH )
+			(*ppu_latch)(( tile_page << 10 ) | ( page2 << 4 ));
+		}
+
+		if(start_x < VISIBLE_SCREEN_WIDTH )
+		{
+			paldata = &color_table[ 4 * ( ( ( color_byte >> color_bits ) & 0x03 ) ) ];
+			start = ( index2 % total_elements ) * char_modulo + scroll_y_fine * line_modulo;
+			sd = &gfx_data[start];
+
+			/* render the pixel */
+			for( i = 0; i < 8; i++ )
 			{
-				if ( sd[i] )
+				if ( ( start_x+i ) >= 0 && ( start_x+i ) < VISIBLE_SCREEN_WIDTH )
 				{
-					plot_pixel( bitmap, start_x+i, scanline, paldata[ sd[ i ] ] );
-					line_priority[ start_x+i ] |= 0x02;
-				}
-				else
-				{
-					plot_pixel( bitmap, start_x+i, scanline, back_pen );
+					if ( sd[i] )
+					{
+						pen = paldata[sd[i]];
+						line_priority[ start_x+i ] |= 0x02;
+					}
+					else
+					{
+						pen = back_pen;
+					}
+					*dest = pen;
 				}
+				dest++;
+			}
+
+			start_x += 8;
+
+			/* move to next tile over and toggle the horizontal name table if necessary */
+			x++;
+			if ( x > 31 )
+			{
+				x = 0;
+				tile_index ^= 0x400;
 			}
 		}
-
-		start_x += 8;
-
-		/* move to next tile over and toggle the horizontal name table if necessary */
-		x++;
-		if ( x > 31 )
-		{
-			x = 0;
-			tile_index ^= 0x400;
-		}
+		tilecount++;
 	}
-
 	/* if the left 8 pixels for the background are off, blank 'em */
 	if ( !( ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_BACKGROUND_L8 ) )
 	{
+		dest = ((UINT16 *) bitmap->base) + (bitmap->rowpixels * scanline);
 		for( i = 0; i < 8; i++ )
-			plot_pixel( bitmap, i, scanline, back_pen );
+		{
+			*(dest++) = back_pen;
+			line_priority[ i ] ^= 0x02;
+		}
 	}
-
 	/* done updating, whew */
 }
 
 static void draw_sprites( const int num, UINT8 *line_priority )
 {
 	/* cache some values locally */
-	struct mame_bitmap *bitmap = chips[num].bitmap;
+	mame_bitmap *bitmap = chips[num].bitmap;
 	const int scanline = chips[num].scanline;
 	const int gfx_bank = intf->gfx_layout_number[num];
 	const int total_elements = Machine->gfx[gfx_bank]->total_elements;
@@ -504,12 +437,16 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 	int drawn;
 	int start;
 
+	int first_pixel;
+
 	const pen_t *paldata;
 	const UINT8 *sd;
 
 	/* determine if the sprites are 8x8 or 8x16 */
 	size = ( ppu_regs[PPU_CONTROL0] & PPU_CONTROL0_SPRITE_SIZE ) ? 16 : 8;
 
+	first_pixel= (ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_SPRITES_L8)? 0: 8;
+
 	for( i = 0; i < SPRITERAM_SIZE; i += 4 ) {
 		y = sprites[i] + 1;
 
@@ -542,6 +479,10 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 
 		index1 = chips[num].nes_vram[page] + ( tile & 0x3f );
 
+		/*27/12/2002 */
+		if ( ppu_latch )
+			(*ppu_latch)(( sprite_page << 10 ) | ( (tile & 0xff) << 4 ));
+
 		/* compute the character's line to draw */
 		sprite_line = scanline - y;
 
@@ -562,7 +503,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 				for ( j = 0; j < 8; j++ )
 				{
 					/* is this pixel non-transparent? */
-					if ( sd[7-j] )
+					if ( sd[7-j] && ((x+j)>= first_pixel))
 					{
 						/* has the background (or another sprite) already been drawn here? */
 						if ( !line_priority[ x + j ] )
@@ -577,7 +518,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 							line_priority[ x + j ] |= 0x01;
 
 						/* set the "sprite 0 hit" flag if appropriate */
-						if ( i == 0 )
+						if ( i == 0 && ((x+j)<255) && ( line_priority[ x + j ] & 0x02 ))
 							ppu_regs[PPU_STATUS] |= PPU_STATUS_SPRITE0_HIT;
 					}
 				}
@@ -587,7 +528,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 				for ( j = 0; j < 8; j++ )
 				{
 					/* is this pixel non-transparent? */
-					if ( sd[j] )
+					if ( sd[j] && ((x+j)>= first_pixel))
 					{
 						/* has the background (or another sprite) already been drawn here? */
 						if ( !line_priority[ x + j ] )
@@ -602,7 +543,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 							line_priority[ x + j ] |= 0x01;
 
 						/* set the "sprite 0 hit" flag if appropriate */
-						if ( i == 0 )
+						if ( i == 0 && ((x+j)<255) && ( line_priority[ x + j ] & 0x02 ))
 							ppu_regs[PPU_STATUS] |= PPU_STATUS_SPRITE0_HIT;
 					}
 				}
@@ -618,7 +559,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 				for ( j = 0; j < 8; j++ )
 				{
 					/* is this pixel non-transparent? */
-					if ( sd[7-j] )
+					if ( sd[7-j] && ((x+j)>= first_pixel))
 					{
 						/* has another sprite been drawn here? */
 						if ( !( line_priority[ x + j ] & 0x01 ) )
@@ -633,7 +574,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 						}
 
 						/* set the "sprite 0 hit" flag if appropriate */
-						if ( ( i == 0 ) && ( line_priority[ x + j ] & 0x02 ) )
+						if ( ( i == 0 ) && ((x+j)<255) && ( line_priority[ x + j ] & 0x02 ) )
 							ppu_regs[PPU_STATUS] |= PPU_STATUS_SPRITE0_HIT;
 					}
 				}
@@ -643,7 +584,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 				for ( j = 0; j < 8; j++ )
 				{
 					/* is this pixel non-transparent? */
-					if ( sd[j] )
+					if ( sd[j] && ((x+j)>= first_pixel))
 					{
 						/* has another sprite been drawn here? */
 						if ( !( line_priority[ x + j ] & 0x01 ) )
@@ -658,7 +599,7 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 						}
 
 						/* set the "sprite 0 hit" flag if appropriate */
-						if ( ( i == 0 ) && ( line_priority[ x + j ] & 0x02 ) )
+						if ( ( i == 0 ) && ((x+j)<255) && ( line_priority[ x + j ] & 0x02 ) )
 							ppu_regs[PPU_STATUS] |= PPU_STATUS_SPRITE0_HIT;
 					}
 				}
@@ -683,14 +624,14 @@ static void draw_sprites( const int num, UINT8 *line_priority )
 
 /*************************************
  *
- *	Scanline Rendering and Update
+ *  Scanline Rendering and Update
  *
  *************************************/
 static void render_scanline( int num )
 {
 	UINT8	line_priority[VISIBLE_SCREEN_WIDTH];
 	int		*ppu_regs = &chips[num].regs[0];
-	int 	i;
+/*  int     i; */
 	int		refresh_data = chips[num].refresh_data;
 
 	/* lets see how long it takes */
@@ -706,13 +647,6 @@ static void render_scanline( int num )
 	if ( ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_BACKGROUND )
 		draw_background( num, line_priority );
 
-	/* if sprites are hidden in the leftmost column, fake a priority flag to mask them */
-	if ( !( ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_SPRITES_L8 ) )
-	{
-		for ( i = 0; i < 8; i++ )
-			line_priority[i] |= 0x01;
-	}
-
 	/* if sprites are on, draw them */
 	if ( ppu_regs[PPU_CONTROL1] & PPU_CONTROL1_SPRITES )
 		draw_sprites( num, line_priority );
@@ -768,22 +702,20 @@ static void update_scanline( int num )
 		/* Render this scanline if appropriate */
 		if ( ppu_regs[PPU_CONTROL1] & ( PPU_CONTROL1_BACKGROUND | PPU_CONTROL1_SPRITES ) )
 			render_scanline( num );
-
-		return;
 	}
 
-	if ( scanline == BOTTOM_VISIBLE_SCANLINE+1 )
+	/* Note: this routine is called at the _end_ of each scanline */
+	if ( scanline == BOTTOM_VISIBLE_SCANLINE )
 	{
 		/* We just entered VBLANK */
 		ppu_regs[PPU_STATUS] |= PPU_STATUS_VBLANK;
-		return;
 	}
 
 	/* If NMI's are set to be triggered, go for it */
-	if ( ( scanline == NMI_SCANLINE ) && ( ppu_regs[PPU_CONTROL0] & PPU_CONTROL0_NMI ) )
+	if ( ( scanline == NMI_SCANLINE-1 ) && ( ppu_regs[PPU_CONTROL0] & PPU_CONTROL0_NMI ) )
 	{
-		if ( intf->handler[num] )
-			(*intf->handler[num])( num );
+		if ( intf->nmi_handler[num] )
+			(*intf->nmi_handler[num])( num, ppu_regs );
 	}
 }
 
@@ -794,7 +726,7 @@ static void scanline_callback( int num )
 	int blanked = ( ppu_regs[PPU_CONTROL1] & ( PPU_CONTROL1_BACKGROUND | PPU_CONTROL1_SPRITES ) ) == 0;
 	int vblank = ( ppu_regs[PPU_STATUS] & PPU_STATUS_VBLANK ) ? 1 : 0;
 
-/*	logerror("SCANLINE CALLBACK %d\n",chips[num].scanline); */
+/*  logerror("SCANLINE CALLBACK %d\n",chips[num].scanline); */
 
 	/* if a callback is available, call it */
 	if ( chips[num].scanline_callback_proc )
@@ -814,7 +746,7 @@ static void scanline_callback( int num )
 		/* cache some values */
 		UINT8 *dirtyarray = chips[num].dirtychar;
 		UINT8 *vram = chips[num].videoram;
-		struct GfxElement *gfx = Machine->gfx[intf->gfx_layout_number[num]];
+		gfx_element *gfx = Machine->gfx[intf->gfx_layout_number[num]];
 
 		/* then iterate and decode */
 		for( i = 0; i < CHARGEN_NUM_CHARS; i++ )
@@ -830,7 +762,7 @@ static void scanline_callback( int num )
 	}
 
 	/* see if we rolled */
-	if ( chips[num].scanline >= SCANLINES_PER_FRAME )
+	if ( chips[num].scanline >= chips[num].scanlines_per_frame )
 	{
 		/* clear the vblank & sprite hit flag */
 		ppu_regs[PPU_STATUS] &= ~( PPU_STATUS_VBLANK | PPU_STATUS_SPRITE0_HIT );
@@ -844,12 +776,12 @@ static void scanline_callback( int num )
 	}
 
 	/* setup our next stop here */
-	chips[num].scanline_timer = timer_set( cpu_getscanlinetime( chips[num].scanline * chips[num].scan_scale ), num, scanline_callback );
+	timer_adjust(chips[num].scanline_timer, cpu_getscanlinetime( chips[num].scanline * chips[num].scan_scale ), num, 0);
 }
 
 /*************************************
  *
- *	PPU Reset
+ *  PPU Reset
  *
  *************************************/
 void ppu2c03b_reset( int num, int scan_scale )
@@ -863,10 +795,6 @@ void ppu2c03b_reset( int num, int scan_scale )
 		return;
 	}
 
-	/* check if we have a previously allocated scanline timer */
-	if ( chips[num].scanline_timer )
-		timer_remove( chips[num].scanline_timer );
-
 	/* reset the scanline count */
 	chips[num].scanline = 0;
 
@@ -874,7 +802,7 @@ void ppu2c03b_reset( int num, int scan_scale )
 	chips[num].scan_scale = scan_scale;
 
 	/* allocate the scanline timer - start at scanline 0 */
-	chips[num].scanline_timer = timer_set( cpu_getscanlinetime(0), num, scanline_callback );
+	timer_adjust(chips[num].scanline_timer, cpu_getscanlinetime(0), num, 0);
 
 	/* reset the callbacks */
 	chips[num].scanline_callback_proc = 0;
@@ -896,6 +824,7 @@ void ppu2c03b_reset( int num, int scan_scale )
 	chips[num].sprite_page = 0;
 	chips[num].back_color = 0;
 	chips[num].chars_are_dirty = 1;
+	chips[num].scanlines_per_frame = 262;	/* default */
 
 	/* initialize the color tables */
 	{
@@ -922,7 +851,7 @@ void ppu2c03b_reset( int num, int scan_scale )
 
 /*************************************
  *
- *	PPU Registers Read
+ *  PPU Registers Read
  *
  *************************************/
 int ppu2c03b_r( int num, int offset )
@@ -941,11 +870,11 @@ int ppu2c03b_r( int num, int offset )
 		logerror( "PPU %d(r): Attempting to read past the chip\n", num );
 
 		offset &= PPU_MAX_REG - 1;
-/*		return 0; */
+/*      return 0; */
 	}
 
 	/* now, see wich register to read */
-	switch( offset )
+	switch( offset & 7 )
 	{
 		case PPU_STATUS:
 			ret = chips[num].regs[PPU_STATUS];
@@ -961,9 +890,16 @@ int ppu2c03b_r( int num, int offset )
 		break;
 
 		case PPU_DATA:
-			ret = chips[num].videoram_data_latch;
+			if (  chips[num].videoram_addr >= 0x3f00  )
+				ret = chips[num].videoram[chips[num].videoram_addr&0x3F1F];
+			else ret = chips[num].videoram_data_latch;
 
-			if ( ( chips[num].videoram_addr >= 0x2000 ) && ( chips[num].videoram_addr <= 0x3fef ) )
+			/*27/12/2002 */
+			if ( ppu_latch )
+				(*ppu_latch)( chips[num].videoram_addr & 0x3fff );
+
+			/*was 3fef, but if it's trying palette ram compensation, it should have been 3eff, and the actual filling of the latch would be wrong...? */
+			if ( ( chips[num].videoram_addr >= 0x2000 ) && ( chips[num].videoram_addr <= 0x3fff ) )
 				chips[num].videoram_data_latch = chips[num].ppu_page[ ( chips[num].videoram_addr & 0xc00) >> 10][ chips[num].videoram_addr & 0x3ff ];
 			else
 				chips[num].videoram_data_latch = chips[num].videoram[ chips[num].videoram_addr & 0x3fff ];
@@ -982,7 +918,7 @@ int ppu2c03b_r( int num, int offset )
 
 /*************************************
  *
- *	PPU Registers Write
+ *  PPU Registers Write
  *
  *************************************/
 void ppu2c03b_w( int num, int offset, int data )
@@ -1000,10 +936,10 @@ void ppu2c03b_w( int num, int offset, int data )
 		logerror( "PPU: Attempting to write past the chip\n" );
 
 		offset &= PPU_MAX_REG - 1;
-/*		return; */
+/*      return; */
 	}
 
-	switch( offset )
+	switch( offset & 7 )
 	{
 		case PPU_CONTROL0:
 			chips[num].regs[PPU_CONTROL0] = data;
@@ -1089,6 +1025,10 @@ void ppu2c03b_w( int num, int offset, int data )
 			{
 				int tempAddr = chips[num].videoram_addr & 0x3fff;
 
+				/*27/12/2002 */
+				if ( ppu_latch )
+					(*ppu_latch)( tempAddr );
+
 				/* if there's a callback, call it now */
 				if ( chips[num].vidaccess_callback_proc )
 					data = (*chips[num].vidaccess_callback_proc)( num, tempAddr, data );
@@ -1122,15 +1062,21 @@ void ppu2c03b_w( int num, int offset, int data )
 					break;
 				}
 
-				/* the only valid background colors are writes to 0x3f00 and 0x3f10			*/
-				/* and even then, they are mirrors of each other. as usual, some games		*/
+				/* the only valid background colors are writes to 0x3f00 and 0x3f10         */
+				/* and even then, they are mirrors of each other. as usual, some games      */
 				/* attempt to write values > the number of colors so we must mask the data. */
 				if ( tempAddr >= 0x3f00 )
 				{
 					int color_base = intf->color_base[num];
 
 					/* store the data */
-					chips[num].videoram[tempAddr] = data;
+					if(tempAddr&0x3)
+						chips[num].videoram[tempAddr&0x3F1F] = data;
+					else
+					{
+						chips[num].videoram[0x3F10+(tempAddr&0xF)] = data;
+						chips[num].videoram[0x3F00+(tempAddr&0xF)] = data;
+					}
 
 					data &= 0x3f;
 
@@ -1183,27 +1129,29 @@ void ppu2c03b_w( int num, int offset, int data )
 
 /*************************************
  *
- *	Sprite DMA
+ *  Sprite DMA
  *
  *************************************/
-void ppu2c03b_spriteram_dma( int num, const UINT8 *source )
+void ppu2c03b_spriteram_dma(const UINT8 page )
 {
-	/* check bounds */
-	if ( num >= intf->num )
-	{
-		logerror( "PPU(render): Attempting to access an unmapped chip\n" );
-		return;
-	}
+	int i;
+	int address=page<<8;
+	/*should last 513 CPU cycles. */
 
-	memcpy( chips[num].spriteram, source, SPRITERAM_SIZE );
+	for(i=0;i<SPRITERAM_SIZE;i++)
+	{
+		UINT8 t=program_read_byte_8(address+i);
+		program_write_byte_8(0x2000+PPU_SPRITE_DATA, t);
+	}
+	activecpu_adjust_icount(-513);
 }
 
 /*************************************
  *
- *	PPU Rendering
+ *  PPU Rendering
  *
  *************************************/
-void ppu2c03b_render( int num, struct mame_bitmap *bitmap, int flipx, int flipy, int sx, int sy )
+void ppu2c03b_render( int num, mame_bitmap *bitmap, int flipx, int flipy, int sx, int sy )
 {
 	/* check bounds */
 	if ( num >= intf->num )
@@ -1212,14 +1160,14 @@ void ppu2c03b_render( int num, struct mame_bitmap *bitmap, int flipx, int flipy,
 		return;
 	}
 
-/*	logerror("PPU %d:VBLANK HIT (scanline %d)\n",num, chips[num].scanline); */
+/*  logerror("PPU %d:VBLANK HIT (scanline %d)\n",num, chips[num].scanline); */
 
 	copybitmap( bitmap, chips[num].bitmap, flipx, flipy, sx, sy, 0, TRANSPARENCY_NONE, 0 );
 }
 
 /*************************************
  *
- *	PPU VideoROM banking
+ *  PPU VideoROM banking
  *
  *************************************/
 void ppu2c03b_set_videorom_bank( int num, int start_page, int num_pages, int bank, int bank_size )
@@ -1255,7 +1203,7 @@ void ppu2c03b_set_videorom_bank( int num, int start_page, int num_pages, int ban
 
 /*************************************
  *
- *	Utility functions
+ *  Utility functions
  *
  *************************************/
 int ppu2c03b_get_pixel( int num, int x, int y )
@@ -1273,7 +1221,7 @@ int ppu2c03b_get_pixel( int num, int x, int y )
 	if ( y >= VISIBLE_SCREEN_HEIGHT )
 		y = VISIBLE_SCREEN_HEIGHT - 1;
 
-	return ((UINT8 *)chips[num].bitmap->line[y])[x];
+	return read_pixel(chips[num].bitmap, x, y);
 }
 
 int ppu2c03b_get_colorbase( int num )
@@ -1338,16 +1286,16 @@ void ppu2c03b_set_mirroring( int num, int mirroring )
 	}
 }
 
-void ppu2c03b_set_irq_callback( int num, ppu2c03b_irq_cb cb )
+void ppu2c03b_set_nmi_callback( int num, ppu2c03b_nmi_cb cb )
 {
 	/* check bounds */
 	if ( num >= intf->num )
 	{
-		logerror( "PPU(set_irq_callback): Attempting to access an unmapped chip\n" );
+		logerror( "PPU(set_nmi_callback): Attempting to access an unmapped chip\n" );
 		return;
 	}
 
-	intf->handler[num] = cb;
+	intf->nmi_handler[num] = cb;
 }
 
 void ppu2c03b_set_scanline_callback( int num, ppu2c03b_scanline_cb cb )
@@ -1374,28 +1322,40 @@ void ppu2c03b_set_vidaccess_callback( int num, ppu2c03b_vidaccess_cb cb )
 	chips[num].vidaccess_callback_proc = cb;
 }
 
+void ppu2c03b_set_scanlines_per_frame( int num, int scanlines )
+{
+	/* check bounds */
+	if ( num >= intf->num )
+	{
+		logerror( "PPU(set_scanlines_per_frame): Attempting to access an unmapped chip\n" );
+		return;
+	}
+
+	chips[num].scanlines_per_frame = scanlines;
+}
+
 /*************************************
  *
- *	Accesors
+ *  Accesors
  *
  *************************************/
 
-READ_HANDLER( ppu2c03b_0_r )
+READ8_HANDLER( ppu2c03b_0_r )
 {
 	return ppu2c03b_r( 0, offset );
 }
 
-READ_HANDLER( ppu2c03b_1_r )
+READ8_HANDLER( ppu2c03b_1_r )
 {
 	return ppu2c03b_r( 1, offset );
 }
 
-WRITE_HANDLER( ppu2c03b_0_w )
+WRITE8_HANDLER( ppu2c03b_0_w )
 {
 	ppu2c03b_w( 0, offset, data );
 }
 
-WRITE_HANDLER( ppu2c03b_1_w )
+WRITE8_HANDLER( ppu2c03b_1_w )
 {
 	ppu2c03b_w( 1, offset, data );
 }
