@@ -1,10 +1,8 @@
-/***************************************************************************
+/*************************************************************************
 
-  vidhrdw.c
+    Sega vector hardware
 
-  Generic functions used by the Sega Vector games
-
-***************************************************************************/
+*************************************************************************/
 
 /*
  * History:
@@ -22,8 +20,6 @@
 #include "vector.h"
 #include <math.h>
 
-#define VEC_SHIFT 15	/* do not use a higher value. Values will overflow */
-
 static int width, height, cent_x, cent_y, min_x, min_y, max_x, max_y;
 static long *sinTable, *cosTable;
 static int intensity;
@@ -48,9 +44,13 @@ void sega_generate_vector_list (void)
 
 	symbolIndex = 0;	/* Reset vector PC to 0 */
 
+	/* Sega games are all clipped to this region at the DAC */
+	vector_add_clip((512-min_x)<<16, (max_y-1536)<<16,
+  	              (1536-min_x)<<16, (max_y-512)<<16);
+
 	/*
-	 * walk the symbol list until 'last symbol' set
-	 */
+     * walk the symbol list until 'last symbol' set
+     */
 
 	do {
 		draw = vectorram[symbolIndex++];
@@ -63,8 +63,8 @@ void sega_generate_vector_list (void)
 			rotate      = vectorram[symbolIndex + 6] | (vectorram[symbolIndex + 7] << 8);
 			scale       = vectorram[symbolIndex + 8];
 
-			currentX = ((currentX & 0x7ff) - min_x) << VEC_SHIFT;
-			currentY = (max_y - (currentY & 0x7ff)) << VEC_SHIFT;
+			currentX = ((currentX & 0x7ff) - min_x) << 16;
+			currentY = (max_y - (currentY & 0x7ff)) << 16;
 			vector_add_point ( currentX, currentY, 0, 0);
 			vectorIndex &= 0xfff;
 
@@ -85,8 +85,8 @@ void sega_generate_vector_list (void)
 				deltax = sinTable[angle] * scale * length;
 				deltay = cosTable[angle] * scale * length;
 
-				currentX += deltax >> 7;
-				currentY -= deltay >> 7;
+				currentX += deltax >> 6;
+				currentY -= deltay >> 6;
 
 				color = VECTOR_COLOR222((attrib >> 1) & 0x3f);
 				if ((attrib & 1) && color)
@@ -110,12 +110,14 @@ void sega_generate_vector_list (void)
 	} while (!(draw & 0x80));
 }
 
+
 /***************************************************************************
 
   Start the video hardware emulation.
 
 ***************************************************************************/
-int sega_vh_start (void)
+
+VIDEO_START( sega )
 {
 	int i;
 
@@ -130,58 +132,37 @@ int sega_vh_start (void)
 	cent_x=(max_x+min_x)/2;
 	cent_y=(max_y+min_y)/2;
 
-	vector_set_shift (VEC_SHIFT);
-
 	/* allocate memory for the sine and cosine lookup tables ASG 080697 */
-	sinTable = malloc (0x400 * sizeof (long));
+	sinTable = auto_malloc (0x400 * sizeof (long));
 	if (!sinTable)
 		return 1;
-	cosTable = malloc (0x400 * sizeof (long));
+	cosTable = auto_malloc (0x400 * sizeof (long));
 	if (!cosTable)
-	{
-		free (sinTable);
 		return 1;
-	}
 
 	/* generate the sine/cosine lookup tables */
 	for (i = 0; i < 0x400; i++)
 	{
-		double angle = ((2. * PI) / (double)0x400) * (double)i;
+		double angle = ((2. * M_PI) / (double)0x400) * (double)i;
 		double temp;
 
 		temp = sin (angle);
 		if (temp < 0)
-			sinTable[i] = (long)(temp * (double)(1 << VEC_SHIFT) - 0.5);
+			sinTable[i] = (long)(temp * (double)(1 << 15) - 0.5);
 		else
-			sinTable[i] = (long)(temp * (double)(1 << VEC_SHIFT) + 0.5);
+			sinTable[i] = (long)(temp * (double)(1 << 15) + 0.5);
 
 		temp = cos (angle);
 		if (temp < 0)
-			cosTable[i] = (long)(temp * (double)(1 << VEC_SHIFT) - 0.5);
+			cosTable[i] = (long)(temp * (double)(1 << 15) - 0.5);
 		else
-			cosTable[i] = (long)(temp * (double)(1 << VEC_SHIFT) + 0.5);
-
+			cosTable[i] = (long)(temp * (double)(1 << 15) + 0.5);
 	}
 
-	return vector_vh_start();
+	return video_start_vector();
 }
 
-/***************************************************************************
 
-  Stop the video hardware emulation.
-
-***************************************************************************/
-void sega_vh_stop (void)
-{
-	if (sinTable)
-		free (sinTable);
-	sinTable = NULL;
-	if (cosTable)
-		free (cosTable);
-	cosTable = NULL;
-
-	vector_vh_stop();
-}
 
 /***************************************************************************
 
@@ -190,8 +171,9 @@ void sega_vh_stop (void)
   the main emulation engine.
 
 ***************************************************************************/
-void sega_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+
+VIDEO_UPDATE( sega )
 {
 	sega_generate_vector_list();
-	vector_vh_screenrefresh(bitmap,full_refresh);
+	video_update_vector(screen,bitmap,0);
 }
