@@ -3,13 +3,13 @@
 #include "konamiic.h"
 
 
-data16_t *tail2nos_bgvideoram;
+UINT16 *tail2nos_bgvideoram;
 
 
-static struct tilemap *bg_tilemap;
+static tilemap *bg_tilemap;
 
 static int charbank,charpalette,video_enable;
-static data16_t *zoomdata;
+static UINT16 *zoomdata;
 static int dirtygfx;
 static unsigned char *dirtychar;
 
@@ -47,11 +47,11 @@ static void zoom_callback(int *code,int *color)
 
 /***************************************************************************
 
-	Start the video hardware emulation.
+    Start the video hardware emulation.
 
 ***************************************************************************/
 
-int tail2nos_vh_start(void)
+VIDEO_START( tail2nos )
 {
 	bg_tilemap = tilemap_create(get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,32);
 
@@ -61,29 +61,19 @@ int tail2nos_vh_start(void)
 	if (K051316_vh_start_0(REGION_GFX3,4,TILEMAP_OPAQUE,0,zoom_callback))
 		return 1;
 
-	if (!(dirtychar = malloc(TOTAL_CHARS)))
-	{
-		K051316_vh_stop_0();
+	if (!(dirtychar = auto_malloc(TOTAL_CHARS)))
 		return 1;
-	}
 	memset(dirtychar,1,TOTAL_CHARS);
 
 	tilemap_set_transparent_pen(bg_tilemap,15);
 
 	K051316_wraparound_enable(0,1);
 	K051316_set_offset(0,-89,-14);
-	zoomdata = (data16_t *)memory_region(REGION_GFX3);
+	zoomdata = (UINT16 *)memory_region(REGION_GFX3);
 
 	return 0;
 }
 
-void tail2nos_vh_stop(void)
-{
-	K051316_vh_stop_0();
-	free(dirtychar);
-	dirtychar = 0;
-}
-
 
 
 /***************************************************************************
@@ -151,11 +141,11 @@ WRITE16_HANDLER( tail2nos_gfxbank_w )
 
 /***************************************************************************
 
-	Display Refresh
+    Display Refresh
 
 ***************************************************************************/
 
-static void drawsprites(struct mame_bitmap *bitmap)
+static void drawsprites(mame_bitmap *bitmap,const rectangle *cliprect)
 {
 	int offs;
 
@@ -178,13 +168,13 @@ static void drawsprites(struct mame_bitmap *bitmap)
 				40 + color,
 				flipx,flipy,
 				sx+3,sy+1,	/* placement relative to zoom layer verified on the real thing */
-				&Machine->visible_area,TRANSPARENCY_PEN,15);
+				cliprect,TRANSPARENCY_PEN,15);
 	}
 }
 
-void tail2nos_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+VIDEO_UPDATE( tail2nos )
 {
-	static struct GfxLayout tilelayout =
+	static const gfx_layout tilelayout =
 	{
 		16,16,
 		TOTAL_CHARS,
@@ -224,10 +214,10 @@ void tail2nos_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
 
 	if (video_enable)
 	{
-		K051316_zoom_draw_0(bitmap,0,0);
-		drawsprites(bitmap);
-		tilemap_draw(bitmap,bg_tilemap,0,0);
+		K051316_zoom_draw_0(bitmap,cliprect,0,0);
+		drawsprites(bitmap,cliprect);
+		tilemap_draw(bitmap,cliprect,bg_tilemap,0,0);
 	}
 	else
-		fillbitmap(bitmap,Machine->pens[0],&Machine->visible_area);
+		fillbitmap(bitmap,Machine->pens[0],cliprect);
 }
