@@ -20,17 +20,19 @@ unsigned char *taitosj_scroll;
 unsigned char *taitosj_colscrolly;
 unsigned char *taitosj_gfxpointer;
 unsigned char *taitosj_colorbank,*taitosj_video_priority;
+unsigned char *kikstart_scrollram;
+unsigned char *taitosj_spritebank;
 static unsigned char taitosj_collision_reg[4];
 static unsigned char *dirtybuffer2,*dirtybuffer3;
-static struct mame_bitmap *taitosj_tmpbitmap[3];
-static struct mame_bitmap *sprite_sprite_collbitmap1,*sprite_sprite_collbitmap2;
-static struct mame_bitmap *sprite_plane_collbitmap1;
-static struct mame_bitmap *sprite_plane_collbitmap2[3];
+static mame_bitmap *taitosj_tmpbitmap[3];
+static mame_bitmap *sprite_sprite_collbitmap1,*sprite_sprite_collbitmap2;
+static mame_bitmap *sprite_plane_collbitmap1;
+static mame_bitmap *sprite_plane_collbitmap2[3];
 static unsigned char dirtycharacter1[256],dirtycharacter2[256];
 static unsigned char dirtysprite1[64],dirtysprite2[64];
 static int taitosj_video_enable;
 static int flipscreen[2];
-static struct rectangle spritearea[32]; /*areas on bitmap (sprite locations)*/
+static rectangle spritearea[32]; /*areas on bitmap (sprite locations)*/
 static int spriteon[32]; /* 1 if sprite is active */
 
 static int playfield_enable_mask[3] = { 0x10, 0x20, 0x40 };
@@ -102,7 +104,7 @@ static int draworder[32][4];
 
 ***************************************************************************/
 
-void taitosj_vh_convert_color_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *color_prom)
+PALETTE_INIT( taitosj )
 {
 	int i;
 	#define COLOR(gfxn,offs) (colortable[Machine->drv->gfxdecodeinfo[gfxn].color_codes_start + offs])
@@ -145,7 +147,7 @@ void taitosj_vh_convert_color_prom(unsigned char *palette, unsigned short *color
 
 
 
-WRITE_HANDLER( taitosj_paletteram_w )
+WRITE8_HANDLER( taitosj_paletteram_w )
 {
 	int bit0,bit1,bit2;
 	int r,g,b,val;
@@ -185,7 +187,7 @@ WRITE_HANDLER( taitosj_paletteram_w )
   Start the video hardware emulation.
 
 ***************************************************************************/
-int taitosj_vh_start(void)
+VIDEO_START( taitosj )
 {
 	int i;
 
@@ -197,92 +199,45 @@ int taitosj_vh_start(void)
 	dirtybuffer3  = dirtybuffer2 = 0;
 
 
-	if (generic_vh_start() != 0)
+	if (video_start_generic() != 0)
 		return 1;
 
-	if ((dirtybuffer2 = malloc(videoram_size)) == 0)
-	{
-		generic_vh_stop();
+	if ((dirtybuffer2 = auto_malloc(videoram_size)) == 0)
 		return 1;
-	}
 	memset(dirtybuffer2,1,videoram_size);
 
-	if ((dirtybuffer3 = malloc(videoram_size)) == 0)
-	{
-		generic_vh_stop();
+	if ((dirtybuffer3 = auto_malloc(videoram_size)) == 0)
 		return 1;
-	}
 	memset(dirtybuffer3,1,videoram_size);
 
-	if ((sprite_plane_collbitmap1 = bitmap_alloc(16,16)) == 0)
-	{
-		generic_vh_stop();
+	if ((sprite_plane_collbitmap1 = auto_bitmap_alloc(16,16)) == 0)
 		return 1;
-	}
 
 	for (i = 0; i < 3; i++)
 	{
-		if ((taitosj_tmpbitmap[i] = bitmap_alloc(Machine->drv->screen_width,Machine->drv->screen_height)) == 0)
-		{
-			generic_vh_stop();
+		if ((taitosj_tmpbitmap[i] = auto_bitmap_alloc(Machine->drv->screen_width,Machine->drv->screen_height)) == 0)
 			return 1;
-		}
 
-		if ((sprite_plane_collbitmap2[i] = bitmap_alloc(Machine->drv->screen_width,Machine->drv->screen_height)) == 0)
-		{
-			generic_vh_stop();
+		if ((sprite_plane_collbitmap2[i] = auto_bitmap_alloc(Machine->drv->screen_width,Machine->drv->screen_height)) == 0)
 			return 1;
-		}
-
 	}
 
-	if ((sprite_sprite_collbitmap1 = bitmap_alloc(32,32)) == 0)
-	{
-		generic_vh_stop();
+	if ((sprite_sprite_collbitmap1 = auto_bitmap_alloc(32,32)) == 0)
 		return 1;
-	}
 
-	if ((sprite_sprite_collbitmap2 = bitmap_alloc(32,32)) == 0)
-	{
-		generic_vh_stop();
+	if ((sprite_sprite_collbitmap2 = auto_bitmap_alloc(32,32)) == 0)
 		return 1;
-	}
 
 	flipscreen[0] = flipscreen[1] = 0;
 
+	taitosj_spritebank = spriteram;
+
 	return 0;
 }
 
 
 
-/***************************************************************************
-
-  Stop the video hardware emulation.
-
-***************************************************************************/
-void taitosj_vh_stop(void)
-{
-	int i;
-
-
-	if (sprite_sprite_collbitmap2) bitmap_free(sprite_sprite_collbitmap2);
-	if (sprite_sprite_collbitmap1) bitmap_free(sprite_sprite_collbitmap1);
-	if (sprite_plane_collbitmap1) bitmap_free(sprite_plane_collbitmap1);
-
-	for (i = 0; i < 3; i++)
-	{
-		if (taitosj_tmpbitmap[i]) bitmap_free(taitosj_tmpbitmap[i]);
-		if (sprite_plane_collbitmap2[i]) bitmap_free(sprite_plane_collbitmap2[i]);
-	}
-
-	if (dirtybuffer3) free(dirtybuffer3);
-	if (dirtybuffer2) free(dirtybuffer2);
-	generic_vh_stop();
-}
-
-
-
-READ_HANDLER( taitosj_gfxrom_r )
+READ8_HANDLER( taitosj_gfxrom_r )
 {
 	int offs;
 
@@ -298,7 +253,7 @@ READ_HANDLER( taitosj_gfxrom_r )
 }
 
 
-WRITE_HANDLER( taitosj_videoram2_w )
+WRITE8_HANDLER( taitosj_videoram2_w )
 {
 	if (taitosj_videoram2[offset] != data)
 	{
@@ -310,7 +265,7 @@ WRITE_HANDLER( taitosj_videoram2_w )
 
 
 
-WRITE_HANDLER( taitosj_videoram3_w )
+WRITE8_HANDLER( taitosj_videoram3_w )
 {
 	if (taitosj_videoram3[offset] != data)
 	{
@@ -322,7 +277,7 @@ WRITE_HANDLER( taitosj_videoram3_w )
 
 
 
-WRITE_HANDLER( taitosj_colorbank_w )
+WRITE8_HANDLER( taitosj_colorbank_w )
 {
 	if (taitosj_colorbank[offset] != data)
 	{
@@ -337,7 +292,7 @@ logerror("colorbank %d = %02x\n",offset,data);
 
 
 
-WRITE_HANDLER( taitosj_videoenable_w )
+WRITE8_HANDLER( taitosj_videoenable_w )
 {
 	if (taitosj_video_enable != data)
 	{
@@ -353,13 +308,22 @@ logerror("videoenable = %02x\n",data);
 			memset(dirtybuffer3,1,videoram_size);
 		}
 
+		/*OBJEX sprite bank select */
+		if( data & 4 )
+			taitosj_spritebank = spriteram_2;
+		else
+			taitosj_spritebank = spriteram;
+
+		if ((taitosj_video_enable & 4) != (data & 4))
+			logerror( "sprite bank[%d]\n", (data & 4) >> 2 );
+
 		taitosj_video_enable = data;
 	}
 }
 
 
 
-WRITE_HANDLER( taitosj_characterram_w )
+WRITE8_HANDLER( taitosj_characterram_w )
 {
 	if (taitosj_characterram[offset] != data)
 	{
@@ -378,7 +342,7 @@ WRITE_HANDLER( taitosj_characterram_w )
 	}
 }
 
-WRITE_HANDLER( junglhbr_characterram_w )
+WRITE8_HANDLER( junglhbr_characterram_w )
 {
 	taitosj_characterram_w(offset, data ^ 0xfc);
 }
@@ -389,12 +353,12 @@ WRITE_HANDLER( junglhbr_characterram_w )
   collision detection.
 
 ***************************************************************************/
-READ_HANDLER( taitosj_collision_reg_r )
+READ8_HANDLER( taitosj_collision_reg_r )
 {
 	return taitosj_collision_reg[offset];
 }
 
-WRITE_HANDLER( taitosj_collision_reg_clear_w )
+WRITE8_HANDLER( taitosj_collision_reg_clear_w )
 {
 	taitosj_collision_reg[0] = 0;
 	taitosj_collision_reg[1] = 0;
@@ -407,8 +371,8 @@ INLINE int get_sprite_xy(UINT8 num, UINT8* sx, UINT8* sy)
 	int offs = num * 4;
 
 
-	*sx = spriteram[offs] - 1;
-	*sy = 240 - spriteram[offs + 1];
+	*sx = taitosj_spritebank[offs] - 1;
+	*sy = 240 - taitosj_spritebank[offs + 1];
 	return (*sy < 240);
 }
 
@@ -448,17 +412,17 @@ static int check_sprite_sprite_bitpattern(int sx1, int sy1, int num1,
 	}
 
 	/* draw the sprites into seperate bitmaps and check overlapping region */
-	drawgfx(sprite_sprite_collbitmap1,Machine->gfx[(spriteram[offs1 + 3] & 0x40) ? 3 : 1],
-			spriteram[offs1 + 3] & 0x3f,
+	drawgfx(sprite_sprite_collbitmap1,Machine->gfx[(taitosj_spritebank[offs1 + 3] & 0x40) ? 3 : 1],
+			taitosj_spritebank[offs1 + 3] & 0x3f,
 			0,
-			spriteram[offs1 + 2] & 1, spriteram[offs1 + 2] & 2,
+			taitosj_spritebank[offs1 + 2] & 1, taitosj_spritebank[offs1 + 2] & 2,
 			sx1,sy1,
 			0,TRANSPARENCY_NONE,0);
 
-	drawgfx(sprite_sprite_collbitmap2,Machine->gfx[(spriteram[offs2 + 3] & 0x40) ? 3 : 1],
-			spriteram[offs2 + 3] & 0x3f,
+	drawgfx(sprite_sprite_collbitmap2,Machine->gfx[(taitosj_spritebank[offs2 + 3] & 0x40) ? 3 : 1],
+			taitosj_spritebank[offs2 + 3] & 0x3f,
 			0,
-			spriteram[offs2 + 2] & 1, spriteram[offs2 + 2] & 2,
+			taitosj_spritebank[offs2 + 2] & 1, taitosj_spritebank[offs2 + 2] & 2,
 			sx2,sy2,
 			0,TRANSPARENCY_NONE,0);
 
@@ -483,38 +447,50 @@ static void check_sprite_sprite_collision(void)
 	UINT8 i,j,sx1,sx2,sy1,sy2;
 
 
-	/* chech each pair of sprites */
-	for (i = 0x00; i < 0x20; i++)
+	if (taitosj_video_enable & 0x80)	/* check OBJOFF */
 	{
-		if ((i >= 0x10) && (i <= 0x17)) continue;	/* no sprites here */
-
-		if (get_sprite_xy(i, &sx1, &sy1))
+		/* chech each pair of sprites */
+		for (i = 0x00; i < 0x20; i++)
 		{
-			for (j = 0x00; j < 0x20; j++)
+			if ((i >= 0x10) && (i <= 0x17)) continue;	/* no sprites here */
+
+			if (get_sprite_xy(i, &sx1, &sy1))
 			{
-				if (j >= i)	 break;		/* only check a pair once and don't check against itself */
-
-				if ((j >= 0x10) && (j <= 0x17)) continue;	  /* no sprites here */
-
-				if (get_sprite_xy(j, &sx2, &sy2))
+				for (j = i+1; j < 0x20; j++)
 				{
-					/* rule out any pairs that cannot be touching */
-					if ((abs((INT8)sx1 - (INT8)sx2) < 16) &&
-						(abs((INT8)sy1 - (INT8)sy2) < 16))
+					if ((j >= 0x10) && (j <= 0x17)) continue;	  /* no sprites here */
+
+					if (get_sprite_xy(j, &sx2, &sy2))
 					{
-						if (check_sprite_sprite_bitpattern(sx1, sy1, i, sx2, sy2, j))
+						/* rule out any pairs that cannot be touching */
+						if ((abs((INT8)sx1 - (INT8)sx2) < 16) &&
+							(abs((INT8)sy1 - (INT8)sy2) < 16))
 						{
-							/* mark sprites as collided */
-							int reg1,reg2;
+							if (check_sprite_sprite_bitpattern(sx1, sy1, i, sx2, sy2, j))
+							{
+								/* mark sprite as collided */
+								/* note that only the sprite with the higher number is marked */
+								/* as collided. This is how the hardware works and required */
+								/* by Pirate Pete to be able to finish the last round. */
+								int reg;
 
-							reg1 = i >> 3;
-							if (reg1 == 3)  reg1 = 2;
+								/* Here I am mimicking what I do in the sprite drawing code. */
+								/* The last sprite has to be moved at the start of the list. */
+								if (j == 0x1f)
+								{
+									reg = i >> 3;
+									if (reg == 3)  reg = 2;
 
-							reg2 = j >> 3;
-							if (reg2 == 3)  reg2 = 2;
+									taitosj_collision_reg[reg] |= (1 << (i & 0x07));
+								}
+								else
+								{
+									reg = j >> 3;
+									if (reg == 3)  reg = 2;
 
-							taitosj_collision_reg[reg1] |= (1 << (i & 0x07));
-							taitosj_collision_reg[reg2] |= (1 << (j & 0x07));
+									taitosj_collision_reg[reg] |= (1 << (j & 0x07));
+								}
+							}
 						}
 					}
 				}
@@ -535,6 +511,11 @@ static void calculate_sprites_areas(void)
 
 		if (get_sprite_xy(i, &sx, &sy))
 		{
+			if (flipscreen[0])
+				sx = 238 - sx;
+			if (flipscreen[1])
+				sy = 242 - sy;
+
 			minx = sx;
 			miny = sy;
 
@@ -578,25 +559,17 @@ static int check_sprite_plane_bitpattern(int num)
 	maxy = spritearea[num].max_y + 1;
 
 
-	flipx = spriteram[offs + 2] & 1;
-	flipy = spriteram[offs + 2] & 2;
+	flipx = taitosj_spritebank[offs + 2] & 1;
+	flipy = taitosj_spritebank[offs + 2] & 2;
 
 	if (flipscreen[0])
-	{
 		flipx = !flipx;
-		minx = MIN(minx + 2, Machine->drv->screen_width);
-		maxx = MIN(maxx + 2, Machine->drv->screen_width);
-	}
 	if (flipscreen[1])
-	{
 		flipy = !flipy;
-		miny = MIN(miny + 2, Machine->drv->screen_height);
-		maxy = MIN(maxy + 2, Machine->drv->screen_height);
-	}
 
 	/* draw sprite into a bitmap and check if playfields collide */
-	drawgfx(sprite_plane_collbitmap1, Machine->gfx[(spriteram[offs + 3] & 0x40) ? 3 : 1],
-			spriteram[offs + 3] & 0x3f,
+	drawgfx(sprite_plane_collbitmap1, Machine->gfx[(taitosj_spritebank[offs + 3] & 0x40) ? 3 : 1],
+			taitosj_spritebank[offs + 3] & 0x3f,
 			0,
 			flipx, flipy,
 			0,0,
@@ -640,31 +613,50 @@ static void check_sprite_plane_collision(void)
 	UINT8 i;
 
 
-	/* check each sprite */
-	for (i = 0x00; i < 0x20; i++)
+	if (taitosj_video_enable & 0x80)	/* check OBJOFF */
 	{
-		if ((i >= 0x10) && (i <= 0x17)) continue;	/* no sprites here */
-
-		if (spriteon[i])
+		/* check each sprite */
+		for (i = 0x00; i < 0x20; i++)
 		{
-			taitosj_collision_reg[3] |= check_sprite_plane_bitpattern(i);
+			if ((i >= 0x10) && (i <= 0x17)) continue;	/* no sprites here */
+
+			if (spriteon[i])
+			{
+				taitosj_collision_reg[3] |= check_sprite_plane_bitpattern(i);
+			}
 		}
 	}
 }
 
 
-static void drawsprites(struct mame_bitmap *bitmap)
+static void drawsprites(mame_bitmap *bitmap)
 {
-	/* Draw the sprites. Note that it is important to draw them exactly in this */
-	/* order, to have the correct priorities (but they are still wrong sometimes.) */
-	if (taitosj_video_enable & 0x80)
+	/* sprite visibility area is missing 4 pixels from the sides, surely to reduce
+       wraparound side effects. This was verified on a real Elevator Action.
+       Note that the clipping is asymmetrical. This matches the real thing.
+       I'm not sure of what should happen when the screen is flipped, though.
+     */
+	static rectangle spritevisiblearea =
 	{
-		int offs;
+		0*8+3, 32*8-1-1,
+		2*8, 30*8-1
+	};
+	static rectangle spritevisibleareaflip =
+	{
+		0*8+1, 32*8-3-1,
+		2*8, 30*8-1
+	};
+
+	if (taitosj_video_enable & 0x80)	/* check OBJOFF */
+	{
+		int sprite;
 
 
-		for (offs = spriteram_size - 4;offs >= 0;offs -= 4)
+		/* drawing order is a bit strange. The last sprite has to be moved at the start of the list. */
+		for (sprite = 0x1f;sprite >= 0;sprite--)
 		{
 			UINT8 sx,sy,flipx,flipy;
+			int offs = ((sprite - 1) & 0x1f) * 4;	/* move last sprite at the head of the list */
 
 
 			if ((offs >= 0x40) && (offs <= 0x5f))  continue;	/* no sprites here */
@@ -672,8 +664,8 @@ static void drawsprites(struct mame_bitmap *bitmap)
 
 			if (get_sprite_xy(offs / 4, &sx, &sy))
 			{
-				flipx = spriteram[offs + 2] & 1;
-				flipy = spriteram[offs + 2] & 2;
+				flipx = taitosj_spritebank[offs + 2] & 1;
+				flipy = taitosj_spritebank[offs + 2] & 2;
 				if (flipscreen[0])
 				{
 					sx = 238 - sx;
@@ -685,27 +677,27 @@ static void drawsprites(struct mame_bitmap *bitmap)
 					flipy = !flipy;
 				}
 
-				drawgfx(bitmap,Machine->gfx[(spriteram[offs + 3] & 0x40) ? 3 : 1],
-						spriteram[offs + 3] & 0x3f,
-						2 * ((taitosj_colorbank[1] >> 4) & 0x03) + ((spriteram[offs + 2] >> 2) & 1),
+				drawgfx(bitmap,Machine->gfx[(taitosj_spritebank[offs + 3] & 0x40) ? 3 : 1],
+						taitosj_spritebank[offs + 3] & 0x3f,
+						2 * ((taitosj_colorbank[1] >> 4) & 0x03) + ((taitosj_spritebank[offs + 2] >> 2) & 1),
 						flipx,flipy,
 						sx,sy,
-						&Machine->visible_area,TRANSPARENCY_PEN,0);
+						flipscreen[0] ? &spritevisibleareaflip : &spritevisiblearea,TRANSPARENCY_PEN,0);
 
 				/* draw with wrap around. The horizontal games (eg. sfposeid) need this */
-				drawgfx(bitmap,Machine->gfx[(spriteram[offs + 3] & 0x40) ? 3 : 1],
-						spriteram[offs + 3] & 0x3f,
-						2 * ((taitosj_colorbank[1] >> 4) & 0x03) + ((spriteram[offs + 2] >> 2) & 1),
+				drawgfx(bitmap,Machine->gfx[(taitosj_spritebank[offs + 3] & 0x40) ? 3 : 1],
+						taitosj_spritebank[offs + 3] & 0x3f,
+						2 * ((taitosj_colorbank[1] >> 4) & 0x03) + ((taitosj_spritebank[offs + 2] >> 2) & 1),
 						flipx,flipy,
 						sx - 0x100,sy,
-						&Machine->visible_area,TRANSPARENCY_PEN,0);
+						flipscreen[0] ? &spritevisibleareaflip : &spritevisiblearea,TRANSPARENCY_PEN,0);
 			}
 		}
 	}
 }
 
 
-static void drawplayfield(int n, struct mame_bitmap *bitmap)
+static void drawplayfield(int n, mame_bitmap *bitmap)
 {
 	static int fudge1[3] = { 3,  1, -1 };
 	static int fudge2[3] = { 8, 10, 12 };
@@ -744,8 +736,47 @@ static void drawplayfield(int n, struct mame_bitmap *bitmap)
 	}
 }
 
+static void kikstart_drawplayfield(int n, mame_bitmap *bitmap)
+{
+	if (taitosj_video_enable & playfield_enable_mask[n])
+	{
+		int i,scrolly,scrollx[32*8];
 
-static void drawplane(int n,struct mame_bitmap *bitmap)
+		for (i = 1;i < 32*8;i++)/* 1-255 ! */
+		{
+			if(flipscreen[1])
+			{
+				switch(n)
+				{
+					case 0:	scrollx[32*8-i] = 0 ;break;
+					case 1:	scrollx[32*8-i] = kikstart_scrollram[i]+((taitosj_scroll[2*n]+0xa)&0xff);break;
+					case 2:	scrollx[32*8-i] = kikstart_scrollram[0x100+i]+((taitosj_scroll[2*n]+0xc)&0xff);break;
+				}
+			}
+			else
+			{
+				switch(n)
+				{
+					case 0:	scrollx[i] = 0 ;break;
+					case 1:	scrollx[i] = 0xff-kikstart_scrollram[i-1]-((taitosj_scroll[2*n]-0x10)&0xff);break;
+					case 2:	scrollx[i] = 0xff-kikstart_scrollram[0x100+i-1]-((taitosj_scroll[2*n]-0x12)&0xff);break;
+				}
+			}
+		}
+		scrolly=taitosj_scroll[2*n+1];/*always 0 ? */
+		copyscrollbitmap(bitmap,taitosj_tmpbitmap[n],32*8,scrollx,1,&scrolly,&Machine->visible_area,TRANSPARENCY_COLOR,0);
+		/* store parts covered with sprites for sprites/playfields collision detection */
+		for (i=0x00; i<0x20; i++)
+		{
+			if ((i >= 0x10) && (i <= 0x17)) continue; /* no sprites here */
+			if (spriteon[i])
+				copyscrollbitmap( sprite_plane_collbitmap2[n],taitosj_tmpbitmap[n],32*8,scrollx,1,&scrolly,&spritearea[i],TRANSPARENCY_NONE,0);
+		}
+	}
+}
+
+
+static void drawplane(int n,mame_bitmap *bitmap)
 {
 	switch (n)
 	{
@@ -755,15 +786,18 @@ static void drawplane(int n,struct mame_bitmap *bitmap)
 	case 1:
 	case 2:
 	case 3:
-		drawplayfield(n-1,bitmap);
+		if(!strcmp(Machine->gamedrv->name,"kikstart"))
+			kikstart_drawplayfield(n-1,bitmap);/*line scroll */
+		else
+			drawplayfield(n-1,bitmap);/*tile scroll */
 		break;
 	}
 }
 
 
-void taitosj_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+VIDEO_UPDATE( taitosj )
 {
-	int offs,i;
+	int offs,i,alldirty = 0;
 
 
 	/* decode modified characters */
@@ -773,13 +807,24 @@ void taitosj_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
 		{
 			decodechar(Machine->gfx[0],offs,taitosj_characterram,Machine->drv->gfxdecodeinfo[0].gfxlayout);
 			dirtycharacter1[offs] = 0;
+			alldirty = 1;
 		}
 		if (dirtycharacter2[offs] == 1)
 		{
 			decodechar(Machine->gfx[2],offs,taitosj_characterram + 0x1800,Machine->drv->gfxdecodeinfo[2].gfxlayout);
 			dirtycharacter2[offs] = 0;
+			alldirty = 1;
 		}
 	}
+
+	/* if characters changed, redraw everything */
+	if (alldirty)
+	{
+		memset(dirtybuffer, 1,videoram_size);
+		memset(dirtybuffer2,1,videoram_size);
+		memset(dirtybuffer3,1,videoram_size);
+	}
+
 	/* decode modified sprites */
 	for (offs = 0;offs < 64;offs++)
 	{
