@@ -44,17 +44,23 @@
 
 static YGV608 ygv608;
 
-static struct tilemap *tilemap_A = NULL;
-static struct tilemap *tilemap_B = NULL;
-static struct mame_bitmap *work_bitmap = NULL;
+static tilemap *tilemap_A_cache_8[3];
+static tilemap *tilemap_A_cache_16[3];
+static tilemap *tilemap_B_cache_8[3];
+static tilemap *tilemap_B_cache_16[3];
+static tilemap *tilemap_A = NULL;
+static tilemap *tilemap_B = NULL;
+static mame_bitmap *work_bitmap = NULL;
 
 #ifdef MAME_DEBUG
 static void ShowYGV608Registers( void );
+#if 0
 static void dump_block( char *name, unsigned char *block, int len );
 #endif
+#endif
 
 /* interrupt generated every 1ms second */
-int ygv608_timed_interrupt( void )
+INTERRUPT_GEN( ygv608_timed_interrupt )
 {
 /*
     this is not quite generic, because we trigger a 68k interrupt
@@ -72,18 +78,16 @@ int ygv608_timed_interrupt( void )
     {
 		ygv608.ports.s.p6 |= p6_fv;;
 		if (ygv608.regs.s.r14 & r14_iev)
-			return( m68_level2_irq() );
+			irq2_line_hold();
 	}
 
 	/* once every 60Hz, set the position detection flag (somewhere) */
-	if( ( timer % (1000/60) ) == 7 )
+	else if( ( timer % (1000/60) ) == 7 )
 	{
 		ygv608.ports.s.p6 |= p6_fp;
 		if (ygv608.regs.s.r14 & r14_iep)
-			return( m68_level2_irq() );
+			irq2_line_hold();
 	}
-
-	return( 0 );
 }
 
 
@@ -122,6 +126,7 @@ static void get_tile_info_A_8( int offset )
 		int sx, sy, page;
 		int i = pattern_name_base + (((row << ygv608.pny_shift) + col) << ygv608.bits16);
 		int j = ygv608.pattern_name_table[i];
+        int f = 0;
 
 		if( ygv608.bits16 )
 		{
@@ -129,6 +134,12 @@ static void get_tile_info_A_8( int offset )
 			/* attribute only valid in 16 color mode */
 			if (set == GFX_8X8_4BIT)
 				attr = ygv608.pattern_name_table[i+1] >> 4;
+
+            if (ygv608.regs.s.r7 & r7_flip)
+            {
+                if (ygv608.pattern_name_table[i+1] & (1<<3)) f |= TILE_FLIPX;
+                if (ygv608.pattern_name_table[i+1] & (1<<2)) f |= TILE_FLIPY;
+            }
 		}
 
 		/* calculate page according to scroll data */
@@ -169,7 +180,17 @@ static void get_tile_info_A_8( int offset )
 			if( set == GFX_8X8_4BIT )
 				attr = ( j >> ( ((ygv608.regs.s.r12 & r12_apf) - 1 ) * 2 ) ) & 0x0f;
 		}
-		SET_TILE_INFO( set, j, attr & 0x0F, 0 );
+		/* banking */
+		if (set == GFX_8X8_4BIT)
+		{
+			j += namcond1_gfxbank * 0x10000;
+		}
+		else /* 8x8x8 */
+		{
+			j += namcond1_gfxbank * 0x8000;
+		}
+
+		SET_TILE_INFO( set, j, attr & 0x0F, f );
 	}
 }
 
@@ -202,11 +223,18 @@ static void get_tile_info_B_8( int offset )
 		int sx, sy, page;
 		int i = pattern_name_base + (((row << ygv608.pny_shift) + col) << ygv608.bits16);
 		int j = ygv608.pattern_name_table[i];
+        int f = 0;
 
 		if (ygv608.bits16)
 		{
 			j += ((int)(ygv608.pattern_name_table[i+1] & ygv608.na8_mask )) << 8;
 			attr = ygv608.pattern_name_table[i+1] >> 4; /*& 0x00; 0xf0;*/
+
+            if (ygv608.regs.s.r7 & r7_flip)
+            {
+                if (ygv608.pattern_name_table[i+1] & (1<<3)) f |= TILE_FLIPX;
+                if (ygv608.pattern_name_table[i+1] & (1<<2)) f |= TILE_FLIPY;
+            }
 		}
 
 		/* calculate page according to scroll data */
@@ -248,7 +276,18 @@ static void get_tile_info_B_8( int offset )
 			/* assume 16 colour mode for now... */
 			attr = ( j >> ( (color - 1 ) * 2 ) ) & 0x0f;
 		}
-		SET_TILE_INFO( set, j, attr, 0 );
+
+		/* banking */
+		if (set == GFX_8X8_4BIT)
+		{
+			j += namcond1_gfxbank * 0x10000;
+		}
+		else /* 8x8x8 */
+		{
+			j += namcond1_gfxbank * 0x8000;
+		}
+
+		SET_TILE_INFO( set, j, attr, f );
 	}
 }
 
@@ -274,6 +313,7 @@ static void get_tile_info_A_16( int offset )
     int sx, sy, page;
     int j;
     int i = ( ( ( row << ygv608.pny_shift ) + col ) << ygv608.bits16 );
+    int f = 0;
     i += pattern_name_base;
 
     j = ygv608.pattern_name_table[i];
@@ -282,6 +322,12 @@ static void get_tile_info_A_16( int offset )
       /* attribute only valid in 16 color mode */
       if( set == GFX_16X16_4BIT )
         attr = ygv608.pattern_name_table[i+1] >> 4;
+
+      if (ygv608.regs.s.r7 & r7_flip)
+      {
+        if (ygv608.pattern_name_table[i+1] & (1<<3)) f |= TILE_FLIPX;
+        if (ygv608.pattern_name_table[i+1] & (1<<2)) f |= TILE_FLIPY;
+      }
     }
 
     /* calculate page according to scroll data */
@@ -319,7 +365,18 @@ static void get_tile_info_A_16( int offset )
 				attr = ( j >> ( ((ygv608.regs.s.r12 & r12_apf)) * 2 ) ) & 0x0f;
 		}
 
-		SET_TILE_INFO( set, j, attr, 0 );
+		/* banking */
+		if (set == GFX_16X16_4BIT)
+		{
+			j += namcond1_gfxbank * 0x4000;
+		}
+		else /* 8x8x8 */
+		{
+			j += namcond1_gfxbank * 0x2000;
+		}
+
+
+		SET_TILE_INFO( set, j, attr, f );
 	}
 }
 
@@ -348,12 +405,19 @@ static void get_tile_info_B_16( int offset )
     int sx, sy, page;
     int j;
     int i = ( ( ( row << ygv608.pny_shift ) + col ) << ygv608.bits16 );
+    int f = 0;
     i += pattern_name_base;
 
     j = ygv608.pattern_name_table[i];
     if( ygv608.bits16 ) {
       j += ((int)(ygv608.pattern_name_table[i+1] & ygv608.na8_mask )) << 8;
       attr = ygv608.pattern_name_table[i+1] >> 4; /*& 0x00; 0xf0;*/
+
+      if (ygv608.regs.s.r7 & r7_flip)
+      {
+        if (ygv608.pattern_name_table[i+1] & (1<<3)) f |= TILE_FLIPX;
+        if (ygv608.pattern_name_table[i+1] & (1<<2)) f |= TILE_FLIPY;
+      }
     }
 
     /* calculate page according to scroll data */
@@ -391,36 +455,64 @@ static void get_tile_info_B_16( int offset )
 			/* assume 16 colour mode for now... */
 			attr = ( j >> (color * 2)) & 0x0f;
 		}
-		SET_TILE_INFO( set, j, attr, 0 );
+
+		/* banking */
+		if (set == GFX_16X16_4BIT)
+		{
+			j += namcond1_gfxbank * 0x4000;
+		}
+		else /* 8x8x8 */
+		{
+			j += namcond1_gfxbank * 0x2000;
+		}
+
+
+		SET_TILE_INFO( set, j, attr, f );
 	}
 }
 
-int ygv608_vh_start(void)
+VIDEO_START( ygv608 )
 {
 	memset( &ygv608, 0, sizeof(ygv608) );
 
 	/* flag rebuild of the tilemaps */
 	ygv608.screen_resize = 1;
 	ygv608.tilemap_resize = 1;
+	namcond1_gfxbank = 0;
+
+	/* create tilemaps of all sizes and combinations */
+	tilemap_A_cache_8[0] = tilemap_create(get_tile_info_A_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 32,32);
+	tilemap_A_cache_8[1] = tilemap_create(get_tile_info_A_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 64,32);
+	tilemap_A_cache_8[2] = tilemap_create(get_tile_info_A_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 32,64);
+
+	tilemap_A_cache_16[0] = tilemap_create(get_tile_info_A_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 32,32);
+	tilemap_A_cache_16[1] = tilemap_create(get_tile_info_A_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 64,32);
+	tilemap_A_cache_16[2] = tilemap_create(get_tile_info_A_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 32,64);
+
+	tilemap_B_cache_8[0] = tilemap_create(get_tile_info_B_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 32,32);
+	tilemap_B_cache_8[1] = tilemap_create(get_tile_info_B_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 64,32);
+	tilemap_B_cache_8[2] = tilemap_create(get_tile_info_B_8, get_tile_offset, TILEMAP_TRANSPARENT, 8,8, 32,64);
+
+	tilemap_B_cache_16[0] = tilemap_create(get_tile_info_B_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 32,32);
+	tilemap_B_cache_16[1] = tilemap_create(get_tile_info_B_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 64,32);
+	tilemap_B_cache_16[2] = tilemap_create(get_tile_info_B_16, get_tile_offset, TILEMAP_TRANSPARENT, 16,16, 32,64);
 
 	return 0;
 }
 
-void ygv608_vh_stop(void)
+VIDEO_STOP( ygv608 )
 {
-#ifdef _ENABLE_ROTATE_ZOOM
-	if( work_bitmap )
-		bitmap_free( work_bitmap );
-#endif
+	tilemap_A = NULL;
+	tilemap_B = NULL;
 }
 
-static void draw_sprites( struct mame_bitmap *bitmap )
+static void draw_sprites( mame_bitmap *bitmap, const rectangle *cliprect )
 {
 #ifdef _ENABLE_SPRITES
 
   /* sprites are always clipped to 512x512 */
   /* - regardless of the visible display dimensions */
-  static struct rectangle spriteClip = { 0, 512, 0, 512 };
+  static rectangle spriteClip = { 0, 512, 0, 512 };
 
   PSPRITE_ATTR sa;
   int flipx = 0, flipy = 0;
@@ -431,6 +523,7 @@ static void draw_sprites( struct mame_bitmap *bitmap )
     return;
 
   /* draw sprites */
+  sect_rect(&spriteClip, cliprect);
   sa = &ygv608.sprite_attribute_table.s[YGV608_MAX_SPRITES-1];
   for( i=0; i<YGV608_MAX_SPRITES; i++, sa-- )
   {
@@ -467,7 +560,7 @@ static void draw_sprites( struct mame_bitmap *bitmap )
 	    code = 0;
       }
       drawgfx( bitmap, Machine->gfx[GFX_8X8_4BIT],
-	       code,
+	       code+namcond1_gfxbank*0x10000,
 	       color,
 	       flipx,flipy,
 	       sx,sy,
@@ -475,14 +568,14 @@ static void draw_sprites( struct mame_bitmap *bitmap )
       /* redraw with wrap-around */
       if( sx > 512-8 )
         drawgfx( bitmap, Machine->gfx[GFX_8X8_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x10000,
 	        color,
 	        flipx,flipy,
 	        sx-512,sy,
 	        &spriteClip,TRANSPARENCY_PEN,0x00);
       if( sy > 512-8 )
         drawgfx( bitmap, Machine->gfx[GFX_8X8_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x10000,
 	        color,
 	        flipx,flipy,
 	        sx,sy-512,
@@ -500,7 +593,7 @@ static void draw_sprites( struct mame_bitmap *bitmap )
 	    code = 0;
       }
       drawgfx( bitmap, Machine->gfx[GFX_16X16_4BIT],
-	       code,
+	       code+namcond1_gfxbank*0x4000,
 	       color,
 	       flipx,flipy,
 	       sx,sy,
@@ -508,14 +601,14 @@ static void draw_sprites( struct mame_bitmap *bitmap )
       /* redraw with wrap-around */
       if( sx > 512-16 )
         drawgfx( bitmap, Machine->gfx[GFX_16X16_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x4000,
 	        color,
 	        flipx,flipy,
 	        sx-512,sy,
 	        &spriteClip,TRANSPARENCY_PEN,0x00);
       if( sy > 512-16 )
         drawgfx( bitmap, Machine->gfx[GFX_16X16_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x4000,
 	        color,
 	        flipx,flipy,
 	        sx,sy-512,
@@ -533,7 +626,7 @@ static void draw_sprites( struct mame_bitmap *bitmap )
 	code = 0;
       }
       drawgfx( bitmap, Machine->gfx[GFX_32X32_4BIT],
-	       code,
+	       code+namcond1_gfxbank*0x1000,
 	       color,
 	       flipx,flipy,
 	       sx,sy,
@@ -541,14 +634,14 @@ static void draw_sprites( struct mame_bitmap *bitmap )
       /* redraw with wrap-around */
       if( sx > 512-32 )
         drawgfx( bitmap, Machine->gfx[GFX_32X32_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x1000,
 	        color,
 	        flipx,flipy,
 	        sx-512,sy,
 	        &spriteClip,TRANSPARENCY_PEN,0x00);
       if( sy > 512-32 )
         drawgfx( bitmap, Machine->gfx[GFX_32X32_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x1000,
 	        color,
 	        flipx,flipy,
 	        sx,sy-512,
@@ -566,7 +659,7 @@ static void draw_sprites( struct mame_bitmap *bitmap )
 	    code = 0;
       }
       drawgfx( bitmap, Machine->gfx[GFX_64X64_4BIT],
-	       code,
+	       code+namcond1_gfxbank*0x400,
 	       color,
 	       flipx,flipy,
 	       sx,sy,
@@ -574,14 +667,14 @@ static void draw_sprites( struct mame_bitmap *bitmap )
       /* redraw with wrap-around */
       if( sx > 512-64 )
         drawgfx( bitmap, Machine->gfx[GFX_64X64_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x400,
 	        color,
 	        flipx,flipy,
 	        sx-512,sy,
 	        &spriteClip,TRANSPARENCY_PEN,0x00);
       if( sy > 512-64 )
         drawgfx( bitmap, Machine->gfx[GFX_64X64_4BIT],
-	        code,
+	        code+namcond1_gfxbank*0x400,
 	        color,
 	        flipx,flipy,
 	        sx,sy-512,
@@ -607,7 +700,7 @@ static char *mode[] = { "2PLANE_8BIT",
 static char *psize[] = { "8x8", "16x16", "32x32", "64x64" };
 #endif
 
-void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
+VIDEO_UPDATE( ygv608 )
 {
 #ifdef _SHOW_VIDEO_DEBUG
     char buffer[64];
@@ -617,6 +710,22 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
     int xc, yc;
     double r, alpha, sin_theta, cos_theta;
 #endif
+	rectangle finalclip;
+
+	/* clip to the current bitmap */
+	finalclip.min_x = 0;
+	finalclip.max_x = Machine->drv->screen_width - 1;
+	finalclip.min_y = 0;
+	finalclip.max_y = Machine->drv->screen_height - 1;
+	sect_rect(&finalclip, cliprect);
+	cliprect = &finalclip;
+
+	/* punt if not initialized */
+	if (ygv608.page_x == 0 || ygv608.page_y == 0)
+	{
+		fillbitmap(bitmap, 0, cliprect);
+		return;
+	}
 
 	if( ygv608.screen_resize )
 	{
@@ -630,7 +739,7 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
 #ifdef _ENABLE_ROTATE_ZOOM
 		if( work_bitmap )
 			bitmap_free( work_bitmap );
-		work_bitmap = bitmap_alloc_depth( Machine->drv->screen_width,
+		work_bitmap = auto_bitmap_alloc_depth( Machine->drv->screen_width,
 										  Machine->drv->screen_height,
 										  Machine->color_depth );
 #else
@@ -643,54 +752,37 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
 
 	if( ygv608.tilemap_resize )
 	{
+		int index;
 
-		if (tilemap_A)
-		{
-			tilemap_dispose( tilemap_A );
-			tilemap_A = NULL;
-		}
+		/* based on the page sizes, pick an index */
+		if (ygv608.page_x == 64)
+			index = 1;
+		else if (ygv608.page_y == 64)
+			index = 2;
+		else
+			index = 0;
 
 		if ((ygv608.regs.s.r9 & r9_pts) == PTS_8X8 )
-			tilemap_A = tilemap_create( get_tile_info_A_8,
-										get_tile_offset,
-										TILEMAP_TRANSPARENT,
-										8, 8,
-										ygv608.page_x, ygv608.page_y );
+			tilemap_A = tilemap_A_cache_8[index];
 		else
-			tilemap_A = tilemap_create( get_tile_info_A_16,
-										get_tile_offset,
-										TILEMAP_TRANSPARENT,
-										16, 16,
-										ygv608.page_x, ygv608.page_y );
+			tilemap_A = tilemap_A_cache_16[index];
+		tilemap_mark_all_tiles_dirty(tilemap_A);
 
 		tilemap_set_transparent_pen( tilemap_A, 0 );
 		/* for NCV1 it's sufficient to scroll only columns */
 		tilemap_set_scroll_cols( tilemap_A, ygv608.page_x );
 
-		if( tilemap_B )
-		{
-			tilemap_dispose( tilemap_B );
-			tilemap_B = NULL;
-		}
-
 		if ((ygv608.regs.s.r9 & r9_pts) == PTS_8X8 )
-			tilemap_B = tilemap_create( get_tile_info_B_8,
-										get_tile_offset,
-										TILEMAP_OPAQUE,
-										8, 8,
-										ygv608.page_x, ygv608.page_y );
+			tilemap_B = tilemap_B_cache_8[index];
 		else
-			tilemap_B = tilemap_create( get_tile_info_B_16,
-										get_tile_offset,
-										TILEMAP_OPAQUE,
-										16, 16,
-										ygv608.page_x, ygv608.page_y );
+			tilemap_B = tilemap_B_cache_16[index];
+		tilemap_mark_all_tiles_dirty(tilemap_B);
 
 		/* for NCV1 it's sufficient to scroll only columns */
 		tilemap_set_scroll_cols( tilemap_B, ygv608.page_x );
 
 		/* now clear the screen in case we change to 1-plane mode */
-		fillbitmap( work_bitmap, Machine->pens[0], &Machine->visible_area );
+		fillbitmap( work_bitmap, Machine->pens[0], cliprect );
 
 		/* reset resize flag */
 		ygv608.tilemap_resize = 0;
@@ -734,8 +826,8 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
 
 
 	/*
-	 *    now we can render the screen
-	 */
+     *    now we can render the screen
+     */
 
 #if 1
 	/* LBO - need to implement proper pen marking for sprites as well as set aside a non-transparent */
@@ -743,12 +835,12 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
 	if ((ygv608.regs.s.r7 & r7_md) & MD_1PLANE)
 	{
 		/* If the background tilemap is disabled, we need to clear the bitmap to black */
-		fillbitmap (work_bitmap,Machine->pens[0],&Machine->visible_area);
-/*		fillbitmap (work_bitmap,1,&Machine->visible_area); */
+		fillbitmap (work_bitmap,Machine->pens[0],cliprect);
+/*      fillbitmap (work_bitmap,1,&Machine->visible_area); */
 	}
 	else
 #endif
-		tilemap_draw( work_bitmap, tilemap_B, 0, 0 );
+		tilemap_draw( work_bitmap,cliprect, tilemap_B, 0, 0 );
 
 #ifdef _ENABLE_ROTATE_ZOOM
 
@@ -772,11 +864,11 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
                     ygv608.ay + 0x10000 * r *
                     ( cos( alpha ) * cos_theta + sin( alpha ) * sin_theta ),
                    ygv608.dx, ygv608.dxy, ygv608.dyx, ygv608.dy, 0,
-                   &Machine->visible_area,
+                   cliprect,
                    TRANSPARENCY_NONE, 0, 0 );
   else
     copybitmap( bitmap, work_bitmap, 0, 0, 0, 0,
-                &Machine->visible_area,
+                cliprect,
                 TRANSPARENCY_NONE, 0 );
 
   /* for some reason we can't use an opaque tilemap_A */
@@ -789,9 +881,9 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
 
 	if ((ygv608.regs.s.r11 & r11_prm) == PRM_ASBDEX ||
 		(ygv608.regs.s.r11 & r11_prm) == PRM_ASEBDX )
-		draw_sprites( bitmap );
+		draw_sprites( bitmap,cliprect );
 
-	tilemap_draw( work_bitmap, tilemap_A, 0, 0 );
+	tilemap_draw( work_bitmap,cliprect, tilemap_A, 0, 0 );
 
 #ifdef _ENABLE_ROTATE_ZOOM
   if( ygv608.regs.s.zron )
@@ -799,37 +891,37 @@ void ygv608_vh_update( struct mame_bitmap *bitmap, int full_refresh )
                    ygv608.ax, /* + ( Machine->visible_area.min_x << 16 ), */
                    ygv608.ay, /* + ( Machine->visible_area.min_y << 16 ), */
                    ygv608.dx, ygv608.dxy, ygv608.dyx, ygv608.dy, 0,
-                   &Machine->visible_area,
+                   cliprect,
                    TRANSPARENCY_PEN, Machine->pens[0], 0 );
   else
     copybitmap( bitmap, work_bitmap, 0, 0, 0, 0,
-                &Machine->visible_area,
+                cliprect,
                 TRANSPARENCY_PEN, Machine->pens[0] );
 #endif
 
 	if ((ygv608.regs.s.r11 & r11_prm) == PRM_SABDEX ||
 		(ygv608.regs.s.r11 & r11_prm) == PRM_SEABDX)
-		draw_sprites( bitmap );
+		draw_sprites( bitmap,cliprect );
 
 
 #ifdef _SHOW_VIDEO_DEBUG
   /* show screen control information */
-  ui_text( bitmap, mode[(ygv608.regs.s.r7 & r7_md) >> 1], 0, 0 );
+  ui_draw_text( mode[(ygv608.regs.s.r7 & r7_md) >> 1], 0, 0 );
   sprintf( buffer, "%02ux%02u", ygv608.page_x, ygv608.page_y );
-  ui_text( bitmap, buffer, 0, 16 );
-  ui_text( bitmap, psize[(ygv608.regs.s.r9 & r9_pts) >> 6], 0, 32 );
+  ui_draw_text( buffer, 0, 16 );
+  ui_draw_text( psize[(ygv608.regs.s.r9 & r9_pts) >> 6], 0, 32 );
   sprintf( buffer, "A: SX:%d SY:%d",
 	   (int)ygv608.scroll_data_table[0][0x80] +
 	   ( ( (int)ygv608.scroll_data_table[0][0x81] & 0x0f ) << 8 ),
 	   (int)ygv608.scroll_data_table[0][0x00] +
 	   ( ( (int)ygv608.scroll_data_table[0][0x01] & 0x0f ) << 8 ) );
-  ui_text( bitmap, buffer, 0, 48 );
+  ui_draw_text( buffer, 0, 48 );
   sprintf( buffer, "B: SX:%d SY:%d",
 	   (int)ygv608.scroll_data_table[1][0x80] +
 	   ( ( (int)ygv608.scroll_data_table[1][0x81] & 0x0f ) << 8 ),
 	   (int)ygv608.scroll_data_table[1][0x00] +
 	   ( ( (int)ygv608.scroll_data_table[1][0x01] & 0x0f ) << 8 ) );
-  ui_text( bitmap, buffer, 0, 64 );
+  ui_draw_text( buffer, 0, 64 );
 #endif
 }
 
@@ -844,7 +936,7 @@ READ16_HANDLER( ygv608_r )
 	static int p3_state = 0;
 	static int pattern_name_base = 0;  /* pattern name table base address */
 	int pn=0;
-	data16_t  data = 0;
+	UINT16  data = 0;
 
 	switch (offset)
 	{
@@ -987,7 +1079,7 @@ READ16_HANDLER( ygv608_r )
 
 		case 0x06:
 		case 0x07:
-			return( (data16_t)(ygv608.ports.b[offset]) << 8 );
+			return( (UINT16)(ygv608.ports.b[offset]) << 8 );
 
 		default :
 			logerror( "unknown ygv608 register (%d)\n", offset );
@@ -1253,7 +1345,7 @@ void HandleRomTransfers( void )
 #endif
 }
 
-void nvsram( offs_t offset, data16_t data )
+void nvsram( offs_t offset, UINT16 data )
 {
   static int i = 0;
 
@@ -1321,7 +1413,7 @@ void SetPostShortcuts( int reg )
 
 			if (yTile >= ygv608.page_y)
 				logerror ("setting pny(%d) >= page_y(%d) @ $%X\n",
-						yTile, ygv608.page_y, cpu_get_pc() );
+						yTile, ygv608.page_y, activecpu_get_pc() );
 			yTile &= (ygv608.page_y - 1);
 			ygv608.regs.s.r0 &= ~r0_pny;
 			ygv608.regs.s.r0 |= yTile;
@@ -1334,7 +1426,7 @@ void SetPostShortcuts( int reg )
 
 			if (xTile >= ygv608.page_x)
 				logerror ("setting pnx(%d) >= page_x(%d) @ $%X\n",
-						xTile, ygv608.page_x, cpu_get_pc() );
+						xTile, ygv608.page_x, activecpu_get_pc() );
 			xTile &= (ygv608.page_x - 1);
 			ygv608.regs.s.r1 &= ~r1_pnx;
 			ygv608.regs.s.r1 |= xTile;
@@ -1468,6 +1560,7 @@ void SetPostShortcuts( int reg )
 
 #define nSHOW_SOURCE_MODE
 
+#if 0
 void dump_block( char *name, unsigned char *block, int len )
 {
   int i;
@@ -1482,7 +1575,7 @@ void dump_block( char *name, unsigned char *block, int len )
   }
   logerror( "};\n" );
 }
-
+#endif
 READ16_HANDLER( debug_trigger )
 {
   static int oneshot = 0;
@@ -1497,7 +1590,7 @@ READ16_HANDLER( debug_trigger )
   ShowYGV608Registers();
 
 #ifdef SHOW_SOURCE_MODE
-
+#if 0
   dump_block( "ygv608_regs",
 	      (unsigned char *)ygv608.regs.b,
 	      64 );
@@ -1513,6 +1606,7 @@ READ16_HANDLER( debug_trigger )
   dump_block( "ygv608_cp",
 	      (unsigned char *)ygv608.colour_palette,
 	      768 );
+#endif
 
 #else
 
